/*
 * CPUTIME: return CPU time in minutes
 *
 *      28-nov-89   non-VMS version uses HZ variable now
 *                  on most machines (in the us) HZ will be 60
 *		    For SUN and BSD it is defined in
 *			<param.h>
 *	26-may-91   re-introduced the correct Cray code + TOOLBOX from starlab
 *		    Whenever we have POSIX compliant code - this
 *		    nasty piece will be better code...
 *	 4-nov-91   Multiflow _trace_ seems to have funny HZ
 *		    better to use getrusage(2) - Cray doesn't know this
 *	17-feb-94   cleanup; return double, not real
 *	
 */
#include <stdinc.h>

#if !defined(vms)

# include <sys/types.h>
# include <sys/times.h>

# if defined(unicos) || defined(cray)
#   include <sys/machd.h>
# else
#   include <sys/param.h>/* This might defined HZ - if not - set yourself */
			 /* define HZ to be the number of ticks/second */
			 /* Some places will have this defined as 50 or 100 */
			 /* Toshiba uses 100 - Cray has 1/60 ?? */
#endif

#else		/* special for Vomit Made System */
    struct tms {	/* VMS - not tested I believe (PJT) */
        int tms_utime, tms_stime, tms_cutime, tms_cstime;
    };
#endif

/* HZ is defined as a sysconf() call in Solaris */
#if !defined(HZ)	/* test if HZ defined ... */
# define HZ 60
#endif


double cputime()
{
    struct tms buffer;

    if (times(&buffer) == -1)
	error("times(2) call failed");
    return (buffer.tms_utime / (60.0*HZ));		/* only in the USA */
}


#if defined(TESTBED)

main(ac,av)
int ac;
char *av[];
{
    int i,j,k, imax=10, jmax=10, kmax=10;
    double x=1,y=2,z=3;
    
    if (ac>1) imax=atoi(av[1]);
    if (ac>2) jmax=atoi(av[2]);
    if (ac>3) kmax=atoi(av[3]);
    
    for(i=0; i<imax;i++) {
       y += sqrt(z);
       for(j=0; j<jmax;j++) {
          z += sqrt(x);
          for(k=0; k<kmax;k++) {
             x += sqrt(y);
          }
       }
    }
#if 0
    if (emesg) printf("%s\n",emesg);
    else printf("HZ defined to be %d\n",HZ);
#endif    
    printf("CPU time = %f min for %d*%d*%d -> %g\n",cputime(),
	imax,jmax,kmax,x+y+z);
}

#endif


/*===========================================================================*/

#if defined(TOOLBOX)

/*-----------------------------------------------------------------------------
 *  main  --  driver to test  cputime() . It also gives useful information
 *            about the floating point speed of the machine at hand.
 *-----------------------------------------------------------------------------
 */
main(ac,av)
int ac;
char *av[];
{
    int  i, cnt = 250000;
    real a, b, c, d, e, f, g;
    real  t0, t1, t2, t3, t4, t5;
    real  addspeed, subspeed, mulspeed, divspeed, sqrtspeed;
    double  cputime();

    if (ac>1) cnt = atoi(av[1]);
    
    a = 0.0000001;
    b = 1.0000001;
    c = d = e = f = g = 1.0;

    t0 = cputime();

    i = cnt;
    while (i--)
        {
	c += a;
	d *= b;
	e -= a;
	f /= b;
	}

    t1 = cputime();

    printf("To calculate %d floating point operations (+,-,*,/)\n",4*cnt);
    printf("  took about %.3f minutes.\n", t1 - t0);
    printf("This implies a rough average floating point speed of ");
    printf("%.0f kflops.\n", 4*cnt/1000.0/(60.0 *(t1 - t0)));

    t0 = cputime();

    i = cnt;
    while (i--)
	c += a;

    t1 = cputime();

    i = cnt;
    while (i--)
	d *= b;

    t2 = cputime();

    i = cnt;
    while (i--)
	e -= a;

    t3 = cputime();

    i = cnt;
    while (i--)
	f /= b;

    t4 = cputime();

    i = cnt;
    while (i--)
        {
	d *= b;                     /* added to fool the optimizing compiler */
	g = sqrt(d);
	}	

    t5 = cputime();

    dummy(a, b, c, d, e, f, g);     /* we have to do something with these    */
                                    /* variables in order to fool the        */
                                    /* optimizing compilers on some machines */

    addspeed = cnt / (t1 - t0);
    mulspeed = cnt / (t2 - t1);
    subspeed = cnt / (t3 - t2);
    divspeed = cnt / (t4 - t3);
    sqrtspeed = cnt / ((t5 - t4) - (t2 - t1));

    printf("\n");
    printf("The following ratios were measured in floating point speed:\n  ");
    printf("add : sub. : mult. : div. : sqrt = ");
    printf("%.2f : %.2f : %.0f : %.2f : %.2f\n", addspeed/mulspeed,
	   subspeed/mulspeed, 1.0, divspeed/mulspeed, sqrtspeed/mulspeed);
}

dummy(a, b, c, d, e, f, g)
real a, b, c, d, e, f, g;
{}

#endif

