/*
 * FILESECRET.H: parameters and definitions for secret (user-invisible)
 *		 part of structured file package.
 * V 1.0: Joshua Barnes 4/87	basic I/O operators implemented,
 * V 1.X: Lyman Hurd 8/87	added f-d coercion, deferred input,
 * V 2.0: Joshua Barnes 4/88	new types, operators, external format.
 *	  4-oct-90		bug in ss_stk[SetStkLen] removed: 1 more
 * V 2.1: Peter Teuben 13-oct-90  random access
 * V 2.2: 26-feb-94   ansi				PJT
 */
 
#define RANDOM  /* allow random access */
#define CHKSWAP /* allow mixed endian datasets */

/*
 * New-style magic numbers, for FITS type machines (like SUN)
 * Note: If they appear swapped, filestruct() runs in swapped mode !
 */

#define SingMagic  ((011<<8) + 0222)		/* singular items */
#define PlurMagic  ((013<<8) + 0222)		/* plural items */

/*
 * ITEM: structure representing data-token.
 */

typedef struct {
    string itemtyp;		/* type string, listed in filestruct.h */
    int    itemlen;		/* length associated with above type */
    string itemtag;		/* name given this item by application */
    int   *itemdim;		/* int-string of dimensions, or NULL */
    void  *itemdat;		/* the real goodies, if any, or NULL */
    long   itempos;		/* where the item began in stream (i/o) */
} item, *itemptr;    

#define ItemTyp(ip)  ((ip)->itemtyp)
#define ItemLen(ip)  ((ip)->itemlen)
#define ItemTag(ip)  ((ip)->itemtag)
#define ItemDim(ip)  ((ip)->itemdim)
#define ItemDat(ip)  ((ip)->itemdat)
#define ItemPos(ip)  ((ip)->itempos)

/*
 * STRSTK: structure used to associate stream with item stack.
 */

#define SetStkLen  8
#define StrTabLen 16

typedef struct {
    stream  ss_str;		    /* pointer to stdio stream */
    itemptr ss_stk[SetStkLen+1];    /* stack of assoc. items (extra dummy) */
    int     ss_stp;		    /* item stack pointer */
    bool    ss_seek;		    /* permit seeks on this stream ? */
#if defined(RANDOM)
    long    ss_pos;                 /* tail of file, in case random access */
    itemptr ss_ran;                 /* pointer to random access item */
#endif
} strstk, *strstkptr;

/*
 * TYPLEN: structure used to associate length with item type.
 */

typedef struct {
    string tl_typ;			/* type string (see filestruct.h)   */
    int    tl_len;			/* length of elements in bytes      */
} typlen, *typlenptr;

/*
 * Function declarations.
 */

/* Local functions */

local bool writeitem   ARGS(( stream str, itemptr ipt ));
local bool putitem     ARGS(( stream str, itemptr ipt ));
local bool puthdr      ARGS(( stream str, itemptr ipt ));
local bool putdat      ARGS(( stream str, itemptr ipt ));
local itemptr scantag  ARGS(( strstkptr sspt, string tag ));
local itemptr nextitem ARGS(( strstkptr sspt ));
local itemptr finditem ARGS(( strstkptr sspt, string tag ));
local itemptr readitem ARGS(( stream str, itemptr first ));
local itemptr getitem  ARGS(( stream str ));
local itemptr gethdr   ARGS(( stream str ));
local void getdat      ARGS(( itemptr ipt, stream str ));
local proc copyfun     ARGS(( string srctyp, string destyp ));
local void copydata    ARGS(( void *dat, int, int, itemptr ipt, stream str ));
local void copydata_f2d ARGS(( double *dat, int, int, itemptr ipt, stream str ));
local void copydata_d2f ARGS(( float  *dat, int, int, itemptr ipt, stream str ));
local float getflt     ARGS(( stream str ));
local double getdbl    ARGS(( stream str ));
local void saferead    ARGS(( void *dat, int siz, int cnt, stream str ));
local void safeseek    ARGS(( stream str, long offset, int key ));
local int eltcnt       ARGS(( itemptr ipt, int skp ));
local int datlen       ARGS(( itemptr ipt, int skp ));
local itemptr makeitem ARGS(( string typ, string tag, void *dat, int *dim ));
local void freeitem    ARGS(( itemptr ipt, bool flg));
local int baselen      ARGS(( string typ ));
local strstkptr findstream ARGS(( stream str ));
local void ss_push     ARGS(( strstkptr sspt, itemptr ipt ));
local void ss_pop      ARGS(( strstkptr sspt ));
local string findtype  ARGS(( string *a, string type ));

/* kludge to make C++ shut up about it's void(char *) declaration etc. */
#if defined(__cplusplus)
extern void free ARGS((void *));
extern int fread ARGS((void *, int, int, stream));
#endif

#if defined(CHKSWAP)
 local bool swap=FALSE;
#endif

