/* 
 * GETPARAM.C: command-line processing functions for NEMO
 *             this also encompassed most of NEMO's user interface
 *             Note: this file is really too big, approaching 2000 lines.
 *
 * Nov. 1986  Joshua Edward Barnes, Princeton NJ. - original version
 * Oct. 1987  Peter Teuben: added the 'host=' system keyword 
 * Mar. 1988  PJT: added 'debug=' and 'yapp =' system keyword
 *              and used data-history mechanism (HeadlineTag, now HistoryTag)
 * Apr. 1988  PJT: added interactive mode using help=1
 * Jun. 1988  JEB + PJT : --merged IAS and UIUC versions -- 
 * Dec. 1988  PJT: help_levels extended : pick your favorite EDITOR mode 
 * Jan. 1989  PJT: cleared up help-levels and some debug output
 * Mar. 1989  PJT: general calls to nemoinp to parse
 * Jun. 1989  PJT: added setparam - added help=a option for help=8 nemotool
 *                 'key=val' is allowed to have some whitespace, i.e.
 *                 ' key = val' This is for NemoTool
 * Jul. 1989  PJT: interrupting to REVIEW section - help=h installed 
 *                 keyword file is now <prognam>.def (Mirth compatible)
 * Nov. 1989  PJT: date_id() added, fixed annoying save_history() feature
 * Jan  1990  PJT: turning off date_id() [bug??? on sub3]; added USAGELOG
 * Feb  1990  PJT: read include files using key=@file  (no nesting)
 * 14-mar-90  PJT: V2.3 made GCC happy and fixed bug introduces by GCC!
 * 19-mar-90  PJT: V2.3a help=t fixed up a but, helpvec formatting too
 *  9-apr-90  PJT: V2.4 review can be used to chain programs... (NONO not yet)
 * 21-may-90  PJT: V2.5 yapp_string  for yapp_sm now installed
 * 17-jul-90  PJT: V2.5a isolated debug_level through function calls
 *  7-sep-90  PJT: V2.6 - REDIR & system keywords stdout, stderr added
 *                            Won't work the way I did it - disabled
 *                      - help=t also shows the default now
 *  1-oct-90  PJT: V2.6a moved MACROREAD into getparam
 *  7-oct-90           b more ANSI - index -> strchr
 * 11-nov-90           c repaired bug with @include files
 * 21-nov-90           d added 'isaparam'
 * 16-jan-91           e added support for MNEM_ malloc() debug
 * 22-mar-91  PJT V2.7 ERROR environment variable
 * 25-mar-91           a include ieee_handler when debug>0
 * 25-may-91           b appease some gcc warning messages - some Cray tests
 *                       also got rid of MICRO and REDIR stuff
 * 27-jun-91  PJT      c Fixed bug when empty string given to getXparam()
 * 11-jul-91           d fixed bug when empty line in .def file
 *                       blank lines will still crap out
 * 12-oct-91  PJT V2.8 Various small compiler complaints for BorlandC compiler
 *  7-nov-91       2.9 Added hasvalue() function
 * 22-nov-91      2.9a AIX can't handle if(getenv()) if non-existing e.v.
 *                     so use 'ev' as temporary pointer
 *  6-feb-92  PJT 2.9b output KHOROS pane file when help=z is used
 * 13-feb-92         c usage string request via help= or so...; help=u
 * 23-feb-92         d implemeted dmm(3) menu interface via help=8 HELP_DMM
 * 25-feb-92	     e happy gcc2.0
 * 12-mar-92	     f added local nemobin: no usage if no $NEMOBIN
 * 27-apr-92         g parname: space is not a key=val separator
 *  6-jul-92         h help=t more verbose output
 * 18-aug-92         i protected matchname() agains NULL entry
 *                     removed some lint complaints
 *                   j special handling of VERSION keyword to be able to
 *                     warn of old VERSION def file is read.
 * 31-oct-92         k fixed bug when VERSION= absent
 * 18-mar-93	     l default USAGE is now off
 *  6-apr-93         m error_level and debug_level back in their .c routines
 *		       stop() moved to error.c -- still not clean !!!
 * 21-apr-93	     n fixed bug in help=2 reading due to VERSION . ...
 *  6-jun-93         o fixed bugs with badly contructed defv[]'s - HatCreek
 * 15-jun-93         p fixed bugs introduced by previous 'fix' - HatCreek
 * 14-nov-93	V2.10 consolidated two separate mod tracks ...		PJT
 * 18-jan-94    V2.11 solaris (5.3) support				pjt
 *                    -> no more host= remote execution
 *                       ieee disabled (problems finding library)
 * 21-feb-94         a - ansi *** unfinished
 *                    
  BUGS:  
      - if keyword file with ??? present, help=1 gracefully
        allows to process it - and of course dies
      - ESC go key in help=2 prompting mode not correct
      - @macro files do not work in help= mode from a keyword file....
        also they screw up writing the keyword file since it's funny
        value is now written in the binddef[] vector
      - GCC cannot handle the TIO stuff in readparam()
      - ?bug?feature?   @file gets replaced by its text in the .def file
        when help=5 was used. I.e. expansion of macro was done in
        a different spot; should fix this by copying all argv[]/defv[]
        stuff since strings are not writable anyhow in default (g)cc
        This also result is coredumps when the macro file was very
        long, internal buffers are only 256 or so chars length
      - gets() fails after somebody has used the scanf function
  	(see promptparam)
 */

#define VERSION_ID  "PJT-2.11a-*** 21-feb-94"

/*************** BEGIN CONFIGURATION TABLE *********************/

#if !defined(unicos) && !defined(__BORLANDC__) && !defined(mc68k)
# define INTERACT        /* want interactive input ?            *//* no cray */
# define NEMOINP         /* want nemoinp parsing in getXparam ? *//* BUG cray? */
#endif

#define PUTPARAM        /* want possibility of putparam ?      */
#define MACROREAD       /* want ablility to use @include files */
#define HELPVEC         /* reset if not "key=val\nhelp"        */
#define HISTORY         /* have history.c implemented ?        */

#if 0
# define INTERRUPT       /* reset when you don't want REVIEW interrupt */
#endif

#if 0
# define USEDMM		/* needs -lcurses and dmm.o into libnemo.a */
#endif

#if 0
# define REMOTE          /* want remote shell execution ?       *//* no cray */
#endif

/*************** END CONFIGURATION TABLE ***********************/


/*  Because <curses.h> also defines bool, we use the #idef in stdinc.h */
/*  this only works on the BSD version, /usr/5include uses a typedef...*/
/*  and this is what we need for dmm.h                                 */

#if defined(USEDMM)
#  include <curses.h>
#  include <dmm.h>
#endif

#include <stdinc.h>
#include <string.h>
#include <getparam.h>
#include <extstring.h>
#include <filefn.h>
#include <strlib.h>

#if defined(sun)
#   include <floatingpoint.h>               /* special IEEE handler for Sun's */
#endif

#if defined(MIRIAD)
#define KEYVAL 1
#endif

#if defined(INTERRUPT)
# include <signal.h>
#endif

#if defined(INTERACT)
# include <sys/ioctl.h>           /* this is a BSD thing - won't work on SYSV */
                                  /* fcntl.h doesn't have the right stuff */
# if defined(SOLARIS)
#  include <sys/termio.h>	/* for Solaris - can it hurt on SUN OS 4.x ?  */
# endif

#endif /* INTERACT */

/*      *       *       *       variables        *       *       */

/* truely local variable */

#if defined(INTERACT)
   local char key_filename[256];                /* filename to store keywords */
   local char keybuf[256];                  /* buffer for keyword i/o to file */
#endif /* INTERACT */
   local string version_e=NULL; /* extern version (keyfile or commandline)    */
   local string version_i="*";  /* internal version (executable)              */

#ifdef INTERRUPT
   local int initparam_done=0;                 /* trap for too early review() */
#endif

   local string *bindvec = NULL;  /* arr. of NULL term'd "name=value" strings */
#if defined(HELPVEC)
   local string *helpvec = NULL;        /* arr. of NULL term'd "help" strings */
#endif

#if defined(USAGELOG)
   local char cmd[256];                /* buffer to hold command for usagelog */
   local int nemobin = 0;              /* if $NEMOBIN exists, usagelog written*/
#endif



/* global variables: I wish could live with a few less - some are only of     */
/*                   historical interest                                      */

extern int debug_level; /* see dprintf.c   from DEBUG env.var. */
extern int error_level; /* see error.c     from ERROR env.var. */

int history=1;          /* 0=no history written; see history.c */

int yapp_dev = 0;       /* interface hidden keyword yapp to plotting pkg */
int help_level = 0;     /* hidden keyword help for interactive prompting */
int bell_level = 0;     /* noisy terminal when prompted */
int review_flag = 0;    /* review keywords and optional help=8 chaining ? */
string yapp_string = NULL;  /* future expansion */
string help_string = NULL;  /* future expansion */

/* local functions needed */

local void scan_environment ARGS((void));
local void save_history ARGS((string *argv, string *defv, string *bndv));
local void printhelp ARGS((string help));
local void newline ARGS((int n));
local void showconfig ARGS((void));
local void printusage ARGS((string *defv));
local string get_macro ARGS((char *mname));
local void eval_bindvec ARGS((int ndef, string *defv, string *bindvec));
local void setparam ARGS((char *par, char *val, char *prompt));
local int readparam ARGS((char *buffer, char *text));
local void beep ARGS((void));
local string parname ARGS((string arg));
local string parvalue ARGS((string arg));
local string parhelp ARGS((string arg));
local int scanbind ARGS((string bvec[], string name));
local bool matchname ARGS((string bind, string name, bool exact));
local string bindpar ARGS((string name, string value));
local void writekeys ARGS((char *mesg));
local bool readkeys ARGS((string mesg, bool useflag, bool needit, string defv[]));

#if 0
local int scanbind(), check_syskey(), readparam(), rsh();
local string parvalue(), parname(), parhelp(), bindpar(), get_macro();
local void review_help(), review(),  showconfig(), printusage();
local void ieee_nemo(), writekeys(), setparam(), beep();
local void eval_bindvec();
local bool matchname(), readkeys();
#endif

/* external NEMO functions needed */

int file_size();
void app_history();

/* Mask values for help levels */

#define HELP_DEFIO  0x01
#define HELP_PROMPT 0x02
#define HELP_EDIT   0x04
#define HELP_DMM    0x08
#define HELP_GLOBAL 0x10		/* Note: not used yet */
#define HELP_FUTURE 0x20		/* Note: not used yet */


/*
 * INITPARAM: initalize parameter lists and handle special help and
 *            review requests. This should be the first routine any
 *            NEMO program calls. See also ../misc/nemomain.c for
 *            a better approach
 */

void initparam(argv, defv)
string argv[];                   /* arg vector passed from main */
string defv[];                   /* default parameter values */
{
    int ndef, i, j;
    char *cp;
    bool posflag, useflag, defflag;
    string help, name;

    scan_environment();              /* scan env.var and set what is needed */
#if defined(sun) && defined(USE_IEEE)
    if(ieee_handler("set","common",ieee_nemo))    /* need to trap FP errors */
        warning("Error setting Sun ieee handler");
#endif
    
#if defined(USAGELOG)
    if (nemobin) {
        sprintf(cmd,
            "echo %s `whoami` `hostname` `date` >> $NEMO/adm/Usage", argv[0]);
        system(cmd);
    }
#endif
    ndef = xstrlen(defv, sizeof(string));       /* count params plus NULL   */
    bindvec = (string *) allocate((ndef + 1) * sizeof(string)); /* PPAP */
    for (i = 0; i <= ndef; i++)                 /* init bindings to NULL    */
        bindvec[i] = NULL;			/* note it's NULL terminated*/
    bindvec[0] = bindpar("argv0", tail(argv[0])); /* argv0 = progr name */
#if defined(HELPVEC)
    helpvec = (string *) allocate((ndef + 1) * sizeof(string)); /* PPAP */
    for (i = 0; i <= ndef; i++)                 /* init bindings to NULL    */
        helpvec[i] = NULL; 		        /* note it's NULL terminated*/
    for (i=0; defv[i] != NULL; i++)
        helpvec[i] = parhelp(defv[i]);
#endif    
    for (i = 0; i < ndef-1; i++)                  /* loop over defaults */
        if (streq(parname(defv[i]),"VERSION")) {  /* until VERSION found */
            version_i = scopy(parvalue(defv[i]));   /* and store */
            break;
        }

    /*
     * the first thing to do is parse over the command line arguments
     * and install them into bindvec[], saves some work for later
     */

    help = NULL;                                /* turn help processing off */
#if defined(KEYVAL)
    posflag = FALSE;                            /* force key=val args       */
#else
    posflag = TRUE;                             /* start scan by position   */
#endif
    for (i = 1; argv[i] != NULL; i++) {         /* loop over stuff in argv  */
        name = parname(argv[i]);                /*   get param name, if any */
        posflag = posflag && (name == NULL);    /*   see how to match args  */
        if (posflag) {                          /*   match by position?     */
            if (i >= ndef)
                error("Too many args");
            bindvec[i] = bindpar(parname(defv[i-1]), argv[i]);
                                                /*     save name=val pair   */
        } else {                                /*   match by name?         */
            if (name == NULL)
                error("Arg \"%s\" must be named", argv[i]);
            j = 1 + scanbind(defv, name);       /*     find index in defv   */
            if (j > 0) {                        /*     was name found?      */
                if (bindvec[j] != NULL)         /*       check slot is free */
                    error("Parameter \"%s\" duplicated", name);
                bindvec[j] = argv[i];           /*       store name=value   */
            } else {                            /*     not listed in defv?  */
                if (streq(name, "help")) {      /*       got help keyword?  */
                    help = parvalue(argv[i]);
                    if ((cp = strpbrk(help,"0123456789"))!=NULL)  /* isnum? */
                        help_level = atoi(cp);  /* if so, change help_level */
                }
                else if (streq(name, "debug"))  /*       got debug keyword? */
                    debug_level = atoi(parvalue(argv[i]));
                else if (streq(name, "yapp")) { /*       got yapp keyword?  */
                    yapp_string = scopy(parvalue(argv[i]));
                    yapp_dev = atoi(yapp_string);
                }
                else if (streq(name, "review")) /*     go into review ?   */
                    review_flag = atoi(parvalue(argv[i]));
                else if (streq(name, "error")) /*     allowed error count? */
                    error_level = atoi(parvalue(argv[i]));
#if defined(REMOTE)
                else if (streq(name, "host"))   /*       got host keyword?  */
                    rsh(parvalue(argv[i]), argv);
#endif /* REMOTE */
                else
                    error("Parameter \"%s\" unknown", name);
            } /* j>0 (i.e. program/system keyword) */
        } /* if(posflag) */
    } /* for (i) */

#if defined(INTERRUPT)
    if (review_flag) {
        signal(SIGTERM, review);                /* catch interrupts */
        signal(SIGQUIT, review);                /* for review section */
        signal(SIGINT, review);                 /* and ^C also */
    }
#endif

    useflag =  FALSE;                         /* set if usage message needed */

#if defined(INTERACT)
    sprintf(&key_filename[strlen(key_filename)],"%s.def", tail(argv[0]));
    dprintf(2,"Keyword file will be %s\n",key_filename);

    if (help_level & HELP_DEFIO) {           /* read any existing keys-file */
	(void) readkeys("iniparam(1)",useflag,FALSE,defv);
    } /* if (help_level & HELP_DEFIO) */

    if (help_level & HELP_GLOBAL) {          /* read any existing keys-file */
        warning("HELP_GLOBAL level not implemented");
    }

#endif /* INTERACT */


    if (!(help_level & HELP_PROMPT)) {          /* if no help was needed */
        for (i=1; i<ndef; i++)                   /* loop installing defaults */
            if (bindvec[i]==NULL) {                /* empty slot to fill ? */
                useflag = useflag || streq(parvalue(defv[i-1]), "???");
                bindvec[i] = defv[i-1];            /* copy default entry */
            }
    } else if (help_level & HELP_PROMPT) {
#if defined(INTERACT)
        char  *cp;
        string key, val;
        int    go=0;            /* set immediate go to false */
        int    v;
        
        if (!isatty(fileno(stdin)) || !isatty(fileno(stdout)))
            error("(initparam) Can only INTERACT with terminal");

        for (i=1; i<ndef; i++) {                 /* loop installing defaults */
            v = 0;
            do {                                 /*       loop for input     */
                if (bindvec[i]==NULL) {        /* had not yet been installed */
                    strcpy(keybuf,defv[i-1]);
                    if (streq(parname(keybuf), "VERSION")) {
                        v = 1;
                        version_i = scopy(parvalue(keybuf));
                        key = parname(keybuf);
                        val = parvalue(keybuf);
                        break;
                    }
                } else {                            /* was already installed */
                    strcpy(keybuf,bindvec[i]);     
                    if (streq(parname(keybuf), "VERSION")) {
                        v = 1;
                        version_e = scopy(parvalue(keybuf));
                        if (!streq(version_i,version_e)) 
                            warning("version_i=%s version_e=%s",version_i,version_e);
                    }
                }

                if (v) {             /* if keyword was "VERSION", break out here */
                    printf("%s: %s\n", argv[0], keybuf);
                    break;
                }
                if (streq(parvalue(keybuf), "???")) {  /* unanswered */
                    defflag = TRUE;
                    cp=strchr(keybuf,'=');
                    if (cp==NULL)
                        error("(initparam) Improper keyword (%s)",keybuf);
                    else
                        cp++;
                    *cp = '\0';
                    go = readparam(keybuf, "No default allowed");  /* get it */
                } else {                  /* keyword with default value */
                    defflag = FALSE;
                    go = readparam(keybuf, "");     /* or use NULL for "" */
                }
                if (go) break;          /* exit loop when premature done */
                key = parname(keybuf);
                val = parvalue(keybuf);
            } while ((key == NULL) ||
                     ((val == NULL || streq(val, "")) && defflag) );
            bindvec[i] = bindpar(key, val);
            if (go) break;
        } /* for (i) */
       /************* GO still has error - not all par's are bount yet ********/
#else   
    error("(initparam) INTERACTIVE input not allowed; check help=?");
#endif /* INTERACT */     
    }


#if defined(INTERACT)
    if (help_level&HELP_DEFIO || help_level&HELP_EDIT) {     /* write keyfile */
	writekeys("iniparam(1)");
    }
    if (help_level&HELP_EDIT) {    /* into editor mode, edit and read keyfile */
        char *edtcmd;
                                
        edtcmd = getenv("EDITOR");
        if (edtcmd != NULL)
            sprintf(keybuf,"%s %s",edtcmd,key_filename);       /* your editor */
        else
            sprintf(keybuf,"vi %s",key_filename);           /* default editor */

        if (system(keybuf))                         /*  edit the keyword file */
            error("(initparam) Error running editor (%s)",keybuf);
                        
        useflag = FALSE;                   /* will be set if usage msg needed */
	useflag=readkeys("iniparam(2)",useflag,TRUE,defv);

    }

    if (help_level&HELP_GLOBAL) {                /* write global keyword file */
        warning("HELP_GLOBAL: Not implemented");
    }
#endif /* INTERACT */

#if defined(USEDMM)
    if (help_level&HELP_DMM) {                          /* dmm user interface */
        int startfield = -1;
        int retcode, ncols, nrows, icol, irow;
        extern DMMRETTYPE       dmmRetType;
        DMMDRAWFLAG             drawflag = dmmNew;
        DMMVALFLAG              valflag = dmmInitVal;

        static DMMMESSAGE       msg1 =  {"Writing keyword file",
                                        5, 10, 0};
        static DMMMESGBLK       wmsgblk = {&msg1, 0};
        static DMMMENUITEM  runitem = {"Run", "Run the program"},
			    writeitem = {"Write", "Write the keyword file"},
			    quititem = {"Quit", "Quit without saving"};
        static DMMMENU 	    menu = { &runitem, &writeitem, &quititem, 0};
        DMMFIELD **screen, *fptr;
        /* some NEMO scope needs */
        extern string usage;
        char *scopy(), *sconc();
        byte *allocate();
        string *burststring();

        screen = (DMMFIELD **) allocate((ndef+1) * sizeof(DMMFIELD *));
        for (i=0; i<ndef; i++) {
            fptr =  (DMMFIELD *) allocate(sizeof(DMMFIELD));

            if (i==0) {          /* header with program name, version & usage */
                fptr->labelValue = sconc(sconc(parvalue(bindvec[0]),"  V"),
                            sconc(parvalue(bindvec[ndef-1]),sconc(" :",usage)));
                fptr->labelCol          = 5;
                fptr->inputCol          = 0;
                fptr->labelAttrib       = A_REVERSE|A_BLINK;
                fptr->inputEditValue    = 0;
                fptr->inputLength       = 0;
                fptr->inputInstructVec  = 0;
            } else {           /* Editable fields with keyword and def values */
                if(bindvec[i]==NULL) {
                    fptr->labelValue        = scopy(parname(defv[i-1]));
                    fptr->inputInitValue    = parvalue(defv[i-1]);
                } else {
                    fptr->labelValue        = scopy(parname(bindvec[i]));
                    fptr->inputInitValue    = parvalue(bindvec[i]);
                }
                fptr->labelCol          = 0;
                fptr->inputCol          = 12;
                fptr->labelAttrib       = 0;
                fptr->inputEditValue    = allocate(80);
                fptr->inputLength       = 60;      /* should be 80-12 ??*/
                fptr->inputInstructVec  = burststring(helpvec[i-1],"\n");
            }
            fptr->labelLine         = i;
            fptr->inputLine         = i;
            fptr->inputMaxCols      = 0;
            fptr->inputFullLines    = 0;
            fptr->inputShortCols    = 0;
            fptr->inputAttrib       = A_REVERSE;
            fptr->inputEditAttrib   = A_UNDERLINE;
            fptr->inputNumInstructs = 0;
            fptr->inputAudit        = NULL;
            screen[i] = fptr;
        }
        screen[ndef] = NULL;          /* terminate DMMFIELD array with a NULL */

        dmmInit();                                           /* set up screen */
        for(;;) {                                         /* loop until happy */
            retcode = dmmRun(startfield, screen, menu, drawflag, valflag);
            if(retcode==0) {                                   /* run program */
                       /* check that keywords are still "???" trigraph */
                break;
            } else if(retcode==1) {                     /* write keyword file */
                dmmClear();
                dmmPutMsgsHit(wmsgblk);
            } else if(retcode==2)                       /* unconditional quit */
                break;            
            else
                break;
        } /* for(;;) */
        dmmClose();                              /* return to a normal screen */

        if(retcode == 2) {                                      /* catch Quit */
            exit(0);
            /*NOTREACHED*/
	}
        else if(retcode == -1)                /* and other bizarre situations */
            error("bad return code from dmmRun using help=8");

        for(i=1; i<ndef; i++) {            /* save new values back in bindvec */
            bindvec[i]=bindpar(screen[i]->labelValue,screen[i]->inputEditValue);
	    dprintf(1,"Bound par %d as %s\n",i,bindvec[i]);
        }
    }
#endif /* USEDMM */

    if (help != NULL)
        if (  strpbrk(help,"apdqntkvh?")!=NULL ||
              ( strpbrk(help,"apdqntkvh?")==NULL && 
                strpbrk(help,"0123456789")==NULL )  )

            printhelp(help);                     /* give some help and exit */
    if (useflag) {
        printusage(defv);                  /* give minimum 'usage' and exit */
        exit(0);
        /*NOTREACHED*/
    }
#if defined(INTERRUPT)
    initparam_done = 1;                 /* flag for interruptability */
    review();
    beep();
#endif
    save_history(argv, defv, bindvec);   /* save activation record */
    eval_bindvec(ndef,defv,bindvec);		/* resolve $key references */
    
} /* initparam */

/*
 * FINIPARAM: finish up any outstanding requests. If INTERACT is not defined
 *            this amazing function does nothing!
 *
 */

void finiparam()
{
#if defined(INTERACT)

    if (help_level&HELP_DEFIO) {
        dprintf(2,"finis_param: writing keyword file for final time\n");
	writekeys("finiparam");

    }
    if (help_level & HELP_GLOBAL) {
        warning("HELP_GLOBAL: Not implemented yet");
    }

#if defined(INTERRUPT)
    review();
#endif

#endif
}


local void scan_environment()
{
    int i;
    char *ev, *date_id();
    extern char **environ;      /* (extern) global database for environment */

    for (i = 0; environ[i] != NULL; i++) {
        if (streq("BELL", parname(environ[i])))
            bell_level = atoi(parvalue(environ[i]));
        else if (streq("DEBUG", parname(environ[i])))
            debug_level = atoi(parvalue(environ[i]));
        else if (streq("YAPP", parname(environ[i]))) {
            yapp_string = scopy(parvalue(environ[i]));
            yapp_dev = atoi(yapp_string);
        } else if (streq("HELP", parname(environ[i])))
            help_level = atoi(parvalue(environ[i]));
        else if (streq("HISTORY", parname(environ[i])))
            history = atoi(parvalue(environ[i]));
        else if (streq("REVIEW", parname(environ[i])))
            review_flag = atoi(parvalue(environ[i]));
        else if (streq("ERROR", parname(environ[i])))
            error_level = atoi(parvalue(environ[i]));
#if defined(USAGELOG)
	else if (streq("NEMOBIN", parname(environ[i])))
            nemobin = 1;
#endif
    }
    dprintf(5, 
    "scan_environment: debug=%d yapp=%d help=%d history=%d review=%d error=%d\n",
     debug_level, yapp_dev, help_level, history, review_flag, error_level);
    dprintf(5,"date_id = %s\n",date_id());
#if defined(INTERACT)
    if ((ev=getenv("NEMODEF")) != NULL) {
        strcpy (key_filename,ev);
        strcat (key_filename,"/");
    } else
        key_filename[0] = '\0';
#endif /* INTERACT */
}

local void save_history(argv, defv, bndv)
string *argv;                   /* arg vector passed from main */
string *defv;                   /* default parameter values */
string *bndv;                   /* the actual found key=val strings */
{
#if defined(HISTORY)
    int i, histlen;
    string hist;

    if (!history)
	return;

    if (help_level) {         /* if external help, cmdln is not appropriate */
        histlen = 0;
        for (i=0; bndv[i] != NULL; i++)            /* count how much needed */
            histlen += strlen(bndv[i]) + 1;
        hist = (char *) allocate(histlen+20);
        strcpy(hist,parvalue(bndv[0]));
        for (i=1; bndv[i] != NULL; i++) {
            strcat(hist," ");
            strcat(hist,bndv[i]);
        }
        sprintf(&hist[strlen(hist)]," help=%d",help_level);     /* add help */

    } else {                /* save the command line */
        histlen = strlen(version_i) + 9; 	/* "VERSION=" + version_i */
        for (i = 0; argv[i] != NULL; i++)
            histlen += strlen(argv[i]) + 1;
        hist = (char *) allocate(histlen);
	hist[0] = '\0';
        for (i = 0; argv[i] != NULL; i++) {     /* copy name and arguments  */
            strcat(hist, argv[i]);
            strcat(hist, " ");
        }
        strcat(hist,"VERSION=");            /* append version identifcation */
        strcat(hist, version_i);
    }
    app_history(hist);
#endif /* HISTORY */
}

/*
 * PRINTHELP: print out help message in various ways.
 */

local void printhelp(help)
string help;                    /* help option */
{
    string *bp, *hp;
    int numl;
    extern string usage;

    if (strchr(help,'?')) {
        printf("Help options are any combination of:\n\n");
        printf("  a       key=val   (all)\n");
        printf("  p,k     key       (parameter,key)\n");
        printf("  d,v     val       (default,value)\n");
        printf("  n       >> add's a newline between above\n");
        printf("  q       >> quit's after above work done\n");
        printf("  u       >> show the usage string\n");
	printf("  h	  >> show key and help strings\n");
        printf("  t       >> show as MIRIAD doc file\n");
        printf("  z       >> show as KHOROS pane file\n");
        printf("  ?       >> this help\n\n");
        printf("Numeric helplevels determine degree and type of assistence:\n");
        printf("They can be added to give combined functionality\n");
        printf("  1       keyword files used (prognam.def)\n");
        printf("  2       interactive prompting\n");
        printf("  4       menu interface using EDITOR environment variable\n");
        printf("  8       use dmm(3) curses menu interface\n");
	printf(" 16       reserved\n");
        printf(" VERSION_ID = %s\n",VERSION_ID);
        showconfig();
        exit(0);
        /*NOTREACHED*/
    }

    if (strchr(help,'h')) {
#if defined(HELPVEC)
        for (bp = bindvec+1, hp = helpvec; *bp != NULL ; bp++, hp++) 
            printf("%-20s : %s\n",parname(*bp), *hp);
#else
        for (bp = bindvec ; *bp != NULL ; bp++)         
            printf("%-20s : Not compiled with HELPVEC option\n",parname(*bp));
#endif
        exit(0);
        /*NOTREACHED*/
    }

    numl = ((strchr(help,'n')) ? 1 : 0);    /* add newlines between key=val ? */

    if (strchr(help,'a') || strpbrk(help,"apdqntvkzu")==NULL) { /* arguments */
        printf("%s", parvalue(*bindvec));
        for (bp = bindvec+1; *bp != NULL; bp++) {
            newline(numl);
            printf(" %s", *bp);
        }
        newline(1);
        if (strpbrk(help,"apdqntvkzu")==NULL) {
            exit(0);
            /*NOTREACHED*/
        }
    }

    if (strchr(help,'p') || strchr(help,'k')) {      /* parameters keywords */
        printf("%s", parvalue(*bindvec));
        for (bp = bindvec+1; *bp != NULL; bp++) {
            newline(numl);
            printf(" %s", parname(*bp));
        }
        newline(1);
    } /* 'p', 'k' */

    if (strchr(help,'d') || strchr(help,'v')) {              /* arguments */
        printf("%s", parvalue(*bindvec));
        for (bp = bindvec+1; *bp != NULL; bp++) {
            newline(numl);
            printf(" %s", parvalue(*bp));
        }
        newline(1);
    } /* 'd' 'v' */
    
    if (strchr(help,'u')) {
        printf("%s\n",usage);
    } /* 'u' */

    if (strchr(help,'t')) {     /* nemotool - special doc file output */
        printf("%%N %s\n",parvalue(bindvec[0]));
        printf("%%D %s\n",usage);
        printf("%%B\n");
        printf("  This doc file has been produced with NEMO help=t option\n");
        printf("  Try 'man %s' for more extensive online help\n",
                                                parvalue(bindvec[0]));
	printf("  Defaults of program keywords are listed between square brackets\n");
#if defined(HELPVEC)
        for (bp=bindvec+1, hp=helpvec; *bp!=NULL; bp++, hp++)
            printf("%%A %s\n\t%s [%s]\n",parname(*bp),*hp,parvalue(*bp));
#else
        for (bp = bindvec+1; *bp != NULL; bp++) {
                printf("%%A %s\n",parname(*bp));
                printf("\tNo help available\n");
        }
#endif
        exit(0);                    /* always exit in this case */
        /*NOTREACHED*/
    } /* 't' */
    if (strchr(help,'z')) {     /* khoros - special pane file output */
        int vcount = 2, opt;
        char *parval;

        printf("-F 4.2 1 0 170x7+10+20 +35+1 'CANTATA for KHOROS' cantata\n");
        printf("-M 1 0 100x40+10+20 +23+1 'A NEMO program' nemo\n");
        printf("-P 1 0 80x38+22+2 +0+0 '%s ' %s\n",usage,getargv0());
#if defined(HELPVEC)
        for (bp=bindvec+1, hp=helpvec; *bp!=NULL; bp++, hp++) {
            parval = parvalue(*bp);
            opt = !streq(parval,"???");

            if (streq(parname(*bp),"in"))
                printf("-I 1 0 %d 1 0 1 50x1+2+%d +0+0 '%s' '%s ' '%s' %s\n",
                           opt,          
                           vcount,
                           (*parval == NULL ? " " : (opt?parval:" ")),
                           parname(*bp), *hp, parname(*bp));

            else if (streq(parname(*bp),"out"))
                printf("-O 1 0 %d 1 0 1 50x1+2+%d +0+0 '%s' '%s ' '%s' %s\n",
                           opt,          
                           vcount,
                           (*parval == NULL ? " " : (opt?parval:" ")),
                           parname(*bp), *hp, parname(*bp));
            else
                printf("-s 1 0 %d 1 0 50x1+2+%d +0+0 '%s' '%s ' '%s' %s\n",
                           opt,          
                           vcount,
                           (*parval == NULL ? " " : (opt?parval:" ")),
                           parname(*bp), *hp, parname(*bp));

            vcount += 2;
        }
#else
        printf("help=z not implemented without helpvec\n");
        exit(0);
        /*NOTREACHED*/
#endif
        vcount++;
        printf("-H 1 13x2+1+%d 'Help' 'Help for %s' nemo.help\n",
                            vcount, getargv0());
        printf("-R 1 0 1 13x2+39+%d 'Run' 'RunMe' khoros2nemo %s\n",
                            vcount, getargv0());
        printf("-E\n-E\n-E\n");
        exit(0);                    /* always exit in this case */
        /*NOTREACHED*/
    } /* 'z' */

    if (strchr(help,'q')) {
        exit(0);                        /* quit - don't run program */
        /*NOTREACHED*/
    }
}

local void newline(n)        /* print 'n' newlines */
int n;
{
    while (n-- > 0)
        printf("\n");
}

/*
 * SHOWCONFIG: show what non-standards have been compiled into the
 *             the interface
 */

local void showconfig()
{
    printf("INTERACT  ");
#if defined(INTERACT)
    printf("on\n");
#else
    printf("off\n");
#endif

    printf("REMOTE    ");
#if defined(REMOTE)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("NEMOINP   ");
#if defined(NEMOINP)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("PUTPARAM  ");
#if defined(PUTPARAM)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("USAGELOG  ");
#if defined(USAGELOG)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("MACROREAD ");
#if defined(MACROREAD)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("HELPVEC   ");
#if defined(HELPVEC)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("HISTORY   ");
#if defined(HISTORY)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("INTERRUPT ");
#if defined(INTERRUPT)
    printf("on\n");
#else
    printf("off\n");
#endif
    printf("USEDMM    ");
#if defined(USEDMM)
    printf("on\n");
#else
    printf("off\n");
#endif
}
/*
 * PRINTUSAGE: print out helpful usage info.
 */

local void printusage(defv)
string *defv;
{
    string *dp;
    bool otherargs;
    extern string usage;

    printf("Insufficient parameters, try 'help=', 'help=?' or 'help=h',\n");
    printf("Usage: %s", parvalue(*bindvec));
    otherargs = FALSE;
    for (dp = defv; *dp != NULL; dp++)
        if (streq(parvalue(*dp), "???"))
            printf(" %s", *dp);
        else
            otherargs = TRUE;
    printf(otherargs ? " ...\n" : "\n");
    if (usage != NULL) 
        printf("%s\n",usage);
}

/*
 * GET_MACRO:  see if string is of a '@file', if so, read it into
 *             memory and return pointer to it.
 *   Note: not hierarchical
 */
local string get_macro(mname)
char *mname;
{
#if defined(MACROREAD)
    char *cp, *mp;
    int n;
    stream fstr;    

    if (*mname != '@')
        return mname;
    mname++;
    dprintf(1,"Opening macro file %s\n",mname);
    n = file_size(mname);
    if (n<0)
        error("(getparam) macro include file \"%s\" does not exist",mname);
    mp = (char *) allocate(n+1);
    if (n==0) {              /* make sure n=0 also works properly */
        *mp = '\0';
        return mp;
    }

    fstr = stropen(mname,"r");
    fread(mp,sizeof(char),n,fstr);
    strclose(fstr);
#if 1
    for (cp=mp; *cp; cp++)      
        if (*cp=='\n')
            *cp=' ';            /* replace newline with blanks */
#endif
    return mp;
#else
    return mname;              /* no processing, return string */
#endif
}

/*
 * EVAL_BINDVEC: scan key=val pairs for cross references
 */
 
local void eval_bindvec(ndef,defv,bindvec)
int ndef;
string *defv;
string *bindvec;
{
    int i;
    char *c1,*c2,*c3,*c4, *cd, newkeyval[256], key[16];
    bool more = TRUE;

    while (more) {                  /* loop until no more $ references found */
        more = FALSE;
        for (i=0; i<ndef; i++) {                  /* loop over the key=val's */
            c1 = bindvec[i];
            c2 = newkeyval;
            cd = strchr(c1,'$');
            if (cd == 0) continue;
            dprintf(1,"eval_bindvec: parsing %s\n",bindvec[i]);
            more = TRUE;
            while (cd) {                       /* loop, while more '$' found */
                while (*c1 && *c1 != '$') *c2++ = *c1++;    /* copy before $ */
                if (*c1 == 0)
                    error("improper $-reference in %s",bindvec[i]);
                c3 = key;
                c1++;
                while (*c1 && isalpha(*c1)) *c3++ = *c1++;
                *c3 = 0; /* terminate key */
                
                c4 = getparam(key);
                dprintf(3,"Patching %s with %s=%s\n",bindvec[i],key,c4);
                while (*c4) *c2++ = *c4++;
                cd = strchr(c1,'$');        /* search for next */
            }
            while (*c1) *c2++ = *c1++;
            *c2 = 0;
            dprintf(3,"eval_bindvec: Modifying %s\n",bindvec[i]);
            /* free(bindvec[i]); ??? */
            bindvec[i] = scopy(newkeyval);
        }
    }
}


/*
 * ISAPARAM: check if this is truely a parameter for this program
 *           The only way to prevent program from crashing through
 *           error(3NEMO)
 */

bool isaparam(name)
string name;
{
    if (bindvec == NULL)
        error("isaparam: called before initparam");
    return scanbind(bindvec, name) >= 0;
}
/*
 * HASVALUE:  check if a parameter has been given a value
 *
 */

bool hasvalue(name)     
string name;
{
#if 1
    return !streq(getparam(name), "");
#else
    string val;
    val = getparam(name);
    return *val != NULL;
#endif
}

/*
 * GETPARAM: look up parameter value on binding vector.
 */

string getparam(name)
string name;
{
    int i;
    char *cp, *cp1;

    if (bindvec == NULL)
        error("(getparam) called before initparam");
    i = scanbind(bindvec, name);
    if (i < 0)
        error("(getparam) \"%s\" unknown", name);
#if defined(MACROREAD)
    cp = parvalue(bindvec[i]);
    if (*cp == '@') {
        cp = get_macro(cp);         /* get the macro in memory  */
        cp1 = (char *) allocate(strlen(cp)+strlen(name)+1);
        strcpy(cp1,name);
        strcat(cp1,"=");
        strcat(cp1,cp);
        bindvec[i]=cp1;
        free(cp);
        cp = strchr(cp1,'=');       /* look again for '=' - is always there */
        cp++;                       /* advance one to point to value part */
    } 
    return cp;                     /* return value part */
#else
    return parvalue(bindvec[i]);
#endif
}

/*
 * GETIPARAM, ..., GETDPARAM: get int, long, bool, or double parameters.
 *          and allow some optional parsing if -DNEMOINP
 */

int getiparam(par)
string par;                     /* name of parameter to look for */
{
    string val;
    int    ipar, nret;

    val = getparam(par);                        /* obtain value of param */
#if !defined(NEMOINP)
    return (atoi(val));                         /* convert to an integer */
#else
    nret = nemoinpi(val,&ipar, 1);
    if (nret < 0)
        warning("getiparam(%s=%s) parsing error %d, assumed %d\n",
                    par,val,nret,ipar);
    if (nret==0)
        return(0);
    else
        return(ipar);
#endif /* !NEMOINP */
}

long getlparam(par)
string par;                     /* name of parameter to look for */
{
    string val;
    long lpar;
    int nret;

    val = getparam(par);                        /* obtain value of param */
#if !defined(NEMOINP)
    return (atol(val));                         /* convert to an integer */
#else
    nret = nemoinpl(val,&lpar,1);
    if (nret < 0)
        warning("getlparam(%s=%s) parsing error %d assumed %l\n",
                    par,val,nret,lpar);
    if (nret==0)
        return(0);
    else
        return(lpar);
#endif /* !NEMOINP */
}

bool getbparam(par)
string par;                     /* name of parameter to look for */
{
    char *val;
    bool bpar;
    int  nret;

    val = getparam(par);                        /* obtain value of param */
#if !defined(NEMOINP)
    if (strchr("tTyY1", *val) != NULL)          /* is value true? */
        return (TRUE);
    if (strchr("fFnN0", *val) != NULL)          /* is value false? */
        return (FALSE);
    error("getbparam: %s=%s not bool", par, val);
    return(0);   /*turboc*/
#else
    nret = nemoinpb(val,&bpar,1);
    if (nret < 0)
        warning("getbparam(%s=%s) parsing error %d, assumed %d\n",
                        par,val,nret,bpar);
    if (nret==0)
        return(FALSE);
    else
        return(bpar);
#endif /* !NEMOINP */
}

double getdparam(par)
string par;                     /* name of parameter to look for */
{
    string val;
    double dpar;
    int    nret;

    val = getparam(par);                        /* obtain value of param */
    if (val == NULL)                            /* insure value is given */
        error("getdparam: %s unknown", par);
#if !defined(NEMOINP)
    return (atof(val));                         /* convert to a double */
#else
    nret = nemoinpd(val,&dpar,1);
    if (nret < 0)
        warning("getdparam(%s=%s) parsing error %d, assumed %g\n",
                            par,val,nret,dpar);
    if (nret==0)
        return(0.0);
    else
        return(dpar);
#endif /* !NEMOINP */
}

/*
 *
 *  PUTPARAM:       resetting the value of a keyword.
 *  PROMPTPARAM:    interactive setting the value of a keyword
 *
 *  both function call a local work horse:
 * 
 *  SETPARAM:
 *      
 *      prompt==NULL:  both 'par' and 'val' are assumed a valid string
 *                      and the keyword 'par' is updated with the value 'val'
 *      prompt!=NULL:  The string 'val' is ignored, the string 'prompt' is
 *                      a user prompt for interactive updating of the value
 *                      of the keyword 'par': the user must enter a new 'val'
 *                      If need be, the bindvec[] pointer to the 'par=val'
 *                      string will be reallocated.
 *
 *      setparam does not check if standard input is a redirected unit,
 *      in which case it's use is questionable or dangerous.
 *
 */
void putparam(par,val)
char *par, *val;
{

    setparam(par,val,(char *)NULL);
}

void promptparam(par,prompt)
char *par, *prompt;
{
    /* BUG: check if OK to do IO, i.e. isatty(stdin) && isatty(stdout).. */
    setparam(par,(char *)NULL,prompt);
}

local void setparam (par, val, prompt)
string par, val, prompt;
{
#if defined(PUTPARAM) && defined(INTERACT)
    char line[80];      
    int  old, new, i;

    if (par==NULL || *par==NULL)
        error("setparam: no parameter supplied?");
    if (bindvec==NULL)
        error("setparam: called before initparam");
    i=scanbind(bindvec,par);        /* find entry of keyword in table */
    if (i<0)
        error("setparam: parameter \"%s\" unknown",par);
    if (prompt!=NULL && *prompt!=NULL) {        /* interactive entry */
        beep();
        fprintf (stderr,"%s: %s=",par, prompt);
        fflush(stderr);
	clearerr(stdin);
        if (gets(line) == NULL) error("Null input");
        val = line;
    }
    new = strlen(val) + strlen(par) + 2;
    old = strlen(bindvec[i]);
    if (new>old)                        /* allocate new space */
        bindvec[i] = (char *) allocate(new);     /* perhaps use realloc() ?? */
    strcpy(bindvec[i], par);
    strcat(bindvec[i],"=");
    strcat(bindvec[i], val);
#else
    error("setparam: not compiled into getparam.c");
#endif /* PUTPARAM && INTERACT */
}

#if defined(INTERACT)
#define ESC 0x1b
/* 
 * READPARAM:  interactive input of a keyword
 *                                                  April 1988      Peter Teuben
 *             ESC hot key to stop prompting        March 1989
 *  GCC bug: cannot deal with ioctl() call
 */

local int readparam(buffer,text)
char *buffer, *text;
{
    char *p;
    int  i,n, go;

    if (!isatty(fileno(stdin)))
        error("readparam: Cannot use redirected input in interactive mode");
    fflush(stdin);                  /* some programs seem to need this */
    beep();
    if (text && *text)
        printf ("%s\n",text);
    n = strlen(buffer);
    for (i=0; i<n; i++)             /* simulate as if this was already input */
        ioctl(fileno(stdin),TIOCSTI,&buffer[i]);
    p = buffer;
    p--;  
    do {                            /* read it */
        p++;
        *p = getchar();
        n++;
    } while (*p != '\n' && *p != ESC);
    go = 0;
    if (* p   ==ESC) { p--; go++;}          /* not really needed in straight mode ? */
    if (*(p-1)==ESC) { p--; go++;}          /* get rid of GO character */
    *p = NULL;                      /* terminate string */
    return(go);
            /* THIS ESC STUFF DOESN'T WORK YET - USE CBREAK MODE?? */
}

#endif /* INTERACT */


#define BELL 0x07

local void beep()
{   
    if (bell_level)
        putchar((char) BELL);
}

/*
 * PARNAME: extract name from name=value string.
 * W A R N I N G :  returns ptr to static storage, that may also be empty
 */

local string parname(arg)
string arg;                     /* string of the form "name=value" */
{
    permanent char namebuf[64];
    char *ap, *np;

    ap = (char *) arg;
    while (*ap == ' ')          /* skip initial whitespace */
        ap++;
    np = &namebuf[0];               /* point to spot where to copy to */
    while ((*np = *ap) != NULL) {       /* copy until '=' or ' ' */
#if 0
        if (*np == '=' || *np == ' ') {
#else
        if (*np == '=') {
#endif
            *np = NULL;
            return ((string) namebuf);
        }
        np++;
        ap++;
    }
#if 0
    namebuf[0] = 0;
    return ((string) namebuf);  /* return pointer to null string */
#else
    return NULL;                /* return NULL (nothing found) */
#endif    
}

/*
 * PARVALUE: extract value from name=value string.
 *              skips initial whitespace        PJT
 *              BUG:  when HELPVEC is not defined, helpvec also returned
 */

local string parvalue(arg)
string arg;                     /* string of the form "name=value" */
{
    char *ap;

    ap = (char *) arg;
    while (*ap != NULL)
        if (*ap++ == '=') {
            while (*ap == ' ')      /* skip whitespace after '=' */
                ap++;
            return ((string) ap);
        }
#if 0        
    return ((string) ap);	    /* return pointer to null string */
#else
    return NULL;                    /* return nothing found */
#endif    
}
/* 
 * PARHELP: extract help from a defv[] "keyword=value\nhelp" string
 * W A R N I N G : patches the '\n' (BOH) with a '\0' (EOS) and returns
 *      pointer to help string - only call this once per 'defv[i]' !!!!
 *      plus need -fwriteable-strings in GNU compiler
 */
local string parhelp(arg)
string arg;
{
    char *cp;

    cp = arg;    
    while (*cp != '\0' && *cp != '\n')      /* scan until EOS or BOH */
        cp++;
    if (*cp == '\n')                        /* if BOH, patch it up */
        *cp++ = '\0';           /* core dump if these string are in text !! */
    
    while (*cp == ' ' || *cp == '\t')       /* skip white spaces after BOH */
        cp++;
    return ( (string) cp);      /* return NULL (no help) or valid help */
}

/*
 * SCANBIND: scan binding vector for name, return index. (>= 0)
 *           the binding vector array needs to be NULL terminated
 */

local int scanbind(bvec, name)
string bvec[];  /* NULL terminated array of "key=val" strings */
string name;    /* 'key' to look for */
{
    int i, j, match=0;

    for (i = 0; bvec[i] != NULL; i++)
        if (matchname(bvec[i], name, TRUE)) return i;
#if 0
    /****  There is a 'bug' in binding unknown names that prevents us
           from taking this path : the shorthand key=val will be the 
           key's name, and the long name will be considered an
           unknown key ****/
           
    /*     
     * have arrived here, no exact match was present, let's try
     * a minimum match
     */
       
    for (i = 0; bvec[i] != NULL; i++) {
        if (matchname(bvec[i], name, FALSE)) {
            j = i;
            match++;
        }
    }
    if (match==0) return -1;
    if (match==1) return j; 
    warning("There were %d matches for keyword %s",match,name);

#endif    
    return -1;
}

/*
 * MATCHNAME: determine if "name=value" matches "name".
 *              also skip blanks around 'name' in 'name=value'
 *      bind : name=value pair, normally from the bindvec[] array
 *      name : name to check, but could be NULL ...
 *      exact: need exact match, or kan 'name' be partial?
 */
#ifdef PROTO
local bool matchname(string bind, string name, bool exact)
#else
local bool matchname(bind, name, exact)
string bind, name;
bool exact;
#endif
{
    char *bp, *np;
    
    if (name==NULL) return FALSE;

    bp = bind;
    np = name;
    while (*bp == ' ')          /* skip initial blanks */
        bp++;
    while (*np == ' ')          /* skip initial blanks */
        np++;
    while (*bp == *np) {
        bp++;
        np++;
    }
    if (exact)
    	return (*bp == '=' || *bp == ' ') && *np == NULL;
    else
        return *np==NULL || *np==' ';
}

/*
 * BINDPAR: construct "name=value" string.
 *      Special warnings with the VERSION keyword,
 *      but do allow it to be set to whatever is requested
 *      At the end though, VERSION written to disk should
 *      always be the internal one.
 */

local string bindpar(name, value)
string name;
string value;
{
    char bindbuf[128];

    if (streq(name,"VERSION")) {
        if (!streq(version_i,value)) {
            if (version_e) {
                warning("Resetting version_e: %s -> %s",version_e,value);
                free(version_e);
            } else
                warning("bindpar: internal[%s] and external[%s] VERSION id differ",
                         version_i,value);
            version_e = scopy(value);
        }
    }
    if ( (int)strlen(name)+(int)strlen(value)+1 > 128)
        error("bindpar: not enough space for key=val");

    sprintf(bindbuf, "%s=%s", name, value);
    return (copxstr(bindbuf, sizeof(char)));
}

#if defined(INTERACT)

/* READKEYS, WRITEKEYS:
 * 
 *  These local routines read and write the keyword files.
 *
 *  Some peculiar notion to the readkeys routine is that
 *  the first time around  ...
 *
 *  The writekeys routine always writes out the internal version
 *  ID, not the external. This may result in excessive warnings
 *  when lot's of keyroutine I/O is done.
 *
 *  Globals used: 
 *
 *          key_filename        the name of the keyword file
 *          bindvec[ndef]       the array of "key=value" bindings
 *
 */

local void writekeys(mesg)
string mesg;     /* identifing caller */
{
    FILE *keyfile;
    int i;

    keyfile = fopen(key_filename,"w");           /* open to (over) write */
    if (keyfile==NULL)
        error("%s: Cannot write to keyfile \"%s\", - no write permission?",
                                                 mesg, key_filename);
    fprintf(keyfile,
        "# Program: %s\n",parvalue(bindvec[0]));
    fprintf(keyfile,
        "# keyword file written by nemo (help level=1,4)\n");
    for (i=1; bindvec[i]; i++) {
        if (streq(parname(bindvec[i]),"VERSION"))
            fprintf(keyfile,"VERSION=%s\n",version_i);
        else
            fprintf(keyfile,"%s\n",bindvec[i]);
    }
    fprintf(keyfile,
        "#### end of keywords - Save file and exit editor to execute program\n");
    fprintf(keyfile,
        "#### To get more help on keywords use help=h option somewhere\n");
    fclose(keyfile);
} /* writekeys */


#ifdef PROTO
local bool readkeys(
string mesg,		/* identifing the caller                              */
bool useflag,		/* incoming 'usage' check on "???" -- no triagraph -- */
bool needit,		/* is file really needed? If true, error if not exist */
string defv[]           /* NULL term'd array of default "key=val" strings     */
)
#else
local bool readkeys(mesg, useflag, needit, defv)
string mesg;		/* identifing the caller                              */
bool useflag;		/* incoming 'usage' check on "???" -- no triagraph -- */
bool needit;		/* is file really needed? If true, error if not exist */
string defv[];           /* NULL term'd array of default "key=val" strings     */
#endif
{
    FILE *keyfile;
    int i, i1;

    keyfile = fopen(key_filename,"r");
    if (keyfile==NULL && needit==TRUE)
        error("%s Cannot read keyfile \"%s\" ",mesg,key_filename);
    if (keyfile==NULL) return useflag;
    while (fgets(keybuf,256,keyfile) != NULL) {
        if (keybuf[0]=='#' || keybuf[0]=='\n' || keybuf[0]==' ')  /* skip these */
            continue;
        if (keybuf[strlen(keybuf)-1] != '\n')
            warning("readkeys: reading incomplete lines from %s",key_filename);
        keybuf[strlen(keybuf)-1] = '\0';          /* get rid of newline */
        if(streq(parname(keybuf),"VERSION")) 
            if(!streq(version_i,parvalue(keybuf)))
                warning("readkeys: internal[%s] and external[%s] VERSION id differ",
                    version_i,parvalue(keybuf));
        if(needit) {            /* second time around: don't need defv */
            i = scanbind(bindvec,parname(keybuf));             /* get index */
            if (i<=0) {                                  /* illegal keyword ? */
                warning("initparam: ignoring erroneous keyword %s",keybuf);
                continue;
            }
            if (streq(parvalue(bindvec[i]),"???") && streq(parvalue(keybuf),"???"))
                useflag = TRUE;
            if ((int) strlen(keybuf) > (int) strlen(bindvec[i]))
                bindvec[i] = (char *) allocate(strlen(keybuf)); /* realloc */
        } else {                /* first time around: must get from defv */
            i = scanbind(defv,parname(keybuf)) + 1;  /* check if valid key */
            if (i<1) {
                warning("initparam: ignoring keyfile parameter %s",keybuf);
                continue;
            }
            i1 = scanbind(bindvec,parname(keybuf));       /* check if now */
            if (i1>=0) continue;            /* ignore: it was already in argv */
            bindvec[i] = (char *)allocate(strlen(keybuf)); /* alloc for default */
        }
        strcpy (bindvec[i],keybuf);                   /* put into place */
    } 
    fclose(keyfile);
    return useflag;
} /* readkeys */

#endif

#if defined(REMOTE)
#include <sys/param.h>          /* max length of getwd name */
/*
 *   RSH : put the command line on a remote host in background.
 *         This is a rather system dependant part, 
 *         and may not work if NFS uses different names on different
 *         machines... automounters may also screw up?? Not tested
 *
 *              Peter Teuben
 */

#define MAXARGS   100

local char *newargv[MAXARGS] = {
        "rsh",                  /* rsh: shell is defined in /etc/passwd */
        NULL,                   /* node name to be filled in */
        "-n",                   /* needed for background processes */
        "(cd",                  /* Note start of subshell */
        NULL,                   /* current directory name t.b.f.i */
        ";",                    /* command separator */
        NULL,                   /* rest of old argv[] will be from here on */
};

local char trailer[5] = "&)";   /* putting in background + end subshell */
                        
local int rsh(node,argv)
char *node;
char *argv[];
{
    char host[32];              /* current host name */
    char cwd[MAXPATHLEN];       /* current directory (full path) */
    char *getwd();
    int  i, j;
        
    gethostname (host,32);                      /* get current host */
    if ((strlen(node)==0) || (node[0]==' ') || (strcmp(node,host)==0))
        return(0);                      /* YES, program can run here */
    newargv[1] = node;          /* replace node-name */             
    getwd(cwd);                 /* get current directory */
    newargv[4] = cwd;           /* replace it */
    j=0;
    while (newargv[j]!=NULL)    /* first part of new cmdline */
        j++;
    i=0;
    while (argv[i] != NULL)     /* now the old part */
        newargv[j++] = argv[i++];
    newargv[j] = trailer;
    newargv[j+1] = NULL;
    dprintf(5,"RSH_DEBUG:");            /* DEBUG output */
    for (j=0; newargv[j]!=NULL; j++)
        dprintf(5," %s ", newargv[j]);
    dprintf(5,"\n");
    execvp("rsh", newargv);             /* this one will scream and die if  */
                                        /* node name doesn't exist          */
    error("getparam(rsh): error executing rsh");
    return(1);                          /* will never get here */
}

#endif /* REMOTE */


#if defined(INTERRUPT)
        /* should not be called if I/O redirection is performed */
local void review()
{
        char cmd[80], *key, *cp, prompt;
        int  i, pid, getpid();
        double cputime();

        if (!initparam_done) {
            fprintf(stderr,"Cannot interrupt yet, let initparam() finish\n");
            return;
        }
        if (review_flag == 0) return;
                
        fflush(stdin);
        fflush(stdout);
        fprintf(stderr,"REVIEW called, enter your command: \n");
        fflush(stderr);

        prompt = '>';

        for (;;) {                              /* requesting loop */
           sprintf(cmd,"(%d)", getpid());       /* get process id */
           if (!debug_level)
              cmd[0] = '\0';
           fprintf(stderr,"REVIEW|%s%s%c ",getargv0(),cmd,prompt);
           fflush(stderr);
           gets(cmd);
           if (strcmp(cmd,"end")==0 || strcmp(cmd,"quit")==0) { /* all done */
              exit(0);                                    /* plain exit */
              /*NOTREACHED*/
           }
           if (strcmp(cmd,"stop")==0) {                 
              stop(0);                                   /* graceful exit */
           }else if (strcmp(cmd,"continue")==0 || strcmp(cmd,"go")==0) {
              fflush(stdin);
              fflush(stdout);
              break;
           }else if (strncmp(cmd,"task ",5)==0) {
              cp = &cmd[5];
              pid = fork();
              if (pid<0) {
                 dprintf(0,"Unable to fork: %s\n",cmd);
              } else if (pid==0) {      /* child - will run the job */
                 /* execvp(path,args); */    /* do it */
                 dprintf(0,"[STILL TO DO] Unable to execute: %s\n",cmd);
                 exit(1);   /* if child didn't work out, kill this thread */
                 /*NOTREACHED*/
              } 
              /* now in parent : can just live on...and on...and on...*/
           }else if (strncmp(cmd,"set ",4)==0) {        /* set new value for key */
              cp = &cmd[4];
              key = parname(cp);
              i = scanbind(bindvec,key);
              if (i==0)         
                  fprintf(stderr,"Cannot change name of program....\n");
              else if (i>0) {   /* regular keyword */
                  if (strlen(bindvec[i]) < strlen(cp))
                       bindvec[i] = (char *) allocate(strlen(cp));
                  strcpy (bindvec[i],cp);
              } else {          /* unknown keyword, check if system */
                  i = check_syskey(cp,"debug",&debug_level) |
                      check_syskey(cp,"help",&help_level)   |
                      check_syskey(cp,"yapp",&yapp_dev)     |
                      check_syskey(cp,"error",&error_level); 
                  if (!i)
                      fprintf(stderr,"Syntax error (%s)\n",cmd);
              }
           } else if (strncmp(cmd,"show ",5)==0) {      /* show a keyword */
              key = &cmd[5];
              i = scanbind(bindvec,key);
              if (i>0)
                  fprintf(stderr,"%s\n",bindvec[i]);
              else
                  fprintf(stderr,"keyword %s unknown\n",key);
           } else if (strcmp(cmd,"keys")==0) {          /* show all keywords */
              for (i=0; bindvec[i] != NULL; i++)
                 fprintf(stderr,"%s\n",bindvec[i]);
           } else if (strcmp(cmd,"syskeys")==0) {   /* show system keywords */
              fprintf(stderr,"debug=%d\n",debug_level);
              fprintf(stderr,"error=%d \n",error_level);
              fprintf(stderr,"help=%d\n",help_level);
              fprintf(stderr,"yapp=%d (%s)\n",yapp_dev,yapp_string);
           } else if (strcmp(cmd,"version")==0) {
              fprintf(stderr,"version id = %s\n",VERSION_ID);
           } else if (strcmp(cmd,"time")==0) {
              fprintf(stderr,"cputime = %g\n",cputime());
           } else if (cmd[0]=='!' && cmd[1] != '\0') {  /* run a command */
              if (system(&cmd[1])<0)
                  fprintf(stderr,"Couldn't spawn another task\n");
           } else if (cmd[0]=='?' || strncmp(cmd,"help",4)==0) {
              if (strlen(cmd) < 6) {
                fprintf(stderr,"help,?             help (this list)\n");
                fprintf(stderr,"continue,go        back to program\n");
                fprintf(stderr,"end,quit           exit() to shell\n");
                fprintf(stderr,"stop               graceful finiparam() exit\n");
                fprintf(stderr,"task name par(s)   start up new task\n");
                fprintf(stderr,"set <key>=<val>    enter new val for key\n");
                fprintf(stderr,"show <key>         show val of key\n");
                fprintf(stderr,"keys,set           show values of all keys\n");
                fprintf(stderr,"syskeys,sets       show values of all syskeys\n");
                fprintf(stderr,"set <syskey>       show value of system keyword\n");
                fprintf(stderr,"set <syskey>=<val> set new val to system keyword\n");
                fprintf(stderr,"time               show cputime used\n");
                fprintf(stderr,"!cmd               run a shell command\n");
                fprintf(stderr,"<HOTKEY>           cause coredump anyhow\n");
                fprintf(stderr,"version            display version of INITPARAM\n");
              } else {
                review_help(&cmd[5]);
              }
           } else if (cmd[0]=='\0')
              continue;
           else
              fprintf(stderr,"Syntax error, try 'help'\n");
        } /* for(;;) */
}

local int check_syskey(cmd,syskey,value)
char *cmd, *syskey;
int  *value;
{
    int len;

    len=strlen(syskey);
    if (strncmp(cmd,syskey,len))
        return(0);
    if (cmd[len]=='\0') {         /* display system keyword */
        fprintf(stderr,"%s=%d\n",cmd,*value);
        return(1);
    } else if (cmd[len]=='=') {     /* set system keyword */
        if (cmd[len+1]=='\0') {
            fprintf(stderr,"System keyword must be given a value\n");
            return(1);
        }
        *value = atoi(&cmd[len+1]);
        return(1);
    } else                          /* error or unknow (system) keyword */
        return(0);
}

local void review_help(cmd)
char *cmd;
{
    char command[256], *pager, *getenv();
    
    pager = getenv("PAGER");

    sprintf(command,"%s %s/%s.doc",
            (pager == NULL ? "more" : pager),
            getenv("NEMODOC"),cmd);
    printf("%s\n",command);
    system(command);
}
#endif  /* INTERRUPT */

#if defined(sun) && defined(USE_IEEE)

local void ieee_nemo(sig, code, scp, addr)
int sig;               /* sig == SIGFPE always */
int code;
struct sigcontext *scp;		/* solaris warning: dubious tag declaration */
char *addr;
{
 /* warning("ieee exception code 0x%x occurred at pc 0X%X", code, scp->sc_pc); */
    if (debug_level>0) abort();
}
                                                   
#endif


#if defined(TESTBED)

string defv[] = {
    "flag1=???\n        This needs a value. Perhaps even more",
    "flag2=false\n      A false flag2",
    "infile=r000.snap\n just a name, but what's is a name",
    "tstop=10.0\n       just a number (you may try parsing an expression)",
    "foobar=12345\n     just some digits",
    "prompt=t\n		Checking interactive prompting routines?",
    "VERSION=1.1b\n     25-feb-92 PJT",
    NULL,
};

string usage = "A testbed for getparam.c";

nemo_main()
{
    bool flag2, prompt;
    double tstop;

    printf("program %s:\n", getargv0());
    check("argv0");
    check("flag1");
    check("flag2");
    check("infile");
    check("tstop");
    check("foobar");
    check("prompt");

    dprintf(0,"debug level 0 printout\n");
    dprintf(1,"debug level 1 printout\n");
    dprintf(2,"debug level 2 printout\n");
    dprintf(9,"debug level 9 printout\n");
    dprintf(11,"debug level 9 printout\n");

    flag2 = getbparam("flag2");    
    tstop = getdparam("tstop");
    printf ("flag2=%d tstop=%f\n",flag2,tstop);

#if defined(PUTPARAM)
    putparam("flag1","false");
    if(getbparam("prompt")) {
        for (;;) {              /* infinite loop */
#if defined(INTERRUPT)
            promptparam("foobar",
                "(REVIEW interruptable) Enter a new value for foobar");
            check("foobar");
#else
            promptparam("flag1","flag1 determines end of infinite loop");
            check("flag1");
#endif 
            if (getbparam("flag1"))
                break;
        }
    }
    putparam("nosuchkey","nosuchprompt");       /* should cause error() */
#endif
} /* nemo_main */

check(par)
string par;
{
    string val;

    val = getparam(par);
    printf("%s has value \"%s\"\n", par, val);
}

#endif  /* TESTBED */


