/*
 * VECTMATH.C: source code for vector/matrix operations.
 * Joshua Barnes  1 December 1986  Princeton, NJ.
 *	29-jul-92	convex fix		PJT
 */

#include <stdinc.h>

#ifndef SINGLEPREC
#  define _dotvp	_dotvp_d
#  define _absv		_absv_d
#  define _distv	_distv_d
#  define _tracem	_tracem_d
#else
#  define _dotvp	_dotvp_f
#  define _absv		_absv_f
#  define _distv	_distv_f
#  define _tracem	_tracem_f
#endif

double _dotvp(v, u, n)
register real *v;
register real *u;
register int n;
{
    real s;

    s = 0.0;
    while (--n >= 0)
	s += (*v++) * (*u++);
    return (s);
}

double _absv(v, n)
register real *v;
register int n;
{
    real s;

    s = 0.0;
    while (--n >= 0) {
	s += (*v) * (*v);
	v++;
    }
    return (sqrt(s));
}

double _distv(v, u, n)
register real *v;
register real *u;
register int n;
{
    real d, s;

    s = 0.0;
    while (--n >= 0) {
	d = (*v++) - (*u++);
	s += d * d;
    }
    return (sqrt(s));
}

double _tracem(p, n)
register real *p;
int n;
{
    double s;
    register int i;

    s = 0.0;
    for (i = n; --i >= 0; ) {
	s += (*p);
	p += (n + 1);			/* next diag. element */
    }
    return (s);
}
