/*
 * XRANDOM: better (portable) floating-point random number routine.
 *
 *	-DNUMREC: a portable one from NumRec can be used (currently ran3)
 *      -DRAND48: the standard 'rand48' pseudo-random are used 
 *                (lrand48/srand48 instead of random/srandom)
 *
 *  xx-xxx-86   created                                     JEB
 *  xx-xxx-90   introduced the NUMREC portable option       PJT
 *  15-apr-91   set_xrandom() now returns it's seed!        PJT
 *   7-jan-92	fixed bug which made seed's in set_xrandom() useless	PJT
 *  18-may-92   documentation 
 *  29-jul-92   convex declaration fix - no more math       PJT
 *  18-feb-94   changed default to NUMREC (Solaris)  - ansi PJT
 *  24-mar-94   made -DRAND48 a full option		    PJT
 */

#include <stdinc.h>
#include <sys/types.h>
#include <sys/times.h>

#if defined(mc68k)
# define RAND48
#endif


#if defined(RAND48)
extern double drand48(void);
extern void   srand48(long);
#endif


#if defined(NUMREC)
# define portable_ran  ran3
real portable_ran(int *);
#endif

local int idum;            /* local variable to store used seed */

int set_xrandom(dum)
int dum;
{
    int retval;

    if (dum==0)
        retval = idum = (int) time(0);     /* get unix time as random seed */
    else
    	retval = idum = dum;	           /* use supplied seed in argument */

#if defined(NUMREC)
    dprintf(2,"set_xrandom(NUMREC portable) seed=%d\n",idum);
    if (idum > 0) idum = -idum; /* ran needs a negative idum as seed */
    portable_ran(&idum);        /* use portable seed: will only work once */
    idum = 0;                   /* reset to avoid reentry ... */
#elif defined(RAND48)
    dprintf(2,"set_xrandom(UNIX srand48) seed=%d\n",retval);
    srand48(retval);
#else    
    dprintf(2,"set_xrandom(UNIX srandom) seed=%d\n",retval);
    srandom(retval);
#endif    
    return retval;
}

double xrandom(xl, xh)
double xl, xh;		 /* lower, upper bounds on random number */
{
#if defined(NUMREC)
    return (xl + (xh - xl) * ((double) portable_ran(&idum)));
#elif defined(RAND48)
    return xl + (xh - xl) * drand48();
#else    
    return (xl + (xh - xl) * ((double) random()) / 2147483647.0);
#endif
}

/*
 * GRANDOM: normally distributed random number (polar method).
 * Reference: Knuth, vol. 2, p. 104.
 */

double grandom(mean, sdev)
double mean, sdev;	/* parameters for normal distribution */
{
    double v1, v2, s;

    do {				/* loop until */
	v1 = xrandom(-1.0, 1.0);	/* two points within */
	v2 = xrandom(-1.0, 1.0);	/* the unit square */
	s = v1*v1 + v2*v2;		/* have radius such that */
    } while (s >= 1.0);			/* point fall inside unit circle */
    return (mean + sdev * v1 * sqrt(-2.0 * log(s) / s));
}



#if defined(TESTBED)

#include <getparam.h>

string defv[] = {
    "n=4\n      Number of random numbers",
    "seed=1\n   Seed [0->time]",
    "gauss=f\n  Test on gaussian noise?",
    NULL,
};

string usage="TESTBED for xrandom";


string defaults[] = {
    "srandom:        0.968071  0.0667306 0.478281  0.909534",
    "srand48:        0.0416303 0.454492  0.834817  0.335986",
    "numrec ran3:    0.715119  0.0330211 0.874394  0.534194",
    NULL,
};

nemo_main()
{
    int i, seed, n;
    double sum[5], mean, sigma, skew, kurt, y, s;
    bool   Qgauss = getbparam("gauss");
    string *sp;

    n = getiparam("n");
    seed = getiparam("seed");

    printf("Will use n=%d seed=%d\n",n,seed);

    seed = set_xrandom(seed);
    printf("Seed used = %d\n",seed);

    if (Qgauss) {
        sum[0] = sum[1] = sum[2] = sum[3] = sum[4] = 0.0;
        while (n-- > 0)
            for (i=0, s=1.0, y = grandom(0.0,1.0); i<5; i++, s *= y)
                sum[i] += s;
        
        mean  = sum[1]/sum[0];
        sigma = sqrt(sum[2]/sum[0]-mean*mean);
        skew  = ( (sum[3]-3*sum[2]*mean)/sum[0] + 2*mean*mean*mean) /
                           (sigma*sigma*sigma);
        kurt  = ( (sum[4]-4*sum[3]*mean+6*sum[2]*mean*mean)/sum[0]
              - 3*mean*mean*mean*mean) /
	     (sigma*sigma*sigma*sigma)  - 3.0;

        printf ("Mean and dispersion  : %f %f\n",mean,sigma); 
        printf ("Skewness and kurtosis: %f %f\n",skew,kurt);
    } else {
        if (n==4) {
            printf("Known n=4 cased are:\n");
            for (sp=defaults; *sp; sp++) printf("%s\n",*sp);
        }
        while (n-- > 0) 
            printf("%g\n",xrandom(0.0,1.0));
            
    }
}

#endif
