/* 
 * AZEL_IO	  get and set last AZ/EL of telescope
 *
 *  21-jul-93  added warning if a source passing close to elev=90
 */

#include <nemo.h>

#define DELMAX  5.0	/* near zenith check */

static FILE *fp = NULL;
static char *file_name = NULL;
static void init_name();

int get_azel(az,el)
float *az, *el;
{
    init_name();

    fp = fopen(file_name,"r");
    if (fp==NULL) return 0;
    if (fscanf(fp,"%f %f",az,el) != 2) {
        warning("get_azel: problem reading az & el"); 
        *az = 0;
        *el = 0;
    }
    fclose(fp);

    return 1;
}

set_azel(az,el)
float az,el;
{
    init_name();
    fp = fopen(file_name,"w");
    if (fp==NULL) {
       printf("### Warning: set_azel: Cannot create %s\n",file_name);
       return 0;
    }
    fprintf(fp,"%f %f",az,el);
    fclose(fp);
    return 1;
}

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

static void init_name()
{
    char *cp, name[MAXPATHLEN];

    if (file_name==NULL) {
        cp = getenv("HATCOM");
        if (cp==NULL) {
            fprintf(stderr,"No env.var. $HATCOM\n");
            exit(1);
        }
        sprintf(name,"%s.azel",cp);
        file_name = malloc(strlen(name)+1);
        strcpy(file_name,name);
    }
}
/* return slowing time in decimal house from 0 -> 1 */
double slew(az0,el0,az1,el1)
float az0,el0,az1,el1;
{
    static double az_rate = 40.0;
    static double el_rate = 80.0;
    int maz, mel;

    /* az and el are assumed in degrees */
    maz = ceil(ABS(az0-az1)/az_rate);       /* round up to minutes */
    mel = ceil(ABS(el0-el1)/el_rate);
    return MAX(maz,mel)/60.0;
}

/* Get az/el for a source */

azel(src,cat,az,el)
string src,cat;
float *az, *el;
{
    char cmd[256];
    FILE *fp;

    strcpy(cmd,"checksource");
    if (*src) {
        strcat(cmd," source=");
        strcat(cmd,src);
    }
    if (*cat) {
        strcat(cmd," catalog=");
        strcat(cmd,cat);
    }
    strcat(cmd," setme=azel");
    fp = popen(cmd,"r");
    if (fscanf(fp,"%f %f",az,el) != 2) {
        warning("azel: problem getting AZ & EL for source=%s catalog=%s",
                 src,cat);
        *az = 0.0;
        *el = 0.0;
    }
    if (ABS(*el - 90) < DELMAX) fprintf(stderr,
               "### Warning; SOURCE %s ELEV = %g too close to zenith\n",
                               src,*el);
    pclose(fp);
}


