/*
 *  CROSS/DIAMOND - pointing program (known under various names...)
 */

#include <nemo.h>

string defv[] = {
    "source=\n		source name",
    "catalog=\n		catalog to use to find source",
    "ants=\n		antenna selection flag",
    "refants=\n		reference ant",
    "line=wide,1,1\n	line",
    "file=\n		",
    "itime=10.0\n	",
#ifndef DIAMOND
    "npoints=3\n	number of points",
#endif
    "step=1\n		stepsize (arcmin)",
    "options=\n		various options (unused?)",
    "times=-1,-1\n	integration times",
    "trakopt=\n		tracking",
    "VERSION=0.3\n	20-aug-93 PJT",
    NULL,
};

#ifdef DIAMOND
string version="Daimond pointing";
#else
string version="Cross pointing";
#endif

nemo_main()
{
    real get_lst(), slew();
    real lst = get_lst();
    float az0, el0, az1, el1, dt, itime;
    string cat = getparam("catalog"), 
           src = getparam("source"),
           name = getargv0();

    check_line("line",getparam("line"));
#ifndef DIAMOND
    check_int_range("npoints",getparam("npoints"),1,9,"");
#endif
    check_real_range("step",getparam("step"),1.0,120.0,"arcmin");
    check_real_range("itime",getparam("itime"),1.0,3600.0,"seconds");
    check_source(src,cat);

    printf("%s: source=%s started at lst=%g\n",name,src,lst);
    azel(src,cat,&az0,&el0);
    printf("%s: started at az,el=%f %f\n",name,az0,el0);
    if (get_azel(&az1,&el1)) {
        dt = slew(az1,el1,az0,el0);
    } else
        dt = 0.0;
    itime = 4 * getdparam("itime")/3600.0;
    printf("SLEW/OBS %s %f %f %f\n",src,lst,dt,itime);
    lst += itime + dt;
    
    set_lst(lst);
    printf("%s: stopped at lst=%g\n",name,lst);

    azel(src,cat,&az1,&el1);
    printf("%s: stopped at az,el=%f %f\n",name,az1,el1);
    if (el0 < 0 || el1 < 0) warning("Source %s was set",src);
    set_azel(az1,el1);
}
