/*
 * QINT program stub for HatCreek
 *	  24-mar-94 cloned from (M)INT - for testing only
 *	   9-may-94 removed slow= and nminor=
 */

#include "checker.h"

string defv[] = {
    "source=\n              source name from catalog, or planet or COMET",
    "catalog=./j2000.cat\n  catalog where to search for source", 
    "ants=\n",
    "itime=50\n         integraton time in seconds",
    "grid=1x1\n         Grid selector (a file)",
    "scale=1\n          Scale factor for grid offsets",
    "nreps=1\n          Number of times to repeat the grid",
    "quick=1\n          1=all lags  32=1 lag only",
    "vis=qintdata\n     output dataset",
    "trakopt=\n         CH should be default",
    "times=-1,-1\n      start and stop time (decimal hours)",
    "start=\n           Start time [now]",
    "stop=\n            Stop time [1 hour] [+100 in future]",
    "VERSION=1.0\n      3-may-94 PJT",    
    NULL,
};

string usage="HatCreek QINT program stub";


char msg1[32], msg2[32], msg3[32], msg4[32];
char *show_lst();
char *strdup();

nemo_main()
{
    bool new_ss;	/* use this for the new start/stop time method */
    real times[2], itime;
    real get_lst(), slew(), get_stop();
    real idle, slew_time, start, stop, new_lst, on_source, lst = get_lst();
    float az0, el0, az1, el1;
    int n, nreps;
    string src, cat, lst0, lst1, name = getargv0();
    char s[32];
    bool check_time();


    strtoupper(name);

    needvalue("source");    
    src = getparam("source");
    cat = getparam("catalog");
    itime = getdparam("itime");

    new_ss = hasvalue("stop");
    if (!new_ss) {
       n = nemoinpd(getparam("times"),times,2);
       if (n != 2) error("times= needs two values");
    } else {
       if (hasvalue("start")) {
         start = getTime(getparam("start"), (int) 60.0*lst) / 60.0;
         if (check_time(start))
           error("Cannot start before the project start time");
         idle = start - lst;
         if (idle > 0.25)
            warning("sleeping a bit (%d minutes)",(int)60*idle);  
         if (idle < 0.0) {
            warning("you started too late!");
            idle = 0.0;
         }
         lst = start;
	 set_lst(lst);	/* sync the clocks */
       } 
       if (check_time(lst)) 
         error("Current time (%g) outside of project time range",lst);
    }

    check_source(src,cat);
    check_filename("vis",getparam("vis"));

    azel(src,cat,&az0,&el0);
    dprintf(2,"%s: source=%s started at lst=%g az=%g el=%g\n",
		name,src,lst,az0,el0);

    lst0 = strdup(show_lst(lst));

    if (get_azel(&az1,&el1))
        slew_time  = slew(az1,el1,az0,el0) + idle;
    else
        slew_time  = idle;

    if (new_ss) {                           /* new Start/Stop */
      stop = getTime(getparam("stop"), (int) 60.0*lst) / 60.0;
      if (check_time(stop)) {
        stop = getTime("stop", (int) 60.0*lst) / 60.0;
        warning("stoptime truncated by project stoptime");
      }
      on_source = stop-lst;
      if (on_source < 0) on_source += 24.0;
      dprintf(2,"SLEW/OBS %s %f %f %f\n",src,lst,slew_time, on_source);
      lst = stop;
    } else {                                /* old Times= */
      if (times[0] < 0) {
        if (times[1] < 0) {              /* incremental */
            on_source = ABS(times[1]);
            dprintf(2,"SLEW/OBS %s %f %f %f\n",src,lst,slew_time, on_source);
            lst += ABS(times[1]);
        } else {                          /* absolute : yuck */
            sprintf(s,"%g",times[1]);
	    check_real_range("times(2)",s,0.0,24.0,"hours");
            on_source = times[1] - lst;
            dprintf(2,"SLEW/OBS %s %f %f %f\n", src, lst, slew_time, on_source);
            lst = times[1];
        }
      } else {
        error("times(1) > 0 not implemented");
      }
    }

    check_real_range("itime",getparam("itime"),0.32,580.0,"seconds");
    check_real_range("scale",getparam("scale"),0.0,999999999.0,"");
    nreps = getiparam("nreps");
    if (nreps>1) warning("Mel sez not to use nreps > 1");

    set_lst(lst);
    dprintf(2,"%s: stopped at lst=%g\n",name,lst);

    azel(src,cat,&az1,&el1);
    dprintf(2,"%s: stopped at az,el=%f %f\n",name,az1,el1);
    lst1 = strdup(show_lst(lst));
    sprintf(msg1,"%s-%s",lst0,lst1);
    sprintf(msg2,"%5.1f-%5.1f",az0,az1);
    sprintf(msg3,"%5.1f-%5.1f",el0,el1);
    sprintf(msg4,"%s  %s",strdup(show_lst(slew_time)), 
			  strdup(show_lst(on_source)));


    if (el0 < 0 || el1 < 0) warning("Source %s was set",src);
    set_azel(az1,el1);

    /* final (and only!) output */

    dprintf(1,"%s\n",OBS_HEADER);   /* this is a printf() in setfreq.c !!! */
    printf("%s: %-12s %2s %5.1f %s %s %s %s\n",name,src,
		"--", itime,msg1,msg2,msg3,msg4);

}
