/*
 *  SETCORR: set correlator - for checker only
 *	assuming the V3 cora software
 *      4-aug-94    added hanning=
 *	8-sep-94    1.1: mode= -> cormode=, added restfqs=
 *	9-sep-94    cormode finally 
 *     13-sep-94    1.1c: more checks on max # restfqs entered using coropt=
 */

#include <nemo.h>

string defv[] = {
    "cormode=8\n              correlator mode (1..8)",
    "corf=200,400,600,800\n   LO settings (Mhz)",
    "corbw=100,100,100,100\n  Bandwidths (6.25/12.5/25/50/100) (Mhz)",
    "ants=\n                  antenna flag",
    "coropt=0\n               0=cross 1=auto correlation",
    "corbits=2\n              number of bits used to digitize",
    "restfqs=\n		      Optional restfreq's for each window",
    "hanning=n\n              Apply hanning ?",
    "VERSION=1.1c\n           13-sep-94 pjt",
    NULL,
};

string version="input the correlator settings (Version 3)";

#define MAXCORF  4
#define MAXCORBW 4
#define MAXWIN  16

nemo_main()
{
    real corshift, corf[MAXCORF], corbw[MAXCORBW], restfqs[MAXWIN], rf;
    int i, mode, ncorf, ncorbw, nrest, automode;
    bool Qhann;
    string *rval, *burststring();

    printf("SETCORR: cormode=%d corf=%s Mhz corbw=%s Mhz\n",
            getiparam("cormode"), getparam("corf"), getparam("corbw"));
    check_int_range("coropt",getparam("coropt"),0,1,"");
    check_int_range("cormode",getparam("cormode"),1,8,"");
    check_int_range("corbits",getparam("corbits"),1,2,"");
    mode = getiparam("cormode");     /* 1..8 */
    automode = getiparam("coropt");	  /* 0= cross 1=auto */
    Qhann = getbparam("hanning");

    nrest = nemoinpd(getparam("restfqs"),restfqs,MAXWIN);
    if (nrest < 0)
        warning("Bad parsing restfqs=%s",getparam("restfqs"));
    if (nrest > 0) {
        rval = burststring(getparam("restfqs"),",");
        /* need to check for 0.0 here, if not, then do the following test */
        for (i=0; i<nrest; i++) {
            rf = atof(rval[i]);
            if (rf == 0.0) continue;
	    check_real_range("restfqs",rval[i],70.0,120.0,"GHz");
	}
    }
    if (automode)
        if (nrest > mode) warning("At most %d restfqs allowed",mode);
    else
        if (nrest > 2*mode) warning("At most %d restfqs allowed",2*mode);

    ncorf = nemoinpd(getparam("corf"),corf,MAXCORF);
    if (ncorf < 0) 
        warning("Bad parsing corf=%s",getparam("corf"));
    if (ncorf > 0) {
        rval = burststring(getparam("corf"),",");
        for (i=0; i<ncorf; i++)
            check_real_range("corf",rval[i],70.0,900.0,"MHz");
    }


    ncorbw = nemoinpd(getparam("corbw"),corbw,MAXCORBW);
    if (ncorbw < 0) 
        warning("Bad parsing corbw=%s",getparam("corbw"));

    for (i=0; i<ncorbw; i++)
        if (corbw[i] != 6.25 &&
	    corbw[i] != 12.5 &&
	    corbw[i] != 25.0 && 
	    corbw[i] != 50.0 && 
	    corbw[i] != 100.0 )
            	warning("corbw[%d]=%g should be 6.25/12.5/25/50/100 Mhz",
			 i+1,corbw[i]);
    mode = getiparam("cormode");
    if (mode == 8) {
        if (corbw[0] != 25.0 && 
	    corbw[0] != 50.0 && 
	    corbw[0] != 100.0 ) 
                warning("corbw1 should be 25/50/100 in mode 8");

    }
    if (hasvalue("ants")) 
        warning("Setting ants= is not done in normal observing");
}
