/*
 *  SETCORR: set correlator - for checker only
 */

#include <nemo.h>

string defv[] = {
    "coropt=0\n		      0=cross 1=auto",
    "cormode=4\n              correlator mode (1..4)",
    "corf=200,400,600,800\n   LO settings (Mhz)",
    "corbw=100,100,100,100\n  Bandwidths (Mhz)",
    "cormpx=\n                Multiplexing (0,1,2) - def. from corbw",
    "corshift=\n              shift rate {12.5,25,50} (Mhz)",
    "ants=\n                  antenna flag",
    "VERSION=0.5b\n           26-apr-94 pjt",
    NULL,
};

string version="input the correlator settings";

#define MAXCORF  4
#define MAXCORBW 4

nemo_main()
{
    real corshift, corf[MAXCORF], corbw[MAXCORBW];
    int i, ncorf, ncorbw;
    string *rval, *burststring();

    printf("SETCORR: mode=%d corf=%s Mhz corbw=%s Mhz\n",
            getiparam("cormode"), getparam("corf"), getparam("corbw"));
    check_int_range("coropt",getparam("coropt"),0,1,"");
    check_int_range("cormode",getparam("cormode"),1,4,"");
    check_int_range("cormpx",getparam("cormpx"),0,2,"");   

    ncorf = nemoinpd(getparam("corf"),corf,MAXCORF);
    if (ncorf < 0) 
        warning("Bad corf=%s",getparam("corf"));
    if (ncorf > 0) {
        rval = burststring(getparam("corf"),",");
        for (i=0; i<ncorf; i++)
            check_real_range("corf",rval[i],90.0,900.0,"MHz");
    }


    ncorbw = nemoinpd(getparam("corbw"),corbw,MAXCORBW);
    if (ncorbw < 0) 
        warning("Bad corbw=%s",getparam("corbw"));
    for (i=0; i<ncorbw; i++)
        if (corbw[i] < 6.25 || corbw[i] > 100.0)
            warning("corbw[%d]=%g out of range",i+1,corbw[i]);

    if (hasvalue("corshift")) {
        corshift = getdparam("corshift");
        if (corshift != 12.5 &&
            corshift != 25.0 &&
            corshift != 50.0) 
            warning("corshift=%g needs to be 12.5, 25 or 50 Mhz",corshift);
    }
    if (hasvalue("ants")) 
        warning("Setting ants= is not done in normal observing");
}
