/*
 *  SETFREQ for checker
 *
 *      20-sep-93  updated a few new parametsr added by NT
 *       3-nov-93  Rick requested more verbose
 *	 1-mar-94  added printf() line for upcoming MINT'
 *	26-mar-94  dont check source if none given
 *	 6-may-94  check ants=
 *       8-sep-94  warn if restfqs= used used here, use setcorr instead
 */

#include "checker.h"

string defv[] = {
    "freq=\n		 observing freq (GHz)",
    "iffreq=\n		 second IF freq (MHz)",
    "source=\n		 source name for doppler correction",
    "catalog=\n          catalog [j2000.cat]",
    "dopsrc=\n           New alternate to replace source=",
    "dopcat=\n           New alternate to replace catalog=",
    "restfqs=\n		 *** OLD *** 1 to 16 rest freqs (ghz) to aid window id",
    "obsline=\n		 description/name of line",
    "ants=\n             Antennas to be tuned",
    "options=\n          {hcheck, notune, musttune}",
    "harm=\n             harmonic check",
    "VERSION=0.9b\n	 8-sep-94 PJT",
    NULL,
};

string version="set frequency parameters";

nemo_main()
{
    string src,cat, ants;

    if (hasvalue("dopsrc")) {
        src = getparam("dopsrc");
        cat = getparam("dopcat");
        if (hasvalue("source")) warning("Redundant keyword source=");
        if (hasvalue("catalog")) warning("Redundant keyword catalog=");
	printf("SETFREQ: freq=%s dopsrc=%s\n",getparam("freq"), src);
    } else {
        src = getparam("source");
        cat = getparam("catalog");
        if (hasvalue("dopsrc")) warning("Redundant keyword dopsrc=");
        if (hasvalue("dopcat")) warning("Redundant keyword dopcat=");
    	printf("SETFREQ: freq=%s source=%s\n",getparam("freq"), src);
    }
    if (*src == 0) 
	warning("No source= or dopsrc= specified; no doppler tracking");
    else
        check_source(src,cat);
    
    needvalue("freq");
    needvalue("iffreq");
    if (hasvalue("restfqs"))
	warning("restfqs=%s should be used in setcorr now",
		getparam("restfqs"));


    ants = getparam("ants");
    if (!hasvalue("ants") || streq(ants,"0"))
        warning("ants= has not been specified");


    check_real_range("freq", getparam("freq"),70.0, 120.0,"GHz");
    check_real_range("iffreq", getparam("iffreq"),-20000.0, 20000.0,"MHz");
    if (strlen(getparam("obsline")) > 24)
        warning("Description for obsline= is too long; <= 24 please");

    dprintf(1,"Header line for mint/int:\n");
    printf("%s\n",OBS_HEADER);


}

