/*
 * various syntax checkers:
 *
 *      check_real_range
 *      check_int_range
 *      check_ants
 *      check_line
 *      check_filename
 *      check_option
 *
 *	needvalue
 *
 * For: check_source, see source.c
 */

#include <stdinc.h>
#include <string.h>
#include <ctype.h>

#define MAX_FILELEN  100

check_real_range(key, s, rmin, rmax, unit)
string key,s,unit;
real rmin, rmax;
{
    double rval, atof();

    if (s==0 || *s == 0) return;    /* don't check if nothing supplied */
    rval = atof(s);
    if (rval<rmin || rval>rmax)
        warning("%s: real value out of range for keyword %s\nRange: (%g,%g) %s",
			s,key,rmin,rmax,unit);
    return;
}

check_int_range(key, s, imin, imax, unit)
string key,s,unit;
int imin, imax;
{
    int ival, atoi();
    char *cp;

    if (s==0 || *s == 0) return;    /* don't check if nothing supplied */
    ival = atoi(s);
    if (ival<imin || ival>imax)
        warning("%s: integer value out of range for keyword %s\nRange: (%d,%d) %s",
			s,key,imin,imax,unit);

    cp = s;
    while (*cp) {
        if (strchr("+-0123456789",*cp++)==NULL) 
            warning("%s: an integer expected for keyword %s",s,key);
    }
}

check_ants(key,s)
string key,s ;
{
    char *cp = s;

    while (*cp) { 
        cp++;
    }
}

check_line(key,s)
string key,s ;
{			/* line={wide|channel),first_channel,width_channels */
    char *cp = s;
    while (*cp) {
        cp++;
    }
}

/*
 * filenames should be POSIX complaint, we'll simply check if the
 * last part is max 14  length 
 */

check_filename(key,s)
string key, s;
{
    char *p = strrchr(s,'/');

    if (p == 0) p = s;
    if ((int)strlen(p) > 14) 
        warning("filename too long for POSIX archiving: %s=%s",key,s);
    if ((int)strlen(s) > MAX_FILELEN)
        warning("filename %s=%s rather long (%d)",key,s,MAX_FILELEN);
}


check_option(key,s,valid)
string key, s, valid;
{
    string *burststring(), *v;
    int partial = 0;
    
    v = burststring(valid,",");
    if (v==0 || *v==0) {
        warning("(check_option) No valid options supplied for %s",key);
        return;
    }
	
    if (s==0 || *s==0) {
        warning("No value supplied for %s=",key);
        return;
    }

    while (*v) {
        if (streq(*v,s)) break;
        if (strncmp(*v,s,strlen(s))==0) partial++;
        v++;
    }
    if (partial==0)
        warning("%s=%s: no valid option from: (%s)",key,s,valid);
    else if (partial>1) 
        warning("%s=%s: more than 1 match from: (%s)",key,s,valid);
}


needvalue(key)
string key;
{
    if (!hasvalue(key)) warning("Need a value for %s=",key);
}


strtoupper(cp)
char *cp;
{
    if (cp)
        while (*cp) {
            if (islower(*cp)) *cp = toupper(*cp);
            cp++;
        }
}
