/*
 *  (M)XPNT - pointing program (known under various names...)
 */

#include <nemo.h>

string defv[] = {
    "source=",
    "catalog=",
    "ants=",
    "line=",
    "file=",
    "itime=10",
    "step=",
#ifdef REGAN
    "reslim=",
#else
    "reslim=0.2",
#endif
    "trakopt=",
#ifdef REGAN
    "threshold=",
    "maxiter=",
#else
    "niter=1",
    "log=",
#endif
    "VERSION=0.2a\n	13-jul-94 PJT",
    NULL,
};


#ifdef REGAN 
string version="Generic pointing - Mike Regan version";
#else
string version="Generic pointing - Arie & Jim version";
#endif

#ifndef MAXANT
#define MAXANT 9
#endif

nemo_main()
{
    int i, len;
    real get_lst(), slew();
    real lst = get_lst();
    float az0, el0, az1, el1, dt, itime;
    string cat = getparam("catalog"), 
           src = getparam("source"),
           name = getargv0();
    char error[80], var[32];

    strtoupper(name);

    needvalue(src);
    check_source(src,cat);
#ifndef REGAN
    check_int_range("niter",getparam("niter"),1,3,"Number of iterations");
    check_real_range("reslim",getparam("reslim"),0.0,1.0,"Resolution limit");
#endif

    printf("%s: source=%s started at lst=%g\n",name,src,lst);
    azel(src,cat,&az0,&el0);
    printf("%s: started at az,el=%f %f\n",name,az0,el0);
    if (get_azel(&az1,&el1)) {
        dt = slew(az1,el1,az0,el0);
    } else
        dt = 0.0;
    itime = 5 * getdparam("itime")/3600.0;
    printf("SLEW/OBS %s %f %f %f\n",src,lst,dt,itime);
    lst += itime + dt;
    
    set_lst(lst);
    printf("%s: stopped at lst=%g\n",name,lst);

    azel(src,cat,&az1,&el1);
    printf("%s: stopped at az,el=%f %f\n",name,az1,el1);
    if (el0 < 0 || el1 < 0) warning("Source %s was set",src);
    set_azel(az1,el1);


    /* do some random messing around with the pointing constants */
    /* change (D) DAZIMLAS, DELEVLAS for all antenna's */

    for (i=0; i<MAXANT; i++) {
        my_computd("dazimlas",i,(i+1)*0.1);
        my_computd("delevlas",i,-(i+1)*0.1);
    }

}


my_computd(string basename, int idx, double value)
{
    int len = MAXANT;
    double oldvalue[MAXANT];
    char name[32], error[80];

    sprintf(name,"%s-%d",basename,idx+1);
    comgetd_(name, oldvalue, &len, error);
    oldvalue[idx] = value;
    computd_(name, oldvalue, &len, error);
    dprintf(1,"computd: %s %g (%s)\n",name,value,error);
}
