c= COMDUMP - dumps out parts of the Interferometer Common Block 
	PROGRAM COMDUMP
c& wh
c: program
c+
c	COMDUMP dumps out parts of the Interferometer Common Block 
c	as denoted by the CLASS type that is associated with each
c	in hatshare:common.tab.  It is also possible to dump the 
c	entire common as class *.  The dump is made by default into
c	file commondump.log.  All dumps also contain a summary of
c	the state of the common block.
c
c@ class
c	Class is an identifier associated with variables in COMMON.TAB.
c	Possible values are:	I= Interferometer   B= Baseline
c	F= frequency		T= Time related     C= Correlator
c	*= All classes
c
c@ options
c	Options modify the output:
c	HMS= print angles as hours:minutes
c	DMS= print angles as degrees:minutes
c	OCTAL= print integers as octal
c	HEX= print integers as hexadecimal
c--
C  JAN 25  85  WH
c  unix-> nov 90 wh
C---------------------------------------------------C
	include '../inc/constants.inc'
	parameter (max=768)
	parameter (nclass=7)
	character*31 par_line(2)
	character*12 tname(max)
	character*1 tclass(max)
	character*8 classes /'CPBTIFR*'/
	character*30 class_title(nclass)
	data class_title /'CONSTANT parameters','POINTING parameters',
	1		'BASELINE parameters','TIME RELATED parameters',
	1		'INTERFEROMETER parameters',
	2		'FREQUENCY parameters','CORRELATOR parameters'/
	character*80 error
	character*1 type
	character*20 class,value,options
	character*14 angles
	real*4 rdata(1000)
	real*8 ddata(1000)
	integer maxsub /0/
	integer*4 idata(1000)
	character*200 string
	logical*1 octal,hex,h_ms,d_ms

	call keyini
	call keya('class',class,'*')
	call keya('options',options,' ')
	call keyfin

	octal = .false.
	if(index(options,'octal').gt.0) octal = .true.
	d_ms = .false.
	if(index(options,'dms').gt.0) d_ms = .true.
	h_ms = .false.
	if(index(options,'hms').gt.0) h_ms = .true.
	hex = .false.
	if(index(options,'hex').gt.0) hex = .true.
	
C	--OPEN OUTPUT FILE--
	TYPE *,'        Dump of COMMON BLOCK'
		
	CALL COMINFO(MAXHASH,LEFTHASH,MAXSTORE,NEXT,ERROR)
	type *
	type *,'Maxstore:',maxstore,'(next:',next,') Maxhash:',maxhash,
	1	'(empty:',lefthash,')'


C	--READ IN THE COMMON.TAB FILE IN SORTED ORDER--
	  CALL INPUT_COMMON(MAX,NUM,TNAME,TCLASS)

C	--LOOP THROUGH THE POSSIBLE CLASSES AND EXTRACT THE RIGHT VARIABLES--
	  DO II=1,NCLASS

	    IF((INDEX(CLASS,CLASSES(II:II)).GT.0) .OR. (CLASS.EQ.'*'))  THEN

C		--CLASS HEADER--
	      type *
	      type *,'     ',CLASS_TITLE(II)
	      LP=1

C		--EXTRACT MEMBERS OF THE CLASS--
	      DO J=1,NUM

		IF(TCLASS(J).EQ.CLASSES(II:II))  THEN

C		--ITERATE THROUGH THE SUBSCRIPTS OF EACH VARIABLE--
		  CALL COMPROBE(TNAME(J),TYPE,MAXSUB,ERROR)
			IF(ERROR(1:2).NE.'OK') TYPE *,ERROR
		  IF (TYPE.EQ.'C') THEN
			CALL COMGETA(TNAME(J),STRING,MAXSUB,ERROR)
			NUMSUB = 1
		  ELSE IF (TYPE.EQ.'I') THEN
			CALL COMGETI(TNAME(J),IDATA,MAXSUB,ERROR)
			NUMSUB = MAXSUB
		  ELSE IF (TYPE.EQ.'R') THEN
			CALL COMGETR(TNAME(J),RDATA,MAXSUB,ERROR)
			NUMSUB = MAXSUB
		  ELSE IF (TYPE.EQ.'D') THEN
			CALL COMGETD(TNAME(J),DDATA,MAXSUB,ERROR)
			NUMSUB = MAXSUB
		  END IF
			IF(ERROR(1:2).NE.'OK') TYPE *,ERROR

		  DO I=1,NUMSUB
			IF((TYPE.EQ.'D').AND.H_MS) THEN
			  VALUE = ANGLES(DDATA(I)*12.d0/dpi)
			ELSE IF((TYPE.EQ.'D').AND.D_MS) THEN
			  VALUE = ANGLES(DDATA(I)*180.D0/DPI)
			ELSE IF(TYPE.EQ.'D') THEN
			 WRITE(VALUE,'(1PD18.10)')DDATA(I)
			ELSE IF (TYPE.EQ.'R') THEN
			 WRITE(VALUE,'(1PE16.8)')RDATA(I)
			ELSE IF (TYPE.EQ.'C') THEN
			 VALUE = STRING(1:MAXSUB)
			ELSE IF (TYPE.EQ.'I') THEN
			 IF (OCTAL) THEN
			  WRITE(VALUE,'(O16.6)')IDATA(I)
			 ELSE
			  WRITE(VALUE,'(I16.6)')IDATA(I)
			 END IF
			END IF

			K=INDEX(TNAME(J),' ')
			WRITE(PAR_LINE(LP),200) 
	1			TNAME(J)(1:8),MOD(I,100),VALUE(1:18)
200			FORMAT(A,'(',I2,')',T13,A)
			LP=LP+1
			IF(LP.EQ.5) THEN
				type *, PAR_LINE
				LP=1
			END IF
		  END DO
		END IF
	      END DO
		IF(LP.NE.1) type*,(PAR_LINE(K),K=1,LP-1)
	    END IF
	  END DO
	END
C-----------------------------------------------------------------C
	SUBROUTINE INPUT_COMMON(MAX,NUM,TNAME,TCLASS)
C	READ IN COMMON.TAB FOR USE BY COMDUMP
C	FEB 85  WH
C-----------------------------------------------------------------C
	character*(*) tname(max),tclass(max)
	character*80 line_in
	logical*1 finished
	character*50 filename

	FINISHED=.FALSE.
	CALL FILEPATH('HATTAB','common.tab',' ',FILENAME)
	OPEN(2,NAME=FILENAME,STATUS='OLD')
	NUM=0

C	--READ THROUGH COMMON.TAB AND INCLUDE ALL VARIABLES
	DO WHILE (.NOT.FINISHED)
		READ(2,'(A)',IOSTAT=IS) LINE_IN
		IF(IS.EQ.-1) FINISHED=.TRUE.
		KE=INDEX(LINE_IN,'!')+1
		IF(KE.GT.5) THEN
			KC=INDEX(LINE_IN,'[')
			KLP=INDEX(LINE_IN,'(')
			KST=INDEX(LINE_IN,'|')
			 NUM=NUM+1
			 TNAME(NUM)=LINE_IN(1:KLP-1)
			 TCLASS(NUM)=LINE_IN(KC+1:KC+1)
		END IF
	END DO
	RETURN
	END
