c= LOADCOM
	PROGRAM LOADCOM
C& WH
C: COMMON
C+
C
C	LOADCOM allows user to update antenna-related parameters 
c	(station description, pointing constants, etc.); stores 
c	these parameters in a binary file for safekeeping; enters 
c	them into appropriate common variables common variables.
c	they can, of course, be altered at any time by other programs
c	or manually with program alter; the advantage of using loadcom is
c	that it carefully documents these changes in the log file, and
c	provides a means for resetting all the values should they become
c	corrupted in common
c	This program no longer initializes the common block - that is now
c	the function of 'CREATECOM'
c
c	Program operation:
c	1.  Common variables are stored in a direct access binary file,
c	HATTAB:LOADCOM.ANS
c	2.  If command line contains 'q', ask question for antennas.
c	otherwise - skip directly to step 4.
c	2a. If command line contains ants=123..., only ask questions
c	for selected antennas
c	3.  For each antenna, ask necessary question using defaults
c	from answer file, then update the answer file.
c	note all changes in log files.
c	4.  After user finishes updating answer file, copy values from
c	answer file into common.
c
c  	The answer file contains 6 records per antenna (all are REAL*8):
c	    rec 6j-5	station,azreal,azmeas
c	    rec 6j-4	best1,best2,best3,apcest,sn,se
c	    rec 6j-3	apc1,apc2,...,epc1,epc2,...,apctv1,...,epctv1,...
c	    rec 6j-2	b1,b2,b3
c	    rec 6j-1	cable,caboff
c	    rec 6j	delay0,phaselo1,phaselo2
c
c@ q
c	If Y, LOADCOM will prompt for parameters for the selected
c	antennas.
c
c@ ants
c	antenna selection flag for questions:
c	 * means use all nants antennas
c	 xyz means use antennas x y and z
c	 -xyz means use all antennas except x y and z
c--
c  Nov 84  WH / May 86  RP
c  adapted to miriad and new parameter names.  June 90 wh
c  unix->  nov 90 wh
c  get rid of target stuff   wh jul 92
c  put bs predict in right place on new station wh nov 92
c  add tv pointing offsets wh dec 92
c----------------------------------------------------------------c
	character version*(*)
	parameter(version='LOADCOM (version 1.4 14-may-93)')
        include '../inc/constants.inc'

	logical		questions	! .true. if answering questions
	logical		ansfile_exists	! .true. if answer file exists
	character*80 	error		! error return from subroutines
	character*10	ques
        integer         jant
	character*60	answerfile

	call mbusopen(error)
	 if(error(1:2).ne.'OK') goto 99
	CALL ANNOUNCE(version,ERROR)
	IF(ERROR(1:2).NE.'OK') GOTO 99

c  open answer file

	call filepath('HATTAB','loadcom.ans',' ',answerfile)
	inquire ( file=answerfile, exist=ansfile_exists )

	if (.not. ansfile_exists) then
	    type *,'LOAD: creating new loadcom.ans'

	    open ( unit=1, file=answerfile, access='direct',
	1		 recl=112, status='new',	form='unformatted' )
					! create new answer file
	    do i=1,6*nants
		write ( unit=1, rec=i) (0.d0, ii=1,14)
	    end do			! zero the new answer 

	else
	    open ( unit=1, file=answerfile, access='direct',
	1		 recl=112, status='old',	form='unformatted' )
					! open old answer file
	endif
c
c  if command line doesn't contain 'q=y' , skip over questions

        call keyini
        call keyants(jant)
        if (.not.ansfile_exists) jant = 123456
	call keya('q',ques,'n')
        questions = .false.
        if ((ques(1:1).eq.'Y').or.(ques(1:1).eq.'y'))  questions=.true.
        call keyfin

	error = 'OK'
        if (questions) call answer_questions(jant,error)

	if(error(1:2).eq.'OK' )  call fill_common(questions,jant)

99	type *,error
	stop
	end
c
c ANSWER_QUESTIONS
c: loadcom
c+
	subroutine answer_questions(jant,error)
c
c  answer_questions:	fills in the answer file for loadcom by asking
c			questions of the user and measuring system values.
c
c  ARGUMENTS:
c  JANT(integer,input)    Antenna flag
c  ERROR(char,output)	  Status return
c--
c	jun 90 wh
c----------------------------------------------------------------c
	integer jant
        include '../inc/constants.inc' 
	character*80 	error,cerror	! error return from subroutines
	character*5	ynrp
	character*3	ynaz,ynca	! y | n answers
	logical		unmeasured /.true./	! measure cables once
	real*8		antposo(3),antpos(3)! baseline array 
	real*8		antmiss,antmisso  ! axis misalignment (ns)
c	real*8		antposref(3)
	real*8		bs(3)		! baselines
	real*8		c_length(nants)	! cable lengths, nsec
	real*8		c_phase(nants)	! 100 Hz phase offsets, turns
	real*8		c_phaseo,c_lengtho
	real*8		best(3)		! baseline estimate from database, nsec
	real*4		apcest		! apc1 estimate , arcmin
	real*8		sn		! antenna offset N from station 9, cm
	real*8		se		! antenna offset E from station 9, cm
	real*8		latit/.7123974d0/	! observatory latitude
	real*4 		apctvo(7),epctvo(7)	! pointing constants from ans
	real*4		apctv(7),epctv(7)	! new pointing constants
	real*4 		apco(7),epco(7)	! pointing constants from ans
	real*4		apc(7),epc(7)	! new pointing constants
c
	character*8	staname,stanameo! station name (e.g., 320 N)
        real*8 	        sta,stao
	integer 	ista		! station number
	logical		new_station 	/.false./
	logical 	tf0,tf2,tf3
				! .true. if 1 or more antennas moved
	real*8		azreal,azmeas	!measured az offsets
	real*8		azrealo,azmeaso
	real*8		delay0,delay0o
	real*8		phaselo1,phaselo1o
	real*8		phaselo2,phaselo2o
	logical		select,if_change	! function
	character*20	pcfile /' '/	! name of pointing file used
					!   to update pointing constants
	character*20	pctvfile /' '/	! name of tv pointing file 
	character*20    bsfile /' '/	! name of cbee correction file
	character*80 	pcpath,bspath
	character*10 	catagories
	character*1 chr
	chr(i) = char(i+ichar('0'))

c  --	choose question types
	type *,'Choose which catagories of questions to answer from:'
	type *,'p = radio pointing     o = optical pointing'
	type *,'b = baseline           d = delay/phase/etc'
	call aska('Question Catagories',0,catagories,'podb')

c -- loop through all selected antennas asking questions
	 j = 1
	  do while (j .le. nants)
	   if (select(j,jant)) then
	    type *
	    type *,'QUESTIONS for ANTENNA',j
c -- get station and azimuth offset information
	    read ( unit=1, rec=6*j-5 )  stao, azrealo, azmeaso

	    call stationname(0,stanameo,int(stao),error)
			! convert from station number to station name
		call write_error('LOAD',error)

10          call aska('Station Name (ex:40N)',j,staname,stanameo)
	     call stationname(1,staname,ista,error)
		sta = ista
		if (error(1:2).ne.'OK') then	! station name not recognized
		    type *, error
		    goto 10			! return to menu
		endif

	    call askd('Measured Azimuth on object <deg>',j,azmeas,azmeaso)
	    call askd('Actual Azimuth on object <deg>',j,azreal,azrealo)
		apcest = (azreal-azmeas) * 60.

	    new_station = if_change( stao, sta, 1E-1, j, 'station', 'F10.0' )
	    tf2 = if_change( azmeaso, azmeas, 1E-3, j, 'azmeas ', 'F10.3' )
	    tf3 = if_change( azrealo, azreal, 1E-3, j, 'azreal ', 'F10.3' )
				! copy any new values into x array

	    if (new_station .or. tf2 .or. tf3 ) then
		write ( unit=1, rec=6*j-5 ) sta,azreal,azmeas
				! write updated values into answer file
		call basepredict(j,int(sta),best,sn,se,error)
				! recalculate predicted  baseline
		  call write_error('LOAD',error)
		write ( unit=1, rec=6*j-4 ) best,apcest,sn,se
		read  ( unit=1, rec=6*j-2 ) antposo, antmisso
		write ( unit=1, rec=6*j-2 ) best, antmisso
				! copy new predictions into answer file
	    endif

c -- 	  pointing constants
	    read ( unit=1, rec=6*j-3 ) apco,epco,apctvo,epctvo
				! default pointing constants from answer file

	   if(index(catagories,'p').gt.0) then
20	    call aska('Filename for radio pointing constants',j,pcfile,pcfile)
	    call filepath('HATPNT',pcfile,' ',pcpath)
	    call check_file(pcpath,lenpc,error)
	    if(error(1:2).ne.'OK') type *,error
	    if(error(1:2).ne.'OK') goto 20

	    if ( len1(pcfile).gt.0 ) then
			! read constants from pntfile
		call readpntfile ( pcpath, j, apc, epc, error )
			! reads pointing constants from file pcfile for ant j
			!  and returns answers in arrays apc,epc
		call write_error('LOAD',error)
	    else
		error = 'ask'
	    end if
	    if(error(1:2).ne.'OK') then		!if bad result, ask for values
	        call askr('APC(1) (*cos(E)) ',j,apc(1),apco(1))
	        call askr('APC(2) (*1) ',j,apc(2),apco(2))
	        call askr('APC(3) (*sin(E)) ',j,apc(3),apco(3))
	        call askr('APC(4) (*sin(A)sin(E)) ',j,apc(4),apco(4))
	        call askr('APC(5) (*cos(A)sin(E)) ',j,apc(5),apco(5))
	        call askr('APC(6) (*sin(A)cos(E)) ',j,apc(6),apco(6))
	        call askr('APC(7) (*cos(A)cos(E)) ',j,apc(7),apco(7))
		call askr('EPC(1) (*1)',j,epc(1),epco(1))
		call askr('EPC(2) (*sin(E)) ',j,epc(2),epco(2))
		call askr('EPC(3) (*cos(E))',j,epc(3),epco(3))
		call askr('EPC(4) (*sin(A))',j,epc(4),epco(4))
		call askr('EPC(5) (*cos(A))',j,epc(5),epco(5))
		call askr('EPC(6) (*sin(2A))',j,epc(6),epco(6))
		call askr('EPC(7) (*cot(E))',j,epc(7),epco(7))
		error = 'OK'
	    end if
	   else
		do ii=1,7
		  apc(ii) = apco(ii)
		  epc(ii) = epco(ii)
		end do
	   end if

	   if(index(catagories,'o').gt.0) then
21	    call aska('Filename for TV pointing constants',j,pctvfile,pctvfile)
	    call filepath('HATPNT',pctvfile,' ',pcpath)
	    call check_file(pcpath,lenpc,error)
	    if(error(1:2).ne.'OK') type *,error
	    if(error(1:2).ne.'OK') goto 21

	    if ( len1(pctvfile).gt.0 ) then
			! read constants from pntfile
		call readpntfile ( pcpath, j, apctv, epctv, error )
			! reads pointing constants from file pcfile for ant j
			!  and returns answers in arrays apctv,epctv
		call write_error('LOAD',error)
	    else
		error = 'ask'
	    end if
	    if(error(1:2).ne.'OK') then		!if bad result, ask for values
	        call askr('APCtv(1) (*cos(E)) ',j,apctv(1),apctvo(1))
	        call askr('APCtv(2) (*1) ',j,apctv(2),apctvo(2))
	        call askr('APCtv(3) (*sin(E)) ',j,apctv(3),apctvo(3))
	        call askr('APCtv(4) (*sin(A)sin(E)) ',j,apctv(4),apctvo(4))
	        call askr('APCtv(5) (*cos(A)sin(E)) ',j,apctv(5),apctvo(5))
	        call askr('APCtv(6) (*sin(A)cos(E)) ',j,apctv(6),apctvo(6))
	        call askr('APCtv(7) (*cos(A)cos(E)) ',j,apctv(7),apctvo(7))
		call askr('EPCtv(1) (*1)',j,epctv(1),epctvo(1))
		call askr('EPCtv(2) (*sin(E)) ',j,epctv(2),epctvo(2))
		call askr('EPCtv(3) (*cos(E))',j,epctv(3),epctvo(3))
		call askr('EPCtv(4) (*sin(A))',j,epctv(4),epctvo(4))
		call askr('EPCtv(5) (*cos(A))',j,epctv(5),epctvo(5))
		call askr('EPCtv(6) (*sin(2A))',j,epctv(6),epctvo(6))
		call askr('EPCtv(7) (*cot(E))',j,epctv(7),epctvo(7))
		error = 'OK'
	    end if
	   else
		do ii=1,7
		  apctv(ii) = apctvo(ii)
		  epctv(ii) = epctvo(ii)
		end do
	   end if
	    if (error(1:2).eq.'OK') then

	        call aska('ADJUST apc(1) by AZreal-AZmeas [y|n]',j,ynaz,'n')
		if ( ynaz.eq.'Y' .or. ynaz.eq.'y' ) then
				! use apc1 prediction
		    call logit('Adjust apc1 by AZreal-AZmeas')
		    apc(1) = apc(1) + apcest
		    apctv(1) = apctv(1) + apcest
		    type *,'    apc(1) now is ',apc(1)
		    type *,'    apctv(1) now is ',apctv(1)
		endif

		do i=1,7
		    tf0 = if_change ( dble(apco(i)), dble(apc(i)), 1E-2, j, 
	1			'APC'//chr(i)//'  ', 'F10.2' )
		    tf0 = if_change ( dble(epco(i)), dble(epc(i)), 1E-2, j, 
	1			'EPC'//chr(i-7)//'  ', 'F10.2' )
		    tf0 = if_change ( dble(apctvo(i)), dble(apctv(i)), 1E-2, j, 
	1			'APC'//chr(i)//'  ', 'F10.2' )
		    tf0 = if_change ( dble(epctvo(i)), dble(epctv(i)), 1E-2, j, 
	1			'EPC'//chr(i-7)//'  ', 'F10.2' )
		enddo

	        write ( unit=1, rec=6*j-3 ) apc,epc,apctv,epctv
			! copy revised pointing constants to answer file
	    end if

c --  enter baseline corrections

	   if(index(catagories,'b').gt.0) then
	    read ( unit=1, rec=6*j-2 ) antposo,antmisso
				! read defaults from answer file

30	    call aska('BEE cal filename for base override',j,bsfile,
	1							bsfile)
	    call filepath('HATBASE',bsfile,' ',bspath)
	    call check_file(bspath,lenbs,error)
	    if (error(1:2).ne.'OK') type *,error
	    if (error(1:2).ne.'OK') goto 30

	    if (len1(bsfile).gt.0) then
				! read from file created by BEE
		call readbeefile ( bspath(1:lenbs), j, bs, error )
		if(error(1:2).eq.'OK') then
		  call logit('LOAD: position read from BEE file '//bsfile)
			do i=1,3
			    antpos(i) = bs(i)
			    tf0 = if_change (antposo(i),antpos(i), 1E-4, j, 
	1				'antpos'//chr(i)//'    ', 'F10.4' ) 
			enddo
		end if
	    else
		error = 'ask'
	    end if
	    if(error(1:2).ne.'OK') then
		    do i=1,3
			call askd('ANTPOS['//chr(i)//'] <ns>',
	1					j,antpos(i),antposo(i))
			tf0 = if_change ( antposo(i), antpos(i), 1E-4, j,
	1				 'antpos'//chr(i)//'    ', 'F10.4' )
		    enddo
	    endif
	    call askd('ANTMISS   <ns>',j,antmiss,antmisso)
	    tf0 = if_change ( antmisso, antmiss, 1E-4, j,
	1				 'antmiss     ', 'F10.4' )
	    error = 'OK'
		write ( unit=1, rec=6*j-2 ) antpos,antmiss
				! copy updated baseline back to answer file
	   end if

c -- update delay or phase offsets
	   if(index(catagories,'d').gt.0) then
	    read ( unit=1, rec=6*j ) delay0o,phaselo1o,phaselo2o

	    call askd('Delay0 <ns>',j,delay0,delay0o)
	    call askd('Phaselo1 <turns>',j,phaselo1,phaselo1o)
	    call askd('Phaselo2 <turns>',j,phaselo2,phaselo2o)
		tf0 = if_change( delay0o, delay0, 1E-3, j,
	1			'DELAY0  ', 'F10.3' )
		tf0 = if_change( phaselo1o, phaselo1, 1E-3, j,
	1			'PHASELO1', 'F10.3' )
		tf0 = if_change( phaselo2o, phaselo2, 1E-3, j,
	1			'PHASELO2', 'F10.3' )

		write ( unit=1, rec=6*j ) delay0,phaselo1,phaselo2
				! write updated values back to answer file 

c  -- update cable lengths
	  read ( unit=1, rec=6*j-1 ) c_lengtho,c_phaseo
				! read defaults from answer file
	  call aska('Measure the cable lengths[y|n]',j,ynca,'n')

	  if (ynca.eq.'Y' .or. ynca.eq.'y') then
				! remeasure all cables automatically
	    if (unmeasured) then
		unmeasured = .false.
		type *,'begin to measure cable lengths'
c	 	call measure_cable ( jant, c_length, c_phase, cerror )
c				! returns lengths and 100 Hz phases
	    end if
	    if (cerror(1:2).ne.'OK') then
		    type *, error
		    type *, 'cable lengths not updated'
		    call write_error('LOAD',error)
	    else
			tf0 = if_change( c_lengtho, c_length(j), 1E-3, j,
	1			'CABLE  ', 'F10.4' )
			tf0 = if_change( c_phaseo, c_phase(j), 1E-2, j,
	1			'CABOFF ', 'F10.3' )
	    endif

	  else		! use values in a array
		call askd('Cable length <ns>',j,c_length(j),c_lengtho)
		call askd('Cable phase <turns>',j,c_phase(j),c_phaseo)
		    tf0 = if_change( c_lengtho, c_length(j), 1E-3, j,
	1			'CABLE  ', 'F10.4' )
		    tf0 = if_change( c_phaseo, c_phase(j), 1E-2, j,
	1			'CABOFF ', 'F10.3' )
	  endif
		write ( unit=1, rec=6*j-1 ) c_length(j),c_phase(j)
	 		! copy updated values back to answer file
	 end if

		call aska('Quit (q) or Repeat (r) ant',j,ynrp,'n')
		if ((ynrp(1:1).ne.'R') .and. (ynrp(1:1).ne.'r')) j = j+1
		if ((ynrp(1:1).eq.'Q') .or. (ynrp(1:1).eq.'q')) j = nants+1
	 else
		j = j+1
	 endif
	end do
	return
	end
c
c FILL_COMMON
c: loadcom
c+
	subroutine fill_common(questions,jants)
c
c  fill_common:	reads the update answer file and fills in the common
c		variables.
c
c  ARGUMENTS:
c  QUESTIONS (logical,input)	true if questions have been asked
c--
c	jun 90 wh
c----------------------------------------------------------------c
        include '../inc/constants.inc'
	integer jants
	logical select

	logical		questions	! input
	logical		database_exists	! .true. if database file exists
	character*80 	error		! error return from 
	character*24 	day
	character*60 	databasefile
	real*8		antposo(3)	! baseline array for reading, writing
	real*8		antmisso	! axis misalignment
	real*8		best(3)		! baseline estimate from database, nsec
	real*8		x(3)		! baseline in database
	real*8		sn		! antenna offset N from station 9, cm
	real*8		se		! antenna offset E from station 9, cm
	real*8		latit/.7123974d0/	! observatory latitude

	character*8	staname(nants)	! station name (e.g., 320 N)
	character*36	allstations
        real*8 	        sta
	character*24	today		! date string
	character*9	move_day	! date of last antenna move
	logical		bastf		! .true. if baseline has been updated
				! .true. if 1 or more antennas moved
	real*8		azreal,azmeas	! measured az offset
	real*8		antpos(nants,3)	! final baseline form
	real*8		antmiss(nants)	! axis misalignment
	real*4 		apc(7,nants),epc(7,nants)
	real*4 		apctv(7,nants),epctv(7,nants)
	real*4		dellong(nants),dellat(nants)
	real*8		sinlat(nants),coslat(nants)
	real*8		delay0,phaselo1,phaselo2
	real*4		rdelay0(nants),rphaselo1(nants),rphaselo2(nants)
	real*8		c_phase,c_length(nants)
	real*4		rc_phase(nants)
	real*8		transaz(nants),transel(nants),tau
	real*4 		focus(nants)

	character*1 chr
	chr(i) = char(i+ichar('0'))

	type *,'LOAD -- copying variables from answer file into common'
	call logit('   copy variables from answer file into common')

c  --	load current focus from antennas
	call logit('   Current focus')
	call getfocus(250,focus,error)
	call computr('FOCUS',focus,9,error)
	write(error,'(7f8.3)') (focus(i),i=1,7)

c  --	station numbers and Azimuth offsets appear in log only, not in common
	call logit('   Station descriptions')
	call logit('   ANT  NAME  (num)   Azreal    Azmeas  '
	1	//' North (met)  East (met)')
	do j=1,nants 
	    read ( unit=1, rec=6*j-5,iostat=is) sta,azreal,azmeas
	    if (sta.eq.0.d0) then
		call write_error('        ',
	1		'station number is 0 for ant '//chr(j))
	    else
	      read ( unit=1, rec=6*j-4,iostat=is) best,apcest,sn,se
	      call stationname(0,staname(j),int(sta),error)	! number-->name
	 		call write_error('LOAD',error)
	      allstations(4*(j-1)+1:4*j) = staname(j)(1:4)
	      write (error,200) j, staname(j), int(sta), azreal,azmeas, sn/100.,
	1	 se/100.
200	      format (x,I2,4x,A,' (',I3,')',2x,2F7.3,2x,2F7.2)
	      call logit(error)
	    end if
	enddo

	    call computa('STANAME',allstations,36,error)
	 		call write_error('LOAD',error)
c
c  pointing constants

	call logit('   Pointing constants')
	do j=1,nants
	    read ( unit=1, rec=6*j-3,iostat=is) (apc(i,j),i=1,7),
	1		(epc(i,j),i=1,7),(apctv(i,j),i=1,7),(epctv(i,j),i=1,7)
	    read ( unit=1, rec=6*j-4,iostat=is ) best,apcest,sn,se

	    dellong(j) = -1.92186E-9 * se
	    dellat(j) = 1.460195E-9 * sn
			! differences in longitude and latitude from sta 9,
			!  used in pointing the antennas
	    sinlat(j) = sin( latit + dellat(j) )
	    coslat(j) = cos( latit + dellat(j) )

	    write (error,110) j, apcest, (apc(i,j),i=1,7)
110	    format ('   ',I2,X,'AZ:  (',F7.2,')',7F7.2)
	    call logit(error)

	    write(error,111) (epc(i,j),i=1,7)
111	    format (6x,'EL:  ',9x,7F7.2)
	    call logit(error)

	    write (error,113) (apctv(i,j),i=1,7)
113	    format (6X,'AZtv:',9x,7F7.2)
	    call logit(error)

	    write(error,114) (epctv(i,j),i=1,7)
114	    format (6x,'ELtv:',9x,7F7.2)
	    call logit(error)

	    write(error,112) dellong(j),dellat(j)
112	    format(9X,'delta long= ',e10.4,5X, 'delta lat=', E10.4)
	    call logit(error)

	    call update_lkt(se,sn,apc,epc,transaz(j),transel(j),tau)
	enddo
	    call computr('APC', apc,nants*7, error)
		    call write_error('        ',error)
	    call computr('EPC', epc,nants*7, error)
		    call write_error('LOAD',error)
	    call computr('APCTV', apctv,nants*7, error)
		    call write_error('        ',error)
	    call computr('EPCTV', epctv,nants*7, error)
		    call write_error('LOAD',error)
	    call computr( 'DELLONG', dellong, nants, error)
		call write_error('        ',error)
	    call computd( 'SINLAT', sinlat, nants, error)
		call write_error('        ',error)
	    call computd( 'COSLAT', coslat, nants, error)
		call write_error('        ',error)
	    call computd('TRANSAZ',transaz,nants,error)
		call write_error('        ',error)
	    call computd('TRANSEL',transel,nants,error)
		call write_error('        ',error)

c  resoff section
	call logit('   Resoff values come from file $HATTAB/resoff.ascii')
	call readresofffile(error)
		call write_error('        ',error)

c  baseline section
	call logit('   Antenna vector from station 9 to antenna j in nsec')
	call logit('   ANT  X (est)    Y (est)    Z (est)      X (used)'
	1	//'  Y (used)  Z(used)')
	do j=1,nants
	    read ( unit=1, rec=6*j-4,iostat=is ) best	! estimated baselines
	    read ( unit=1, rec=6*j-2,iostat=is ) antposo,antmisso  ! actual baselines
	    do i=1,3
		antpos(j,i) = antposo(i)
	    enddo
	    antmiss(j) = antmisso
	    write (error,210) j, best, antposo, antmisso
210	    format(x,I2,2X,'(',3F10.4,')',X,4F10.4 )
	    call logit(error)
	enddo
	    call computd('ANTPOS',antpos, nants*3, error)
 		call write_error('        ',error)
	    call computd('ANTMISS',antmiss, nants, error)
 		call write_error('        ',error)

c -- cable lengths, delay and phase offsets

	call logit('   Cable lengths, Delay and Phase Offsets')
	call logit('   ANT  DELAY0   PHASELO2  PHASELO1  CABLE    CABOFF')
	call logit('         (ns)     (turns)   (turns)   (ns)    (turns)')
	do j=1,nants
	    read ( unit=1, rec=6*j,iostat=is ) delay0,phaselo1,phaselo2
	    rdelay0(j) = delay0
	    rphaselo1(j) = phaselo1
	    rphaselo2(j) = phaselo2
	    read ( unit=1, rec=6*j-1) c_length(j),c_phase
	    rc_phase(j) = c_phase
	    write (error,230,iostat=ios) j,delay0,phaselo2,phaselo1,
	1		c_length(j),c_phase
230	    format(x,I2,F10.3,2F10.4,F11.4,F7.3)
	    call logit(error)
	enddo
	    call computr( 'DELAY0',rdelay0, nants, error )
		call write_error('        ',error)
	    call computr( 'PHASELO2', rphaselo2,nants, error )
		call write_error('        ',error)
	    call computr( 'PHASELO1', rphaselo1,nants, error )
		call write_error('        ',error)
	    call computd( 'CABLE', c_length,nants, error )
		call write_error('        ',error)
	    call computr( 'CABOFF', rc_phase,nants, error) 
		call write_error('        ',error)
	    call fdate(day)
	    call computa( 'LOADDATE',day(:11),12,error)
		call write_error('        ',error)
c
c  if necessary, update database.stn

	if (questions) then
			! if interactive mode was used, update the database

	  call filepath('HATTAB','database.stn',' ',databasefile)
	  inquire ( file=databasefile, exist=database_exists )

	  if (.not. database_exists) then
	    type *,'        : creating new database file'
	    call logit('   creating new database file')
	    open ( unit=2, file=databasefile, status='new', form='formatted' )
					! create new database file
	  else
	    type *,'LOAD -- updating database file'
	    call logit('   updating database file')
	    open ( unit=2, file=databasefile, status='old', 
	1		access='append',form='formatted' )
				! open old database file
	  endif

		call fdate(today)
		move_day = today(5:11)//today(23:24)

	  do j=1,nants
	   if(select(j,jants)) then
	    read ( unit=1, rec=6*j-5 ) sta,azreal,azmeas
	    read ( unit=1, rec=6*j-3 ) apc1
	    read ( unit=1, rec=6*j-4 ) best
	    read ( unit=1, rec=6*j-2 ) x
		bastf = .false.
		do i=1,3
		    if (x(i).ne.best(i)) bastf = .true.
		enddo
			! true if baseline has been updated from prediction
	    write ( unit=2, fmt='(i2,x,i2,x,a9,x,l,4(1pe11.3),f10.4,3(1pe13.5))')
	1	j,int(sta),move_day,bastf,0.d0,0.d0,0.d0,0.d0,apc1,(x(i),i=1,3)
			! one record per antenna per configuration
	   end if
	  enddo
	  close (unit=2)
	end if
	return
	end
c	    
c STATIONNAME
c: loadcom
c+
	subroutine stationname(iaction,staname,stanum,error)
c
c  stationname:	
c  converts between 'new' station names and old station numbers
c	 ('320 N' <--> 23, e.g.)
c  stations added in july 86 are numbered from 29 to 32
c  stations 15E and 15W are numbered 33 and 34
c  before examining name, all blanks are removed and all letters are
c	capitalized; thus, '320 N', '320N', '320 n', '320n' all are OK
c
c  ARGUMENTS:
c  IACTION(integer,input)    0 for number to name, 1 for name to number
c  STANAME(char,in/out)	     station name
c  STANUM (integer,out/in)   station number
c  ERROR (char,output)       status return
c--
c  RP  July 86
c-------------------------------------------------------------------------c
	integer*4	iaction		! (IN) 0 for number to name
					!      1 for name to number
	character*(*)	staname		! (IN or OUT) station name
	integer*4	stanum		! (OUT or IN) station number
	character*(*)	error		! (OUT) standard error return
c
	character*4	names(37)	! table of names
	integer*4	numbers(37)	! table of numbers
	character*70 list, temponly, letter*1
c
	data names     /'40SE','20SE','500E','475E','400E','320E','180E',
	1		'40E ','0   ','0E  ','0W  ','0N  ','100W','240W',
	2		'380W','440W','475W','500W','20SW','40SW','40N ',
	3		'100N','140N','200N','260N','320N','380N','440N',
	4		'500N','560N','580N','80E ','40W ','20N ','60N ',
	5		'15E ','15W ' /
	data numbers   /    1,     2,     3,     4,     5,     6,     7,
	1		    8,     9,     9,     9,     9,    10,    11,
	2		   12,    13,    14,    15,    16,    17,    18,
	3		   19,    20,    21,    22,    23,    24,    25,
	4		   26,    27,    28,    29,    30,    31,    32,
	5		   33,    34  /
c
	if (iaction.eq.0) then
	    do i=1,37
		if (stanum.eq.numbers(i)) then
		    staname = names(i)
		    error = 'OK'
		    return
		endif
	    enddo
	    write(error,'(a,i3)')'Cannot identify station number ',stanum
	    staname = 'none'
	    return
c
	else
	    ic=len(staname)
	    do i=1,ic-1
		if (staname(i:i).eq.' ') then
		    do j=i,ic-1
			staname(j:j) = staname(j+1:j+1)
		    enddo
		    staname(ic:ic)=' '
		endif
	    enddo				! note: removes only 1 blank
	    call ucase(staname)			! convert to upper case

	    do i=1,37
		if (staname.eq.names(i)) then
		    stanum = numbers(i)
		    error = 'OK'
		    return
		endif
	    enddo
	    list = ' number[N|E|W]'
	    lngth = len1(staname)
	    letter = staname(lngth:lngth)
	    do i=1,37
	      if(index(names(i),letter).gt.0) then
		temponly = list(:len1(list))//' '//names(i)
		list = temponly
	      end if
	    end do
	    write(error,'(a,a)')'Try:',list
	    stanum=0.d0
	    return

	endif
	end
c-----------------------------------------------------------------------------c
	logical function if_change( xold, xnew, dif, jant, label, form )
c
c  copies xnew into xold
c  if abs(xnew-xold) > dif, writes message to short and long log files
c  called by LOAD
c  rp  May 86
c-----------------------------------------------------------------------------c
	real*8		xold		! (IN/OUT) old value from answer file
	real*8		xnew		! (IN) new value, usually from menu
	real*8		dif		! (IN) significant difference
	integer		jant		! (IN) which antenna
	character*(*)	label		! (IN) name of variable
	character*(*)	form		! (IN) format for variable
c
	character*80	log_line	! message for log files
	character*60	f		! run-time format

	if ( abs(xnew-xold) .le. dif ) then
	    if_change = .false.

	else
	    if_change = .true.
	    f = '(''LOAD: change ant'',I2,X,A,T32,'
	1	//form//','' -->'','//form//')'
	    write ( log_line, fmt=f ) jant,label,xold,xnew
	    call logit(log_line)
	    xold=xnew

	endif
	return
	end
c--------------------------------------------------------------------------c
c
c READRESOFFFILE
c: loadcom
c+
	subroutine readresofffile( error)
c
c  readresofffile:	gets the resoffs from $HATTAB/resoff.ascii and
c			puts them into common.
c
c  ARGUMENTS:
c  ERROR (char,output)	Status return
c--
c	wh apr 92
c--------------------------------------------------------------------------c
	character*80 error,rlabel
	character*60 rfile
	integer resoff(2,9)

	call filepath('HATTAB','resoff','ascii',rfile)
	open(11,file=rfile(:len1(rfile)),status='old',err=99)
	read(11,'(a)') rlabel
	do i=1,9
	   read(11,*,end=98,err=98) i,resoff(1,i),resoff(2,i)
	end do
	close(11)
	call computi('RESOFF',resoff,18,error)
	return
98	error = 'can''t read resoffs from $HATTAB/resoff.ascii'
	return
99	error = 'can''t open $HATTAB/resoff.ascii'
	return
	end

c--------------------------------------------------------------------------c
c
c READPNTFILE
c: loadcom
c+
	subroutine readpntfile( filename, jant, apc,epc, error)
c
c  readpntfile:
c  called by LOAD; reads pointing constants from file created by program PNT
c  pointing fit file is formatted, contains 2 header lines followed by 2 lines
c	for each antenna
c
c  ARGUMENTS:
c  FILENAME(char,input)	name of file written by PNT
c  JANT (integer,input) antenna flag
c  APC (real(7), output)	Azimuth pointing errors
c  EPC (real(7), output)	Elevation pointing errors
c  ERROR (char,output)	Status return
c--
c  RP  6-Apr-87   
c	mod wh jun 90
c--------------------------------------------------------------------------c
	include '../inc/constants.inc'
	character*(*) 	filename	! (IN) name of file written by PNT
	integer*4	jant		! (IN) antenna flag (1, 2, or 3)
	real		azfit(7),elfit(7)! (OUT) 7 az, 7 el pointing constants
	real		apc(7),epc(7)
	character*(*)	error		! (OUT) standard error return
c
	character*80	line
	integer*4	jin
	character*4	axis
c
	write (line,100) jant,filename(:len1(filename))
100	format(' Pointing for ant',I2,' comes from ',A)
	call logit(line)
	call output(line)

	open ( unit=2, file=filename, status='old', form='formatted')

c   --	skip header + previous antennas
	do nn=1,jant
	 do i=1,2
	    read ( unit=2, fmt='(A)', err=95, end=95 ) line
			! skip over two header lines
	 enddo
	end do
c   --	read in the lines for this (jant) antenna
	    read ( 2, 104, err=95, end=95 ) jin,axis,azfit
104	    format(x, i3, x, a, 3x, 7f8.2)
		if (jin.ne.jant) goto 95
		if (axis.ne.'azim') goto 95
		    write (line,104) jin,axis,azfit	! copy into log
		    call logit(line)
		    call output(line)
	    read ( 2, 104, err=95, end=95 ) jin,axis,elfit
		if (jin.ne.jant) goto 95
		if (axis.ne.'elev') goto 95
		    write (line,104) jin,axis,elfit	! copy into log
		    call logit(line)
		    call output(line)
		do i=1,7
		    apc(i) = azfit(i)
		    epc(i) = elfit(i)
		enddo
	close (2)
c
c  --	if first pointing constant = 0, repeat menu

	if (apc(1).eq.0) then
	    call logit('   error - pointing constants are zero')
	    error = 'Pointing constants for ant '//char(jant+'060'o)//
	1	' are 0 in file '//filename(:len1(filename))
	else
	    error = 'OK'
	endif
	return

c  --	if error in reading the file, repeat menu

95	close(2)
	error = 'File '//filename(:len1(filename))//' contains wrong stuff'
	return
	end
c------------------------------------------------------------c
c
c CHECK_FILE
c: loadcom
c+
	 subroutine check_file(name,length,error)
c
c  check_file:	return OK if file exists 
c
c  ARGUMENTS:
c  NAME(char,input)   	filename
c  LENGTH(integer,out)	trimmed length of name
c  ERROR (char,output)	Status return
c--
c	jun 90 wh
c----------------------------------------------------------------c
	character*(*) name,error
	integer length
	logical is

	error='OK'
	length = len1(name)

	inquire(file=name, exist=is)
	if(is.eq..true.) then
		error='OK'
	else
		error='File '//name(1:length)//' doesn''t exist'
	end if
	return
	end
c
c READBEEFILE
c: loadcom
c+
	subroutine readbeefile( filename, j, bs, error )
c
c  readbeefile:
c  reads (antenna-based) absolute baselines from file created by BEE
c
c  ARGUMENTS:
c  FILENAME(char,input)	name of file from bee
c  J(integer,input)	antenna number
c  BS(dp(3),input)	current antenna-based baselines
c  ERROR (char,output)	Status return
c--
c  WH  Dec 91 - changed to read file from mel's bee/miriad program
c----------------------------------------------------------------c
	include '../inc/constants.inc'
	character*(*) 	filename	! (IN) name of file to read
	real*8		bs(3)		! (OUT) correct antenna-based baselines
	integer		j		! (IN) antenna
	character*80	error		! (OUT) standard error return
	character*80 	line
	real*8		b(3,nants)	! antenna-positions from file
	logical	corbase(nants)		! non-zero position in file

	do jb = 1,nants
	    corbase(jb) = .false.
	    do i=1,3
		b(i,jb) = 0.
	    enddo
	enddo
	ncor = 0
c
c  step 1: read file produced by BEE

	open ( unit=2, file=filename,status='old',err=95,form='formatted')
	read(2,*,err=95) line
	type *,'Antenna position read from file '//filename(:len1(filename))

	jb=1
1	read (2, fmt=*, err=95,end=2) (b(i,jb), i=1,3)
	    if (abs(b(1,jb)).ge.0.001 .or. 
	1	  abs(b(2,jb)).ge.0.001 .or.
	2	     abs(b(3,jb)).ge.0.001) corbase(jb) = .true.
	    jb = jb+1
	goto 1
2	close(2)

	if (j.le.nants .and. corbase(j).eq..true.) then
	  do i=1,3
		bs(i) = b(i,j)
	  end do
	  write(line,fmt=125) j,bs
125	   format(,I4, ' antenna position ',3F12.5)
	  call output(line)
	  error = 'OK'
	else
	  error = 'This antenna was not fit in the BEE file'
	endif
	return

95	type *,'Error in file ',filename(:len1(filename))
	error='repeat'
	close(unit=2)
	return
	end
c
c aska
c: loadcom
c+
	subroutine aska(prompt,ant,answer,lastans)
c
c  aska:	prompt for character input in loadcom
c
c  ARGUMENTS:
c  PROMPT(char,input)	prompt string
c  ANT (integer,input)	antenna number for prompt
c  ANSWER (char,output)	returned answer
c  LASTANS(char,input)	default answer
c--
c	july 90 wh
c----------------------------------------------------------------c
	character*(*) prompt,answer,lastans
	integer ant
	character*80 answeris

	type 100,prompt,ant,lastans
100	format(a,'{ant',i1,'}',t45,'[',a15,'] >',$)
	accept 101,la,answeris
101	format(q,a)
	if(la.eq.0) then
		answer = lastans
	else
		answer = answeris
	end if
	end
c
c askd
c: loadcom
c+
	subroutine askd(prompt,ant,answer,lastans)
c
c  askd:	prompt for dp answer in loadcom
c
c  ARGUMENTS:
c  PROMPT(char,input)	prompt string
c  ANT (integer,input)	antenna number for prompt
c  ANSWER(dp,output)	returned answer
c  LASTANS(dp,input)	default answer
c--
c	july 90 wh
c----------------------------------------------------------------c
	character* (*) prompt
	integer ant
	real*8 answer,lastans,answeris

	type 100,prompt,ant,lastans
100	format(a,'{ant',i1,'}',t45,'[',g15.5,'] >',$)
	accept 101,la,answeris
101	format(q,f20.0)
	if(la.eq.0) then
		answer = lastans
	else
		answer = answeris
	end if
	end
c
c askr
c: loadcom
c+
	subroutine askr(prompt,ant,answer,lastans)
c
c  askd:	prompt for real answer in loadcom
c
c  ARGUMENTS:
c  PROMPT(char,input)	prompt string
c  ANT (integer,input)	antenna number for prompt
c  ANSWER(real,output)	returned answer
c  LASTANS(real,input)	default answer
c--
c	july 90 wh
c----------------------------------------------------------------c
	character* (*) prompt
	integer ant
	real*4 answer,lastans,answeris

	type 100,prompt,ant,lastans
100	format(a,'{ant',i1,'}',t45,'[',g15.5,'] >',$)
	accept 101,la,answeris
101	format(q,f20.0)
	if(la.eq.0) then
		answer = lastans
	else
		answer = answeris
	end if
	end

c-----------------------------------------------------------------------c
	subroutine basepredict(iant,ista,best,sn,se,error)
c
c  predicts baseline and azimuth encoder offset from database
c  reads antenna parameters from file target.stn
c  rp   apr 86	adapted from STPEST.FTN
c  rp   nov 86  adapted to read in 32 station positions
c  rp	may 88  format of target.stn changed; now accomodates
c		arbitrary number of stations; comment lines begin with !
c	wh no more targets
c-----------------------------------------------------------------------c
	integer*4	iant		! (IN) antenna number (1,2,3)
	integer*4	ista		! (IN) station number (1,2,...,32)
	real*8		best(3)		! (OUT) estimated baseline, in
					!  ha,dec coordinate system, nsec
	real*8		sn,se		! (OUT) N,E offsets from sta 9, cm
	character*(*)	error		! (OUT) standarad error return		
c
c	----these parameters must be the same as in program sta.for----
	real*8	sinl/.65364999d0/	! sine(observatory latitude)
	real*8	cosl/.75679699d0/	! cosine(observatory latitude)
	real*8	nstocm/29.97929d0/	! nanoseconds to cm
	real 	factr1(3) /3*6.9226/	! 
	real	factr2(3) /3*.2445/	!
c	---------------------------------------------------------------
	logical*1	file_exists
	logical*1	tf		! true if station coordinates are known
	real*8		sz		! height difference from station 9, cm
	character*80	line
	character*10	wantsta,wantant
	logical*1	found_sta,found_ant
	character*60	targetfile

c  initialize all values to zero in case prediction fails

	best(1)=0.d0
	best(2)=0.d0
	best(3)=0.d0
	sn=0.d0
	se=0.d0
c
c  make sure that target.stn exists; open it
        call filepath('HATTAB','target.stn',' ',targetfile)
	inquire ( file=targetfile, exist=file_exists )
	if (.not. file_exists) then
	    error = 'basepredict: cannot open file target.stn'
	    return
	endif

	open ( unit=2, file=targetfile, form='formatted',status='old')
c
c  read through entire file; ignore lines beginning with "!"; strip
c  off desired station and antenna offsets

	found_sta = .false.
	found_ant = .false.
	write (wantsta,fmt='(a,i3)') 'STATION',ista
	write (wantant,fmt='(a,i3)') 'ANTENNA',iant

	do while (.true.)
	    read ( 2, fmt='(a)',end=50, err=95) line
	    if (line(1:1).eq.'!') then
		continue
	    else if (index(line,wantsta).ne.0) then
		read (line,fmt=*) snrth,seast,selev,sapc,tf
		found_sta = .true.
	    else if (index(line,wantant).ne.0) then
		read (line,fmt=*) anrth,aeast,aelev,aapc
		found_ant = .true.
	    endif
	enddo

50	close(2)

	if (.not.found_sta) then
	    error = 'predict: cannot find '''//wantsta//
	1	''' in TARGET.STN'
	    return
	else if (.not.found_ant) then
	    error = 'predict: cannot find '''//wantant//
	1	''' in TARGET.STN'
	    return
	else if (.not.tf) then
	    write (error, fmt='(A,I3,A)')
	1	' warning: we have not been to station',ista,' before'
	else
	    error = 'OK'
	endif
c
c  predict baseline (cm) in north,east,elev system

	sn=snrth+anrth
	se=seast+aeast
	sz=selev+aelev
c
c  convert to (nsec) ha,dec system

	best(1) = (-sinl*sn+cosl*sz)/nstocm
	best(2) = se/nstocm
	best(3) = (cosl*sn+sinl*sz)/nstocm
	return
c
c  error message

95	error = 'predict: error reading target.stn'
	return
	end

c-----------------------------------------------------------------------------c
	subroutine update_lkt(seast,snrth,apc,epc,encaz,encel,taul)
c  predicts azimuth and elevation of lookout tower, and delay relative
c	to station 9 for signals from lookout tower, for an antenna
c	located at (se,sn) with pointing constants (apc,epc)
c  called by update
c  RP  22 Jan 86
c-----------------------------------------------------------------------------c
	real*8		seast	! (IN) ant position, cm E from 9
	real*8		snrth	! (IN) ant location, cm N from 9
	real*4		apc(7)	! (IN) az pointing constants, arcmin
	real*4		epc(7)	! (IN) el pointing constants, arcmin
	real*8		encaz	! (OUT) encoder azim of lkt, decimal deg
	real*8		encel	! (OUT) encoder elev of lkt, decimal deg
	real*8		taul	! (OUT) delay relative to 9, in nsec
c
	real*4		dnrth	/-88820./	! cm N from station 9
	real*4		deast	/448396./	! cm E from station 9
	real*4		dz	/ 52405./	! cm above station 9
	real*8		nstocm	/29.97929d0/
	real*4		pi	/3.1415926/
	real*8		d9sq,d,dsq
c
c  compute true azim and elev in radians, delay in nsec

	d9sq = dnrth**2 + deast**2 + dz**2
				! distance from station 9 to lookout tower
	truaz = atan( (deast-seast)/(-dnrth+snrth) )
	d = (deast-seast)/sin(truaz)
	truel = atan(dz/d)
	dsq = (-dnrth+snrth)**2 + (deast-seast)**2 + dz**2
	taul=(dsqrt(dsq)-dsqrt(d9sq))/nstocm
c
c  compute raw encoder readings 

c	do i=1,7
c	    apcr(i) = pi * apc(i)/180./60.
c	    epcr(i) = pi * epc(i)/180./60.
c	enddo

c	sinaz = sin(truaz)
c	cosaz = cos(truaz)
c	sinel = sin(truel)
c	cosel = cos(truel)
c	encaz=truaz+(apc(1)+apc(2)/cosel+sinel/cosel*(apc(3)
c	1	+apc(4)*sinaz+apc(5)*cosaz)+ apc(6)*sin(2.*truaz)
c	2	+apc(7)*cos(2.*truaz))
c	encel=truel+(epc(1)+epc(2)*sinel+epc(3)*cosel+epc(4)*
c	1	sinaz+epc(5)*cosaz+epc(6)*sin(2.*truaz)+epc(7)
c	2	*cosel/sinel)

c	encaz = encaz*180./pi
c	encel = encel*180./pi
	encaz = truaz*180./pi
	encel = truel*180./pi

	return
	end


