/******************************************************************************
 *
 *	Filename:	btsio.h
 *
 *	Purpose:	Bit 3 SBus Adaptor include files for IOCTL, structures,
 *			and constants.
 *
 *	Copyright (c) 1990 by Bit 3 Computer Corporation.
 *	All rights reserved.
 *	License governs use and distribution.
 *
 ******************************************************************************/

/******************************************************************************
 *			Include files
 *****************************************************************************/

#include	<sys/ioctl.h>

/******************************************************************************
 *			Definitions
 *****************************************************************************/

/* Start, size, and end of each window on SBus card. */
#define BTS_MODEL	464
#define	BT_RRAM		0x1000000	/* Remote memory */
#define	BT_SZ_RRAM	0x1000000
#define	BT_MAX_RRAM	(BT_RRAM + BT_SZ_RRAM)

#define	BT_DPRT		0x0800000	/* Dual-Port memory */
#define	BT_SZ_DPRT	0x0800000
#define	BT_MAX_DPRT	(BT_DPRT + BT_SZ_DPRT)

#define	BT_RBIO		0x0400000	/* Remote Bus I/O */
#define	BT_SZ_RBIO	0x0010000
#define	BT_MAX_RBIO	(BT_RBIO + BT_SZ_RBIO)

#define	BT_NODE		0x0200000	/* Adaptor node I/O */


#ifdef	KERNEL

/* Size for each rmalloc() of remote bus and dual-port bus space. */
#define	BT_CHUNK	0x100000

/* Information from SBus adaptor PROM */

#define	BTS_NAME	"BIT3,bts"	/* device name */
#define	MAX_NBTS	3		/* Maximum number of units */
#define	BTS_VER_NAME	"Bit3-version"	/* Minimum PROM version number */
#define	BTS_CUR_VER	"1.1"

#endif				/* KERNEL */

/******************************************************************************
 *			MACRO definitions
 *****************************************************************************/

#ifdef	KERNEL
#define	BTS_UNIT(dev)	minor(dev)
#endif

/*
*/
/******************************************************************************
 *			Structure definitions
 *****************************************************************************/

/******************************************************************************
 *
 *			SBus local node registers (All models)
 *
 *****************************************************************************/

struct bts_reg {
    u_char          loc_cmd;	/* Local Node Command Register */
#define	CLEAR_ERR	0x80	/* =1, Clear error status bits */
#define	SWAP_BYTE	0x40	/* =1, Swap Bytes */
#define	SWAP_WORD	0x20	/* =1, Swap Words */
#define	INT_ON_ERR	0x04	/* =1, Enable error interrupts, if INT_ENABLE */
#define	INT_ENABLE	0x01	/* =1, Enable device interrupts */

    u_char          nused1;

    u_char          loc_status;	/* Local Node Status Register (Read Only) */
#define	ERR_PARITY	0x80	/* =1, Interface parity error */
#define	ERR_BERR	0x40	/* =1, Remote bus error */
#define	ERR_RCVPARITY	0x20	/* =1, Parity error on received parity */
#define	ERR_TIMEOUT	0x04	/* =1, Interface timeout */
#define	NO_POWER	0x01	/* =1, Cable Disconnected or Remote Power Off */


#define	ERROR_MASK	(ERR_PARITY | ERR_BERR | ERR_RCVPARITY | ERR_TIMEOUT)

    u_char          nused3;

    /* Only the device driver should access the interrupt control register. */

#ifdef	KERNEL
    u_char          loc_intctl;	/* Local Node Interrupt Control Register */
#define	INT_PENDING	0x10	/* =1, This Adaptor is asserting an interrupt */
#define	INTREQ_SHFT	5	/* left-shift amount, INTREQ(1:7) level used */
#define	INTREQ_MASK	0x1f	/* Mask of interrupt pending bits */

#else				/* KERNEL */

    u_char          nused4;	/* Don't access from application level */

#endif				/* KERNEL */

    u_char          nused5, nused6, nused7;

/*
*/
#if	BTS_MODEL == 464

/******************************************************************************
 *
 *			Model 464 SBus-Multibus I remote node registers
 *
 *****************************************************************************/

    u_char          rem_cmdtwo;	/* Remote Node Command Register 2 */
#define	PGSZ_64		0x00	/* Set page size to  64 Kbytes */
#define	PGSZ_128	0x01	/* Set page size to 128 Kbytes */
#define	PGSZ_256	0x03	/* Set page size to 256 Kbytes */
#define	PGSZ_512	0x07	/* Set page size to 512 Kbytes */
#define	PGSZ_1024	0x0f	/* Set page size to   1 Mbytes */

    u_char          rem_cmd;	/* Remote Node Command/Status Register */
    /* Caution: The register bits for reading and writing change meaning. */
#define	REM_RESET	0x80	/* w = 1, reset Multibus I */
#define	CLEAR_PT	0x40	/* w = 1, clear PT programmed interrupt */
#define	SEND_PR		0x20	/* w = 1, cause PR programmed interrupt */
#define	LOCK_BUS	0x10	/* w = 1, LOCK BUS; r = 0, BUS IS LOCKED */
#define	USE_PAGE	0x08	/* r/w = 1, use paged mode */

#define	PT_PENDING	0x02	/* r = 1, PT interrupt received */

    /* Since the bit definitions change between a read and a write of the
       remote node command/status register, need a little help modifying it. */

#define	REM_CMD_MASK	(LOCK_BUS | USE_PAGE)

    u_char          nuseda;

    u_char          rem_page;	/* Address page register */

    u_short         nusedc, nusede;

/*
*/
#else				/* default to Model 466 SBus-VMEbus adaptor */

#define	BTS_MODEL	466

/******************************************************************************
 *
 *			Model 466 SBus-VMEbus remote node registers
 *
 *****************************************************************************/

    u_char          rem_cmdtwo;	/* Remote Node Command Register 2 */
#define	USE_AMOD	0x40	/* =1, Use Address Modifier register */
#define	PGSZ_64		0x00	/* Set page size to  64 Kbytes */
#define	PGSZ_128	0x01	/* Set page size to 128 Kbytes */
#define	PGSZ_256	0x03	/* Set page size to 256 Kbytes */
#define	PGSZ_512	0x07	/* Set page size to 512 Kbytes */
#define	PGSZ_1024	0x0f	/* Set page size to   1 Mbytes */

    u_char          rem_cmd;	/* Remote Node Command/Status Register */
    /* Caution: The register bits for reading and writing change meaning. */
#define	REM_RESET	0x80	/* w = 1, cause SYSRESET; r=1, was reset */
#define	CLEAR_PT	0x40	/* w = 1, clear PT programmed interrupt */
#define	SEND_PR		0x20	/* w = 1, cause PR programmed interrupt */
#define	LOCK_BUS	0x10	/* w = 1, LOCK BUS; r = 0, BUS IS LOCKED */
#define	USE_PAGE	0x08	/* r/w = 1, use paged mode */

#define	PT_PENDING	0x02	/* r = 1, PT interrupt received */

    /* Since the bit definitions change between a read and a write of the
       remote node command/status register, need a little help modifying it. */

#define	REM_CMD_MASK	(LOCK_BUS | USE_PAGE)

    u_short         rem_page;	/* Address page register */

    u_char          rem_amod;	/* Address modifier register */

    /*-
        Address modifier	Function
    
	    0x0e		A32 Extended Supervisory Program Access
	    0x0d		A32 Extended Supervisory Data Access
	    0x0a		A32 Extended Non-Privileged Program Access
	    0x09		A32 Extended Non-Privileged Data Access
    
	    0x3e		A24 Standard Supervisory Program Access
	    0x3d		A24 Standard Supervisory Data Access
	    0x3a		A24 Standard Non-Privileged Program Access
	    0x39		A24 Standard Non-Privileged Data Access
   
	    0x2d		A16 Short Supervisory Data Access
	    0x29		A16 Short Non-Privileged Data Access
    */

    u_char          nusedd;

    u_short         rem_iack;	/* Interrupt Acknowledge register */
#endif				/* BTS_MODEL */
};

/*
*/
/******************************************************************************
 *			IOCTL structures
 *****************************************************************************/

struct sig_def {		/* Used for BIOCGSIG & BIOCSSIG calls. */
    int             PT_sig;	/* signal type sent for PT interrupt */
    int             err_sig;	/* signal type sent for error interrupt */
    int             other_sig;	/* signal type sent for other interrupts */
};

struct shdw_regs {		/* Used for BIOCGREGS calls */
    int             loc_stat;	/* Local status register */
    int             loc_intctl;	/* Local interrupt control register */
    int             rem_stat;	/* Remote command/status register */
};

union btioctl {
    int             pgrp;	/* Process group to send signals to. */
    struct sig_def  signal;	/* Signal numbers to send by event type. */
    int             intreq;	/* SBus INTREQ level to interrupt on. */
    struct shdw_regs shadow;	/* Shadow copy of selected registers at last
				   interrupt. */
};

/******************************************************************************
 *			IOCTL definitions
 *****************************************************************************/

/* Enable/Disable error interrupt logging */

#define	BIOCNO_LOG_ERRORS	_IO(b,7)
#define	BIOCLOG_ERRORS		_IO(b,8)

/* Set/Get Process Group number */

#define	BIOCSPGRP		_IOW(b,118,int)
#define	BIOCGPGRP		_IOR(b,119,int)

/* Set/Get signal numbers to send. */

#define	BIOCSSIG		_IOW(b,5,struct	sig_def)
#define	BIOCGSIG		_IOR(b,6,struct sig_def)

/* Get register values at time of last interrupt */

#define	BIOCGREGS		_IOR(b,4,struct shdw_regs)

/* Get current INTREQ(1:7) line device interrupts on */

#define	BIOCGINTREQ		_IOR(b,2,int)

/* Set INTREQ(1:7) line device will interrupt on */
/* Must be superuser for this call */

#define	BIOCSINTREQ		_IOW(b,3,int)

/******************************************************************************
 *			External function prototypes
 *****************************************************************************/

/* Device driver procedure declarations */

int             bts_open();
int             bts_close();
int             bts_read();
int             bts_write();
int             bts_ioctl();
int             bts_vdcmd();
int             bts_mmap();

