// complex class
//

#ifndef COMPLEX_H
#define COMPLEX_H

#include <stdio.h>
#include <math.h>

class complex
{
public:
	float Re, Im;
	complex( )				// Creator -- no args.
	{	Re = 0.0;	Im = 0.0;	}
	complex( float re, float im )		// Creator -- initialized
	{	Re = re;	Im = im;	}				
	complex complex::operator=( complex z )	// Complex assignment
	{	Re = z.Re;	Im = z.Im;	return * this;
	}
	complex complex::operator=( float x )	// Complex = Real assign.
	{	Re = x;		Im = 0.0;	return * this;
	}
//	char * complex::str( char * txt )
//	{	sprintf( txt, "(%f,%f)", Re, Im );
//		return( txt );
//	}
	friend char * str( complex );
	friend complex operator+( complex, complex );
	friend complex operator+( complex, float );
	friend complex operator+( float, complex );
	friend complex operator-( complex, complex );
	friend complex operator-( complex, float );
	friend complex operator-( float, complex );
	friend complex operator-( complex );
	friend complex operator*( complex, complex );
	friend complex operator*( float, complex );
	friend complex operator*( complex, float );
	friend complex operator/( complex, complex );
	friend complex operator/( complex, float );
	friend complex operator/( float, complex );
	friend complex conj( complex );		// Complex conjugate
	friend float real( complex );		// Real part
	friend float imag( complex );		// Imaginary part
	friend float abs( complex );		// Absolute value
	friend float arg( complex );		// Argument of complex #
	friend complex sqrt( complex );		// Complex square root
};

inline complex operator+( complex z1, complex z2 )
{	return complex(z1.Re+z2.Re, z1.Im+z2.Im);	}
inline complex operator+( complex z, float x )
{	return complex(z.Re + x, z.Im);		}
inline complex operator+( float x, complex z )
{	return complex(z.Re + x, z.Im);		}
inline complex operator-( complex z1, complex z2 )
{	return complex(z1.Re-z2.Re, z1.Im-z2.Im);	}
inline complex operator-( complex z, float x )
{	return complex(z.Re - x, z.Im);		}
inline complex operator-( float x, complex z )
{	return complex(x - z.Re, z.Im);		}
inline complex operator-( complex z )
{	return complex(-z.Re, -z.Im);	}
inline complex operator*( complex z1, complex z2 )
{	return complex(z1.Re*z2.Re - z1.Im*z2.Im,
		       z1.Re*z2.Im + z1.Im*z2.Re);	}
inline complex operator*( float s, complex z )
{	return complex( s*z.Re, s*z.Im );	}
inline complex operator*( complex z, float s )
{	return complex( s*z.Re, s*z.Im );	}
inline complex operator/( complex z1, complex z2 )
{	float q = 1.0/(z2.Re*z2.Re + z2.Im*z2.Im);
	return complex( q*(z1.Re*z2.Re + z1.Im*z2.Im),
			q*(-z1.Re*z2.Im + z2.Re*z1.Im) );	}
inline complex operator/( complex z, float s )
{	return complex(z.Re/s, z.Im/s);		}
inline complex operator/( float s, complex z )
{	float q = 1.0/(z.Re*z.Re + z.Im*z.Im);
	return complex( s*q*z.Re, -s*q*z.Im );	}
static char pdq_complex_text[800];
static int  pdq_complex_index = 0;
inline char * str( complex z )
{	char * ptr = &pdq_complex_text[pdq_complex_index];
	pdq_complex_index = (pdq_complex_index + 40) % 800;
	sprintf( ptr, "(%f, %f)", z.Re, z.Im );
	return ptr;
}
inline complex conj( complex z )
{	return complex( z.Re, -z.Im );	}
inline float real( complex z )
{	return z.Re;	}
inline float imag( complex z )
{	return( z.Im );	}
inline float abs( complex z )
{	return( (float) sqrt(z.Re*z.Re + z.Im*z.Im) );	}
inline float arg( complex z )
{	return( (float) atan2( z.Im, z.Re ) );	}
inline complex sqrt( complex z )
{	float r = sqrt(z.Re*z.Re + z.Im*z.Im);
	if( z.Im > 0 )
		return complex( 0.70710678*sqrt(r+z.Re), 
			0.70710678*sqrt(r-z.Re) );
	else
		return complex( 0.70710678*sqrt(r+z.Re),
			- 0.70710678*sqrt(r-z.Re) );
}
#endif // COMPLEX_H
