/*
	Header file for control program

	consisting mostly of common storage
*/

#define NANTS 9
#define NAX 18
#define NRMSMAX 8
#define REALNANTS 7
#define REALNAX 14

/* --	ITRACK bits defined		*/
#define AZEL	1
#define RADEC	2
#define SERVO	3
#define CONSVELAZ  8
#define CONSVELEL  16
#define HORNOFF	   32
#define ELOVER90   64
#define ADDRA	   128
#define CORRECT	   256
#define	ELUNDER90  512
#define OFFSETS	   1024
#define TVOFF	   2048
/* --	globally used parameters	*/
#define SLEWHI 3000 	/* normal slew speed	 	*/
#define SLEWSP 500 	/* separating slew speed	*/
int TIMEPAST ;		/* ticks for APAST 		*/
int TIMENEXT ;		/* ticks for ANEXT 		*/
int TIMENOW ;		/* ticks from mbus now 		*/

int HIWIND ;		/* T/F wind indicator 		*/
int WINDCOUNT ;		/* counts up to 15 and back 	*/
int N12 ;		/* 0-11 which frac of sec   	*/
int TOGGLE ;		/* 0-0x8000 added to veloc  	*/

/* --	per ANTENNA parameters		*/
struct perant {
  int itrack ;		/* RA/HA/AZ/CV (n-s/pc) */
  int azax ;		/* number of my azim axis */
  int elax ;		/* number of my elev axis */
  int close ;		/* T/F am I close to anybody */
  int safe ;		/* T/F am I in unhittable pos */
  int decentinterval ;	/* counter to 3 secs  */
  double azbefore ;	/* azim previous second	*/
}  ANT[NANTS+1] ;

/* -- per AXIS parameters		*/
struct peraxis {
  int ant ;		/* which antenna 		*/
  int state ;		/* 'T'/'S'/'C'/'W'		*/
  int seq ;		/* sequence 0-7 for slew	*/
  int consvel ;		/* constant velocity used by CV	*/
  int onlimit ;		/* T/F set by pointup		*/
  int broken ;		/* count down from 15		*/
  int stalltimer ;	/* count down from 20 		*/
  int isazim ;		/* T if this is azimuth		*/
  double ain[12] ;	/* holds incoming raw angles 	*/
  double apast ;	/* old angle from pointup    	*/
  double anext ;	/* next angle from pointup   	*/
  double anow ;		/* ain[n12]			*/
  double alast ;	/* ain[n12-1]			*/
  double target ;	/* interpolated apast-anext	*/
  double starget ;	/* current slewing target	*/
  double ratio ;	/* deg/ratio = vel output	*/
  double asafe ;	/* save angle for this axis	*/
  double error[NRMSMAX];/* errors each sec  (")		*/
  double rms ;		/* rms error for last secs	*/
  int resoff ;		/* resolver offset		*/
  int vel ;		/* velocity just sent to motor	*/
  int lastvel ;		/* old velocity to motor	*/
  int laststalltime ;	/* time of last axis stall	*/
  int mbus ;		/* telemetry register for axis  */
} AX[NAX] ;

