//========================================================================
//				exports
//========================================================================
//  Export routines for correlator object, callable from straight C
//  or Fortran.
//========================================================================

#ifndef EXPORTS_H
#define EXPORTS_H

#ifndef BOOLEAN
#define BOOLEAN int
#define TRUE	1
#define FALSE	0
#endif

#ifndef SECONDS
#define SECONDS float
#endif

/* Structure for representing spectra	*/
typedef struct Spectra
{
	int ants;	/* Antennas on-line, bit set for each.		*/
	int mode;	/* Spectrometer mode, 1..4			*/
	int ctype;	/* Type of correlation, 0=>Cross, 1=>Auto	*/
	int mpx;	/* 0=>none, 1=>2X, 2=>4X multiplexing		*/
	int han;	/* Whether to hanning-smooth the spectra	*/
			/* 0=>no,   1=>yes				*/
	int process;	/* Processing flags, bit set for each:		*/
			/* 0=>norm, 1=>fixbadchan, 2=>fixclip,		*/
			/* 4=>halfshift,				*/
			/* 8=>FFT, 9=>fixbirdies, 10=>fixdelays,	*/
			/* 11=>aplpass, 12=>aplgain, 13=>hanning	*/
	float corf[4];	/* Correlator LO frequencies, MHz		*/
	float corbw[4];	/* Correlator band widths, MHz			*/
	float shrate[2];/* Correlator processor shift rates, MHz	*/
	int numants;	/* Number of antennas on-line			*/
	int numbls;	/* Number of active correlators			*/
	int numchans;	/* Number of complex channels/spectrum		*/
	float tint;	/* Integration time, seconds			*/
	int * blno;	/* Pointer to array of active BL identifiers	*/
	float * data;	/* Pointer to array of data[bl][chan],		*/
			/* chan=0..numchans-1, Complex data type.	*/
} Spectra;

/* Fringes structure */

typedef struct Fringes
{
	int numbls;	/* Number of active correlators			*/
	int * blno;	/* Pointer to array of active BL identifiers	*/
	float * data;	/* Array data[bl][sb], bl = 0..numbls-1,	*/
			/* sb = 0..1 (sideband).  Complex type.		*/
} Fringes;


/* Procedure prototypes */
BOOLEAN	cora_cal( void );	/* Calibrate upper, lower sbs, K/count	*/
BOOLEAN cora_test( void );	/* Test correlator for bad channels	*/
BOOLEAN cora_passband(		/* Get passband cal data		*/
(	Spectra & s );		/* out	Spectra data structure=passband	*/
BOOLEAN cora_comset( void );	/* Set up correlator from Global Common	*/
BOOLEAN	cora_get_lags(	 	/* Read correlator			*/
	Spectra * s );		/* out	Spectra data structure with lags*/
BOOLEAN cora_make_spectra(	/* Process lag data into spectra	*/
	Spectra * s );		/* out	Spectra data structure		*/
void cora_msg( char * txt );	/* Read text of correlator error msg	*/
void cora_start			/* Start correlator integration		*/
(	SECONDS * tint );	/* in	Integrating time		*/
void cora_idle( void );		/* Idle until correlator is done	*/
void cora_wide			/* Compute wb fringe by summing chan's	*/
(	Spectra * s,		/* in	Spectra data structure		*/
	Fringes * f );		/* out	Fringes data structure		*/
void cora_writespectra		/* Write spectra to external file	*/
(	Spectra * s,		/* in	Spectra data structure		*/
	char * sname );		/* in	Root name for files:<nam>xx.dat	*/
void cora_adc_power		/* Read IF power from ADC's		*/
	int ant,		/* in   Physical antenna ID		*/
	float p[8] );		/* out  Power level, each sampler	*/
void coracom( void );		/* Write info about spectra to G.Common */
int cora_los(			/* Set Correlator LO's (3rd LO's )	*/
	float corfs[4] );	/* in	Frequencies for LO1..4, MHz	*/

#endif
