//=======================================================================
//	gpibio
//
//	Header file to be used in conjunction with National Instruments'
//	cib library for their GPIB interface board.  These declarations
//	are intended for use with the C++ compiler.
//=======================================================================

// Status bits returned by most functions
#define ERR	0x8000		// An error has occurred
#define TIMO	0x4000		// Time limit exceeded
#define END	0x2000		// End or EOS message occurred during read
#define SRQI	0x1000		// GPIB line SRQ asserted
#define RQS	0x0800		// Device requesting service
#define CMPL	0x0100		// Prev. I/O operation complete
#define LOK	0x0080		// Whether board in lockout state
#define REM	0x0040		// Whether board in remote state
#define CIC	0x0020		// Whether board is Controller-In-Charge
#define ATN	0x0010		// Whether line ATN is asserted
#define TACS	0x0008		// Whether board addressed to talk
#define LACS	0x0004		// Whether board addressed to listen
#define DTAS	0x0002		// Whether board detected dev. trigger cmd
#define DCAS	0x0001		// Whether board detected dev. clear cmd

// Error numbers
#define EDVR	0		// Unix error--look in ibcnt for code
#define ECIC	1		// Function requires GPIB board to be CIC
#define ENOL	2		// Write handshake error (no listener?)
#define	EADR	3		// GPIB board not addressed correctly
#define	EARG	4		// Invalid argument to function call
#define ESAC	5		// GPIB board not system controller
#define EABO	6		// I/O operation aborted
#define ENEB	7		// Non-existent GPIB board
#define EDMA	8		// DMA hardware error
#define EBTO	9		// DMA hardware bus timeout
#define ECAP	11		// No capability for type of operation
#define EFSO	12		// File system error
#define EBUS	14		// GPIB bus error
#define ESTB	15		// Serial Poll queue overflow
#define ESRQ	16		// SRQ line asserted by unknown device

// Prototypes for selected functions

extern int ibsta;				// Status word
extern int iberr;				// Error code
extern int ibcnt;				// Count of bytes xferred

extern "C"
{
	int ibclr( int devh );			// Clear
	int ibfind( char * devicename );	// Get handle for device
	int ibllo( int devh );			// Put in local locout mode
	int ibloc( int devh );			// Return to local mode
	int ibonl( int devh, int parm );	// Reinit GPIB; parm=0 =>
						// close device
	int ibpad( int devh, int addr );	// Change GPIB primary addr.
	int ibrd( int devh, char * buf, int charcount );
						// Read chars from device
	int ibsad( int devh, int addr );	// Change secondary address
						// 0 => disable s.a. recog.
	int ibwait( int devh, int mask );	// Wait for events to occur
	int ibwrt( int devh, char * buf, int charcount );
						// Write chars to device
}

