/* rcvr.h Header file which contains declarations of structures used
	in the tuning programs	*/

/* the motor structure stores the settings for each motor */

struct motor {
	char name[12]; /* name of the motor */
	int address; /* address of the motor, 0 to 7 */
	int ad_add; /* address of the potentiometer readout on the a/d card */
	int card_add; /* address of the motor driver card */
	int coast; /* distance motor coasts in a/d counts */
	int pulse; /* distance travelled in 1 pulse, in counts */
	int shoot; /* overshoot (required for backlash) in counts */
	float speed; /* motor speed in rpm */
};

struct modulator {
	int v_add; /* DAC address to set the modulator voltage */
	int i_add; /* address to monitor the modulator current */
};

struct mixer {
	int v_add; /* DAC address to set the mixer bias voltage */
	int v_mon_add; /* address to monitor the mixer voltage */
	int i_mon_add; /* address to monitor the mixer current */
	int card_add; /* address of the card controlling open/closed loop */
	int loop_add; /* two bytes indicating which bit is wired to the
		open/closed loop control */
	int temp_add; /* a/d address for the mixer temperature sensor */
};

/* the band structure stores settings relevant to each band */

struct band {
	char name; /* which band, A,B,C or D */
	char mixer[10]; /* type of mixer, "sis" or "schottky" */
	char osc[5]; /* number of the oscillator, used to look up table */
	int v_sense_add; /* address of the mm oscillator voltage sense */
					/* address refers to tripler for band D */
	int power; /* address of the total power detector */
	float low_freq; /* low limit of band frequency range */
	float high_freq; /* high limit of band frequency range */
	struct motor *mmosc; /* pointer to the motor structure for mmosc */
	struct motor *mmback; /* pointer to the motor for mm backshort */
	struct motor *atten; /* pointer to the motor for the attenuator */
	struct motor *mxrback; /* pointer to the motor for the mixer 
		backshort */
	struct motor *cavity; /* pointer to the motor for the cavity */
	struct modulator *mod; /* pointer to the modulator structure */
	struct mixer *mxr; /* pointer to the mixer structure */
	char touchy; /* y/n type variable for touchy oscillators */
};
	
/* these structures are band independent */

struct mm_lock {
	int err_add; /* address for error volts */
	int if_add; /* address for if level */
	int stat_card; /* address of the card with the status bit */
	int stat_add; /* two bytes which determine which bit is wired
		to the phase lock status, relevant bit is high */
	int v_op_add; /* address for the DAC vop setting */
	int lg_add; /* address for the DAC phase lock loop gain setting */
	int lg_mon_add; /* address for monitoring the ph. lock loop gain */
	int ref_card; /* address of the card with the phase lock. ref bit */
	int ref_add; /* two bytes which determine which bit is wired to 
		the mm phase lock reference status, relevant bit is high */
};

struct x_lock {
	int err_add; /* address for error volts */
	int if_add; /* address for if level */
	int stat_card; /* address of the card with the status bit */
	int stat_add; /* two bytes which determine which bit is wired
		to the phase lock status, relevant bit is high */
	int yig_add; /* address for the DAC yig coarse tune voltage set */
	int yig_mon_add; /* monitor address for yig coarse setting */
};

struct l_lock {
	int err_add; /* address for error volts */
	int if_add; /* address for if level */
	int stat_card; /* address of the card with the status bit */
	int stat_add; /* two bytes which determine which bit is wired
		to the phase lock status, relevant bit is high */
};

struct select {
/* this structure has bit out addresses, all two bytes, with the
	relevant bit being high */
	int if_card; /* address of the card with the if bits */
	int if_1; /* if select, bit 1 */
	int if_2; /* if select, bit 2 */
	int hemt_card; /* address of the card with the hemt bits */
	int hemt_1; /* hemt amplifier select bit 1 */
	int hemt_2; /* hemt amplifier select bit 2 */
	int ph_card; /* address of the card with the phase lock bits */
	int phlock_1; /* phase lock select bit 1 */
	int phlock_2; /* phase lock select bit 2 */
	int osc_card; /* address of the card with the osc bit */
	int osc; /* mm oscillator on/off bit */
	int sweep_card; /* address of the card with the sweep bit */
	int sweep; /* mm phase lock sweep on/off bit */
	int mm_card; /* address of the card with the if monitor bit */
	int if_mon; /* mm if level monitor on/off bit */
};

struct table {
/* this stucture holds a table of entries, one for each motor
position, which is passed from the table lookup program to the 
main porgram */
	int mmosc; /* target for the mm oscillator */
	int mmback; /* target for the mm backshort */
	int mxrback; /* target for the mixer backshort */
	int atten; /* target for the attenuator */
	int cavity; /* target for the cavity */
	float bias; /* target for the mixer bias */
	float vop; /* target for the millimeter operating volts */
};
