/*
 * User GPIB Include File
 * (%M% %I% - %G% %U%)
 */
typedef short word_t;     /* two byte object */

/* structure exits one per gpib board in system */
struct board {
        word_t  b_uflags;          /* user flags (see below) */
	struct dmaregs * b_dmag;   /* I/0 port address for DMA controller */
        struct ibregs * b_addr;    /* I/O port address for 7210 and TUBO */
        char    b_pad,b_sad;       /* primary and secondary gpib address */
        char    b_eos;             /* end-of-string character */
        char    b_ppe;             /* parallel poll enable byte */
        char    b_clk;             /* clock setting (imr) */
	char	b_dma;		   /* 0:programmed I/O; 1:DMA */
        char    b_sig;             /* signal to send */
        word_t  b_tmo;             /* timeout value (see below) */
        char    b_res[3];          /* (reserved) */
};

/* board user flags (b_uflags) */
#define EOT     000001  /* assert EOI with last byte of each write */
#define HLD     000002  /* holdoff handshake at end of each read */
#define EOSM    000034  /* end-of-string modes */
#define REOS    000004  /* terminate read on eos */
#define XEOS    000010  /* assert EOI with eos byte */
#define BIN     000020  /* eight-bit compare */
#define TDCL    000040  /* terminate I/O on device clear */
#define TASC    000100  /* terminate I/O on address status change */
#define PP2	000200	/* support local parallel poll */
#define TTICK   000400  /* if set, b_tmo is ticks not scale factor */
#define IST     001000  /* individual status bit */
#define SRE     002000  /* assert REN when SC */
#define SC      004000  /* interface is system controller */
#define TRI     010000  /* use high-speed timing */
#define VSTD    040000  /* use Very Short T1 delay(very high-speed) */

struct device {
        word_t d_uflags;     /* (see below) */
        char d_bna;       /* access board number */
        char d_pad,d_sad; /* primary and secondary gpib address */
        char d_eos;       /* end-of-string character */
        char d_tmo;       /* timeout value */
        char d_ppe;       /* parallel poll enable byte */
	char d_dname[15]; /* device name */
        char d_rtlk_m;    /* request to talk spb (AND mask) */
        char d_rtlk_v;    /* request to talk spb (value) */
        char d_rltn_m;    /* request to listen spb (AND mask) */
        char d_rltn_v;    /* request to listen spb (value) */
};

/* device user flags (d_uflags) */
/*      EOT     000001  /* assert EOI with last byte of each write */
/*      HLD     000002  /* holdoff handshake at end of each read */
/*      EOSM    000034  /* end-of-string modes */
/*       REOS   000004  /*  terminate read on eos */
/*       XEOS   000010  /*  assert EOI with eos byte */
/*       BIN    000020  /*  eight-bit compare */
/*      TDCL    000040  /* terminate I/O on device clear */
/*      TASC    000100  /* terminate I/O on address status change */
#define DSPQ    001000  /* save serial poll responses in queue */
#define WRQS    002000  /* wait for RQS before starting I/O */

/* GPIB status bit vector */

#define ERR     (1<<15) /* Error detected */
#define TIMO    (1<<14) /* timeout */
#define END     (1<<13) /* EOI or eos detected */
#define SRQI    (1<<12) /* SRQ detected by CIC */
#define RQS     (1<<11) /* Device requires service */
#define CMPL    (1<<8)  /* DMA completed (SH/AH synch'd) */
#define LOK     (1<<7)  /* local lockout state */
#define REM     (1<<6)  /* remote state */
#define CIC     (1<<5)  /* controller-in-charge */
#define ATN     (1<<4)  /* attention asserted */
#define TACS    (1<<3)  /* talker active */
#define LACS    (1<<2)  /* listener active */
#define DTAS    (1<<1)  /* device trigger state */
#define DCAS    (1<<0)  /* device clear state */

/* GPIB Bus Control Lines bit vector */
#define BUS_DAV         0x100           /* DAV line status bit  */
#define BUS_NDAC        0x200           /* NDAC line status bit */
#define BUS_NRFD        0x400           /* NRFD line status bit */
#define BUS_IFC         0x800           /* IFC line status bit  */
#define BUS_REN         0x1000          /* REN line status bit  */
#define BUS_SRQ         0x2000          /* SRQ line status bit  */
#define BUS_ATN         0x4000          /* ATN line status bits */
#define BUS_EOI         0x8000          /* EOI line status bits */

/* Error messages */

#define EDVR  0  /* System error */
#define ECIC  1  /* Not CIC (or lost CIC during command) */
#define ENOL  2  /* write detected no listeners */
#define EADR  3  /* board not addressed correctly */
#define EARG  4  /* bad argument to function call */
#define ESAC  5  /* function requires board to be SAC */
#define EABO  6  /* asynchronous operation was aborted */
#define ENEB  7  /* non-existent board */
#define EDMA  8  /* DMA hardware error detected */
#define EBTO  9  /* DMA hardware uP bus timeout */
#define EOIP 10  /* new I/O attempted with old I/O in progress  */
#define ECAP 11  /* no capability for intended opeation         */
#define EFSO 12  /* file system operation error                 */
#define EOWN 13  /* shareable board exclusively owned */
#define EBUS 14  /* bus error */
#define ESTB 15  /* serial poll queue overflow */
#define ESRQ 16  /* SRQ line 'stuck' on */

extern  unsigned   int      ibsta,iberr,ibcnt;

/* Timeout values and meanings                          */
#define TNONE    0   /* Infinite timeout (disabled)     */
#define T10us    1   /* Timeout of 10 us (ideal)        */
#define T30us    2   /* Timeout of 30 us (ideal)        */
#define T100us   3   /* Timeout of 100 us (ideal)       */
#define T300us   4   /* Timeout of 300 us (ideal)       */
#define T1ms     5   /* Timeout of 1 ms (ideal)         */
#define T3ms     6   /* Timeout of 3 ms (ideal)         */
#define T10ms    7   /* Timeout of 10 ms (ideal)        */
#define T30ms    8   /* Timeout of 30 ms (ideal)        */
#define T100ms   9   /* Timeout of 100 ms (ideal)       */
#define T300ms  10   /* Timeout of 300 ms (ideal)       */
#define T1s     11   /* Timeout of 1 s (ideal)          */
#define T3s     12   /* Timeout of 3 s (ideal)          */
#define T10s    13   /* Timeout of 10 s (ideal)         */
#define T30s    14   /* Timeout of 30 s (ideal)         */
#define T100s   15   /* Timeout of 100 s (ideal)        */
#define T300s   16   /* Timeout of 300 s (ideal)        */
#define T1000s  17   /* Timeout of 1000 s (maximum)     */

extern unsigned ibsta,iberr,ibcnt;

/* ioctl codes */
#define IIOC    ('I'<<8)
#define U_OUT	0x40000000	/* parameters are read into user space */
#define K_IN	0x80000000	/* parameters are read into kernal space */
#define PARAS	0x400000	/* parameter size, not larger than 127 */

/* IBGET/IBSET use the form: ioctl(f,IBGET,&board); */
#define IBGET    (K_IN|U_OUT|PARAS|IIOC|0)   /* get board/device structure */
#define IBSET    (K_IN|U_OUT|PARAS|IIOC|1)   /* set board/device structure */

#define IBINB    (IIOC|2)       /* special */
#define IBOUTB   (IIOC|3)       /* special */

/* Secondary address constant used by IBLN() */
#define NO_SAD	       0	/* No secondary address */
#define ALL_SAD	    0xFF	/* send all secondary addresses */

struct ibinfo { 
        unsigned char bcnt,     /* number of boards */
             	      dcnt,     /* number of devcies */
             	      isdev;    /* non-zero if called with device file descriptor */
};
/* IBINFO uses the form: ioctl(f,IBINFO,&ibinfo); */
#define IBINFO   (IIOC|4)       /* get board and device info */

struct ibarg {
        long     buf;           /* buffer address */
        unsigned n;             /* count or argument */
        unsigned ibsta;         /* return status */
        unsigned iberr;         /* return error code */
        unsigned ibcnt;         /* return count */
};

/* the following codes use the form: ioctl(f,IBRD,&ibarg);  */
#define IBTRG   (IIOC|10)
#define IBCLR   (IIOC|11)
#define IBCMD   (IIOC|12)
#define IBXTRC  (IIOC|14)
#define IBRSP   (IIOC|15)
#define IBRPP   (IIOC|16)
#define IBBNA   (IIOC|17)
#define IBWAIT  (IIOC|18)
#define IBONL   (IIOC|19)
#define IBRSV   (IIOC|20)
#define IBGTS   (IIOC|21)
#define IBCAC   (IIOC|22)
#define IBPOKE  (IIOC|23)
#define IBSGNL  (IIOC|24)
#define IBSRE   (IIOC|25)
#define IBRSC   (IIOC|26)
#define IBRD    (IIOC|27)
#define IBWRT   (IIOC|28)
#define IBLOC   (IIOC|29)
#define IBPCT   (IIOC|30)
#define IBSIC   (IIOC|31)
#define IBIST   (IIOC|32)
#define IBPPC   (IIOC|33)
#define IBLINE  (IIOC|34)
#define IBEOT   (IIOC|35)
#define IBPAD   (IIOC|36)
#define IBSAD   (IIOC|37)
#define IBDMA   (IIOC|38)
#define IBEOS   (IIOC|39)
#define IBTMO   (IIOC|40)
#define IBWRTF  (IIOC|41)
#define IBRDF   (IIOC|42)
#define IBLLO   (IIOC|43)
#define IBSTAT  (IIOC|44)
#define IBLN	(IIOC|45)
#define IBDEV	(IIOC|46)
