      PROGRAM CHECKSOURCE
c= CHECKSOURCE - calculates the current LST and elevation of source.
c: observing
c+
c	CHECKSOURCE calculates the current LST and elevation of source.
c	If the source is a planet, CHECKSOURCE  also calculates the 
c	predicted flux for each baseline. It is useful for command files 
c	(cf. POINTING.CSH) and checking catalog inputs.
c       Note 1: NAIF saves the last source/catalog found so when looking
c       up a source twice in succession, the second time repeats the
c       first. Look up a different source to force a new catalog lookup. 
c	Note 2: In checker mode NAIF last source/catalog procedure is not
c	used.
c@ source
c       Source name [max 8 characters] from catalog or planet
c       Other objects on the NAIF approved list can be accessed
c       as _xxxxxx where xxxxxx is their NAIF number. Comet 
c       shoemaker-levy has number 1000130 and is loaded into
c       our database.
c
c@ epoch
c	Julian Day number of observation (0 means present)
c 	Can also enter yymmmdd:[hh:mm... format, need to : to
c	trigger.
c	Default: 0
c
c@ catalog
c       Catalog name; first catalog is searched (this is a path, not
c       just a filename). If it is not found then the filename is
c       stripped off and $HATCAT/filename is searched.
c       If blank, $HATCAT/j2000.cat is assumed.
c@ elevlim
c	Elevation limit (deg) is only really needed when checksource
c	is called from command files to determine if observing can be done.
c	Default: 10 degrees

c@ setme
c     Setting this causes checksource to only output the
c     value that is desired; this allows the shell command:
c        set=`checksource ... setme=xxx` to work.
c     Current values can be:
c        visible     'y' or 'n' (is the source above elevlim?)
c        azel        azim(deg) elev(deg)
c     Note: if a program/script relies on output from these, be sure
c	to include an option that the source/catalog may have been
c	misspecified, and an error message may be returned.
c     Default: none.
c@ latitude
c     Latitude of the observatory, in degrees.
c     Default: 40.817361 (Hat Creek)
c
c--
C	JC FEB 87 ,added rise and set time stuff JC JUL 87
C	WH NOV 89 keyini added
c	unix->  nov 90 wh
c	setme added mar 93
c     added setme=azel to print out (az,el) in radians        jun 93   - pjt
c     fixed bug in computing azimuth                         5-aug-93    pjt
c	merged newstyle catalog cache for NAIF from wilson
c	fixed up epoch= to allow yymmmdd
c     merged online and checker version once more           11-nov-93    pjt
c>>c       2aug93 wh - match changes in get_source
c>>c       27aug93 wh - add exit status
c>>c       30aug  wh - fix bug from 27aug
c>>c       16sep  wh - fix units for dRA,dDEC
c     no more local prepending of path                              feb-94 pjt
c     declared catalog*100 to be consistent with lower level code   feb-94 pjt

C----------------------------------------------------------------------------
      include '../inc/constants.inc'
      include '../inc/ephem.inc'
      record /asource/ s
      real*4 plflx/9999./
      real*8 epoch
      real*4 secs /206264.8/
      real*4 degs /57.29577/
      real*8 lo,lst,u_ns(nants),v_ns(nants),frequp,freqlow,ut,freq
      real*8 sini, cosi, elev, acosarg
      real*4 az,el_lim
      real*8 cosl,sinl,latitude,elow,rise,set,flxupr,flxlwr
      real*8 ub,vb,beta_ns
      integer*4 month,day,year, i, j, k, ticks
      character*14 angles,a1,a2
      character*9 splflx(7),sdis(7)
      character cat*100, src*12, setme*10, error*80
c
      integer len1
      real*4 j1xbyx
c
      call keyini
      call keya('source',src,' ')
      call keya('catalog',cat,' ')
      call keyr('elevlim',el_lim,10.)
      call keyepoch('epoch',epoch,0.d0)
      call keya('setme',setme,' ')
      call keyd('latitude',latitude,40.817361d0)
      call keyfin

      IF (src.eq.' ') CALL bug('f','Need source=')

      call mbusopen(error)
      if(error(1:2).ne.'OK') CALL bug('f',error)

      CALL GET_TIMES(TICKS,2,MONTH,DAY,YEAR,UT,LST,ERROR)
      IF(ERROR(1:2).NE.'OK') CALL bug('f',error)

      CALL GET_SOURCE_NOLOCK(SRC,CAT,EPOCH,S,ERROR)
      IF(ERROR(1:2).NE.'OK') CALL bug('f',error)

C  --  compute elevation
      sinl = SIN(latitude/degs)
      cosl = COS(latitude/degs)
      ELEV = ASIN(sinl*SIN(S.DEC0)+cosl*COS(S.DEC0)*COS(LST-S.RA0))
      elow = el_lim/degs
      acosarg=(sin(elow)-sin(S.dec0)*sinl)/(cos(S.dec0)*cosl)
      if (acosarg.gt..9999907) then 
		acosarg=.9999907
      else if (acosarg.lt.-.9999907) then
		acosarg=-.9999907
      endif
      rise=S.ra0-acos(acosarg)
      set=S.ra0+acos(acosarg)
      if (rise.lt.0.0) rise = tupi + rise

c  --	handle various setme= cases (visible, azel)
c       they all stop the program right here, no more processing done
      IF(SETME(1:7).EQ.'visible') THEN
	   IF(ELEV.GT.ELOW) THEN
                TYPE *,'y'
	   ELSE
		TYPE *,'n'
	   END IF
           STOP
      ELSE IF (SETME(1:4).EQ.'azel') THEN
           az = ACOS((SIN(s.dec0) - sinl*SIN(elev))/(cosl*COS(elev)))
           IF(SIN(lst-s.ra0).GT.0) az = tupi - az
           TYPE *,AZ*DEGS,ELEV*DEGS
           STOP
      END IF

      TYPE *
      TYPE *,' Source: ',S.NAME,'   [',CAT(1:LEN1(CAT)),']'
      IF (epoch.ne.0.d0) TYPE *,'  Computed for epoch:',epoch
      IF (ACOSARG.EQ..9999907) THEN
	   type *,' source never rises to ',el_lim,' degrees'
      ELSE IF(ACOSARG.EQ.-.9999907) THEN
	   type *,' source never sets below ',el_lim,' degrees'
      ELSE
	   a1 = ANGLES(RISE*12.D0/DPI)
	   a2 = ANGLES(SET*12.D0/DPI)
	   TYPE 996,a1,a2,el_lim
996	   format(1x,' Rises at LST = ',A,' sets at ',A,
     *        	   ' (elev limit ',f5.0,' Deg)')
      ENDIF
      a1 = ANGLES(LST*12.D0/DPI)
      a2 = ANGLES(ELEV*180.D0/DPI)
      TYPE 992,a1,a2
992   FORMAT (1x,' Present LST = ',A,6X,'source elevation: ',A)
      TYPE 995,S.VELOC + S.VREST + S.VSUN
995   FORMAT(1x,' Doppler velocity: ',F7.1)
      IF(S.PLMAJ.EQ.0.) THEN
	   a1 = ANGLES(S.RA2000*12.D0/DPI)
	   a2 = ANGLES(S.DEC2000*180.D0/DPI)
	   TYPE 990,a1,a2
      END IF
990   FORMAT('  J2000    RA: ',A,5X,'  DEC: ',A)

      a1 = ANGLES(S.RA0*12.D0/DPI)
      a2 = ANGLES(S.DEC0*180.D0/DPI)
      TYPE 991,a1,a2
991   FORMAT('  Present  RA: ',A,5x,'  DEC: ',A)

c --  convert rad/tick to hr/deg per minute of time
      a1 = ANGLES(S.RA1*12.D0/DPI*6000.d0)
      a2 = ANGLES(S.DEC1*180.D0/DPI*6000.d0)
      TYPE 998,a1,a2
998   FORMAT('  per min dRA: ',A,5x,' dDEC: ',A)

      IF(S.PLMAJ.NE.0.) THEN
         CALL COMGETD('FREQ',FREQ,1,ERROR)
	 TYPE *
	 TYPE 993,S.PLMAJ*SECS,S.PLMIN*SECS,S.PLANGLE
	 TYPE 994,S.PLTB,S.PLFLUXCR,FREQ
      END IF

993   FORMAT('  Planet major/minor axes: ',F8.1,'" ',F8.1,'"',
     *         ' Inclin: ',F6.1,' degrees')
994   FORMAT('              temperature: ',F6.1,
     *       ' K      flux: ',F6.1,' Jy at',F7.2,' GHz')

      IF (S.PLMAJ.NE.0.) THEN

         CALL COMGETD('LO1',LO,1,ERROR)
	 CALL WRITE_ERROR('CHECKSOURCE',ERROR)
	 FREQUP=LO+1.5
	 FREQLOW=LO-1.5
	 if(S.PLTB.GT.0.) then
              FLXUPR=S.PLMAJ*S.PLMIN*1158.1*(FREQUP)**3/
     *   		(EXP(.04801*(FREQUP)/S.PLTB)-1.)
	      FLXLWR=S.PLMAJ*S.PLMIN*1158.1*(FREQLOW)**3/
     *           	(EXP(.04801*(FREQLOW)/S.PLTB)-1.)
	 else		!fix-up for sun,moon,mercury, where PLTB=0.
	      FLXUPR=0.
	      FLXLWR=0.
         endif

	 type *
         type *,'ANT      FLUX (Up+Low)/2 \\ -SQRT(U*U+V*V) '
	 TYPE *,'       1        2        3        4        5',
     *        '        6        7'
         CALL CALCUNSVNS(LST,U_NS,V_NS,ERROR)
	 CALL WRITE_ERROR('CHECKSOURCE',ERROR)
         COSI=COS(S.PLANGLE)
         SINI=SIN(S.PLANGLE)
	 DO I=1,7
	    DO J=1,7
	       IF((I.NE.J).AND.(J.NE.2).AND.(I.NE.2)) THEN
                  UB = U_NS(J)-U_NS(I)
		  VB = V_NS(J)-V_NS(I)
		  BETA_NS=DPI*SQRT((S.PLMAJ*(UB*COSI-VB*SINI))**2
     *   		+(S.PLMIN*(UB*SINI+VB*COSI))**2)
		  PLFLX=ABS(J1XBYX(SNGL(BETA_NS*FREQUP)))*FLXUPR
     *   		   +ABS(J1XBYX(SNGL(BETA_NS*FREQLOW)))*FLXLWR
		  WRITE(SPLFLX(J),'(F9.2)') PLFLX
		  WRITE(SDIS(J),'(F9.2)') -SQRT(UB*UB+VB*VB)

	       ELSE
	    	  SPLFLX(J) = ' '
		  SDIS(J) = ' '
	       END IF
	    END DO
            TYPE '(I2,7A)',I,(SPLFLX(K),K=1,I-1),'         ',
     *           (SDIS(K),K=I+1,7)
	 END DO
      END IF

99    TYPE *,ERROR
      END
c***********************************************************************
      SUBROUTINE GET_SOURCE_NOLOCK(SOURCE,CATALOG,PRESENT_EPOCH,S,ERROR)
C
C  SENDS MESSAGE TO EPHEMERIS TO PRECESS A SOURCE WITHOUT LOCKS
C   THIS VERSION DOES NOT FILL IN COMMON VARIABLES
C  WH  JAN 87
C-------------------------------------------------------------C
	include '../inc/ephem.inc'
	record /asource/ s
	include '../inc/constants.inc'
	character*(*) source,catalog
	real*8 present_epoch,freq,TJD
	integer ticks(2)
	integer*4 mon,day,year, getcwd, ie, len1
	real*8 ut,lst, julian
	character*80 error,cwd,catpath

C	--NO ACTION IF SOURCE=TEST
	IF(SOURCE(1:4).EQ.'TEST' .OR. SOURCE(1:4).EQ.'test') THEN
	  ERROR='OK'
	  S.NAME = 'TEST'
	  S.VELOC=0.
	  S.VSUN=0.
	  S.VREST=0.
	  S.PLMAJ=0.
	  S.PLMIN=0.
	  S.PLANGLE=0.
	  S.RA2000=0.d0
	  S.DEC2000=0.d0
	  S.RA0=0.D0
	  S.RA1=0.D0
	  S.DEC0=0.D0
	  S.DEC1=0.D0
	ELSE
C	  --otherwise get position from ephemeris routines--
	  CALL COMGETD('FREQ',FREQ,1,ERROR)
	   IF (FREQ.LT.50.D0) FREQ = 90.D0
	  IF (PRESENT_EPOCH.EQ.0.D0) THEN
	    CALL GET_TIMES(TICKS,1,MON,DAY,YEAR,UT,LST,ERROR)
		TJD = JULIAN(DAY,MON,YEAR,UT)
	  ELSE
		TJD = PRESENT_EPOCH
	  END IF
c
c dubious method of catalog searching ....
c
cpjt	  IF(CATALOG(1:1).NE.' ') THEN
cpjt	   IF(INDEX(CATALOG,'/').GT.0) THEN
cpjt	    CATPATH = CATALOG
cpjt	   ELSE
cpjt	    IE = GETCWD(CWD)
cpjt	    CATPATH = CWD(:LEN1(CWD))//'/'//CATALOG(:LEN1(CATALOG))
cpjt	   END IF
cpjt	  ELSE
cpjt	   CATPATH = ' '
cpjt	  END IF
cpjt
cpjt	  CALL EPHEMERIS(source,catpath,TJD,FREQ,S,ERROR)

	  CALL EPHEMERIS(source,catalog,TJD,FREQ,S,ERROR)

	END IF

	RETURN
	END

c***********************************************************************
      SUBROUTINE keyepoch(key,epoch, default)
c
      CHARACTER key*(*)
      DOUBLE PRECISION epoch, default
c
      CHARACTER*30 estr
      LOGICAL ok
      INTEGER len1

      CALL keya(key,estr,' ')
      CALL lcase(estr)

c       check if ':' or month (vowels 'aeuo' will do it) seems to appear 
c       in string, if so, try the dayjul conversion, else assume it's 
c       already a julian day...

      ok = INDEX(estr,':').GT.0 .OR.
     *     INDEX(estr,'a').GT.0 .OR.
     *     INDEX(estr,'e').GT.0 .OR.
     *     INDEX(estr,'u').GT.0 .OR.
     *     INDEX(estr,'o').GT.0

      IF (ok) THEN
         CALL dayjul(estr,epoch)
      ELSE
         IF (estr.EQ.' ') THEN
            epoch = default
         ELSE
            CALL atodf(estr(1:len1(estr)),epoch,ok)
            IF (.NOT.ok) CALL bug('w','(atodf) Decoding '//estr)
         ENDIF
      ENDIF
      END
