c History:
c	cloned off checksource - 3aug93  pjt
c
c	(note: 	subroutine GET_SOURCE_NOLOCK and KEYEPOCH are both
c	in this code and checksource, need to put this in library
c	or share from an include file...)

c
      PROGRAM DISTANCE
c
c= distance - angular separation between two sources
c& pjt
c: observing
c+
c     DISTANCE calculates the angular distance between two sources
c     at any given epoch. The computations are done in azimuthm-
c     elevation coordinates, to aid HatCreek specific observations.
c     For unknown sources use CATALOG to create your own catalog.
c
c@ source
c     Source name [max 8 characters]. You can enter
c     either one or two sources. If two are used,
c     it limits catalog search to the catalog= keyword
c     only. Else source1= and catalog1= are used for
c     the comparison source.
c     No default.
c
c@ catalog
c     Catalog name used to search for all entries from
c     source=.
c     Default: $HATCAT/j2000.cat
c
c@ source1
c     Comparison source to compute the distance to. 
c     No default if it had not been used in the second 
c     entry on source=
c
c@ catalog1
c     Catalog used to find source1, if used.
c     Default: j2000.cat.
c
c@ epoch
c     Julian Day number of observation (0 means present).
c     The ``[yymmmdd:][hh[:mm[:ss.s]]]'' format
c     is also supported as long as the ':' symbol is present.
c     This will be converted internally to julian days.
c     Default: 0.
c
c@ elevlim
c     Elevation limit in degrees. For each source that is below the
c     elevation limit, a warning is given.
c     Default: 10.
c
c@ latitude
c     Latitude of the observatory in degrees.
c     Default 40.817361 (hatcreek)
c
c--
c-----------------------------------------------------------------------
      INCLUDE '../inc/constants.inc'
      INCLUDE '../inc/ephem.inc'
      RECORD /asource/ s
      REAL*4 plflx/9999./
      REAL*8 am /.004363323/
      REAL*8 epoch
      REAL*4 secs /206264.8/
      REAL*4 degs /57.29577/

      INTEGER ticks
      INTEGER*4 month,day,year
      REAL*8 az0,el0,az1,el1,daz,del,d,elevlim
      REAL*8 ra0,dec0,ra1,dec1,cosl,sinl
      REAL*8 lst,ut,latitude
      CHARACTER*12 src0,src1
      CHARACTER*50 cat0,cat1
      CHARACTER*80 error
c externals
      LOGICAL keyprsnt

      CALL mbusopen(error)
      IF(error(1:2).NE.'OK') CALL bug('f',error)

      CALL keyini
      CALL keya('source',src0,' ')
      CALL keya('catalog',cat0,' ')
      CALL keyepoch('epoch',epoch)
      IF (keyprsnt('source')) THEN
         CALL keya('source',src1,' ')
         cat1 = cat0
      ELSE
         CALL keya('source1',src1,' ')
         CALL keya('catalog1',cat1,' ')
      ENDIF
      CALL keyd('elevlim',elevlim,10.0d0)
      CALL keyd('latitude',latitude,40.817361d0)
      CALL keyfin
	
      IF (src0.EQ.' ') CALL bug('f','Need source=')
      IF (src1.EQ.' ') CALL bug('f','Need source1=')

      CALL get_times(ticks,2,month,day,year,ut,lst,error)
      IF(error(1:2).NE.'OK') CALL bug('f',error)

      cosl = COS(latitude/degs)
      sinl = SIN(latitude/degs)

C Get source

      CALL get_source_nolock(src0,cat0,epoch,s,error)
      IF(error(1:2).NE.'OK') CALL bug('f',error)
      el0 = ASIN(sinl*SIN(s.dec0)+cosl*COS(s.dec0)*COS(lst-s.ra0))
      az0 = ACOS( (SIN(s.dec0) - sinl*SIN(el0))/(cosl*COS(el0)) )
      IF (SIN(lst-s.ra0).GT.0) az0 = dtupi - az0
        ra0 = s.ra0
	dec0 = s.dec0

C Get comparison source

      CALL get_source_nolock(src1,cat1,epoch,s,error)
      IF(error(1:2).NE.'OK') CALL bug('f',error)
      el1 = ASIN(sinl*SIN(s.dec0)+cosl*COS(s.dec0)*COS(lst-s.ra0))
      az1 = ACOS( (SIN(s.dec0) - sinl*SIN(el1))/(cosl*COS(el1)) )
      IF (SIN(lst-s.ra0).GT.0) az1 = dtupi - az1
	ra1 = s.ra0
	dec1 = s.dec0


      daz = (az1-az0)*degs
      del = (el1-el0)*degs
      d = ACOS(SIN(el0)*SIN(el1) + COS(el0)*COS(el1)*COS(az0-az1))

c	TYPE *,'AZEL0: ',az0*degs,el0*degs
c	TYPE *,'AZEL1: ',az1*degs,el1*degs
      TYPE *,d*degs

c This is always good of course and independant of LST

c	d = ACOS(SIN(dec0)*SIN(dec1) + COS(dec0)*COS(dec1)*COS(ra0-ra1))
c      	TYPE *,d*degs

      elevlim = elevlim/degs
c      IF (el0.LT.elevlim) CALL bug('w','source below elevlim: '//src0)
c      IF (el1.LT.elevlim) CALL bug('w','source below elevlim: '//src1)

      END
c***********************************************************************
	SUBROUTINE GET_SOURCE_NOLOCK(SOURCE,CATALOG,PRESENT_EPOCH,S,
	1	ERROR)
C  SENDS MESSAGE TO EPHEMERIS TO PRECESS A SOURCE WITHOUT LOCKS
C   THIS VERSION DOES NOT FILL IN COMMON VARIABLES
C  WH  JAN 87
C-------------------------------------------------------------C
	include '../inc/ephem.inc'
	record /asource/ s
	include '../inc/constants.inc'
	character*(*) source,catalog
	real*8 present_epoch,freq,TJD
	integer ticks(2)
	integer*4 mon,day,year, getcwd
	real*8 ut,lst, julian
	character*80 error,cwd,catpath

C	--NO ACTION IF SOURCE=TEST
	IF(SOURCE(1:4).EQ.'TEST' .OR. SOURCE(1:4).EQ.'test') THEN
	  ERROR='OK'
	  S.NAME = 'TEST'
	  S.VELOC=0.
	  S.VSUN=0.
	  S.VREST=0.
	  S.PLMAJ=0.
	  S.PLMIN=0.
	  S.PLANGLE=0.
	  S.RA2000=0.d0
	  S.DEC2000=0.d0
	  S.RA0=0.D0
	  S.RA1=0.D0
	  S.DEC0=0.D0
	  S.DEC1=0.D0
	ELSE
C	  --otherwise get position from ephemeris routines--
	  CALL COMGETD('FREQ',FREQ,1,ERROR)
	   IF (FREQ.LT.50.D0) FREQ = 90.D0
	  IF (PRESENT_EPOCH.EQ.0.D0) THEN
	    CALL GET_TIMES(TICKS,1,MON,DAY,YEAR,UT,LST,ERROR)
		TJD = JULIAN(DAY,MON,YEAR,UT)
	  ELSE
		TJD = PRESENT_EPOCH
	  END IF

	  IF(CATALOG(1:1).NE.' ') THEN
	   IF(INDEX(CATALOG,'/').GT.0) THEN
	    CATPATH = CATALOG
	   ELSE
	    IE = GETCWD(CWD)
	    CATPATH = CWD(:LEN1(CWD))//'/'//CATALOG(:LEN1(CATALOG))
	   END IF
	  ELSE
	   CATPATH = ' '
	  END IF

	  CALL EPHEMERIS(source,catpath,TJD,FREQ,S,ERROR)

	END IF

	RETURN
	END
c***********************************************************************
      SUBROUTINE keyepoch(key,epoch)
      CHARACTER key*(*)
      DOUBLE PRECISION epoch
c
      CHARACTER*30 estr
      LOGICAL ok

      CALL keya(key,estr,' ')
      CALL lcase(estr)

c       check if ':' or month name seems to appear in string,
c       if so, try the dayjul conversion, else assume it's 
c       already a julian day...

      ok = INDEX(estr,':').GT.0 .OR.
     *     INDEX(estr,'a').GT.0 .OR.
     *     INDEX(estr,'e').GT.0 .OR.
     *     INDEX(estr,'u').GT.0 .OR.
     *     INDEX(estr,'o').GT.0

      IF (ok) THEN
         CALL dayjul(estr,epoch)
      ELSE
         IF (estr.EQ.' ') THEN
            epoch = 0.0d0
         ELSE
            CALL atodf(estr,epoch,ok)
            IF (.NOT.ok) CALL bug('w','(atodf) Decoding '//estr)
         ENDIF
      ENDIF
      END


