/*	FUNCTIONS TO ACCESS THE COMMON AREA				*/
/*	add cpid lock  oct 91  wh					*/
#include "hatlib.h"
#include <sys/mman.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define TRUE 1
#define FALSE 0
#define MAXHASH	768
#define MAXSTORE 4096
#define NULL 0
static struct hashentry {char name[12]; int index;} *hashb, *hashp ;
struct chead {int length; char type;} ;
static union commentry {
    struct chead ahead ;
    int anint[2] ;
    float afloat[2] ;
    double adouble ;
    char achar[8] ; 
}  *commb ;
static int *cnext, *cpid ;
static int mypid = 0 ;

static int hash(char [],int *) ;
static int commap() ;
static int comcreate(char [],char *,int *,int *,char *) ;
static void fixname(char *,char *) ;
static void lock_common(), unlock_common() ;

/** HASH	- convert a hash code into a pointer			*/
/*: common								*/
/*+
	INTEGER FUNCTION HASH(NAME,INDEX)

	(this is internal to comgetx and computx routines)
  ARGUMENTS:
  NAME (char,input)		parameter name to be added to the common
				 hash table
  INDEX (integer,output)	pointer to found name or next empty place
  HASH  (logical,output)	true if newname is found
									*/
/*--									*/
int hash(char name[],int *index)
{
  int h, i, cont, j ;
  int found ;

	h = 0 ;
	for (i=0; i<11; i++)  h = h + (int)name[i] ;
	h = (h*7) % MAXHASH ;

	if(hashb == NULL) {
	  if(commap() == FALSE) {
		printf("Unable to open common file\n") ;
		return FALSE ;
	  }
	}

	hashp = hashb + h;
	cont = TRUE ;
	while (cont)  {
		if (strcmp(name,hashp->name) == 0) {
			found = TRUE ;
			cont = FALSE ;
		}
		else {
		  if (hashp->index == 0) {
			found = FALSE ;
			cont = FALSE ;
		  }
		  else {
			hashp-- ;
			if (hashp < hashb) hashp = hashb + MAXHASH ;
			if (hashp == hashb + h + 1) {
				printf ("Hash Table Full - common dies\n") ;
				cont = FALSE ;
				found = FALSE ;
			}
		  }
		}
	}
	*index = hashp->index ;
	return found ;
}
/** COMMAP 	- map the common block and fill in pointers		*/
/*: common								*/
/*+
	INTEGER FUNCTION COMMAP()

	(this is internal to comgetx and computx routines)

  FILLS IN STATIC VARIABLES:
  HASHB (*char,output)		pointer to hash table
  COMMB (*char,output)		pointer to storage
  CNEXT (*int, output)		pointer to index of first avialable slot
  CPID  (*int, output)		pointer to proc pid used by comcreate
  COMMAP (logical,output)	true if map works
									*/
/*--									*/
int commap()
{
  caddr_t point ;
  int fd, size ;

	if((fd = open(getenv("HATCOM"),O_RDWR,0)) == -1)
		return FALSE ;
	size = 8 + MAXHASH * sizeof(struct hashentry) 
		 + MAXSTORE * sizeof(union commentry) ;
	point = mmap((caddr_t)0,size,PROT_READ|PROT_WRITE,MAP_SHARED,fd,0) ;

	hashb = (struct hashentry *)(point + 2*sizeof(int)) ;
	size = 2*sizeof(int) + MAXHASH*sizeof(struct hashentry) ;
	commb = (union commentry *)(point + size) ;
	cnext = (int *)point ;
	cpid  = (int *)(point+sizeof(int)) ;
	(void) close(fd) ;
	return TRUE ;
}
/** COMINIT	- initialize the common block by writing to it		*/
/*: common								*/
/*+
	INTEGER FUNCTION COMINIT()

  USES:
  MAXHASH		size of hash table
  MAXSTORE		size of storage block
  COMINIT (logical,output)	true if OK

  Storage block is laid out as follows:
  cnext (int)		      index of next available storage slot (starts at 1)
  MAXHASH* (struct hashentry) hash table				
  MAXSTORE*(union commentry)  storage slots of 8 bytes each can contain
				either a header of type and length or
				data of 8*char/2*int/2*float/double
									*/
/*--									*/
int cominit_()
{
  int fd, size, wstat, i ;
  char buf[64] ;
  static char firstnext[4] = {0,0,0,1} ;

	if((fd = open(getenv("HATCOM"),O_CREAT|O_RDWR,
		S_IRWXU|S_IRWXG|S_IRWXO,0766)) == -1) return FALSE ;

	if(fchmod(fd,S_IRWXU|S_IRWXG|S_IRWXO) == -1) return FALSE ;

	for (i=0; i<64; i++) buf[i] = 0 ;
	size = 40 + MAXHASH * sizeof(struct hashentry) 
		 + MAXSTORE * sizeof(union commentry) ;
	size = (size / 64) + 1 ;
		wstat = write(fd,firstnext,4) ;
		if(wstat < 0) return FALSE ;
	for (i=0; i<=size; i++)  {
		wstat = write(fd,buf,64) ;
		if(wstat < 0) return FALSE ;
	}
	(void) close(fd) ;
	return TRUE ;
}
/** COMGETA	- get a string from the common				*/
/*: common								*/
/*+
	COMGETA(NAME,STRING,N,ERROR)

	
  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  STRING (char,output)	The value of the variable
  N (integer,input)	The size of string
  ERROR (char,output)	Status return
									*/
/*--									*/
int comgeta_(char name[],char string[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int index,i, nsubs ;
  char *bytes ;

	fixname(name,uname) ;
	
	if( hash(uname,&index) == FALSE) {
	  sprintf(error,"%s is not in common\n",uname) ;
	  return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;

	if (head->ahead.type != 'C')  {
	  sprintf(error,"%s (%c) is not type C\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n != nsubs) {
	  sprintf(error,"%s (%d) length doesn't match (%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    bytes = (char *)head ;
	    for (i=0; i<*n; i++) string[i] = *bytes++ ; 
	    string[*n] = '\0' ;
	}
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMPUTA	- put a string to the common				*/
/*: common								*/
/*+
	COMPUTA(NAME,STRING,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  STRING (char,output)	The value of the variable
  N (integer,input)		The size of string
  ERROR (char,output)	Status return 
									*/
/*--									*/
int computa_(char name[],char string[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int i, index, nsubs ;
  char *bytes ;
  static char ccc = {'C'} ;

	fixname(name,uname) ;
	lock_common() ;
	if( hash(uname,&index) != TRUE) {
		comcreate(uname,&ccc,n,&index,error) ;
		if (strcmp(error,"OK") != 0) return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;
	unlock_common() ;
	if (head->ahead.type != ccc)  {
	  sprintf(error,"%s (%c) is not type C\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n > nsubs) {
	  sprintf(error,"%s (%d) is too big for slot(%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    bytes = (char *)head ;
	    for (i=0; i<*n; i++) *bytes++ = string[i] ; 
	}
	strncpy(error, "OK",80) ;
	return TRUE ;
}
/** COMGETI	- get an integer array from the common			*/
/*: common								*/
/*+
	COMGETI(NAME,INTS,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  INTS (int,output)	The value of the variables
  N (integer,input)		The size of the array
  ERROR (char,output)	Status return
									*/
/*--									*/
int comgeti_(char name[],int ints[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int index,i, nsubs ;
  int *integers ;

	fixname(name,uname) ;

	if( hash(uname,&index) == FALSE) {
	  sprintf(error,"%s is not in common\n",uname) ;
	  return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;

	if (head->ahead.type != 'I')  {
	  sprintf(error,"%s (%c) is not type I\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n != nsubs) {
	  sprintf(error,"%s (%d) length doesn't match (%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    integers = (int *)head ;
	    for (i=0; i<*n; i++) ints[i] = *integers++ ; 
	}
	strncpy(error,"OK",80) ;
	return TRUE  ;
}
/** COMPUTI	- put an integer array into the common			*/
/*: common								*/
/*+
	COMPUTI(NAME,INTS,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  INTS (int,input)	The value of the variable
  N (integer,input)		The size of the array
  ERROR (char,output)	Status return
									*/
/*--									*/
int computi_(char name[],int ints[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int i, index, nsubs ;
  static char ccc = {'I'} ;
  int *integers ;

	fixname(name,uname) ;
	lock_common() ;
	if( hash(uname,&index) != TRUE) {
		comcreate(uname,&ccc,n,&index,error) ;
		if (strcmp(error,"OK") != 0) return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;
	unlock_common() ;
	if (head->ahead.type != 'I')  {
	  sprintf(error,"%s (%c) is not type I\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n > nsubs) {
	  sprintf(error,"%s (%d) is too big for slot(%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    integers = (int *)head ;
	    for (i=0; i<*n; i++) *integers++ = ints[i] ; 
	}
	strncpy(error, "OK",80) ;
	return TRUE ;
}
/** COMGETR	- get an array of reals from the common			*/
/*: common								*/
/*+
	COMGETR(NAME,REALS,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  REALS (float,output)	The value of the variable
  N (integer,input)		The size of the array
  ERROR (char,output)	Status return
									*/
/*--									*/
int comgetr_(char name[],float reals[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int index,i, nsubs ;
  float *floats ;

	fixname(name,uname) ;

	if( hash(uname,&index) == FALSE) {
	  sprintf(error,"%s is not in common\n",uname) ;
	  return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;

	if (head->ahead.type != 'R')  {
	  sprintf(error,"%s (%c) is not type R\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n != nsubs) {
	  sprintf(error,"%s (%d) length doesn't match (%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    floats = (float *)head ;
	    for (i=0; i<*n; i++) reals[i] = *floats++ ; 
	}
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMPUTR	- put an array of reals into the common			*/
/*: common								*/
/*+
	COMPUTR(NAME,REALS,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  REALS (float,output)	The value of the variable
  N (integer,input)		The size of string
  ERROR (char,output)	Status return
									*/
/*--									*/
int computr_(char name[],float reals[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int i, index, nsubs ;
  static char ccc = {'R'} ;
  float *floats ;

	fixname(name,uname) ;
	lock_common() ;
	if( hash(uname,&index) != TRUE) {
		comcreate(uname,&ccc,n,&index,error) ;
		if (strcmp(error,"OK") != 0) return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;
	unlock_common() ;
	if (head->ahead.type != 'R')  {
	  sprintf(error,"%s (%c) is not type R\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n > nsubs) {
	  sprintf(error,"%s (%d) is too big for slot(%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    floats = (float *)head ;
	    for (i=0; i<*n; i++) *floats++ = reals[i] ; 
	}
	strncpy(error, "OK",80) ;
	return TRUE ;
}
/** COMGETD	- get an array of doubles from the common		*/
/*: common								*/
/*+
	COMGETD(NAME,DP,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  DP (double,output)	The value of the variable
  N (integer,input)		The size of string
  ERROR (char,output)	Status return
									*/
/*--									*/
int comgetd_(char name[],double dp[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int index,i, nsubs ;
  double *doubles ;

	fixname(name,uname) ;

	if( hash(uname,&index) == FALSE) {
	  sprintf(error,"%s is not in common\n",uname) ;
	  return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;

	if (head->ahead.type != 'D')  {
	  sprintf(error,"%s (%c) is not type D\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n != nsubs) {
	  sprintf(error,"%s (%d) length doesn't match (%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    doubles = (double *)head ;
	    for (i=0; i<*n; i++) dp[i] = *doubles++ ; 
	}
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMPUTD	- put an array of doubles into the common		*/
/*: common								*/
/*+
	COMPUTD(NAME,DP,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  DP (double,output)	The value of the variable
  N (integer,input)		The size of string
  ERROR (char,output)	Status return
									*/
/*--									*/
int computd_(char name[],double dp[],int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int i, index, nsubs ;
  static char ccc = {'D'} ;
  double *doubles ;

	fixname(name,uname) ;
	lock_common() ;
	if( hash(uname,&index) != TRUE) {
		comcreate(uname,&ccc,n,&index,error) ;
		if (strcmp(error,"OK") != 0) return FALSE ;
	}
	head = commb+index ;
	nsubs = head->ahead.length ;
	unlock_common() ;
	if (head->ahead.type != ccc)  {
	  sprintf(error,"%s (%c) is not type D\n",uname,head->ahead.type) ;
	  return FALSE ;
	}
	else if (*n > nsubs) {
	  sprintf(error,"%s (%d) is too big for slot(%d)\n",uname,*n,nsubs) ;
	  return FALSE ;
	}
	else {
	    head++ ;
	    doubles = (double *)head ;
	    for (i=0; i<*n; i++) *doubles++ = dp[i] ; 
	}
	strncpy(error, "OK",80) ;
	return TRUE ;
}
/** COMPROBE	- find size and type for a common variable		*/
/*: common								*/
/*+
	COMPROBE(NAME,TYPE,N,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  TYPE (char,output)	C=string I=integer R=float D=double
  N (integer,output)	The size of the variable array
  ERROR (char,output)	Status return
									*/
/*--									*/
int comprobe_(char name[],char *type,int *n,char *error)
{
  char uname[12] ;
  union commentry *head ;
  int index ;

	fixname(name,uname) ;
	if( hash(uname,&index) == FALSE) {
	  sprintf(error,"%s is not in common\n",uname) ;
	  return FALSE ;
	}
	head = commb+index ;
	*n = head->ahead.length ;
	*type = head->ahead.type ;
	strncpy(error,"OK",80) ;
	return TRUE  ;
}
/** COMHASHPROBE	- find size and type for a common variable	*/
/*: common								*/
/*+
	COMHASHPROBE(HASHN,NAME,INDEX,ERROR)

  ARGUMENTS:
  HASHN(int,input)	Slot number in hash table
  NAME (char,output)	The common variable name (11 chars max)
  INDEX (integer,output)The index into the storage array
  ERROR (char,output)	Status return
									*/
/*--									*/
int comhashprobe_(int *hashn,char name[],int *index,char *error)
{

	if( *hashn<0 || *hashn >= MAXHASH) {
	  sprintf(error,"Slot %d is out of hash table bounds\n",*hashn) ;
	  return FALSE ;
	}

	if(hashb == NULL) {
	  if(commap() == FALSE) {
		sprintf(error,"Unable to open common file\n") ;
		return FALSE ;
	  }
	}

	hashp = hashb + *hashn ;
	if (hashp->index == 0) {
	  sprintf(error,"Slot %d is empty\n",*hashn) ;
	  strcpy(name," ") ;
	  *index = 0 ;
	  return FALSE ;
	}
	strcpy(name,hashp->name) ;
	*index = hashp->index ;
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMINFO	- find sizes for common blocks				*/
/*: common								*/
/*+
	COMINFO(MAXHASH,LEFTHASH,MAXSTORE,NEXT,ERROR)

  ARGUMENTS:
  MAXHASH(int,output)	Size of hash table
  LEFTHASH(int,output)	Number of unused hash slots
  MAXSTORE(int,output)	Size of storage table
  NEXT (int,output)	Next unused storage slot
  ERROR (char,output)	Status return
									*/
/*--									*/
int cominfo_(int *maxhash,int *lefthash,int *maxstore,int *next,char *error)
{
  int i ;

	if(hashb == NULL) {
	  if(commap() == FALSE) {
		sprintf(error,"Unable to open common file\n") ;
		return FALSE ;
	  }
	}

	*maxhash = MAXHASH ;
	*maxstore = MAXSTORE ;
	*next = *cnext ;

	*lefthash = 0 ;
	hashp = hashb ;
	for (i=0; i<MAXHASH;i++) {
		if(hashp->index == 0) *lefthash = *lefthash + 1 ; 
		hashp++ ;
	}
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMRAW	- format a specific line from the common storage block  */
/*: common								*/
/*+
	COMRAW(INDEX,TYPE,NSUB,CHARS,INTS,REALS,DP,ERROR)

  ARGUMENTS:
  INDEX (integer,input)	The index into the storage array
  TYPE  (char,output)	C=char I=ints R=reals D=double
  NSUB  (int,output)	Size of array
  INTS  (int*2,output)	integers
  REALS (float*2,output)Reals
  DP	(double,output)	Double precision
  ERROR (char,output)	Status return
									*/
/*--									*/
int comraw_(int *index,char *type,int *nsub,char *chars,int *ints,
				float *reals,double *dp,char *error)
{
  union commentry *head ;
  int *hints ;
  float *hreals ;

	if( *index<0 || *index >= MAXSTORE) {
	  sprintf(error,"Index %d is out of storage table bounds\n",*index) ;
	  return FALSE ;
	}

	if(hashb == NULL) {
	  if(commap() == FALSE) {
		sprintf(error,"Unable to open common file\n") ;
		return FALSE ;
	  }
	}

	head = commb + *index ;
	*type = head->ahead.type ;
	*nsub = head->ahead.length ;
	*dp = head->adouble ;
	strncpy(chars,head->achar,8) ;
	hreals =(float *)head ;
	*reals = *hreals++ ;
	*(reals+1) = *hreals ;
	hints = (int *)head ;
	*ints = *hints++ ;
	*(ints+1) = *hints ;
	strncpy(error,"OK",80) ;
	return TRUE ;
}
/** COMCREATE	- enter a new common variable name			*/
/*: common								*/
/*+
	COMCREATE(NAME,TYPE,N,INDEX,ERROR)

  ARGUMENTS:
  NAME (char,input)	The common variable name (11 chars max)
  TYPE (char,input)	C=char I=integ R=real D=double
  N (integer,input)		The size of string
  INDEX (integer,output)	location of data space
  ERROR (char,output)	Status return
									*/
/*--									*/
int comcreate(char name[],char *type,int *n,int *index,char *error)
{
  int i, j, size ;
  char padname[12] ;
  union commentry *next ;

	switch (toupper(*type)) {
	  case 'C':
		size = (*n+7) /8	;
		break ;
	  case 'D':
		size= *n ;
		break ;
	  case 'R':
		size = (*n+1) /2 ;
		break ;
	  case 'I':
		size = (*n+1) /2 ;
		break ;
	  default:
		sprintf(error,"%s(%c) is not a legal type\n",
						name,toupper(*type)) ;
		return FALSE ;
	}
	for (i=0;(name[i] !='\0') && i<11; i++) padname[i] = name[i] ;
	for (j=i; j<11; j++)  padname[j] = ' ' ;
	padname[11] = '\0' ;
	strcpy(hashp->name,padname) ;
	hashp->index = *cnext ;
	*index = *cnext ;
	next = commb + *cnext ;
	next->ahead.type = toupper(*type) ;
	next->ahead.length = *n ;
	*cnext = *cnext + size + 1 ;
	strncpy(error, "OK",80) ;
	return TRUE ;
}
/* --	a little function to put the input names into standard form   */
void fixname(char *in,char *out)
{
  int i,j ;

	for (i=0;(in[i] !='\0') && (in[i] !=' ') && i<11; i++) 
					out[i] = toupper(in[i]) ;
	for (j=i; j<11; j++)  out[j] = ' ' ;
	out[11] = '\0' ;
}

/* --	function to lockup the common block so changes can be made  */
	void lock_common()
{
  int retries, one = 1 ;

	if(hashb == NULL) {
	  if(commap() == FALSE) {
		printf("Unable to open common file\n") ;
	  }
	}
	if(mypid == 0) mypid = getpid() ;
	retries = 10 ;
	while (*cpid != mypid) {
	 while (*cpid != 0 && retries > 0) { 
	  wait_ticks_(&one) ;
	  retries-- ;
	 }
	 *cpid = mypid ;
	}
}
/* --	function to unlock the common block after lock_common   */
	void unlock_common()
{
	if(*cpid == mypid) *cpid = 0 ;
}
