c
c* COMET_POSITION
c: ephemeris
c+
	SUBROUTINE COMET_POSITION(TOBS,S,ERROR)
C 
c   COMET_POSITION:	the two body comet ephemeris generator (tbceg)
C			is designed to produce predicted observables 
c			of a comet from  osculating elements.
c			It accomplishes this through the computation 
c			of a two-body orbit of the comet from the osculating 
c			elements and the computation of solar coordinates from 
c			newcomb's theory. thus the program avoids the necessity 
c			of numerically integrating the comet orbit, as well 
c 			as reading an external planetary ephemeris.   
c 
c			NOTE: comet circulars still give 1950 coordinates. 
c		This program does not use more than one set of osculating 
c		elements for a given ephemeris interval; that is, each 
c		ephemeris interval must correspond to one and only one set 
c		of osculating elements.
c
c		On the first entry to the subroutine, file:
c		 hatcatalogs:comet.tbl is read.	Each record contains
c		either a ! in column 1 for a comment or
c		Tday,Tmon,Tyr, q, e, omega, omicron, i  (from IAU telegram)
c		or Julian_time,q, e, omega, omicron, i
c		(Date of perihelion passage, Perihelion distance(AU),
c		Eccentricity, Argument of perihelion(deg), 
c		Longitude of ascending node(deg), Inclination(deg) )
c 
c  ARGUMENTS:
C  TOBS(dp,input)		Julian day/time of calculation
c  S (RECORD,output)		RECORD containing source parameters
C  ERROR (char,output)		Status return
c--
c	dec 89 wh
c	unix-> nov 90 wh
C-------------------------------------------------------------------------C
	real*8 tobs,ra0,ra1,dec0,dec1,inc
	character*80 error
	include '../../inc/constants.inc'
	include '../../inc/ephem.inc'
	record /asource/ s
	real*8 au,rem,stacor(3),etmutc,distkm,distkm1
      COMMON/TOPO/IOPT,iberk,AU,REM,STACOR,ETMUTC     
      DATA REM/6378.140D0/    
      DATA AU/1.4959787066D8/ 
      DATA IOPT/2/
      DATA DPR/57.2957795130823252D0/   
      STACOR(1)=238.5282083D0/DPR	!coordinates of Hat Creek <longitude>
      STACOR(2)=.758D0			!<cos(lat)>
      STACOR(3)=.65032D0		!<sin(lat)>
c      ETMUTC=53.18439D0/86400.D0	!Ephemeris time - UTC in days
	etmutc=0.d0			!this is corrected in julian func

	ERROR='OK'
	CALL COMET (TOBS,RA0,DEC0,DISTKM,ERROR)
	 IF (ERROR(1:2).NE.'OK') RETURN

	INC = .001D0		!increment in days for calc dRA and vel
	CALL COMET(TOBS+INC,RA1,DEC1,DISTKM1,ERROR)
	 IF (ERROR(1:2).NE.'OK') RETURN

c  --   ra1 and dec1 are derivatives   (radians per radian of UT)
	S.RA0 = RA0
	S.RA2000 = RA0
	S.DEC0 = DEC0
	S.DEC2000 = DEC0
	S.RA1 = (RA1-RA0) / (INC * DTUPI)
	S.DEC1 = (DEC1-DEC0) / (INC * DTUPI)
	S.VELOC = 0.
	S.VREST = 0.
	S.VSUN = (DISTKM1 - DISTKM) / (INC * 86400.d0)
	S.PLMAJ = 0.
	S.PLMIN = 0.
	S.PLANGLE = 0.
	S.PLTB = 0.
	S.PLFLUXCR = 0.
	S.PLBEAM = 0.
	RETURN
	END

c			The underlying sub was written by Dr. m.s.w. keesey.
      SUBROUTINE COMET(TOBS,RAOUT,DECOUT,DISTKM,ERROR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
	REAL*8 JULIAN
	character*80 error
	character*130 aline
      DIMENSION STACOR(3)    
      DIMENSION RSG(6),RCH(3),RC(6),PUT(4),ROG(6) 
      COMMON/ELM/TT,Q,ECR,PX,PY,PZ,QX,QY,QZ       
      COMMON/TOPO/IOPT,iberk,AU,REM,STACOR,ETMUTC       
      DATA TWOPI,RAD,CSE,SNE/6.283185307179586D0,0.0174532925199D0,   
     *0.9174369524418883D0,0.3978811861525251D0/  
      DATA DPR/57.2957795130823252D0/   
	character*50 comettbl
	external gerror
 
C  --   read the table of orbital elements. 
c  --   Arguments: Tday,Tmon,Tyr, q, e, omega, omicron, i  (from IAU telegram)
	  call filepath('HATCAT','comet.tbl',' ',comettbl)
	  open(14,name=comettbl,status='old',iostat=is)
	  if (is.ne.0) then
		write(error,'(a,a)')'COMET: open error ',gerror()
		return
	  end if
	  aline = '!'
	  do while (aline(1:1).eq.'!')
	   READ(14,'(A)',iostat=is) aline
	   if (is.ne.0) then
		write(error,'(a,a)')'COMET: read error ',gerror()
		return
	   end if
	  end do
	   READ(aline,*,IOSTAT=IS)Tday,Tmon,Tyr,Q,ECR,WR,OMR,XIR
	   if (is.ne.0) then
		error='COMET: can''t decipher>'//aline(1:40)
		return
	   end if
	   WR =WR * RAD    
	   OMR=OMR * RAD    
	   XIR=XIR * RAD    
	   IF(TMON.GT.0.D0 .OR. TYR.GT.0.D0) THEN
	     TT=JULIAN(INT(TDAY),INT(TMON),INT(TYR),(TDAY-INT(TDAY))*TWOPI)
	   ELSE
	     TT=TDAY
	   END IF
	  close(14)
 
C  --   compute the p and q rotation factors. 
      PP=DSIN(WR)*DCOS(XIR)*DCOS(OMR)+DCOS(WR)*DSIN(OMR)    
      QQ=DCOS(WR)*DCOS(XIR)*DCOS(OMR)-DSIN(WR)*DSIN(OMR)    
      PX=-(DSIN(WR)*DCOS(XIR)*DSIN(OMR)-DCOS(WR)*DCOS(OMR)) 
      PY=PP*CSE-SNE*DSIN(XIR)*DSIN(WR)  
      PZ=CSE*DSIN(XIR)*DSIN(WR)+PP*SNE  
      QX=-(DCOS(WR)*DCOS(XIR)*DSIN(OMR)+DSIN(WR)*DCOS(OMR)) 
      QY=-(SNE*DSIN(XIR)*DCOS(WR)-QQ*CSE) 
      QZ=QQ*SNE+CSE*DSIN(XIR)*DCOS(WR)  
 
C  --   ntrig=0 section - 
c  --   compute the heliocentric coordinates of the comet, the geocentric solar
c  --   coordinates, and the heliocentric  comet distance.      
c  --   if iopt=2, compute the topocentric comet distance, and apply the  
c  --   light time correction and the elliptic portion of the
c  --   annual part of the stellar aberration.      
      CALL APPAD2(0,TOBS,RU,ECLOBL,PUT,RSG,RCH,RC,ROG,POSANG)     
 
C  --   compute the 1950 alpha, delta, and change to hours and deg.
C      RHO=DSQRT(RC(1)**2+RC(2)**2+RC(3)**2)       
C      DECOUT= DASIN(RC(3)/RHO)   
C      RAOUT=DATAN2(RC(2),RC(1))  
C      IF(RAOUT.LT.0.D0) RAOUT=RAOUT+TWOPI

C  --   compute earth-comet-sun angle beta, and the total and nuclear mag
C      GRHO=DSQRT((RSG(1)+RCH(1))**2+(RSG(2)+RCH(2))**2+     
C     *(RSG(3)+RCH(3))**2)     
C      RCOM=DSQRT(RCH(1)*RCH(1)+RCH(2)*RCH(2)+RCH(3)*RCH(3)) 
C      RSUNSQ=RSG(1)*RSG(1)+RSG(2)*RSG(2)+RSG(3)*RSG(3)      
C      BETA= DACOS((RCOM*RCOM+GRHO*GRHO-RSUNSQ)/(2.D0*RCOM*GRHO))/RAD  
 
C  --   compute the sun-earth-comet angle.  
C      RSUN=DSQRT(RSUNSQ)      
C      THETA= DACOS((GRHO*GRHO+RSUNSQ-RCOM*RCOM)/(2.D0*GRHO*RSUN))/RAD 
 
C  --   compute the topocentric ecliptic longitude and latitude.
C      YY1=RC(2)*CSE+RC(3)*SNE 
C      ZZ1=-RC(2)*SNE+RC(3)*CSE
C      RRP=DSQRT(RC(1)*RC(1)+YY1*YY1)    
C      RRE=DSQRT(RC(1)*RC(1)+YY1*YY1+ZZ1*ZZ1)      
C      TOPLAT=DASIN(ZZ1/RRE)/RAD 
C      TOPLNG=DATAN2(YY1/RRP,RC(1)/RRP)/RAD
C      IF(TOPLNG.LT.0.D0) TOPLNG=TOPLNG+360.D0     
 
C  --   ntrig set to 1 in order to apply precession, nutation, and the    
c  --   circular portion of the annual part and the diurnal part of the   
c  --   stellar aberration to the topocentric comet coordinates.
  722 CALL APPAD2(1,TOBS,STOUT,ECLOBL,PUT,RSG,RCH,RC,ROG,POSANG)     
 
C  --  compute apparent right ascension and declination.
  723 RHO=DSQRT(RC(1)*RC(1)+RC(2)*RC(2)+RC(3)*RC(3))
      DECOUT=DASIN(RC(3)/RHO)   
      RAOUT=DATAN2(RC(2),RC(1)) 
      IF(RAOUT.LT.0.D0) RAOUT=RAOUT+TWOPI     
      DISTKM = RHO * AU 
	RETURN
      END       
C*******************************************************************************
      SUBROUTINE ANABR3(DJ,X)   
       
c   this subroutine applies the circular portion of the annual part of  
c   the stellar aberration to input coordinates.
c       
c   ref. - meeus,j., astronomical formulae for calculators, monografieen
c   over astronomie en astrofysica, uitgegeven door volkssterrenwacht   
c   urania v.z.w. en vereniging voor sterrenkunde v.z.w. vol. 4, 1979.  
c       
c   note - the input rectangular coordinates are referred to the mean   
c   equator and equinox of date.
C       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(6)    
      DATA DPR/57.29577951D0/   
      DATA CONSAB/-9.933832328D-5/      
      TJC=(DJ-2415020.0D0)/36525.D0     
      CMASUN=(358.47583D0+(35999.04975D0-(1.5D-4+3.3D-6*TJC)*TJC)*TJC)  
     */DPR      
      CC=(1.919460D0-(4.789D-3+1.4D-5*TJC)*TJC)*DSIN(CMASUN)    
     *+(2.0094D-2-1.00D-4*TJC)*DSIN(2.D0*CMASUN)
     *+2.93D-4*DSIN(3.D0*CMASUN)
      CMLSUN=279.69668D0+(36000.76892D0+3.025D-4*TJC)*TJC       
      TLSUN=(DMOD(CMLSUN+CC,360.D0))/DPR
      ECLOBL=(23.452294D0-(1.30125D-2+(1.64D-6-5.03D-7*TJC)*TJC)*TJC)   
     */DPR      
      SR=DSQRT(X(1)*X(1)+X(2)*X(2)+X(3)*X(3))   
      RA=DATAN2(X(2),X(1))      
      DEC=DASIN(X(3)/SR)
      SINRA=DSIN(RA)    
      COSRA=DCOS(RA)    
      SINDEC=DSIN(DEC)  
      COSDEC=DCOS(DEC)  
      SINTL=DSIN(TLSUN) 
      COSTL=DCOS(TLSUN) 
      SINEPS=DSIN(ECLOBL)       
      COSEPS=DCOS(ECLOBL)       
      TANEPS=SINEPS/COSEPS      
      DELRA=CONSAB*(COSRA*COSTL*COSEPS+SINRA*SINTL)/COSDEC      
      DELDEC=CONSAB*(COSTL*COSEPS*(TANEPS*COSDEC-SINRA*SINDEC)  
     *+COSRA*SINDEC*SINTL)      
      RA=RA+DELRA       
      DEC=DEC+DELDEC    
      COSDEC=DCOS(DEC)  
      X(1)=SR*COSDEC*DCOS(RA)   
      X(2)=SR*COSDEC*DSIN(RA)   
      X(3)=SR*DSIN(DEC) 
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE APPAD1(NTRIG,ETOBS,PUT,RSG,RCH,RCG)    
       
c   this subroutine computes the geocentric comet coordinates.  
c       
c   if ntrig=0, the light time correction is applied to the comet       
c   coordinates. if ntrig=1, precession, nutation, and the circular     
c   portion of the annual aberration are also applied.  
c       
c   input -     
c      ntrig - 0 or 1.  
c      etobs - ephemeris time of observation, in j.d.   
c      put   - an array containing delta psi and delta epsilon, used    
c       in the nutation subroutine, in radians. 
c   output-     
c      rsg - geocentric solar coordinates, wrt the mean equator and     
c     equinox of 1950.0, in au's.       
c      rch - heliocentric comet coordinates, wrt the mean equator       
c     and equinox of 1950.0, in au's.   
c      rcg - geocentric comet coordinates, in au's.     
c    if ntrig=0, wrt mean equator and equinox of 1950.0.
c    if ntrig=1, wrt apparent equator and equinox of date.      
       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION EL(21),C(16),RSG(6),RCH(3),RCG(6)       
      DIMENSION PUT(4),OBM(2)   
      COMMON/ELM/TT,Q,ECR,PX,PY,PZ,QX,QY,QZ     
      DATA CAUPD/173.145D0/     
      DATA B1950/2433282.423D0/ 
      DATA TWOPI/6.283185307179586D0/   
C       
      IF(NTRIG.EQ.1) GO TO 50   
      DO 5 I=1,4
    5 PUT(I)=0.D0       
       
c --  compute heliocentric comet coordinates.     
      CALL HELXYZ(ETOBS,TT,Q,ECR,PX,PY,PZ,QX,QY,QZ,RCH(1),RCH(2),RCH(3))
       
c --  compute geocentric solar coordinates, and precess them to the       
c --  mean equator and equinox of 1950.0. 
      CALL SUN(ETOBS,EL,C,RSG(1),RSG(2),RSG(3)) 
      CALL PRECESC(B1950,ETOBS,.FALSE.,RSG)      
       
c   start light time iteration. 
   11 DO 12 I=1,3       
   12 RCG(I)=RCH(I)+RSG(I)      
      SR=DSQRT(RCG(1)**2+RCG(2)**2+RCG(3)**2)   
      DT=SR/CAUPD       
      IF(ABS(DT-DTZ).LT.1.D-7) GO TO 15 
      ETINC=ETINC+DTZ-DT
      ETT2=ETOBS+ETINC  
      DTZ=DT    
      CALL HELXYZ(ETT2,TT,Q,ECR,PX,PY,PZ,QX,QY,QZ,RCH(1),RCH(2),RCH(3)) 
      GO TO 11  
       
   15 CONTINUE  
       
c --  apply eterms.       
      DEC=DASIN(RCG(3)/SR)      
      RA=DATAN2(RCG(2),RCG(1))  
      IF(RA.LT.0.D0) RA=RA+TWOPI
      CALL ETERMSC(RA,DEC)       
      FACT=SR*DCOS(DEC) 
      RCG(1)=FACT*DCOS(RA)      
      RCG(2)=FACT*DSIN(RA)      
      RCG(3)=SR*DSIN(DEC)       
      GO TO 80  
       
c --  precess the comet geocentric coordinates to the mean equator and    
c --  equinox of date, and apply the circular portion of the annual part  
c --  of the stellar aberration.  
   50 CALL PRECESC(B1950,ETOBS,.TRUE.,RCG)       
      CALL ANABR3(ETOBS,RCG)    
       
c --  nutate the coordinates to the apparent equator and equinox of date. 
      CALL NUTANG(ETOBS,DPSI,DEPS)      
      PUT(1)=DPSI       
      PUT(2)=DEPS       
      CALL CNUTATE(ETOBS,PUT,OBM,.TRUE.,RCG)     
   80 CONTINUE  
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE APPAD2(NTRIG,UTOBS,RU,ECLOBL,PUT,RSG,RCH,RCT,ROG,PSANG)
       
c   this subroutine computes the topocentric comet coordinates. 
       
c   if ntrig=0, the light time correction and the elliptic portion      
c   of the annual aberration are applied to the comet coordinates.      
c   if ntrig=1, precession, nutation, the circular portion of the       
c   annual aberration, and the diurnal aberration are also applied.     
c   the position angle of the extended radius vector is also computed.  
       
c   input -     
c      ntrig  - 0 or 1. 
c      utobs  - universal time of observation, in j.d.  
c   output-     
c       ru     - local sidereal time, in radians.       
c       eclobl - obliquity of the ecliptic, in radians. 
c       put    - array containing delta psi and delta epsilon, used in  
c the nutation subroutines, in radians. 
c       rsg    - geocentric solar coordinates, wrt the mean equator     
c and equinox of 1950.0, in au's.       
c       rch    - heliocentric comet coordinates, wrt the mean equator   
c and equinox of 1950.0, in au's.       
c       rct    - topocentric comet coordinates, in au's.
c if ntrig=0, wrt mean equator and equinox of 1950.0.   
c if ntrig=1, wrt apparent equator and equinox of date. 
c       rog    - geocentric observatory coordinates, wrt the mean       
c equator and equinox of 1950.0, in au's.       
       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION EL(21),C(16),RSG(6),RCH(3),RCT(6),ROG(6)
      DIMENSION RCTTMP(6),RCHTMP(3),RCTAIL(6)   
      DIMENSION PUT(4),OBM(2)   
      DIMENSION STACOR(3)       
      LOGICAL FIRST     
      COMMON/ELM/TT,Q,ECR,PX,PY,PZ,QX,QY,QZ     
      COMMON/TOPO/IOPT,iberk,AU,REM,STACOR,ETMUTC     
      DATA TWOPI/6.283185307179586D0/   
      DATA CAUPD/173.145D0/     
      DATA DJ1900/2415020.0D0/  
      DATA B1950/2433282.423D0/ 
      DATA DPR/57.2957795130823252D0/   
      DATA RPTS/7.27220521664303933D-5/ 
       
      FIRST=.TRUE.      
      ETOBS=ETMUTC+UTOBS
      AUPER=REM/AU      
      IF(NTRIG.EQ.1) GO TO 50   
      ETINC=0.D0
      DTZ=0.D0  
      DO 5 I=1,4
    5 PUT(I)=0.D0       
       
c --  compute heliocentric comet coordinates.     
      CALL HELXYZ(ETOBS,TT,Q,ECR,PX,PY,PZ,QX,QY,QZ,RCH(1),RCH(2),RCH(3))
       
c --  compute geocentric solar coordinates and preces them to mean
c --  equator and equinox of 1950.0.      
      CALL SUN(ETOBS,EL,C,RSG(1),RSG(2),RSG(3)) 
      CALL PRECESC(B1950,ETOBS,.FALSE.,RSG)      
       
c --  convert observatory coordinates from cylindrical to rectangular,    
c --  and rotate them to mean equator and equinox of 1950.0.      
      ROG(1)=AUPER*STACOR(2)    
      ROG(2)=0.D0       
      ROG(3)=AUPER*STACOR(3)    
      TU=(UTOBS-DJ1900)/36525.D0
      RU=23925.836D0+(8640184.54D0+0.0929D0*TU)*TU      
      J=UTOBS   
      DJ5=J     
      DJ5=DJ5-0.5D0     
      TSEC=(UTOBS-DJ5)*TWOPI    
      IF(TSEC.GE.TWOPI) TSEC=TSEC-TWOPI 
      CALL NUTANG(ETOBS,DPSI,DEPS)      
      PUT(1)=DPSI       
      PUT(2)=DEPS       
      TE=(ETOBS-DJ1900)/36525.D0
      ECLOBL=(23.45229444D0-(1.30125D-2+(1.64D-6-5.03D-7*TE)*TE)*TE)/DPR
      RU=DMOD((RU*RPTS+DPSI*DCOS(ECLOBL)+TSEC+STACOR(1)),TWOPI) 
      CALL ROTAT6(ROG,RU,-3,ROG)
      CALL CNUTATE(ETOBS,PUT,OBM,.FALSE.,ROG)    
      CALL PRECESC(B1950,ETOBS,.FALSE.,ROG)      
       
c --  compute topocentric comet coordinates and perform the light time    
c --  iteration.  
   11 DO 12 I=1,3       
   12 RCT(I)=RCH(I)+RSG(I)-ROG(I)       
      SR=DSQRT(RCT(1)*RCT(1)+RCT(2)*RCT(2)+RCT(3)*RCT(3))       
      DT=SR/CAUPD       
      IF(ABS(DT-DTZ).LT.1.D-7) GO TO 15 
      ETINC=ETINC+DTZ-DT
      ETT2=ETOBS+ETINC  
      DTZ=DT    
      CALL HELXYZ(ETT2,TT,Q,ECR,PX,PY,PZ,QX,QY,QZ,RCH(1),RCH(2),RCH(3)) 
      GO TO 11  
       
c --  here after convergence      
   15 CONTINUE  
       
c --  apply eterm corrections.    
      DEC=DASIN(RCT(3)/SR)      
      RA=DATAN2(RCT(2),RCT(1))  
      IF(RA.LT.0.D0) RA=RA+TWOPI
      CALL ETERMSC(RA,DEC)       
      FACT=SR*DCOS(DEC) 
      RCT(1)=FACT*DCOS(RA)      
      RCT(2)=FACT*DSIN(RA)      
      RCT(3)=SR*DSIN(DEC)       
       
      IF(IOPT.EQ.2) GO TO 80    
      IF(NTRIG.EQ.0) GO TO 80   
       
c --  precess topocentric comet coordinates to mean equator and equinox   
c --  of date, and apply circular portion of annual aberration.   
   50 CALL PRECESC(B1950,ETOBS,.TRUE.,RCT)       
      CALL ANABR3(ETOBS,RCT)    
       
c  --  nutate coordinates to apparent equator and equinox of date, and apply       
c  --  diurnal portion of stellar aberration.      
      CALL CNUTATE(ETOBS,PUT,OBM,.TRUE.,RCT)     
      SR=DSQRT(RCT(1)*RCT(1)+RCT(2)*RCT(2)+RCT(3)*RCT(3))       
      DEC=DASIN(RCT(3)/SR)      
      RA=DATAN2(RCT(2),RCT(1))  
      IF(RA.LT.0.D0) RA=RA+TWOPI
      CALL DIABER(STACOR,RU,RA,DEC)     
      FACT=SR*DCOS(DEC) 
      RCT(1)=FACT*DCOS(RA)      
      RCT(2)=FACT*DSIN(RA)      
      RCT(3)=SR*DSIN(DEC)       
       
   80 IF(IOPT.EQ.2) GO TO 99    
      IF(NTRIG.EQ.0) GO TO 99   
      IF(.NOT.FIRST) GO TO 90   
       
c  --   compute position angle of comet tail.       
      FIRST=.FALSE.     
      DO 82 I=1,6       
   82 RCTTMP(I)=RCT(I)  
      DO 84 I=1,3       
      RCHTMP(I)=RCH(I)  
      RCH(I)=1.1D0*RCH(I)       
   84 RCT(I)=RCH(I)+RSG(I)-ROG(I)       
      SR=DSQRT(RCT(1)*RCT(1)+RCT(2)*RCT(2)+RCT(3)*RCT(3))       
      GO TO 15  
   90 CONTINUE  
      DO 92 I=1,3       
   92 RCH(I)=RCHTMP(I)  
      DO 94 I=1,6       
      RCTAIL(I)=RCT(I)  
   94 RCT(I)=RCTTMP(I)  
      RHOT=DSQRT(RCTAIL(1)**2+RCTAIL(2)**2+RCTAIL(3)**2)
      DECT=DASIN(RCTAIL(3)/RHOT)
      RAT=DATAN2(RCTAIL(2),RCTAIL(1))   
      IF(RAT.LT.0.D0) RAT=RAT+TWOPI     
      SR=DSQRT(RCT(1)*RCT(1)+RCT(2)*RCT(2)+RCT(3)*RCT(3))       
      DEC=DASIN(RCT(3)/SR)      
      RA=DATAN2(RCT(2),RCT(1))  
      IF(RA.LT.0.D0) RA=RA+TWOPI
      DRAT=RAT-RA       
      DDECT=DECT-DEC    
      SINP=DSIN(DRAT)   
      COSP=DTAN(DECT)*DCOS(DEC)-DSIN(DEC)*DCOS(DRAT)    
      PSANG=DATAN2(SINP,COSP)   
      IF(PSANG.LT.0.D0) PSANG=PSANG+TWOPI       
       
   99 CONTINUE  
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE DIABER(STACOR,RU,RA,DEC)       
       
c   this subroutine applies the diurnal portion of the stellar  
c   aberration to the right ascension and declination.  
c   reference - p.49 of the explanatory supplement to the aena. 
       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION STACOR(3)       
      DATA TWOPI/6.283185307179586D0/   
      HAL=RU-RA 
      IF(HAL.LT.0.D0) HAL=HAL+TWOPI     
      DRA=1.5490D-6*STACOR(2)*DCOS(HAL)/DCOS(DEC)       
      DDEC=1.5514D-6*STACOR(2)*DSIN(HAL)*DSIN(DEC)      
      RA=RA+DRA 
      DEC=DEC+DDEC      
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE ETERMSC(RA,DEC) 
       
c   this subroutine applies the elliptic portion (e-terms) of the annual
c   part of stellar aberration to the right ascension and declination.  
c   reference - p.144 of the explanatory supplement to the aena.
       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DRA=-1.653215D-6*DSIN(2.945243113D0+RA)/DCOS(DEC) 
      DDEC=-1.653215D-6*DCOS(2.945243113D0+RA)*DSIN(DEC)
     *-1.40596D-7*DCOS(DEC)     
      RA=RA+DRA 
      DEC=DEC+DDEC      
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE HELXYZ(TOBS,TT,Q,ECR,PX,PY,PZ,QX,QY,QZ,XG,YG,ZG)       
       
c   this subroutine computes heliocentric equatorial rectangular coord- 
c   inates for the equator and equinox of 1950.0 from cometary two-body 
c   elements and the p and q rotation factors.  
       
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DATA XK/0.01720209895D0/  
      EXCR=ECR-1.D0     
      IF(EXCR) 85,950,951       
  950 CONTINUE  
       
c   parabolic case      
      R2=2.0**(1.0/2.0) 
      Q32=Q**(3.0/2.0)  
      XN=XK*(TOBS-TT)/(R2*Q32)  
      KOUT=1    
      SIG=1.0   
   70 SIG2=SIG**2       
      SIG3=SIG*SIG2     
      SIGMA=(XN+(2.0/3.0)*SIG3)/(1.0+SIG2)      
      DEL=DABS(SIGMA)-DABS(SIG) 
      KOUT=KOUT+1       
      IF(KOUT.GT.10) GO TO 80   
      SIG=SIGMA 
      GO TO 70  
   80 CONTINUE  
      S=SIGMA   
      S2=SIGMA**2       
      XG=Q*PX*(1.0-S2)+2.0*Q*QX*S       
      YG=Q*PY*(1.0-S2)+2.0*Q*QY*S       
      ZG=Q*PZ*(1.0-S2)+2.0*Q*QZ*S       
      RETURN    
  951 CONTINUE  
       
c   hyperbolic case     
      AN1=Q/(1.D0-ECR)  
      AN2=-AN1  
      AN2=AN2**(3.D0/2.D0)      
      AN3=XK/AN2
      ANON=AN3*(TOBS-TT)
      FHV=ANON  
  953 FHVN=FHV-(ECR*DSINH(FHV)-FHV-ANON)/(ECR*DCOSH(FHV)-1.D0)  
      TST=FHVN-FHV      
      IF(DABS(TST).LT.1.D-13) GO TO 952 
      FHV=FHVN  
      GO TO 953 
  952 CONTINUE  
      XOB=AN1*(DCOSH(FHV)-ECR)  
      AN4=ECR**2-1.D0   
      YOB=-AN1*DSQRT(AN4)*DSINH(FHV)    
      XG=XOB*PX+YOB*QX  
      YG=XOB*PY+YOB*QY  
      ZG=XOB*PZ+YOB*QZ  
      RETURN    
       
c   elliptic case       
   85 A=Q/(1.0-ECR)     
      ART=DSQRT(A)      
      AR=XK*(TOBS-TT)/(ART**3)  
      I=1       
      EE1=AR    
      EE=EE1    
   86 DE=EE1-EE+ECR*DSIN(EE)    
      IF(I.GE. 10) GO TO 87     
      I=I+1     
      EE=EE+DE/(1.0D0-ECR*DCOS(EE +0.5D0*DE))   
      GO TO 86  
   87 CONTINUE  
      B=A*DSQRT(1.0D0-ECR**2)   
      FAT1=DCOS(EE)-ECR 
      FAT2=DSIN(EE)     
      XG=A*PX*FAT1+B*QX*FAT2    
      YG=A*PY*FAT1+B*QY*FAT2    
      ZG=A*PZ*FAT1+B*QZ*FAT2    
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE NUTANG(TJD,DPSI,DEPS)  
       
c   this subroutine computes nutation angles.   
       
c   input -     
c      tjd - ephemeris time of observation (jed).       
c       
c   output -    
c      dpsi - delta psi (radians).      
c      deps - delta epsilon (radians).  
c       
c   ref. - j. meeus, astronomical formulae for calculators, monografieen
c   over astronomie en astrofysica, uitgegeven door volkssterrenwacht   
c   urania v.z.w. en vereniging voor sterrenkunde v.z.w. vol. 4, 1979.  
       
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DATA ASPR/206264.806247096355D0/  
      DATA DPR/57.2957795130823252D0/   
      TJC=(TJD-2415020.0D0)/36525.D0    
      CMLSUN=(279.6967D0+(36000.7689D0+3.03D-4*TJC)*TJC)/DPR    
      CMLMN=(270.4342D0+(481267.8831D0-1.133D-3*TJC)*TJC)/DPR   
      CMASUN=(358.4758D0+(35999.0498D0-1.5D-4*TJC)*TJC)/DPR     
      CMAMN=(296.1046D0+(477198.8491D0+9.192D-3*TJC)*TJC)/DPR   
      CLANMN=(259.1833D0-(1934.1420D0-2.078D-3*TJC)*TJC)/DPR    
      DPSI=-(17.2327D0+1.737D-2*TJC)*DSIN(CLANMN)       
     *-(1.2729D0+1.3D-4*TJC)*DSIN(2.D0*CMLSUN)  
     *+2.088D-1*DSIN(2.D0*CLANMN)-2.037D-1*DSIN(2.D0*CMLMN)     
     *+(1.261D-1-3.1D-4*TJC)*DSIN(CMASUN)+6.75D-2*DSIN(CMAMN)   
     *-(4.97D-2-1.2D-4*TJC)*DSIN(2.D0*CMLSUN+CMASUN)    
     *-3.42D-2*DSIN(2.D0*CMLMN-CLANMN)-2.61D-2*DSIN(2.D0*CMLMN+CMAMN)   
     *+2.14D-2*DSIN(2.D0*CMLSUN-CMASUN) 
     *-1.49D-2*DSIN(2.D0*CMLSUN-2.D0*CMLMN+CMAMN)       
     *+1.24D-2*DSIN(2.D0*CMLSUN-CLANMN)+1.14D-2*DSIN(2.D0*CMLMN-CMAMN)  
      DEPS=(9.21D0+9.1D-4*TJC)*DCOS(CLANMN)     
     *+(5.522D-1-2.9D-4*TJC)*DCOS(2.D0*CMLSUN)-9.04D-2*DCOS(2.D0*CLANMN)
     *+8.84D-2*DCOS(2.D0*CMLMN)+2.16D-2*DCOS(2.D0*CMLSUN+CMASUN)
     *+1.83D-2*DCOS(2.D0*CMLMN-CLANMN)+1.13D-2*DCOS(2.D0*CMLMN+CMAMN)   
     *-9.3D-3*DCOS(2.D0*CMLSUN-CMASUN)-6.6D-3*DCOS(2.D0*CMLSUN-CLANMN)  
      DPSI=DPSI/ASPR    
      DEPS=DEPS/ASPR    
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE CNUTATE(DJ,PUT,OBM,FRWD,X)      
       
c     frwd = true implies nutation from 1950,  = false implies to 1950  
       
c     ref. - melbourne, w.g., et.al., constants and related information 
c     for astrodynamic calculations, 1968, jpl tr 32-1306, 1968, p. 9.  
       
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION OBM(2),OBT(2),DPSI(2),X(6),PUT(4)       
      LOGICAL FRWD      
      DATA SPR/206264.8062D0/   
      T=(DJ-2433282.5D0)/36525.D0       
C----  CONSTANTS DOCUMENT   TR32-1306  1968  PG. 9      
      OBM(1)=(84404.84D0+T*(-46.850D0+T*(-0.0034D0+T*0.0018D0)))/SPR    
      OBM(2)=(-46.850D0-0.0068D0*T+0.0054D0*T*T)/(SPR*36525.D0) 
      DPSI(1)=PUT(1)    
      DPSI(2)=PUT(3)    
      OBT(1)=OBM(1)+PUT(2)      
      OBT(2)=OBM(2)+PUT(4)      
      IF(.NOT. FRWD) GO TO 1    
      CALL ROTAT6(X,OBM,+1,X)   
      CALL ROTAT6(X,DPSI,-3,X)  
      CALL ROTAT6(X,OBT,-1,X)   
      RETURN    
   1  CALL ROTAT6(X,OBT,+1,X)   
      CALL ROTAT6(X,DPSI,+3,X)  
      CALL ROTAT6(X,OBM,-1,X)   
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE PRECESC(TZ,TF,FRWD,X)   
c ----  reference  -  lieske  jpl tech report  tr32-1044
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION ZE(2),TH(2),Z(2),X(3)   
      DOUBLE PRECISION JD1900   
      LOGICAL FRWD      
      DATA JD1900/2415020.313516463D0/  
      DATA TCENT/36524.219879D0/
      DATA SPR/206264.8062D0/   
      SCALE=SPR*TCENT   
      IF(TZ .EQ. TZZ) GO TO 2   
      T1=(TZ-JD1900)/TCENT      
      TZZ=TZ    
      P1=2304.253D0+T1*(1.3972D0+T1*0.125D-3)   
      P2=0.3023D0-0.211D-3*T1   
      P3=0.0180D0       
      Q1=P1     
      Q2=1.0949D0+0.460D-3*T1   
      Q3=0.0183D0       
      R1=2004.684D0+T1*(-.8532D0-T1*0.317D-3)   
      R2=-0.4266D0-0.320D-3*T1  
      R3=-0.0418D0      
   2  T=(TF-TZ)/TCENT   
      ZE(1)=T*(P1+T*(P2+T*P3))/SPR      
      ZE(2)=(P1+T*(2.D0*P2+T*3.D0*P3))/SCALE    
      Z (1)=T*(Q1+T*(Q2+T*Q3))/SPR      
      Z (2)=(Q1+T*(2.D0*Q2+T*3.D0*Q3))/SCALE    
      TH(1)=T*(R1+T*(R2+T*R3))/SPR      
      TH(2)=(R1+T*(2.D0*R2+T*3.D0*R3))/SCALE    
      IF(FRWD) GO TO 1  
c ----  here for precessing from tf to tz       
      CALL ROTAT6(X,Z ,+3,X)    
      CALL ROTAT6(X,TH,-2,X)    
      CALL ROTAT6(X,ZE,+3,X)    
      RETURN    
c ----  here for precessing from tz to tf       
   1  CALL ROTAT6(X,ZE,-3,X)    
      CALL ROTAT6(X,TH,+2,X)    
      CALL ROTAT6(X,Z ,-3,X)    
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE ROTAT6(X,A,L,Y)
      DOUBLE PRECISION X(6),A(2),Y(6),C,S,T,SIGN
      K=IABS(L) 
      I=MOD(K,3)+1      
      J=6-I-K   
      SIGN=1.D0 
      IF(L .LT. 0) SIGN=-1.D0   
      C=DCOS(A(1))      
      S=DSIN(A(1)*SIGN) 
      T=X(I+3)*C+X(J+3)*S+SIGN*A(2)*(X(J)*C-X(I)*S)     
      Y(J+3)=X(J+3)*C-X(I+3)*S-SIGN*A(2)*(X(I)*C+X(J)*S)
      Y(I+3)=T  
      Y(K+3)=X(K+3)     
      T=X(I)*C+X(J)*S   
      Y(J)=X(J)*C-X(I)*S
      Y(I)=T    
      Y(K)=X(K) 
      RETURN    
      END       
C*******************************************************************************
      SUBROUTINE SUN (DJ,EL,C,XSUN,YSUN,ZSUN)   
       
c     newcomb's theory of the earth     
       
C     from the 'tables of the sun' (a.p.a.e. vi, part 1), newcomb's 
c     theory of the motion of the object is evaluated, giving geometric 
c     coordinates referred to the mean equinox and equator of date as a 
c     function of ephemeris time expressed in julian centuries of 36525 
c     days.     
       
c     this program originated at the u.s. naval observatory     
C*******************************************************************************
       
c     definitions       
c       
c     dj    = input argument time in julian ephemeris days      
c       
c     el( 2)= orbital eccentricity      
c     el( 5)= longitude of perigee      
c     el( 8)= mean sidereal motion     not given
c     el( 9)= unperturbed mean longitude, radians       
c     el(10)= mean anomaly, affected by long period perturbations,      
c     radians   
c     el(11)= unperturbed radius, astronomical units    
c     el(12)= equation of the center, radians   
c     el(14)= mean longitude of moon    
c     el(15)= mean anomaly of moon      
c     el(16)= lunar mean argument of latitude   
c     el(17)= mean longitude of lunar ascending node    
c     el(21)= mean longitude of moon's perigee  
c     remaining elements of el array not used   
c       
c     c( 1) = perturbed radius vector, astronomical units       
c     c( 2) = true longitude, degrees   
c     c( 3) = latitude, degrees 
c     c( 4) = ecliptic longitude mean equinox of date, radians  
c     c( 5) = ecliptic latitude mean equinox and equator of date, radians       
c     remaining elements of c array not used    
c     el( 1)= semi-major axis   
C*******************************************************************************
      DOUBLE PRECISION DJ,T,TP,RO,GME,GV,GMA,GJ,GS,DL,DR,DBLARG,DB,DG,  
     1D,T18,EL(21),C(16),TWOPI,STR,RTD  
      DOUBLE PRECISION S,EA,EB,EQ,ED,EPS,XSUNE,YSUNE,ZSUNE,     
     *XSUN,YSUN,ZSUN,FOUR,CONV11,CONV12 
      DIMENSION X(8,121),X1(160),X2(160),X3(160),X4(160),X5(160),X6(8)  
      DIMENSION X0(160) 
      EQUIVALENCE(X(1,  1),X0(1))       
      EQUIVALENCE(X(1, 21),X1(1))       
      EQUIVALENCE(X(1, 41),X2(1))       
      EQUIVALENCE(X(1, 61),X3(1))       
      EQUIVALENCE(X(1, 81),X4(1))       
      EQUIVALENCE(X(1,101),X5(1))       
      EQUIVALENCE(X(1,121),X6(1))       
      DATA TWOPI/6.283185307179586D0/   
      DATA STR/206264806.2470964D0/     
      DATA RTD/57.295779513082321D0/    
       
c     part i    tables of the perturbations     
c       
c     perturbations by mercury  
       
C  J    I     VC      VS    RHOC    RHOS      BC     BS 
      DATAX0/    - 1.,+ 1., -   6.,-  11., +  26.,-  12.,     0.,    0.,
     1   - 1.,+ 2., -   3.,-   3., -   4.,+   5.,     0.,    0.,
     2   - 1.,+ 3., +  15.,-   1., -   1.,-  18.,     0.,    0.,
     3   - 1.,+ 4., +  19.,-  13., -   2.,-   4.,     0.,    0.,
       
c     perturbations by venus    
c       
C  J    I     VC      VS    RHOC    RHOS      BC     BS 
     4   - 1.,  0., +  33.,-  67., -  85.,-  39., +  24.,-  17.,
     5   - 1.,+ 1., +2353.,-4228., -2062.,-1146., -   4.,+   3.,
     6   - 1.,+ 2., -  65.,-  34., +  68.,-  14., +   6.,-  92.,
     7   - 1.,+ 3., -   3.,-   8., +  14.,-   8., +   1.,+   7.,
     8   - 2.,  0.,     0.,    0.,     0.,+   4.,     0.,    0.,
     9   - 2.,+ 1., -  99.,+  60., +  84.,+ 136., +  23.,-   3.,
     1   - 2.,+ 2., -4702.,+2903., +3593.,+5822., +  10.,-   6.,
     2   - 2.,+ 3., +1795.,-1737., - 596.,- 632., +  37.,-  56.,
     3   - 2.,+ 4., +  30.,-  33., +  40.,+  33., +   5.,-  13.,
     4   - 3.,+ 2., -  13.,    0.,     0.,+  21., +  13.,+   5.,
     5   - 3.,+ 3., - 666.,+  27., +  44.,+1044., +   8.,+   1.,
     6   - 3.,+ 4., +1508.,- 397., - 381.,-1448., + 185.,- 100.,
     7   - 3.,+ 5., + 763.,- 684., + 126.,+ 148., +   6.,-   3.,
     8   - 3.,+ 6., +  12.,-  12., +  14.,+  13., -   2.,+   4.,
     9   - 4.,+ 3.,     0.,    0.,     0.,+   6., +   4.,+   5.,
     1   - 4.,+ 4., - 188.,-  93., - 166.,+ 337.,     0.,    0./
      DATAX1/    - 4.,+ 5., - 139.,-  38., -  51.,+ 189., -  31.,-   1.,
     1   - 4.,+ 6., + 146.,-  42., -  25.,-  91., +  12.,    0.,
     2   - 4.,+ 7., +   5.,-   4., +   3.,+   5.,     0.,    0.,
     3   - 5.,+ 5., -  47.,-  69., - 134.,+  93.,     0.,    0.,
     4   - 5.,+ 6., -  28.,-  25., -  39.,+  43., -   8.,-   4.,
     5   - 5.,+ 7., - 119.,-  33., -  37.,+ 136., -  18.,-   6.,
     6   - 5.,+ 8., + 154.,    0.,     0.,-  26.,     0.,    0.,
     7   - 6.,+ 5.,     0.,    0.,     0.,    0., -   2.,+   6.,
     8   - 6.,+ 6., -   4.,-  38., -  80.,+   8.,     0.,    0.,
     9   - 6.,+ 7., -   4.,-  13., -  24.,+   7., -   2.,-   3.,
     1   - 6.,+ 8., -   6.,-   7., -  10.,+  10., -   2.,-   3.,
     2   - 6.,+ 9., +  14.,+   3., +   3.,-  12.,     0.,    0.,
     3   - 7.,+ 7., +   8.,-  18., -  38.,-  17.,     0.,    0.,
     4   - 7.,+ 8., +   1.,-   6., -  12.,-   3.,     0.,    0.,
     5   - 7.,+ 9., +   1.,-   3., -   4.,+   1.,     0.,    0.,
     6   - 7.,+10., -   2.,-   2., -   3.,+   3.,     0.,    0.,
     7   - 8.,+ 8., +   9.,-   7., -  14.,-  19.,     0.,    0.,
     8   - 8.,+ 9., +   2.,-   3., -   5.,-   4.,     0.,    0.,
     9   - 8.,+12., -  33.,-  54., -  43.,+   8., -   5.,-   9.,
     1   - 8.,+13.,     0.,    0., -   9.,-   8.,     0.,    0./
      DATAX2/    - 8.,+14.,     0.,    0., -  25.,+  22.,     0.,    0.,
     1   - 9.,+ 9., +   6.,-   1., -   2.,-  13.,     0.,    0.,
     2   - 9.,+10.,     0.,    0., -   1.,-   4.,     0.,    0.,
     3   -10.,+10.,     0.,    0., +   3.,-   7.,     0.,    0.,
       
c     perturbations by mars     
c       
C  J    I     VC      VS    RHOC    RHOS      BC     BS 
     4   + 1.,- 2., -   5.,-   4., -   5.,+   6.,     0.,    0.,
     5   + 1.,- 1., - 216.,- 167., -  92.,+ 119.,     0.,    0.,
     6   + 1.,  0., -   8.,-  47., +  27.,-   6.,     0.,    0.,
     7   + 2.,- 3., +  40.,-  10., -  13.,-  50.,     0.,    0.,
     8   + 2.,- 2., +1963.,- 567., - 573.,-1976.,     0.,-   8.,
     9   + 2.,- 1., -1659.,- 617., +  64.,- 137.,     0.,    0.,
     1   + 2.,  0., -  24.,+  15., -  18.,-  25., -   8.,+   2.,
     2   + 3.,- 4., +   1.,-   4., -   6.,    0.,     0.,    0.,
     3   + 3.,- 3., +  53.,- 118., - 154.,-  67.,     0.,    0.,
     4   + 3.,- 2., + 396.,- 153., -  77.,- 201.,     0.,    0.,
     5   + 3.,- 1., +   8.,+   1.,     0.,+   6.,     0.,    0.,
     6   + 4.,- 4., +  11.,+  32., +  46.,-  17.,     0.,    0.,
     7   + 4.,- 3., - 131.,+ 483., + 461.,+ 125., +   7.,+   1.,
     8   + 4.,- 2., + 526.,- 256., +  43.,+  96.,     0.,    0.,
     9   + 4.,- 1., +   7.,-   5., +   6.,+   8.,     0.,    0.,
     1   + 5.,- 5., -   7.,+   1.,     0.,+  12.,     0.,    0./
      DATAX3/    + 5.,- 4., +  49.,+  69., +  87.,-  62.,     0.,    0.,
     1   + 5.,- 3., -  38.,+ 200., +  87.,+  17.,     0.,    0.,
     2   + 5.,- 2., +   3.,+   1., -   1.,+   3.,     0.,    0.,
     3   + 6.,- 6.,     0.,    0., -   4.,-   3.,     0.,    0.,
     4   + 6.,- 5., -  20.,-   2., -   3.,+  30.,     0.,    0.,
     5   + 6.,- 4., - 104.,- 113., - 102.,+  94.,     0.,    0.,
     6   + 6.,- 3., -  11.,+ 100., -  27.,-   4.,     0.,    0.,
     7   + 7.,- 6., +   3.,-   5., -   9.,-   5.,     0.,    0.,
     8   + 7.,- 5., -  49.,+   3., +   4.,+  60.,     0.,    0.,
     9   + 7.,- 4., -  78.,-  72., -  26.,+  28.,     0.,    0.,
     1   + 8.,- 7., +   1.,+   3., +   5.,-   1.,     0.,    0.,
     2   + 8.,- 6., +   6.,-   8., -  12.,-   9.,     0.,    0.,
     3   + 8.,- 5., +  51.,-  10., -   8.,-  44.,     0.,    0.,
     4   + 8.,- 4., -  17.,-  12., +   5.,-   6.,     0.,    0.,
     5   + 9.,- 7., +   2.,+   3., +   5.,-   3.,     0.,    0.,
     6   + 9.,- 6., +  13.,-  25., -  30.,-  16.,     0.,    0.,
     7   + 9.,- 5., +  60.,-  15., -   4.,-  17.,     0.,    0.,
     8   +10.,- 7., +   2.,+   5., +   7.,-   3.,     0.,    0.,
     9   +10.,- 6., -   7.,+  18., +  14.,+   6.,     0.,    0.,
     1   +10.,- 5., +   5.,-   2.,     0.,    0.,     0.,    0./
      DATAX4/    +11.,- 7., +   9.,+  15., +  17.,-  10.,     0.,    0.,
     1   +11.,- 6., -  12.,+  42., +   8.,+   3.,     0.,    0.,
     2   +12.,- 7., -   4.,-   5., -   4.,+   3.,     0.,    0.,
     3   +13.,- 8., -  13.,-   1., -   1.,+  15.,     0.,    0.,
     4   +13.,- 7., -  30.,-  33., -   4.,+   3.,     0.,    0.,
     5   +15.,- 9., +  13.,-  16., -  17.,-  14.,     0.,    0.,
     6   +15.,- 8., + 200.,-  30., -   1.,-   6.,     0.,    0.,
     7   +17.,-10., -   2.,-   4., -   4.,+   2.,     0.,    0.,
     8   +17.,- 9., -  10.,+  24.,     0.,    0.,     0.,    0.,
       
c     perturbations by jupiter  
c       
C  J    I     VC      VS    RHOC    RHOS      BC     BS 
     9   + 1.,- 3., -   3.,-   1., -   2.,+   5.,     0.,    0.,
     1   + 1.,- 2., - 155.,-  52., -  78.,+ 193., +   7.,    0.,
     2   + 1.,- 1., -7208.,+  59., +  56.,+7067., -   1.,+  17.,
     3   + 1.,  0., - 307.,-2582., + 227.,-  89., +  16.,    0.,
     4   + 1.,+ 1., +   8.,-  73., +  79.,+   9., +   1.,+  23.,
     5   + 2.,- 3., +  11.,+  68., + 102.,-  17.,     0.,    0.,
     6   + 2.,- 2., + 136.,+2728., +4021.,- 203.,     0.,    0.,
     7   + 2.,- 1., - 537.,+1518., +1376.,+ 486., +  13.,+ 166.,
     8   + 2.,  0., -  22.,-  70., -   1.,-   8.,     0.,    0.,
     9   + 3.,- 4., -   5.,+   2., +   3.,+   8.,     0.,    0.,
     1   + 3.,- 3., - 162.,+  27., +  43.,+ 278.,     0.,    0./
      DATAX5/    + 3.,- 2., +  71.,+ 551., + 796.,- 104., +   6.,-   1.,
     1   + 3.,- 1., -  31.,+ 208., + 172.,+  26., +   1.,+  18.,
     2   + 4.,- 4., -   3.,-  16., -  29.,+   5.,     0.,    0.,
     3   + 4.,- 3., -  43.,+   9., +  13.,+  73.,     0.,    0.,
     4   + 4.,- 2., +  17.,+  78., + 110.,-  24.,     0.,    0.,
     5   + 4.,- 1., -   1.,+  23., +  17.,+   1.,     0.,    0.,
     6   + 5.,- 5.,     0.,    0., -   1.,-   3.,     0.,    0.,
     7   + 5.,- 4., -   1.,-   5., -  10.,+   2.,     0.,    0.,
     8   + 5.,- 3., -   7.,+   2., +   3.,+  12.,     0.,    0.,
     9   + 5.,- 2., +   3.,+   9., +  13.,-   4.,     0.,    0.,
       
c     perturbations by saturn   
c       
C  J    I     VC      VS    RHOC    RHOS      BC     BS 
     1   + 1.,- 2., -   3.,+  11., +  15.,+   3.,     0.,    0.,
     2   + 1.,- 1., -  77.,+ 412., + 422.,+  79., +   1.,+   6.,
     3   + 1.,  0., -   3.,- 320., +   8.,-   1.,     0.,    0.,
     4   + 1.,+ 1.,     0.,-   8., +   8.,    0., -   1.,+   6.,
     5   + 2.,- 3.,     0.,    0., -   3.,-   1.,     0.,    0.,
     6   + 2.,- 2., +  38.,- 101., - 152.,-  57.,     0.,    0.,
     7   + 2.,- 1., +  45.,- 103., - 103.,-  44.,     0.,    0.,
     8   + 2.,  0., +   2.,-  17.,     0.,    0.,     0.,    0.,
     9   + 3.,- 2., +   7.,-  20., -  30.,-  11.,     0.,    0.,
     1   + 3.,- 1., +   6.,-  16., -  16.,-   6.,     0.,    0./
      DATAX6/    + 4.,- 2., +   1.,-   3., -   4.,-   1.,     0.,    0./
       
c     part ii   reduction of argument time      
       
c     t  = time in julian centuries from 1900 january 0, greenwich mean noon    
      T  = (DJ - 2415020.D0)/36525.D0   
       
c     tp = time in julian years     from 1850 january 0, greenwich mean noon    
      TP = (DJ - 2396758.D0)/365.25D0   
       
c     t18= time in julian centuries from 1800 january 0, greenwich mean noon    
      T18= (DJ - 2378496.D0)/36525.D0   
       
c     part iii  computation of elliptic elements and secular terms      
       
      FOUR = 0.6283019457267408D+03 + 4.78D0/STR
      EL( 2) =  0.1675104D-01 - T*(0.4180D-04 + 0.126D-06*T)    
      EL( 5) =  0.4908229466868880D+01 + (0.3000526416797348D-01
     1+(0.7902463002085429D-05 +  0.5817764173314427D-07*T)*T)*T
      EL( 9) =  0.4881627934111871D+01 + (0.6283319509909086D+03
     1+ 0.5279620987282842D-05*T)*T     
      EL(10) =  0.6256583580497099D+01 + (  FOUR
     1-(0.2617993877991491D-05 +  0.5817764173314427D-07*T)*T)*T
       
c     reduction to the first revolution 
      EL( 9) = DMOD (EL( 9),TWOPI)      
      EL(10) = DMOD (EL(10),TWOPI)      
       
c     part iv   lunar terms     
       
c     fundamental arguments of hansen with newcomb corrections  
      EL(14) =  0.5859485105530519D+01 + (0.8399709200339593D+04
     1+(0.4619304753611657D-04 +  0.6544984694978728D-07*T18)   
     2*T18)*T18 
      EL(17) =  0.5807638839584459D+00 - (0.3375728238223387D+02
     1-(0.3964806284113784D-04 +  0.3470781143063166D-07*T18)   
     2*T18)*T18 
      EL(21) =  0.3933938487925745D+01 + (0.7101833330913285D+02
     1-(0.1752456013106637D-03 -  0.1773109075921903D-06*T18)   
     2*T18)*T18 
      EL(14) = DMOD (EL(14),TWOPI)      
      EL(17) = DMOD (EL(17),TWOPI)      
      EL(21) = DMOD (EL(21),TWOPI)      
       
c     derived arguments 
      EL(15) = EL(14) - EL(21)  
      EL(16) = EL(14) - EL(17)  
      EL(15) = DMOD (EL(15),TWOPI)      
      EL(16) = DMOD (EL(16),TWOPI)      
c     mean elongation   
      D      = EL(14) - EL(9)   
       
c     combinations of arguments and the perturbations   
      D = DMOD (D,TWOPI)
      ARG = D   
      DL =    + 6454.*SIN(ARG) + 13.*SIN(3.*ARG)
      DR =    + 13360.*COS(ARG) +  30.*COS(3.*ARG)      
       
      DBLARG = D + EL(15)       
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DL = DL +  177.*SIN(ARG)  
      DR = DR +  370.*COS(ARG)  
       
      DBLARG = D - EL(15)       
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      DL = DL -  424.*SIN(ARG)  
      DR = DR - 1330.*COS(ARG)  
       
      DBLARG = 3.D0*D - EL(15)  
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DL = DL +   39.*SIN(ARG)  
      DR = DR +   80.*COS(ARG)  
       
      DBLARG = D + EL(10)       
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DL = DL -   64.*SIN(ARG)  
      DR = DR -  140.*COS(ARG)  
       
      DBLARG = D - EL(10)       
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DL = DL +  172.*SIN(ARG)  
      DR = DR + 360.*COS(ARG)   
       
      EL(16) = DMOD (EL(16),TWOPI)      
      ARG = EL(16)      
      DB =    + 576.*SIN(ARG)   
       
      DBLARG = EL(16) + EL(15)  
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DB = DB +  16.*SIN(ARG)   
       
      DBLARG = EL(16) - EL(15)  
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DB = DB -  47.*SIN(ARG)   
       
      DBLARG = EL(16) - 2.D0*(EL(9) - EL(17))   
      DBLARG = DMOD (DBLARG,TWOPI)      
      ARG = DBLARG      
      DB = DB +  21.*SIN(ARG)   
       
c     part v    computation of periodic perturbations   
       
c     the perturbing mean anomalies     
C       
      GME = 0.43296383D+01 + .2608784648D+02*TP 
      GV  = 0.19984020D+01 + .1021322923D+02*TP 
      GMA = 0.19173489D+01 + .3340556174D+01*TP 
      GJ  = 0.25836283D+01 + .5296346478D+00*TP 
      GS  = 0.49692316D+01 + .2132432808D+00*TP 
       
c     reduction to first revolution     
      GME = DMOD (GME,TWOPI)    
      GV  = DMOD (GV ,TWOPI)    
      GMA = DMOD (GMA,TWOPI)    
      GJ  = DMOD (GJ ,TWOPI)    
      GS  = DMOD (GS ,TWOPI)    
       
c     modification of fundamental arguments     
       
c     application of the jupiter-saturn great inequality
c     to jupiter's mean anomaly 
       
      GJ = GJ + 0.579904067D-02 * DSIN(5.D0*GS - 2.D0*GJ + 1.1719644977 
     1     D0 - 0.397401726D-03 * TP)   
       
c     long period perturbations of mean anomaly 
       
c     time represented in single precision by 'st'      
      ST = T    
c     argument ( 4mars - 7earth + 3venus ) of second order as to masses 
      DG = 266.*SIN(.555015 + 2.076942*ST)      
       
c     argument ( 3jupiter - 8mars + 4earth ) of 2nd order as to masses  
     1    +6400.*SIN(4.035027 + .3525565*ST)    
       
c     argument ( 13earth - 8venus ) first order in masses, 5th in eccentricities
c  5th in inclinations  
     2    +(1882.-16.*T)*SIN(.9990265 + 2.622706*ST)    
       
      GJ = DMOD (GJ,TWOPI)      
       
c     computation of the equation of the center 
       
c     form perturbed mean anomaly       
      EL(10) = DG/STR + EL(10)  
      EL(10) = DMOD(EL(10),TWOPI)       
      EL(12) =   DSIN(     EL(10)) * (6910057. - (17240. + 52.*T)*T)    
     1 + DSIN(2.D0*EL(10)) * (  72338. -    361.*T)     
     2 + DSIN(3.D0*EL(10)) * (   1054. -      1.*T)     
     3 + DSIN(4.D0*EL(10)) *       18.  
       
c     the unperturbed radius vector     
      RO     =  30570. -    150.*T      
     1 - DCOS(     EL(10)) * (7274120. - (18140. + 50.*T)*T)    
     2 - DCOS(2.D0*EL(10)) * (  91380. -    460.*T)     
     3 - DCOS(3.D0*EL(10)) * (   1450. -     10.*T)     
     4 - DCOS(4.D0*EL(10)) *       20.  
      EL(11) = 10.D0**(RO*1.D-09)       
       
c     perturbations by mercury in longitude and radius vector   
      DO 10 K=1,4       
c     argument  j*mercury +   i * earth 
      DBLARG = X(1,K)*GME + X(2,K)*EL(10)       
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      CS  = COS(ARG)    
      SS  = SIN(ARG)    
      DL  =(X(3,K)*CS  + X(4,K)*SS )+ DL
      DR  =(X(5,K)*CS  + X(6,K)*SS )+ DR
   10 CONTINUE  
       
c     perturbations by venus    
      DO 20 K=5,44      
c     argument j * venus +   i * earth  
      DBLARG = X(1,K)*GV + X(2,K)*EL(10)
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      CS  = COS(ARG)    
      SS  = SIN(ARG)    
      DL  =(X(3,K)*CS  + X(4,K)*SS )+ DL
      DR  =(X(5,K)*CS  + X(6,K)*SS )+ DR
      DB  =(X(7,K)*CS  + X(8,K)*SS )+ DB
   20 CONTINUE  
       
c     perturbations by mars     
      DO 30 K=45,89     
c     argument  j * mars  +   i * earth 
      DBLARG = X(1,K)*GMA + X(2,K)*EL(10)       
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      CS  = COS(ARG)    
      SS  = SIN(ARG)    
      DL  =(X(3,K)*CS  + X(4,K)*SS )+ DL
      DR  =(X(5,K)*CS  + X(6,K)*SS )+ DR
      DB  =(X(7,K)*CS  + X(8,K)*SS )+ DB
   30 CONTINUE  
       
c     perturbations by jupiter  
      DO 40 K=90,110    
c     argument j*jupiter +   i * earth  
      DBLARG = X(1,K)*GJ + X(2,K)*EL(10)
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      CS  = COS(ARG)    
      SS  = SIN(ARG)    
      DL  =(X(3,K)*CS  + X(4,K)*SS )+ DL
      DR  =(X(5,K)*CS  + X(6,K)*SS )+ DR
      DB  =(X(7,K)*CS  + X(8,K)*SS) +DB 
   40 CONTINUE  
       
c     perturbations by saturn   
      DO 50 K=111,121   
c     argument j*saturn  +   i * earth  
      DBLARG = X(1,K)*GS + X(2,K)*EL(10)
      DBLARG = DMOD(DBLARG,TWOPI)       
      ARG = DBLARG      
      CS  = COS(ARG)    
      SS  = SIN(ARG)    
      DL  =(X(3,K)*CS  + X(4,K)*SS )+ DL
      DR  =(X(5,K)*CS  + X(6,K)*SS )+ DR
      DB  =(X(7,K)*CS  + X(8,K)*SS )+ DB
   50 CONTINUE  
       
c     part vi   computation of ecliptic coordinates     
       
      C(1) = EL(11)*10.D0**(DR*1.D-09)  
      C(4) = (DL + DG + EL(12))/STR + EL(9)     
      C(4) = DMOD(C(4),TWOPI)   
      C(5) = DB/STR     
      C(2) = C(4)*RTD   
      C(3) = C(5)*RTD   
       
c     the following computes the rectangular coordinates
c     referred to the mean equator and equinox of date. 
       
      CONV11=(TWOPI/360.D0)/3600.D0     
      CONV12=36524.219879D0-.00000614D0*T       
      S=(DJ-2415020.3135D0)/CONV12      
      EA=(23.D0*3600.D0+27.D0*60.D0+8.26D0)*CONV11      
      EB=(-46.845D0*S)*CONV11   
      EQ=(-.0059D0*S*S)*CONV11  
      ED=(.00181D0*S*S*S)*CONV11
      EPS=EA+EB+EQ+ED   
      XSUNE=C(1)*DCOS(C(4))*DCOS(C(5))  
      YSUNE=C(1)*DSIN(C(4))*DCOS(C(5))  
      ZSUNE=C(1)*DSIN(C(5))     
      XSUN=XSUNE
      YSUN=YSUNE*DCOS(EPS)-ZSUNE*DSIN(EPS)      
      ZSUN=YSUNE*DSIN(EPS)+ZSUNE*DCOS(EPS)      
      RETURN    
      END       
