c
c* EPHEMER
c: observing
c+
	SUBROUTINE EPHEMER(INSOURCE,CATALOG,TJD,FREQ,S,ERROR)
c
C  EPHEMER:	Ephemer is used by NAIF to convert from catalog positions or JPL
c		ephemerides (naif) to apparent positions.  Uses subroutines
c		from the naval observatory for stellar objects and uses the
c		JPL subroutines and tape files for planets and moons.
c		This version only provides the current position and first
c		derivative so it must be rerun each integration.
c  ARGUMENTS:
c  INSOURCE (char,input)		Source name
c  CATALOG (char,input)		Catalog name (none for planets)
c  TJD (dp,input)		Julian day.frac 
c  FREQ (dp,input)		Frequency used by planet calculation
c  S (RECORD,output)		RECORD containing all source parameters
c  ERROR (char,output)		Status return
c--
C	REWRITTEN nov 89  WH
c	addon dra,ddec  nov 91 wh
c	naif version apr 92 wh mod may 92
c	name change to ephemer to avoid name conflict
c	add comet code   sep 93 wh
C-------------------------------------------------------C
	include '../../inc/constants.inc'
	include '../../inc/ephem.inc'
	record /asource/ s
	logical planet_flag, is_planet
	real*8 tjd, freq
	character*100 catalog
	real*8 lst,gst,ha,cosha,sind,cosd,sinha
	character*80 error
	character*(*) insource
	character*10 source
	real*8 dra,ddec

C	--CONVERT SOURCE TO UPPER CASE ONLY AND PAD WITH BLANKS--
	SOURCE = INSOURCE
	CALL UCASE(SOURCE)
        DO I=1,10
	  IF(SOURCE(I:I).LT.ICHAR(' ')) SOURCE(I:I) = ' '
	END DO
	S.NAME = SOURCE

C	--CALCULATE local siderial time from epoch
	CALL SIDTIM (TJD,0.D0,1,GST,ERROR)
	  IF(ERROR(1:2).NE.'OK') RETURN
	  LST=DMOD((GST*DTUPI/24.D0 - 2.120082713D0) , DTUPI)	
	  IF(LST.LT.0.D0) LST = LST+DTUPI
				
C	--IS THE SOURCE A PLANET??--
	PLANET_FLAG = IS_PLANET(SOURCE,NPLANET)

C	--DO THE WORK HERE--
	IF(PLANET_FLAG) THEN
	  CALL PLANET_POSITION(TJD,LST,FREQ,NPLANET,S,ERROR)
	   IF(ERROR(1:2).NE.'OK') RETURN
	ELSE
	  IF(CATALOG(1:1).EQ. ' ') CATALOG = 'j2000.cat'
	  CALL CATALOG_POSITION(TJD,SOURCE,CATALOG,S,ERROR)
	   IF(ERROR(1:2).NE.'OK') RETURN
	END IF

c  --	addon dra and dec (arcminutes)
	CALL COMGETD('DRA',DRA,1,ERROR)
	 IF(ERROR(1:2).NE.'OK') RETURN
	CALL COMGETD('DDEC',DDEC,1,ERROR)
	 IF(ERROR(1:2).NE.'OK') RETURN
	S.RA0 = S.RA0 + DRA *DPI/10800.D0 /COS(DEC0)
	S.DEC0 = S.DEC0 + DDEC *DPI/10800.D0

c  --	addon diurnal aberration
	 HA = LST - S.RA0 
	 COSHA = COS(HA) 
	 SINHA = SIN(HA) 
	 COSD = COS(S.DEC0)
	 SIND = SIN(S.DEC0)
	 S.RA0 = S.RA0 + .235*4.84813681E-6*COSHA/COSD 
	 S.DEC0 =S.DEC0 + .235*4.84813681E-6*SINHA*SIND 

	RETURN
	END
c
c* IS_PLANET
c: ephemeris
c+
	LOGICAL FUNCTION IS_PLANET(NAME,NPLANET)
c
c  IS_PLANET:	tests NAME against a list of planets and satellites
c		and returns TRUE if there is a match along with the
c		planet number used in the jpl ephemerides.
c
c  ARGUMENTS:
c  NAME (char,input)		Source name to be tested
c  NPLANET (integer,output)	Planet number or 0
c--
c	dec 89 wh
c	naifed apr 92  wh
c----------------------------------------------------------------c
	character*10 name, namepad
	integer nplanet
	character*10 planet(25)
	character*10 pnumber
	DATA PLANET/'MERCURY   ','VENUS     ','EARTH     ','MARS      ',
	1'JUPITER   ','SATURN    ','URANUS    ','NEPTUNE   ','PLUTO     ',
	2'MOON      ','SUN       ','TITAN     ','IO        ','EUROPA    ',
	3'CALLISTO  ','GANYMEDE  ','MIMAS     ','ENCELADUS ','TETHYS    ',
	4'DIONE     ','RHEA      ','HYPERION  ','IAPETUS   ','TRITON    ',
	5'NEREID    '/
	integer np(25)/199,299,399,499,599,699,799,899,999,301,10,
	1	606,501,502,504,503,601,602,603,604,605,607,608,801,802/

	NAMEPAD = NAME
	IF (NAMEPAD(1:1).EQ.'_') THEN
	  PNUMBER = NAMEPAD(2:)
	  READ(PNUMBER(1:LEN1(PNUMBER)),'(I<LEN1(PNUMBER)>)') NPLANET
	  IS_PLANET = .TRUE.
	  RETURN
	END IF
	I=1
	DO WHILE((NAMEPAD.NE.PLANET(I)).AND.(I.LE.25))
		I=I+1
	END DO
	IF (I.EQ.26) THEN
		IS_PLANET = .FALSE.
	ELSE
		IS_PLANET = .TRUE.
		NPLANET = NP(I)
	END IF
	RETURN
	END

