c
c* PLANET_POSITION
c: ephemeris
c+
	SUBROUTINE PLANET_POSITION(TJD,LST,FREQ,NUMPL,S,ERROR)
C
C  PLANET_POSITION:	find positions for planets using the jpl ephemeris
c			subroutines for j2000.
c
c  ARGUMENTS:
c  TJD (dp,input)		Julian date/time
c  LST (dp,input)		Local Siderial Time (rad)
c  NUMPL (integer,input)	JPL NAIF planet/satellite number
c  S (RECORD,output)		RECORD containg source parameters
c  ERROR (char,output)		Status return
c--
C	MAR 85  WH
C	nov 89  wh -revamped
c	unix-> nov 90 wh
C--------------------------------------------------------C
	include '../../inc/constants.inc'
	include '../../inc/ephem.inc'
	record /asource/ s
	real*8 tjd,lst
	real*8 freq,distance
	real*8 ras,decs,ra0,ra1,dec0,dec1
	real*8 dum,vdum,rdum,ddum,inc,sinc
	character*80 error

c	----data block for planet fluxes----
	real diam(0:11),aratio(0:11),rap(0:11),decp(0:11),temp(0:11)
C  --   **sun***
	data diam(0),aratio(0),rap(0),decp(0),temp(0)/
	1		1392000.,0.,0.,0.,0./
C  --   **mercury***
	data diam(1),aratio(1),rap(1),decp(1),temp(1)/
	1		4878.,0.,200.,50.,200./
C  --   **venus***
	data diam(2),aratio(2),rap(2),decp(2),temp(2)/
	1		12104.,0.,273.,67.,367./
C  --   **earth***
	data diam(3),aratio(3),rap(3),decp(3),temp(3)/
	1		5*0./
C  --   **mars***
	data diam(4),aratio(4),rap(4),decp(4),temp(4)/
	1		6794.4,.006,318.,53.,207./
C  --   **jupiter***
	data diam(5),aratio(5),rap(5),decp(5),temp(5)/
	1		142796.,.061,268.,64.,179./
C  --   **saturn***
	data diam(6),aratio(6),rap(6),decp(6),temp(6)/
	1		120000.,.109,40.,83.,149./
C  --   **uranus***
	data diam(7),aratio(7),rap(7),decp(7),temp(7)/
	1		50800.,.03,257.,-15.,134./
C  --   **neptune***
	data diam(8),aratio(8),rap(8),decp(8),temp(8)/
	1		48600.,.03,295.,41.,126./
C  --   **pluto***
	data diam(9),aratio(9),rap(9),decp(9),temp(9)/
	1		3400.,0.,285.,-20.,68./
C  --   **moon***
	data diam(11),aratio(11),rap(11),decp(11),temp(11)/
	1		1738.,0.,0.,0.,0./
c	--------				--------

	ERROR='OK'
	CALL APPLAN (TJD,NUMPL,3,RA0,DEC0,DISTANCE,LST,VPLANET,RAS,DECS,ERROR)
	 IF (ERROR(1:2).NE.'OK') RETURN

	INC = .001D0
	SINC = INC * DTUPI * RATIO
	CALL APPLAN 
	1	(TJD+INC,NUMPL,3,RA1,DEC1,DUM,LST+SINC,VDUM,RDUM,DDUM,ERROR)
	 IF (ERROR(1:2).NE.'OK') RETURN

c  --   ra1 and dec1 are derivatives   (radians per radian of UT)
	S.RA0 = RA0
	S.RA2000 = RA0
	S.DEC0 = DEC0
	S.DEC2000 = DEC0
	S.RA1 = (RA1-RA0) / 8640.d0	!change per tick (.01s)
	S.DEC1 = (DEC1-DEC0) / 8640.d0

C  --   fill in velocities from planet file (feb 86)----
	S.VREST=0.
	S.VSUN=-VPLANET
	S.VELOC = 0.

C  --   planet flux calculation----
	N = NUMPL/100
	IF((NUMPL-N*100).EQ.99 .or. (NUMPL-N*100).eq.0 .or. NUMPL.eq.10) THEN
		S.PLMAJ=DIAM(N)/DISTANCE/1.49598E8
		S.PLMIN=S.PLMAJ*(1.-ARATIO(N))
		S.PLTB=TEMP(N)

C  --   special calculation for mars effective brightness temperature
c  --      (depends on mars-sun-earth angle)
		IF(N.EQ.4) THEN
		 RAS = RAS * DPI/12.D0
		 DECS = DECS * DPI/180.D0
		 COSMS=COS(DECS0)*COS(DEC0)*(COS(RAS)*COS(RA0)
	1		+SIN(RAS)*SIN(RA0))+SIN(DECS)*SIN(DEC0)
		 RMS=SQRT(DISTANCE**2+DISS**2-2.*DISTANCE*DISS*COSMS)
		 S.PLTB=TEMP(4)*(1.5236/RMS)**.5
		END IF

		RARAD=RAP(N)*.01745329
		DCRAD=DECP(N)*.01745329
		SINP = COS(DCRAD)*SIN(RARAD-RA0)
		COSP = SIN(DCRAD)*COS(DEC0)-COS(DCRAD)*SIN(DEC0)*COS(RARAD-RA0)
		S.PLANGLE = ATAN2(SINP,COSP) * 180. / PI
		IF (S.PLTB.GT.0.) THEN
		  FLUX=S.PLMAJ*S.PLMIN*1158.1*FREQ**3/
	1			(EXP(.04801*FREQ/S.PLTB)-1.)
		ELSE
		  FLUX=0.
		END IF

		XX=0.5*(S.PLMAJ+S.PLMIN)*2862.11/2.3
		S.PLBEAM=(1.-EXP(-XX*XX))/XX**2
		S.PLFLUXCR=FLUX*S.PLBEAM
	ELSE
		S.PLMAJ=0.
		S.PLMIN=0.
		S.PLTB = 0.
		S.PLANGLE=0.
		S.PLBEAM=0.
		S.PLFLUXCR=0.
	END IF

	RETURN
	END

