c
c* PLEPH
c: ephemeris
c+
      SUBROUTINE PLEPH(JD,TARGET,OBSRVR,STATE,TLIGHT,ERROR)

C  PLEPH:	This subroutine reads the jpl planetary ephemeris
c		and gives the position and velocity of the point 'targ'
c		with respect to 'cent'. The calling sequence has been
c		modified for Hat Creek.
C
c  ARGUMENTS:
c  JD (dp,input)	Julian ephemeris date at which interpolation
C			is wanted.
C  TARGET(integer,input)	Number of 'target' point.
C  OBSRVR(integer,input)	Number of center point.
C
C                199 = MERCURY           899 = NEPTUNE
C                299 = VENUS             999 = PLUTO
C                399 = EARTH             301 = MOON
C                499 = MARS               10 = SUN
C                599 = JUPITER             0 = SOLAR-SYSTEM BARYCENTER
C                699 = SATURN            501 = io
C                799 = URANUS            601 = Mimas
C
C  STATE(dp(6),output)	ARRAY containing position and velocity
c			of point 'targ' relative to 'cent'. The units 
c			are km and km/sec.
C  TLIGHT(dp,output)	Light time from source to obsrvr
c--
c   unix-> nov 90 wh
c   naifed-> apr 92 wh
c   23-jun-93 pjt     increased lenght of 'ephem' and 'kernel'
c   sep 93 wh   add comet code
c----------------------------------------------------------------------c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DOUBLE PRECISION JD
      INTEGER TARGET
      INTEGER OBSRVR
      DOUBLE PRECISION STATE(6)
      DOUBLE PRECISION      EPOCH
      DOUBLE PRECISION J2000
      DOUBLE PRECISION SPD
	logical failed
	integer handle(7)
      character*80 error,spice
	logical FAILED,firsttime/.true./
	character*12 bsp(6) /  'de202', 'jupiter', 'saturn', 'uranus', 
	1	'neptune', 'comet'/
	character*80 ephem,kernel

C  --	Load the binary SPK file containing the ephemeris data
C  --	   that we need.

	ERROR = 'OK'
	if (firsttime) then
	  firsttime = .false.
          CALL ERRPRT (  'SET',   'NONE, LONG'  )
	  CALL ERRACT (  'SET',   'RETURN'  )
c  --	load the leapseconds file
	  call filepath('HATCAT','leapseconds','naif',kernel)
	  CALL CLPOOL
          CALL LDPOOL ( KERNEL )

c  --	load some ephemeris files	    
	 do i=1,6
	  call filepath('HATCAT',bsp(i),'bsp',ephem)
	  CALL SPKLEF ( EPHEM, HANDLE(i) )
          IF (  FAILED()  )  THEN
	     CALL GETSMS ( spice )
             CALL EXPLN ( SPICE, ERROR )
	     CALL RESET
	     RETURN     
	  END IF
	 end do
 	end if

        UTCSEC = ( JD - J2000() ) * SPD()
        CALL DELTET ( UTCSEC, 'UTC', DELTA )
	EPOCH = UTCSEC + DELTA

	CALL SPKEZ ( TARGET, EPOCH, 'J2000', 'LT+S', OBSRVR, STATE, TLIGHT )
          IF (  FAILED()  )  THEN
	     CALL GETSMS ( spice )
             CALL EXPLN ( SPICE, ERROR )
	     CALL RESET
	  END IF

	RETURN
	END 

