c
c* SIDTIM
c: ephemeris
c+
      SUBROUTINE SIDTIM (TJDH,TJDL,K,GST,ERROR)
C
C  SIDTIM:	This subroutine computes the greenwich sidereal time
c  		(either mean or apparent) at julian date tjdh + tjdl.
c		see aoki, et al. (1982) astronomy and astropysics 105, 359-361.
c
c  ARGUMENTS:
c  TJDH (dp,input)	julian date, high-order part
c  TJDL (dp,input)	julian date, low-order part 
c  		          julian date may be split at any point, but
c    			  for highest precision, set tjdh to be the integral
c			  part of the julian date, and set tjdl to be the
c  			  fractional part
c  K (integer,input)	time selection code
c 			  set k=0 for greenwich mean sidereal time
c 			  set k=1 for greenwich apparent sidereal time
c  GST (dp,output)	greenwich (mean or apparent) sidereal time
c			  in hours
C  ERROR (char,output)	Status return
c--
c----------------------------------------------------------------c
      DOUBLE PRECISION TJDH,TJDL,TH,TL,T,T2,T3,GST,SECCON,
     /     EQEQ,OBLM,OBLT,DPSI,DEPS,ST,DCOS,DMOD,TLAST
	CHARACTER*80 ERROR
      DATA SECCON/206264.8062470964D0/
C
	ERROR = 'OK'
      TH = (TJDH - 2451545.0D0) / 36525.0D0
      TL =  TJDL                / 36525.0D0
      T = TH + TL
      T2 = T * T
      T3 = T2 * T
C
C     FOR APPARENT SIDEREAL TIME, COMPUTE EQUATION OF THE EQUINOXES
      EQEQ = 0.0D0
      IF (K.LE.0) GO TO 20
      OBLM = 84381.4480D0 - 46.8150D0*T - 0.00059D0*T2
     /     + 0.001813D0*T3
      OBLM = OBLM / SECCON

C --	only call nod every 63 seconds to save time
      IF(ABS(T-TLAST).GT.2.e-8) THEN
        CALL NOD (T,DPSI,DEPS)
        DEPS = DEPS / SECCON
	TLAST = T
      END IF
      OBLT = OBLM + DEPS
      EQEQ = DPSI / 15.0D0 * DCOS(OBLT)
C
  20  ST = EQEQ - 6.2D-6*T3 + 0.093104D0*T2 + 67310.54841D0
     /     + 8640184.812866D0 *TL
     /     + 3155760000.0D0   *TL
     /     + 8640184.812866D0 *TH
     /     + 3155760000.0D0   *TH
C
      GST = DMOD (ST / 3600.0D0, 24.0D0)
      IF (GST.LT.0.0D0) GST = GST + 24.0D0
      RETURN
      END

