C$Procedure            CHBASE ( Character set base )
 
      INTEGER FUNCTION CHBASE ()
 
C$ Abstract
C
C     Return the base value used to encode unsigned integer values
C     in character strings.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONSTANTS
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     The function returns the base value used to encode unsigned
C     integer values in character strings.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     CHBASE is the base used by ENCHAR and DECHAR to encode and decode
C     non-negative integers to and from character strings. Its value is
C     determined by the size of the character set available for a given
C     machine and compiler. Strictly speaking, CHBASE is one more than
C     the biggest positive integer which can be handled by both the
C     CHAR and ICHAR intrinsic functions (which are used by ENCHAR and
C     DECHAR). That is, CHBASE is the first positive integer for which
C     the logical expression
C
C           ( ICHAR ( CHAR ( CHBASE ) ) .EQ. CHBASE )
C
C     is false.
C
C     Note that CHBASE can be (and probably is) different from the
C     number of characters in the character set used by the processor.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The function always returns a constant value, set by the user
C     prior to compilation.
C
C     CHBASE should always be at least 128 (the size of the ASCII
C     character set), and will usually be 256 for machines which use
C     eight bits to represent a single character. The following list
C     contains the values of CHBASE for a range of environments.
C
C     Environment: VAX/VMS, VAX FORTRAN
C     Value:       256
C
C     Environment: Sun, Sun FORTRAN
C     Value:       256
C
C     Environment: PC, MS FORTRAN
C     Value:       256
C
C     Environment: Macintosh, Language Systems FORTRAN
C     Value:       256
C
C     Environment: PC, Lahey F77 EM/32 Version 4.0
C     Value:       256
C
C     Environment: HP-UX 9000/750, FORTRAN/9000 Series 700 computers
C     Value:       256
C
C
C     For other machines, the value can always be determined by running
C     the following simple program:
C
C            INTEGER          CHBASE
C            DATA             CHBASE    / 0 /
C
C            DO WHILE ( .TRUE. )
C
C               IF ( ICHAR (CHAR ( CHBASE ) ) .EQ. CHBASE ) THEN
C                  CHBASE = CHBASE + 1
C               ELSE
C                  WRITE (6,*) 'CHBASE for this machine is : ', CHBASE
C                  STOP
C               END IF
C
C            END DO
C            END
C
C$ Examples
C
C     See ENCHAR, DECHAR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     1. "Programming in VAX FORTRAN", Digital Equipment Corporation,
C        September 1984, Section 8.3, page 8-6.
C
C     2. "Microsoft FORTRAN Reference", Microsoft Corporation,
C        1989, Section 5.1.1, page 241.
C
C     3. "Language Systems FORTRAN Reference Manual", Language Systems
C        Corporation, version 1.2.1, page 3-20.
C
C     4. "Lahey F77L EM/32 FORTRAN Language Reference Manual", page
C        222, Note 20.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C     M.J. Spencer    (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.4.0, 6-OCT-1992 (HAN)
C
C         Module was updated to include the character base
C         value for the Hewlett Packard UX 9000/750 environment,
C         and the value for the Sun was changed from 128 to 256.
C         Both changes are the result of running the program in
C         the Particulars section of the header on both machines.
C
C-     SPICELIB Version 1.3.1, 10-MAR-1999 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the character base
C         value for the Lahey FORTRAN EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the character base
C         value for the Macintosh.
C
C-     SPICELIB Version 1.1.0, 9-MAR-1990 (HAN)
C
C         Module was updated to include the character base
C         value for the Sun. Sources for the values contained
C         in this module are now specified in the Literature_References
C         section.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     base for encoding integers in character_string
C
C-&
 
 
C$ Revisions
C
C-     SPICELIB Version 1.4.0, 6-OCT-1992 (HAN)
C
C         Module was updated to include the character base
C         value for the Hewlett Packard UX 9000/750 environment,
C         and the value for the Sun was changed from 128 to 256.
C         Both changes are the result of running the program in
C         the Particulars section of the header on both machines.
C
C         The previous Sun value was computed on the Sun3 and was
C         not updated when we moved to the Sun4. Everything passed
C         the suite of test programs that would have indicated a bug.
C
C         The code was also reformatted so that a utility program can
C         create the file for each environment.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the character base
C         value for the Lahey FORTRAN EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the character base
C         value for the Macintosh.
C
C-     SPICELIB Version 1.1.0, 9-MAR-1990 (HAN)
C
C         Module was updated to include the character base
C         value for the Sun. Sources for the values contained
C         in this module are now specified in the Literature_References
C         section.
C
C-     Beta Version 1.1.0, 16-FEB-1989 (HAN) (NJB)
C
C         Contents of the Exceptions section was changed
C         to "error free" to reflect the decision that the
C         module will never participate in error handling.
C
C         Missing parentheses added to CHBASE declaration.
C
C-&
 
 
 
C
C     We have provided values for several popular machines. Remove
C     the comment character in front of the value for your machine,
C     or provide your own value. Numbers are provided in a variety
C     of formats: decimal, hex, and binary. These last two formats
C     are not portable; but then, neither are the values.
C
 
 
C
C     VAX, VAX FORTRAN
C     Sun, Sun FORTRAN
C     IBM PC, Microsoft FORTRAN, Lahey EM/32 FORTRAN
C     Macintosh, Language Systems FORTRAN
C     HP-UX 9000/750, FORTRAN/9000 Series 700 computers
C
      CHBASE = 256
 
      RETURN
      END
