C$Procedure DAFAH ( DAF, assign handles )
 
      SUBROUTINE DAFAH ( FNAME,
     .                   ND,
     .                   NI,
     .                   IFNAME,
     .                   RESV,
     .                   HANDLE,
     .                   UNIT,
     .                   FHSET,
     .                   ACCESS  )
 
C$ Abstract
C
C     Assign handles to DAFs as they are opened.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL  = -5 )
 
      CHARACTER*(*)         FNAME
      INTEGER               ND
      INTEGER               NI
      CHARACTER*(*)         IFNAME
      INTEGER               RESV
      INTEGER               HANDLE
      INTEGER               UNIT
      INTEGER               FHSET ( LBCELL : * )
      CHARACTER*(*)         ACCESS
 
C
C     The record length should be big enough to hold 128 double
C     precision numbers.
C
 
 
 
C
C     For the following environments, record length is measured in
C     characters (bytes) with eight characters per double precision
C     number.
C
C     Environment: Sun, Sun FORTRAN
C     Source:      Sun Fortran Programmer's Guide
C
C     Environment: PC, MS FORTRAN
C     Source:      Microsoft Fortran Optimizing Compiler User's Guide
C
C     Environment: Macintosh, Language Systems FORTRAN
C     Source:      Language Systems FORTRAN Reference Manual,
C                  Version 1.2, page 12-7
C
C     Environment: PC, Lahey F77 EM/32 Version 4.0
C     Source:      Lahey F77 EM/32 Language Reference Manual,
C                  page 144
C
C     Environment: HP-UX 9000/750, FORTRAN/9000 Series 700 computers
C     Source:      FORTRAN/9000 Reference-Series 700 Computers,
C                  page 5-110
C
C
      INTEGER               RECL
      PARAMETER           ( RECL  = 1024 )
 
 
      INTEGER               FILEN
      PARAMETER           ( FILEN  =  256 )
 
      INTEGER               FTSIZE
      PARAMETER           ( FTSIZE =  20 )
 
C$ Brief_I/O
C
C     Variable  I/O  Entry points
C     --------  ---  --------------------------------------------------
C     FNAME     I,O  OPR, OPW, OPN, HFN, FNH
C     ND        I,O  OPN, HSF
C     NI        I,O  OPN, HSF
C     IFNAME     I   OPN
C     RESV       I   OPN
C     HANDLE    I,O  OPR, OPW, OPN, CLS, HLU, LUH, HFN, FNH, SIH
C     UNIT      I,O  HLU, LUH
C     FHSET      O   HOF
C     ACCESS     I   SIH
C     RECL       P   OPR, OPW, OPN
C     FTSIZE     P   OPR, OPW, OPN, CLS, HLU, LUH, HFN, FNH
C     FILEN      P   SIH
C
C$ Detailed_Input
C
C     FNAME       on input is the name of a DAF to be opened, or
C                 the name of a DAF about which some information
C                 (handle, logical unit) is requested.
C
C     ND          on input is the number of double precision components
C                 in array summary of a new file.
C
C     NI          on input is the number of integer components in each
C                 array summary in a new file.
C
C     IFNAME      is the internal file name for a DAF to be created.
C
C     RESV        is the number of records to be reserved in a DAF
C                 to be created.
C
C     HANDLE      on input is the handle of a DAF about which some
C                 information (file name, logical unit) is requested,
C                 or the handle of a DAF to be closed.
C
C     UNIT        on input is the logical unit connected to a DAF
C                 about which some information (file name, handle) is
C                 requested.
C
C     ACCESS      is the type of access a DAF is open for, that is,
C                 either reading or writing.  The values of ACCESS
C                 may be
C
C                    'READ'
C                    'WRITE'
C
C                 Leading and trailing blanks are ignored, and case
C                 is not significant.
C
C$ Detailed_Output
C
C     FNAME       on output is the name of a DAF for which
C                 the corresponding handle or logical unit has been
C                 supplied.
C
C     ND          on output is the number of double precision
C                 components in each array summary of an existing file.
C
C     NI          on output is the number of integer components in
C                 each array summary in an existing file.
C
C     HANDLE      on output is the handle of a DAF for which
C                 the corresponding file name or logical unit has been
C                 supplied.
C
C     UNIT        on output is the logical unit connected to a DAF
C                 for which the corresponding file name or handle has
C                 been supplied.
C
C     FHSET       is a SPICELIB set containing the handles of the
C                 currently open DAFs.
C
C$ Parameters
C
C     RECL        is the record length of a DAF. Each record
C                 must be large enough to hold 128 double
C                 precision numbers or 1000 characters, whichever
C                 is greater. The units in which the record length
C                 must be specified vary from environment to
C                 environment. For example, VAX Fortran requires
C                 record lengths to be specified in longwords,
C                 where two longwords equal one double precision
C                 number.
C
C     FTSIZE      is the size of the file table maintained internally
C                 by DAFAH. In effect, FTSIZE is the maximum number
C                 of DAFs that the DAF routines allow to be open
C                 simultaneously.  A lower limit may be imposed by
C                 your operating system.
C
C     FILEN       is the maximum filename length.
C
C$ Files
C
C     All DAFs opened by this routine are specified by name.
C
C$ Exceptions
C
C     1) If DAFAH is called directly, the error SPICE(BOGUSENTRY)
C        is signalled.
C
C     2) See entry points DAFOPR, DAFOPW, DAFOPN, DAFCLS, DAFHSF,
C        DAFHLU, DAFLUH, DAFHFN, DAFNFH, DAFHOF, and DAFSIH for
C        exceptions specific to those entry points.
C
C$ Particulars
C
C     DAFAH serves as an umbrella, allowing data to be shared by its
C     entry points:
C
C        DAFOPR         Open for read.
C        DAFOPW         Open for write.
C        DAFOPN         Open new.
C
C        DAFCLS         Close.
C
C        DAFHSF         Handle to summary format.
C
C        DAFHLU         Handle to logical unit.
C        DAFLUH         Logical to handle.
C
C        DAFHFN         Handle to name.
C        DAFFNH         File name to handle.
C
C        DAFHOF         Handles of open files.
C        DAFSIH         Signal invalid handles.
C
C     Before a DAF can be used, it must be opened. Entry points
C     DAFOPR and DAFOPW provide the only means for opening an
C     existing DAF.
C
C     Several files may be opened for use simultaneously. (This makes
C     it convenient to combine data from several files to produce a
C     single result.) As each DAF is opened, it is assigned a file
C     handle, which is used to keep track of the file internally, and
C     which is used by the calling program to refer to the file in all
C     subsequent calls to DAF routines.
C
C     DAFs may be opened for two kinds of access: read, and write.
C     Files opened for read access may not be changed in any way. Files
C     opened for write access may be both read and written.
C
C     Entry point DAFOPN provides the only means for creating a new
C     DAF. Since the only reason for creating a new file is to
C     write something in it, all new files are opened for write access.
C
C     Entry point DAFCLS provides the only official means of closing
C     a DAF that is currently open. Closing a DAF any other
C     way (for example, by determining its logical unit and using the
C     Fortran CLOSE statement directly) may affect your calling program
C     in mysterious ways.
C
C     Entry point DAFHSF allows you to determine the summary format
C     of any DAF that is currently open, without calling DAFRFR to
C     re-read the file record.
C
C     Entry point DAFHOF allows you to determine which DAFs are open
C     at any time.  In particular, you can use DAFHOF to determine
C     whether any file handle points to an open DAF.
C
C     Entry point DAFSIH signals errors when it is supplied with invalid
C     handles, so it serves to centralize error handling associated
C     with invalid handles.
C
C     The remaining entry points exist mainly to translate between
C     alternative representations of DAFs. There are three ways to
C     identify any open DAF: by name, by handle, and by logical
C     unit. Given any one of these, you may use these entry points to
C     find the other two.
C
C$ Examples
C
C     See entry points DAFOPR, DAFOPW, DAFOPN, DAFCLS, DAFHSF,
C     DAFHLU, DAFLUH, DAFHFN, DAFNFH, DAFHOF, and DAFSIH for examples
C     specific to those entry points.
C
C$ Restrictions
C
C     1) The value of parameter RECL may need to be changed when DAFAH
C        and its entry points are ported to a new environment (CPU and
C        compiler).
C
C     2) An integer overflow may occur if the number of files opened
C        by a single program exceeds the maximum number that can be
C        stored in an integer variable.
C
C$ Literature_References
C
C     1) NAIF Document 167.0, "Double Precision Array Files (DAF)
C        Specification and User's Guide"
C
C     2) Sun Fortran Programmer's Guide
C
C     3) Microsoft Fortran Optimizing Compiler User's Guide
C
C     4) Lahey F77 EM/32 Language Reference Manual, page 144
C
C     5) Language Systems FORTRAN Reference Manual, Version 1.2,
C        page 12-7
C
C     6) "FORTRAN/9000 Reference HP 9000 Series 700 Computers",
C        First Edition, June 1991, Hewlett Packard Company, page 5-110.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.M. Lynch      (JPL)
C     J.E. McLean     (JPL)
C     H.A. Neilan     (JPL)
C     M.J. Spencer    (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 4.0.1, 01-AUG-1993 (HAN)
C
C        Deleted an irrelevant comment.
C
C-    SPICELIB Version 4.0.0, 25-FEB-1993 (JML)
C
C        In the entry points DAFOPR, DAFOPW, and DAFFNH, the INQUIRE
C        statement that checks if the file is already open now also
C        checks that the file exists.
C
C        IOSTAT is now checked after all INQUIRE statements.
C
C        A new variable LUN is used in DAFOPR, DAFOPW, and DAFOPN
C        for the logical unit number returned by GETLUN.
C
C        The IF-THEN statements in DAFOPR and DAFOPW were reorganized
C        to make the routines more readable.
C
C        In DAFOPR and DAFOPW, a long error message was added for the
C        case when the NAIF/DAF id word was not recognized. Also, the
C        file is closed when this error is signalled.
C
C        In DAFOPR and DAFOPW, IOSTAT is now checked after the file
C        record is read.
C
C        In DAFOPR, DAFOPW, DAFOPN, and DAFFNH, the file name is
C        checked to see if it is blank.
C
C        In DAFOPR, DAFOPW, DAFOPN, and DAFFNH, the file name passed
C        to the FORTRAN OPEN and INQUIRE statements has been chopped
C        at the last non-blank character.
C
C        A minor error in the particulars section of the header of
C        DAFCLS was corrected.  It formely stated that a file could be
C        open more than once for read or write access instead of just
C        read access.
C
C-    SPICELIB Version 3.2.0, 6-OCT-1992 (HAN)
C
C        Module was updated to include the record length and source
C        for the Hewlett Packard UX 9000/750 environment. Moved FILEN
C        to the Declarations section, and corrected Revisions section
C        to include the last code change description, 3.1.0.
C
C-    SPICELIB Version 3.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 3.1.0, 13-NOV-1991 (MJS)
C
C        Module was updated to operate in the Lahey F77 EM/32
C        PC environment.
C
C-    SPICELIB Version 3.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        DAFAH and its entry points were modified to permit multiple
C        DAFs to be open for writing at the same time.  Also, the
C        entry points DAFHOF and DAFSIH were added.
C
C-    SPICELIB Version 2.0.0, 25-MAR-1991 (JEM) (MJS)
C
C        The variable MINHAN was initialized to zero and the variable
C        NEXT was saved.  DAFOPW now accepts the ID word 'NAIF/NIP'
C        as well 'NAIF/DAF'.  Spelling mistakes were corrected.
C
C-    SPICELIB Version 1.1.0, 5-NOV-1990 (HAN)
C
C        The parameter FTSIZE was increased from 4 to 20.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     assign daf handles
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 4.0.0, 25-FEB-1993 (JML)
C
C        In the entry points DAFOPR, DAFOPW, and DAFFNH, the INQUIRE
C        statement that checks if the file is already open now also
C        checks that the file exists.
C
C        IOSTAT is now checked after all INQUIRE statements.
C
C        A new variable LUN is used in DAFOPR, DAFOPW, and DAFOPN
C        for the logical unit number returned by GETLUN.
C
C        The IF-THEN statements in DAFOPR and DAFOPW were reorganized
C        to make the routines more readable.
C
C        In DAFOPR and DAFOPW, a long error message was added for the
C        case when the NAIF/DAF id word was not recognized. Also, the
C        file is closed when this error is signalled.
C
C        In DAFOPR and DAFOPW, IOSTAT is now checked after the file
C        record is read.
C
C        In DAFOPR, DAFOPW, DAFOPN, and DAFFNH, the file name is
C        checked to see if it is blank.
C
C        In DAFOPR, DAFOPW, DAFOPN, and DAFFNH, the file name passed
C        to the FORTRAN OPEN and INQUIRE statements has been chopped
C        at the last non-blank character.
C
C        A minor error in the particulars section of the header of
C        DAFCLS was corrected.  It formely stated that a file could be
C        open more than once for read or write access instead of just
C        read access.
C
C-    SPICELIB Version 3.2.0, 6-OCT-1992 (HAN)
C
C        Module was updated to include the record length and source
C        for the Hewlett Packard UX 9000/750 environment. Moved FILEN
C        to the Declarations section, and Revisions section was
C        corrected to include the last code change description, 3.1.0.
C
C        The code was also reformatted so that a utility program can
C        create the source file for a specific environment given a
C        master source file.
C
C-    SPICELIB Version 3.1.0, 13-NOV-1991 (MJS)
C
C        Module was updated to operate in the Lahey F77 EM/32
C        PC environment.
C
C-    SPICELIB Version 3.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        DAFAH and the entry point DAFOPW were modified to permit
C        multiple DAFs to be open for writing at the same time.
C        Also, the entry points DAFHOF and DAFSIH were added.  DAFHOF
C        returns a set containing the handles of currently open DAFs.
C        To accommodate the addition of DAFHOF, the argument FHSET
C        was added to DAFAH's argument list, and local declarations
C        for DAFHOF were added to DAFAH's declaration section.  DAFSIH
C        signals an error if the file indicated by the handle is not
C        open for the specified type of access.
C
C-    SPICELIB Version 2.0.0, 24-JAN-1991 (JEM) (MJS)
C
C        The entry point DAFOPW accepted only 'NAIF/DAF' as a valid
C        ID word.  It now accepts 'NAIF/NIP' as well for
C        backwards compatability.  The entry point DAFOPR did not need
C        this fix because it already accepts both ID words.
C
C-    SPICELIB Version 1.1.0,  5-NOV-1990 (HAN)
C
C        The parameter FTSIZE was increased from 4 to 20. The number
C        4 was chosen for testing purposes and was not removed.
C
C-&
 
 
C
C     SPICELIB functions
C
      INTEGER               ISRCHI
      INTEGER               LTRIM
      INTEGER               RTRIM
 
      LOGICAL               ELEMI
      LOGICAL               FAILED
      LOGICAL               RETURN
 
 
 
C
C     Local parameters
C
      INTEGER               ACCLEN
      PARAMETER           ( ACCLEN =   10 )
 
      INTEGER               CRLEN
      PARAMETER           ( CRLEN  = 1000 )
 
      INTEGER               DPRSIZ
      PARAMETER           ( DPRSIZ =  128 )
 
      INTEGER               IDLEN
      PARAMETER           ( IDLEN  =    8 )
 
      INTEGER               IFNLEN
      PARAMETER           ( IFNLEN =   60 )
 
      INTEGER               MAXND
      PARAMETER           ( MAXND  =  124 )
 
      INTEGER               MAXNI
      PARAMETER           ( MAXNI  =  250 )
 
      INTEGER               NIL
      PARAMETER           ( NIL    =   -1 )
 
      INTEGER               MAXSUM
      PARAMETER           ( MAXSUM =  125 )
 
 
C
C     Local variables
C
 
C
C     As each file is opened, it is assigned a handle, and the
C     internal file name is stored for comparison with other files.
C     All names in the file table begin with FT.
C
C        HAN      Handle
C        LUN      Logical unit
C        LNK      Number of links
C        ND,
C        NI       Summary format
C
C     The columns are stored in no particular order. New files are
C     added to the end of the list; the list is repacked whenever a
C     file is removed from the list.
C
C     NFT is the number of files currently opened: this may not be
C     greater than FTSIZE. FINDEX refers to a file of interest within
C     the table.
C
C     NEXT is incremented each time a file is opened to become the
C     next file handle assigned.
C
      INTEGER               NFT
      INTEGER               FTHAN  ( FTSIZE )
      INTEGER               FTLUN  ( FTSIZE )
      INTEGER               FTLNK  ( FTSIZE )
      INTEGER               FTND   ( FTSIZE )
      INTEGER               FTNI   ( FTSIZE )
      INTEGER               FINDEX
 
 
C
C     Other local variables
C
      CHARACTER*(ACCLEN)    ACC
      CHARACTER*(CRLEN)     CREC
      CHARACTER*(FILEN)     DAF
      CHARACTER*(IDLEN)     DAFWRD
      CHARACTER*(IFNLEN)    IFN
 
      DOUBLE PRECISION      DREC   ( DPRSIZ )
 
      INTEGER               BWARD
      INTEGER               FCHAR
      INTEGER               FHLIST ( LBCELL : FTSIZE )
      INTEGER               FND
      INTEGER               FNI
      INTEGER               FREE
      INTEGER               FWARD
      INTEGER               I
      INTEGER               IOSTAT
      INTEGER               LCHAR
      INTEGER               LUN
      INTEGER               MINHAN
      INTEGER               NEXT
      INTEGER               NUMBER
 
      LOGICAL               EXIST
      LOGICAL               FIRST
      LOGICAL               OPENED
 
 
C
C     Saved variables
C
 
C
C     Save everything between calls.
C
      SAVE
 
 
C
C     Initial values
C
      DATA                  FIRST    / .TRUE. /
      DATA                  MINHAN   / 0      /
      DATA                  NEXT     / 0      /
      DATA                  NFT      / 0      /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN  ( 'DAFAH' )
         CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
         CALL CHKOUT ( 'DAFAH' )
      END IF
 
      RETURN
 
 
 
 
C$Procedure DAFOPR ( DAF, open for read )
 
      ENTRY DAFOPR ( FNAME, HANDLE )
 
C$ Abstract
C
C     Open a DAF for subsequent read requests.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of DAF to be opened.
C     HANDLE     O   Handle assigned to DAF.
C
C$ Detailed_Input
C
C     FNAME       is the file name of a DAF to be opened for read
C                 access.
C
C$ Detailed_Output
C
C     HANDLE      is the file handle associated with the file. This
C                 handle is used to identify the file in subsequent
C                 calls to other DAF routines.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     See argument FNAME.
C
C$ Exceptions
C
C     1) If the specified file has already been opened for read
C        access, the handle already associated with the file is
C        returned.
C
C     2) If the specified file has already been opened for write
C        access, the error SPICE(DAFRWCONFLICT) is signalled.
C
C     3) If the specified file has already been opened by a non-DAF
C        routine, the error SPICE(DAFIMPROPOPEN) is signalled.
C
C     4) If the specified file cannot be opened without exceeding
C        the maximum number of files, the error SPICE(DAFFTFULL)
C        is signalled.
C
C     5) If (for some reason) the file cannot be opened properly,
C        the error SPICE(DAFOPENFAIL) is signalled.
C
C     6) If the specified file is not a DAF (does not contain
C        the proper standard keyword), the error SPICE(DAFNOIDWORD)
C        is signalled.
C
C     7) If no logical units are available, the error will be
C        signalled by routines called by this routine.
C
C     8) If the file does not exist, the error SPICE(FILEDOESNOTEXIST)
C        is signalled.
C
C     9) If the INQUIRE fails, the error SPICE(INQUIREFAILED)
C        is signalled.
C
C    10) If the file record cannot (for some reason) be read,
C        the error SPICE(DAFFRNOTFOUND) is signalled.
C
C    11) If the file name is blank, the error SPICE(BLANKFILENAME)
C        is signalled.
C
C$ Particulars
C
C     Most DAFs require only read access. If you do not need to
C     change the contents of a file, you should open it with DAFOPR.
C
C$ Examples
C
C     In the following code fragment, DAFOPR is used to open a file,
C     which is then searched for DAFs containing data for a particular
C     object.
C
C        CALL DAFOPR ( FNAME, HANDLE )
C        CALL DAFBFS ( HANDLE )
C        CALL DAFFNA ( FOUND  )
C
C        DO WHILE ( FOUND )
C           CALL DAFGS ( SUM )
C           CALL DAFUS ( SUM, ND, NI, DC, IC )
C
C           IF ( IC(1) .EQ. TARGET_OBJECT ) THEN
C            .
C            .
C
C           END IF
C
C           CALL DAFFNA ( FOUND )
C        END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.M. Lynch      (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 3.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is already open
C        now also checks that the file exists.
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The IF-THEN statements were reorganized to improve readability.
C
C        A long error message is now set when the DAF id word is not
C        recognized.  Also, the file is closed when this error is
C        signalled.
C
C        IOSTAT is checked after the file record is read.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
 
C$ Index_Entries
C
C     open daf for read
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 3.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is already open
C        now also checks that the file exists.
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The IF-THEN statements were reorganized to improve readability.
C
C        A long error message is now set when the DAF id word is not
C        recognized.  Also, the file is closed when this error is
C        signalled.
C
C        IOSTAT is checked after the file record is read.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C        Some error messages were changed so that they specify
C        names of relevant DAFs.
C
C-&
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFOPR' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Check if the file name is blank. If it isn't, then find the
C     index in the input string of the first and last characters
C     of the name.
C
      IF ( FNAME .EQ. ' ' ) THEN
 
         CALL SETMSG ( 'The file name is blank. ' )
         CALL SIGERR ( 'SPICE(BLANKFILENAME)'     )
         CALL CHKOUT ( 'DAFOPR'                   )
         RETURN
 
      END IF
 
      FCHAR = LTRIM ( FNAME )
      LCHAR = RTRIM ( FNAME )
 
C
C     INQUIRE as to whether the file exists, if it is open, and what
C     logical unit number it is attached to.
C
      INQUIRE ( FILE   = FNAME(FCHAR:LCHAR),
     .          EXIST  = EXIST,
     .          OPENED = OPENED,
     .          NUMBER = NUMBER,
     .          IOSTAT = IOSTAT  )
 
C
C     First make sure that the file exists and that the INQUIRE worked.
C
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'INQUIRE failed. Value of IOSTAT was #.' )
         CALL ERRINT ( '#', IOSTAT                              )
         CALL SIGERR ( 'SPICE(INQUIREFAILED)'                   )
         CALL CHKOUT ( 'DAFOPR'                                 )
         RETURN
 
      END IF
 
      IF ( .NOT. EXIST ) THEN
 
         CALL SETMSG ( 'The file # does not exist.'             )
         CALL ERRCH  ( '#', FNAME                               )
         CALL SIGERR ( 'SPICE(FILEDOESNOTEXIST)'                )
         CALL CHKOUT ( 'DAFOPR'                                 )
         RETURN
 
      END IF
 
 
C
C     The file may or may not be open already. If it is, it should not
C     have been opened either for writing (handle < 0) or by another
C     routine.  If opened for reading, just increment the number of
C     links and return the handle.
C
      IF ( OPENED ) THEN
 
         FINDEX = ISRCHI ( NUMBER, NFT, FTLUN )
 
         IF ( FINDEX .GT. 0 ) THEN
 
            IF ( FTHAN(FINDEX) .LT. 0 ) THEN
 
               CALL SETMSG ( '# already opened for write access.' )
               CALL ERRCH  ( '#', FNAME                           )
               CALL SIGERR ( 'SPICE(DAFRWCONFLICT)'               )
 
            ELSE
 
               FTLNK(FINDEX) = FTLNK(FINDEX) + 1
               HANDLE        = FTHAN(FINDEX)
 
            END IF
 
         ELSE
 
            CALL SETMSG ( '# is already connected to unit #.' )
            CALL ERRCH  ( '#', FNAME                          )
            CALL ERRINT ( '#', NUMBER                         )
            CALL SIGERR ( 'SPICE(DAFIMPROPOPEN)'              )
 
         END IF
 
         CALL CHKOUT ( 'DAFOPR' )
         RETURN
 
      END IF
 
C
C     If it hasn't been opened, it needs to be, but only if there
C     is room for another file.
C
      IF ( NFT .EQ. FTSIZE ) THEN
 
         CALL SETMSG ( 'The file table is full, with # entries. '     //
     .                 'Could not open #.'                             )
         CALL ERRINT ( '#', FTSIZE                                     )
         CALL ERRCH  ( '#', FNAME                                      )
         CALL SIGERR ( 'SPICE(DAFFTFULL)'                              )
         CALL CHKOUT ( 'DAFOPR'                                        )
         RETURN
 
      END IF
 
C
C     To open for reading: get a free unit, open the file, get the
C     internal file name, and set the number of links to one.
C
C     Look out for:
C
C        -- No free logical units.
C
C        -- Error opening the file.
C
C        -- No NAIF/DAF ID word in the first record.
C
      CALL GETLUN ( LUN )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'DAFOPR' )
         RETURN
      END IF
 
      OPEN ( UNIT   = LUN,
     .       FILE   = FNAME(FCHAR:LCHAR),
     .       ACCESS = 'DIRECT',
     .       RECL   = RECL,
     .       STATUS = 'OLD',
     .       IOSTAT = IOSTAT    )
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'Attempt to open file # failed. Value of '  //
     .                 'IOSTAT was #.'                              )
         CALL ERRCH  ( '#', FNAME                                   )
         CALL ERRINT ( '#', IOSTAT                                  )
         CALL SIGERR ( 'SPICE(DAFOPENFAIL)'                         )
         CALL CHKOUT ( 'DAFOPR'                                     )
         RETURN
 
      END IF
 
 
C
C     Read the file record and check the NAIF/DAF id word.
C     If there is no match then close the file and signal
C     an error.
C
      READ ( LUN, REC=1, IOSTAT=IOSTAT ) DAFWRD, FND, FNI
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CLOSE ( LUN )
 
         CALL SETMSG ( 'File record not found. Value of IOSTAT was #' )
         CALL ERRINT ( '#', IOSTAT                                    )
         CALL SIGERR ( 'SPICE(DAFFRNOTFOUND)'                         )
         CALL CHKOUT ( 'DAFOPR'                                       )
         RETURN
 
      END IF
 
      IF (        DAFWRD .NE. 'NAIF/DAF'
     .     .AND.  DAFWRD .NE. 'NAIF/NIP' ) THEN
 
         CLOSE ( LUN )
 
         CALL SETMSG ( 'The DAF file id word # was not recognized.' )
         CALL ERRCH  ( '#', DAFWRD          )
         CALL SIGERR ( 'SPICE(DAFNOIDWORD)' )
         CALL CHKOUT ( 'DAFOPR'             )
         RETURN
 
      END IF
 
C
C     If we've come this far, we've succeeded.
C
C     Update the file table to include information about
C     our newly opened DAF.  Set the output argument
C     HANDLE as well.
C
      NEXT       = NEXT + 1
      NFT        = NFT  + 1
      FTHAN(NFT) = NEXT
      FTLUN(NFT) = LUN
      FTND(NFT)  = FND
      FTNI(NFT)  = FNI
      FTLNK(NFT) = 1
      HANDLE     = FTHAN(NFT)
C
C     Insert the new handle into our handle set.
C
      CALL INSRTI ( HANDLE, FHLIST )
 
      CALL CHKOUT ( 'DAFOPR' )
      RETURN
 
 
 
C$Procedure DAFOPW ( DAF, open for write )
 
      ENTRY DAFOPW ( FNAME, HANDLE )
 
C$ Abstract
C
C     Open a DAF for subsequent write requests.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of DAF to be opened.
C     HANDLE     O   Handle assigned to DAF.
C
C$ Detailed_Input
C
C     FNAME       is the name of a DAF to be opened for write
C                 access.
C
C$ Detailed_Output
C
C     HANDLE      is the file handle associated with the file. This
C                 handle is used to identify the file in subsequent
C                 calls to other DAF routines.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     See argument FNAME.
C
C$ Exceptions
C
C     1) If the specified file has already been opened, either by
C        the DAF routines or by other code, the error
C        SPICE(DAFOPENCONFLICT) is signalled.  Note that this
C        response is not paralleled by DAFOPR, which allows you
C        to open a DAF for reading even if it is already open for
C        reading.
C
C     2) If the specified file cannot be opened without exceeding
C        the maximum number of files, the error SPICE(DAFFTFULL)
C        is signalled.
C
C     3) If (for some reason) the file cannot be opened properly,
C        the error SPICE(DAFOPENFAIL) is signalled.
C
C     4) If the file record cannot (for some reason) be read,
C        the error SPICE(DAFFRNOTFOUND) is signalled.
C
C     5) If the specified file is not a DAF (does not contain
C        the proper standard keyword), the error SPICE(DAFNOIDWORD)'
C        is signalled.
C
C     6) If no logical units are available, the error will be
C        signalled by routines called by this routine.
C
C     7) If the file does not exist, the error SPICE(FILEDOESNOTEXIST)
C        is signalled.
C
C     8) If the INQUIRE fails, the error SPICE(INQUIREFAILED)
C        is signalled.
C
C     9) If the file name is blank, the error SPICE(BLANKFILENAME)
C        is signalled.
C
C$ Particulars
C
C     Most DAFs require only read access. If you do not need to
C     change the contents of a file, you should open it with DAFOPR.
C     Use DAFOPW when you need to
C
C        -- change (update) one or more summaries, names, or
C           arrays within a file; or
C
C        -- add new arrays to a file.
C
C$ Examples
C
C     In the following code fragment, DAFOPW is used to open a
C     file, which is then searched for arrays containing data for
C     a particular object. The code for the object is then changed
C     (perhaps to reflect some new convention).
C
C        CALL DAFOPW ( FNAME, HANDLE )
C        CALL DAFBFS ( HANDLE )
C        CALL DAFFNA ( FOUND  )
C
C        DO WHILE ( FOUND )
C           CALL DAFGS ( SUM )
C           CALL DAFUS ( SUM, ND, NI, DC, IC )
C
C           IF ( IC(1) .EQ. OLD_CODE ) THEN
C              IC(1) = NEW_CODE
C
C              CALL DAFPS ( ND, NI, DC, IC, SUM )
C              CALL DAFRS ( SUM )
C           END IF
C
C           CALL DAFFNA ( FOUND )
C        END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.M. Lynch      (JPL)
C     J.E. McLean     (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 4.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is already open
C        now also checks that the file exists.
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The IF-THEN statements were reorganized to improve readability.
C
C        A long error message is now set when the DAF id word is not
C        recognized.  Also, the file is closed when this error is
C        signalled.
C
C        IOSTAT is now checked after the file record is read.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 3.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 3.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        DAFOPW now allows multiple files to be open for writing.
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C
C-    SPICELIB Version 2.0.0, 24-JAN-1991 (JEM)
C
C        DAFOPW now accepts the ID word 'NAIF/NIP' as well 'NAIF/DAF'.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     open daf for write
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 4.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is already open
C        now also checks that the file exists.
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The IF-THEN statements were reorganized to improve readability.
C
C        A long error message is now set when the DAF id word is not
C        recognized.  Also, the file is closed when this error is
C        signalled.
C
C        IOSTAT is now checked after the file record is read.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        DAFOPW now allows multiple files to be open for writing.
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C        Some error messages were changed so that they specify
C        names of relevant DAFs.
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFOPW' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Check if the file name is blank. If it isn't, then find the
C     index in the input string of the first and last characters
C     of the name.
C
      IF ( FNAME .EQ. ' ' ) THEN
 
         CALL SETMSG ( 'The file name is blank. ' )
         CALL SIGERR ( 'SPICE(BLANKFILENAME)'     )
         CALL CHKOUT ( 'DAFOPW'                   )
         RETURN
 
      END IF
 
 
      FCHAR = LTRIM ( FNAME )
      LCHAR = RTRIM ( FNAME )
 
C
C     INQUIRE as to whether the file exists, if it is open, and what
C     logical unit number it is attached to.
C
      INQUIRE ( FILE   = FNAME(FCHAR:LCHAR),
     .          EXIST  = EXIST,
     .          OPENED = OPENED,
     .          NUMBER = NUMBER,
     .          IOSTAT = IOSTAT  )
 
C
C     First make sure that the file exists and that the INQUIRE worked.
C
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'INQUIRE failed. Value of IOSTAT was #.' )
         CALL ERRINT ( '#', IOSTAT                              )
         CALL SIGERR ( 'SPICE(INQUIREFAILED)'                   )
         CALL CHKOUT ( 'DAFOPW'                                 )
         RETURN
 
      END IF
 
      IF ( .NOT. EXIST ) THEN
 
         CALL SETMSG ( 'The file # does not exist.'             )
         CALL ERRCH  ( '#', FNAME                               )
         CALL SIGERR ( 'SPICE(FILEDOESNOTEXIST)'                )
         CALL CHKOUT ( 'DAFOPW'                                 )
         RETURN
 
      END IF
C
C     If the file is already open then it is an error.
C
      IF ( OPENED ) THEN
 
         CALL SETMSG ( 'File # already opened.'  )
         CALL ERRCH  ( '#', FNAME                )
         CALL SIGERR ( 'SPICE(DAFOPENCONFLICT)'  )
         CALL CHKOUT ( 'DAFOPW'                  )
         RETURN
 
      END IF
 
C
C     If it hasn't been opened, it needs to be, but only if there
C     is room for another file.
C
      IF ( NFT .EQ. FTSIZE ) THEN
 
         CALL SETMSG ( 'The file table is full, with # entries. '     //
     .                 'Could not open #.'                             )
         CALL ERRINT ( '#', FTSIZE                                     )
         CALL ERRCH  ( '#', FNAME                                      )
         CALL SIGERR ( 'SPICE(DAFFTFULL)'                              )
         CALL CHKOUT ( 'DAFOPW'                                        )
         RETURN
 
      END IF
 
C
C     To open for writing: get a free unit, open the file, and set
C     the number of links to one.
C
C     Look out for:
C
C        -- No free logical units.
C
C        -- Error opening the file.
C
C        -- Invalid NAIF/DAF id word.
C
 
      CALL GETLUN ( LUN )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'DAFOPW' )
         RETURN
      END IF
 
      OPEN ( UNIT   = LUN,
     .       FILE   = FNAME(FCHAR:LCHAR),
     .       ACCESS = 'DIRECT',
     .       RECL   = RECL,
     .       STATUS = 'OLD',
     .       IOSTAT = IOSTAT    )
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'Attempt to open file # failed. Value of '  //
     .                 'IOSTAT was #.'                              )
         CALL ERRCH  ( '#', FNAME                                   )
         CALL ERRINT ( '#', IOSTAT                                  )
         CALL SIGERR ( 'SPICE(DAFOPENFAIL)'                         )
         CALL CHKOUT ( 'DAFOPW'                                     )
         RETURN
 
      END IF
 
C
C     Read the file record and check the NAIF/DAF id word. If
C     there is no match then close the file and signal an error.
C
      READ ( LUN, REC=1, IOSTAT=IOSTAT ) DAFWRD, FND, FNI
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CLOSE ( LUN )
 
         CALL SETMSG ( 'File record not found. Value of IOSTAT was #' )
         CALL ERRINT ( '#', IOSTAT                                    )
         CALL SIGERR ( 'SPICE(DAFFRNOTFOUND)'                         )
         CALL CHKOUT ( 'DAFOPW'                                       )
         RETURN
 
      END IF
 
      IF (        DAFWRD .NE. 'NAIF/DAF'
     .     .AND.  DAFWRD .NE. 'NAIF/NIP' ) THEN
 
         CLOSE ( LUN )
 
         CALL SETMSG ( 'The DAF file id word # was not recognized.' )
         CALL ERRCH  ( '#', DAFWRD          )
         CALL SIGERR ( 'SPICE(DAFNOIDWORD)' )
         CALL CHKOUT ( 'DAFOPW'             )
         RETURN
 
      END IF
 
C
C     If we have made it this far then everything went ok.
C     Update the file table to include information about our
C     newly opened DAF.  Set the output argument HANDLE as well.
C
      NEXT       =  NEXT + 1
      NFT        =  NFT  + 1
      FTHAN(NFT) = -NEXT
      FTLUN(NFT) =  LUN
      FTND(NFT)  =  FND
      FTNI(NFT)  =  FNI
      FTLNK(NFT) =  1
      HANDLE     =  FTHAN(NFT)
C
C     Insert the new handle into our handle set.
C
      CALL INSRTI ( HANDLE, FHLIST )
 
      CALL CHKOUT ( 'DAFOPW' )
      RETURN
 
 
 
 
C$Procedure DAFOPN ( DAF, open new )
 
      ENTRY DAFOPN ( FNAME, ND, NI, IFNAME, RESV, HANDLE )
 
C$ Abstract
C
C     Open a new DAF for subsequent write requests.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               ND
C     INTEGER               NI
C     CHARACTER*(*)         IFNAME
C     INTEGER               RESV
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of DAF to be opened.
C     ND         I   Number of double precision components in summaries.
C     NI         I   Number of integer components in summaries.
C     IFNAME     I   Internal file name.
C     RESV       I   Number of records to reserve.
C     HANDLE     O   Handle assigned to DAF.
C
C$ Detailed_Input
C
C     FNAME       is the name of a new DAF to be created (and
C                 consequently opened for write access).
C
C     ND          is the number of double precision components
C                 in each array summary of the new file.
C
C     NI          is the number of integer components in each
C                 array summary in the new file.
C
C     IFNAME      is the internal file name (containing as many as 60
C                 characters) for the new file. This should uniquely
C                 identify the file.
C
C     RESV        is the number of records in the new file to be
C                 reserved for non-DAF use. The user may reserve
C                 records 2 through (2 + RESV - 1) in the file.
C                 These records are not used to store DAF data,
C                 and are in fact invisible to all DAF routines.
C
C$ Detailed_Output
C
C     HANDLE      is the file handle associated with the file. This
C                 handle is used to identify the file in subsequent
C                 calls to other DAF routines.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     See argument FNAME.
C
C$ Exceptions
C
C     1) If the specified file cannot be opened without exceeding
C        the maximum number of files, the error SPICE(DAFFTFULL)
C        is signalled.
C
C
C     2) If the input argument ND is out of the range [0, 124]
C        or if NI is out of the range [2, 250], the error
C        SPICE(DAFINVALIDPARAMS) is signalled.
C
C
C     3) If
C
C           ND + ( NI + 1 ) / 2   >  125
C
C        the error SPICE(DAFINVALIDPARAMS) is signalled.
C
C
C     4) If the number of records to be reserved is not zero or
C        positive, the error SPICE(DAFNORESV) is signalled.
C
C
C     5) If (for some reason) the file cannot be opened properly,
C        the error SPICE(DAFOPENFAIL) is signalled.
C
C
C     6) If (for some reason) the initial records in the file cannot
C        be written, the error SPICE(DAFWRITEFAIL) is signalled.
C
C
C     7) If no logical units are available, the error will be
C        signalled by routines called by this routine.
C
C
C     8) If the file name is blank, the error SPICE(BLANKFILENAME)
C        is signalled.
C
C$ Particulars
C
C     The DAFs created by DAFOPN have initialized file records but
C     do not yet contain any arrays.  See the DAF Required Reading
C     for a discussion of file records.
C
C$ Examples
C
C     In the following code fragment, DAFOPN is used to open a file,
C     to which a new array is then added.
C
C        CALL DAFOPN   ( FNAME,  ND,  NI,  IFNAME, 0, HANDLE )
C
C        CALL DAFBNA   ( HANDLE, SUM, NAME  )
C        CALL GET_DATA ( DATA,   N,   FOUND )
C
C        DO WHILE ( FOUND )
C           CALL DAFADA   ( DATA, N        )
C           CALL GET_DATA ( DATA, N, FOUND )
C        END DO
C
C        CALL DAFENA
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.M. Lynch      (JPL)
C     H.A. Neilan     (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.1.0, 25-FEB-1993 (JML)
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (HAN) (WLT)
C
C        Updated to allow multiple DAFs to be open for write
C        access simultaneously.  An error in a calling sequence
C        shown in the Examples section was corrected.
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     open new daf
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 2.1.0, 25-FEB-1993 (JML)
C
C        A new variable LUN is used for the logical unit number
C        returned by GETLUN.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN OPEN and
C        INQUIRE statements has been chopped at the last non-blank
C        character.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (HAN) (WLT)
C
C        Updated to allow multiple DAFs to be open for write
C        access simultaneously.
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C        Invalid values of ND and NI are now screened; two new
C        exceptions were added to the $Exceptions header section.
C
C        The calling sequence of DAFADA shown in the first example
C        in the Examples section was reversed; this was fixed.
C
C        Some error messages were changed so that they specify
C        names of relevant DAFs.
C-&
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFOPN' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Check if the file name is blank. If it isn't, then find the
C     index in the input string of the first and last characters
C     of the name.
C
      IF ( FNAME .EQ. ' ' ) THEN
 
         CALL SETMSG ( 'The file name is blank. ' )
         CALL SIGERR ( 'SPICE(BLANKFILENAME)'     )
         CALL CHKOUT ( 'DAFOPN'                   )
         RETURN
 
      END IF
 
      FCHAR = LTRIM ( FNAME )
      LCHAR = RTRIM ( FNAME )
 
C
C     Make sure ND and NI are in range.
C
      IF (  ( ND .LT. 0 )  .OR.  ( ND .GT. MAXND )  ) THEN
 
         CALL SETMSG ( 'ND was #, should be in range [0,#].' )
         CALL ERRINT ( '#', ND                               )
         CALL ERRINT ( '#', MAXND                            )
         CALL SIGERR ( 'SPICE(DAFINVALIDPARAMS)'             )
         CALL CHKOUT ( 'DAFOPN'                              )
         RETURN
 
 
      ELSE IF (  ( NI .LT. 2 )  .OR.  ( NI .GT. MAXNI )  ) THEN
 
         CALL SETMSG ( 'NI was #, should be in range [2,#].' )
         CALL ERRINT ( '#', NI                               )
         CALL ERRINT ( '#', MAXNI                            )
         CALL SIGERR ( 'SPICE(DAFINVALIDPARAMS)'             )
         CALL CHKOUT ( 'DAFOPN'                              )
         RETURN
 
 
      ELSE IF (  ( ND + (NI+1)/2 )  .GT.  MAXSUM  ) THEN
 
         CALL SETMSG ( 'Summary size was #, should not exceed #.' )
         CALL ERRINT ( '#', ND + (NI+1)/2                         )
         CALL ERRINT ( '#', MAXSUM                                )
         CALL SIGERR ( 'SPICE(DAFINVALIDPARAMS)'                  )
         CALL CHKOUT ( 'DAFOPN'                                   )
         RETURN
 
      END IF
 
 
C
C     The file can be opened only if there is room for another file.
C
      IF ( NFT .EQ. FTSIZE ) THEN
 
         CALL SETMSG ( 'The file table is full, with # entries. '     //
     .                 'Could not open #.'                             )
         CALL ERRINT ( '#', FTSIZE                                     )
         CALL ERRCH  ( '#', FNAME                                      )
         CALL SIGERR ( 'SPICE(DAFFTFULL)'                              )
         CALL CHKOUT ( 'DAFOPN'                                        )
         RETURN
 
C
C     The user must reserve some non-negative number of records.
C
      ELSE IF ( RESV .LT. 0 ) THEN
 
         CALL SETMSG ( 'An attempt was made to reserve a negative '   //
     .                 'number (#) of records.'                        )
         CALL ERRINT ( '#', RESV                                       )
         CALL SIGERR ( 'SPICE(DAFNORESV)'                              )
         CALL CHKOUT ( 'DAFOPN'                                        )
         RETURN
 
C
C     To open a new file: get a free unit, open the file, write
C     the file record, write an empty pair of summary/name records,
C     and set the number of links to one.
C
C     Look out for:
C
C        -- No free logical units.
C
C        -- Error opening the file.
C
C        -- Error writing to the file.
C
C     If anything goes wrong after the file has been opened, delete
C     the file.
C
 
      ELSE
 
         CALL GETLUN ( LUN )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'DAFOPN' )
            RETURN
         END IF
 
         OPEN ( UNIT   = LUN,
     .          FILE   = FNAME(FCHAR:LCHAR),
     .          ACCESS = 'DIRECT',
     .          RECL   = RECL,
     .          STATUS = 'NEW',
     .          IOSTAT = IOSTAT    )
 
         IF ( IOSTAT .NE. 0 ) THEN
            CALL SETMSG ( 'Attempt to open file # failed. Value '     //
     .                    'of IOSTAT was #.'                           )
            CALL ERRCH  ( '#', FNAME                                   )
            CALL ERRINT ( '#', IOSTAT                                  )
            CALL SIGERR ( 'SPICE(DAFOPENFAIL)'                         )
            CALL CHKOUT ( 'DAFOPN'                                     )
            RETURN
 
         ELSE
            IFN    = IFNAME
            FND    = ND
            FNI    = NI
            FWARD  = RESV + 2
            BWARD  = FWARD
            CREC   = ' '
            CALL CLEARD ( 128, DREC )
            CALL DAFRWA ( FWARD+2, 1, FREE )
 
            WRITE ( LUN, REC=1, IOSTAT=IOSTAT ) 'NAIF/DAF',
     .                                           FND,
     .                                           FNI,
     .                                           IFN,
     .                                           FWARD,
     .                                           BWARD,
     .                                           FREE
 
            IF ( IOSTAT .EQ. 0 ) THEN
               WRITE ( LUN, REC=FWARD, IOSTAT=IOSTAT ) DREC
 
               IF ( IOSTAT .EQ. 0 ) THEN
                  WRITE ( LUN, REC=FWARD+1, IOSTAT=IOSTAT ) CREC
               END IF
            END IF
 
            IF ( IOSTAT .NE. 0 ) THEN
 
               CLOSE ( UNIT   =  LUN,
     .                 STATUS = 'DELETE' )
 
               CALL SETMSG ( 'Attempt to write file # failed. '       //
     .                       'Value of IOSTAT was #.'                  )
               CALL ERRCH  ( '#', FNAME                                )
               CALL ERRINT ( '#', IOSTAT                               )
               CALL SIGERR ( 'SPICE(DAFWRITEFAIL)'                     )
               CALL CHKOUT ( 'DAFOPN'                                  )
               RETURN
 
            ELSE
C
C              Update the file table to include information about
C              our newly opened DAF.  Set the output argument
C              HANDLE as well.
C
               NEXT       =  NEXT + 1
               NFT        =  NFT + 1
               FTHAN(NFT) = -NEXT
               FTLUN(NFT) =  LUN
               FTND(NFT)  =  FND
               FTNI(NFT)  =  FNI
               FTLNK(NFT) =  1
               HANDLE     =  FTHAN(NFT)
C
C              Insert the new handle into our handle set.
C
               CALL INSRTI ( HANDLE, FHLIST )
 
 
            END IF
 
         END IF
 
      END IF
 
      CALL CHKOUT ( 'DAFOPN' )
      RETURN
 
 
 
C$Procedure DAFCLS ( DAF, close )
 
      ENTRY DAFCLS ( HANDLE )
 
C$ Abstract
C
C     Close the DAF associated with a given handle.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of DAF to be closed.
C
C$ Detailed_Input
C
C     HANDLE      is the handle assigned to a DAF by a previous
C                 call to DAFOPR, DAFOPW, or DAFOPN.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified handle does not belong to a DAF
C        that is currently open, nothing happens.
C
C$ Particulars
C
C     Because DAFAH and its entry points must keep track of what
C     files are open at any given time, it is important that DAF
C     files be closed only with DAFCLS, to prevent the remaining
C     DAF routines from failing, sometimes mysteriously.
C
C     Note that when a file is opened more than once for read access,
C     DAFOPR returns the same handle each time it is re-opened.
C     Each time the file is closed, DAFCLS checks to see if any other
C     claims on the file are still active before physically closing
C     the file.
C
C$ Examples
C
C     In the following code fragment, the arrays in a file are
C     examined in order to determine whether the file contains
C     any arrays whose names begin with the word TEST.
C     The complete names for these arrays are printed to
C     the screen. The file is closed at the end of the search.
C
C        CALL DAFOPR ( FNAME, HANDLE )
C        CALL DAFBFS ( HANDLE )
C        CALL DAFFNA ( FOUND  )
C
C        DO WHILE ( FOUND )
C           CALL DAFGN ( NAME )
C
C           IF ( NAME(1:5) .EQ. 'TEST ' ) THEN
C              WRITE (*,*) NAME
C           END IF
C
C           CALL DAFFNA ( FOUND )
C        END DO
C
C        CALL DAFCLS ( HANDLE )
C
C     Note that if the file has been opened already by a DAF routine
C     at some other place in the calling program, it remains open.
C     This makes it possible to examine files that have been opened for
C     use by other modules without interfering with the operation of
C     those routines.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.2, 25-FEB-1993 (JML)
C
C        A minor error in the particulars section of the header was
C        corrected.  It formely stated that a file could be open more
C        than once for read or write access instead of just read access.
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        This routine was updated so that it now keeps current the set
C        of DAF handles returned by DAFHOF.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
 
C$ Index_Entries
C
C     close daf
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 03-SEP-1991 (NJB) (WLT)
C
C        Upgraded to support file handle checking routines
C        DAFHOF and DAFSIH.  DAFCLS now initializes the file
C        handle list if necessary, and removes from the list
C        the handles of files it closes.
C
C-&
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFCLS' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Is this file even open? If so, decrement the number of links
C     to the file. If the number of links drops to zero, physically
C     close the file and remove it from the file buffer.
C
C     If the file is not open: no harm, no foul.
C
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
 
      IF ( FINDEX .GT. 0 ) THEN
 
         FTLNK(FINDEX) = FTLNK(FINDEX) - 1
 
 
         IF ( FTLNK(FINDEX) .EQ. 0 ) THEN
 
            CLOSE ( FTLUN(FINDEX) )
 
            DO 50001
     .         I = FINDEX, NFT - 1
               FTHAN(I) = FTHAN(I+1)
               FTLUN(I) = FTLUN(I+1)
               FTLNK(I) = FTLNK(I+1)
               FTND(I)  = FTND(I+1)
               FTNI(I)  = FTNI(I+1)
50001       CONTINUE
 
            NFT = NFT - 1
C
C           Delete the handle from our handle set.
C
            CALL REMOVI ( HANDLE, FHLIST )
 
         END IF
 
 
      END IF
 
      CALL CHKOUT ( 'DAFCLS' )
      RETURN
 
 
 
 
 
C$Procedure DAFHSF ( DAF, handle to summary format )
 
      ENTRY DAFHSF ( HANDLE, ND, NI )
 
C$ Abstract
C
C     Return the summary format associated with a handle.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C     INTEGER               ND
C     INTEGER               NI
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of a DAF.
C     ND         O   Number of double precision components in summaries.
C     NI         O   Number of integer components in summaries.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF previously
C                 opened by a call to DAFOPR, DAFOPW, or DAFOPN.
C
C$ Detailed_Output
C
C     ND,
C     NI          are the numbers of double precision and integer
C                 components, respectively, in each array summary
C                 in the specified file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified handle does not belong to any file that is
C        currently known to be open, the error SPICE(DAFNOSUCHHANDLE)
C        is signalled.
C
C$ Particulars
C
C     The summary format must be known in order to pack or unpack
C     an array summary.  See the DAF Required Reading for a discussion
C     of summary formats.
C
C$ Examples
C
C     1)  Find the number of d.p. `words' in a DAF having an
C         arbitrary summary format.
C
C
C                  PROGRAM NWORDS
C            C
C            C     Count the number of d.p. words of data in a
C            C     DAF.  Exclude array summaries, reserved records,
C            C     the file record, and character records.
C            C
C                  INTEGER               FILEN
C                  PARAMETER           ( FILEN  = 128 )
C
C                  INTEGER               MAXND
C                  PARAMETER           ( MAXND  = 124 )
C
C                  INTEGER               MAXNI
C                  PARAMETER           ( MAXNI  = 250 )
C
C                  INTEGER               MAXSUM
C                  PARAMETER           ( MAXSUM = 125 )
C
C                  CHARACTER*(FILEN)     DAF
C
C                  DOUBLE PRECISION      DC    ( MAXND  )
C                  DOUBLE PRECISION      SUM   ( MAXSUM )
C
C                  INTEGER               FA
C                  INTEGER               HANDLE
C                  INTEGER               IA
C                  INTEGER               IC    ( MAXNI )
C                  INTEGER               N
C                  INTEGER               ND
C                  INTEGER               NI
C
C                  LOGICAL               FOUND
C
C                  DATA                  N   /  0  /
C
C                  WRITE (*,*)          'Enter file name'
C                  READ  (*,FMT='(A)')   DAF
C
C            C
C            C     Open the DAF and find the summary format.
C            C
C                  CALL DAFOPR ( DAF,    HANDLE )
C                  CALL DAFHSF ( HANDLE, ND, NI )
C
C            C
C            C     Start a forward search and examine each array in
C            C     turn.
C            C
C                  CALL DAFBFS ( HANDLE )
C                  CALL DAFFNA ( FOUND  )
C
C                  DO WHILE ( FOUND )
C            C
C            C        Obtain the array summary, unpack it, and get
C            C        the initial and final array addresses from
C            C        the integer descriptor component.
C            C
C                     CALL DAFGS ( SUM )
C                     CALL DAFUS ( SUM, ND, NI, DC, IC )
C
C                     IA  =  IC ( NI - 1 )
C                     FA  =  IC ( NI     )
C
C                     N   =  FA - IA + 1 + N
C
C                     CALL DAFFNA ( FOUND )
C
C                  END DO
C
C                  WRITE (*,*) 'Number of d.p. words is ', N
C
C                  END
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.3, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.2, 03-SEP-1990 (NJB)
C
C        Example added to the $Examples section.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     handle to daf summary format
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFHSF' )
      END IF
 
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
      IF ( FINDEX .GT. 0 ) THEN
         ND = FTND(FINDEX)
         NI = FTNI(FINDEX)
 
      ELSE
         CALL SETMSG ( 'There is no DAF open with handle = #' )
         CALL ERRINT ( '#', HANDLE                            )
         CALL SIGERR ( 'SPICE(DAFNOSUCHHANDLE)'               )
      END IF
 
      CALL CHKOUT ( 'DAFHSF' )
      RETURN
 
 
 
 
C$Procedure DAFHLU ( DAF, handle to logical unit )
 
      ENTRY DAFHLU ( HANDLE, UNIT )
 
C$ Abstract
C
C     Return the logical unit associated with a handle.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C     INTEGER               UNIT
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of a DAF.
C     UNIT       O   Corresponding logical unit.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF previously
C                 opened by a call to DAFOPR, DAFOPW, or DAFOPN.
C
C$ Detailed_Output
C
C     UNIT        is the Fortran logical unit to which the file is
C                 connected.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified handle does not belong to any file that is
C        currently known to be open, the error SPICE(DAFNOSUCHHANDLE)
C        is signalled.
C
C$ Particulars
C
C     The best reason for knowing the logical unit to which a DAF
C     is connected is to read or write from the records reserved in a
C     file. Since these records are by definition invisible to the DAF
C     routines, you must read and write them directly.
C
C$ Examples
C
C     In the following code fragment, the first reserved record in
C     a newly created DAF is used to store the name and address
C     of the person who created it.
C
C        CALL DAFOPN ( FNAME, 3, 6, IFNAME, 5, HANDLE )
C        CALL DAFHLU ( HANDLE, UNIT )
C
C        WRITE (UNIT,REC=2) 'Ellis Wyatt, JPL ',
C       .                   '4800 Oak Grove Drive ',
C       .                   'Room 301-125A ',
C       .                   'Pasadena, CA 91109'
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     daf handle to logical unit
C
C-&
 
 
C$ Revisions
C
C-     Beta Version 1.1.0, 1-NOV-1989 (RET)
C
C        DAFHLU now only checks in and checks out if the one exception
C        occurs. The purpose of this change was to help speed up a
C        routine that gets called constantly by higher level DAF
C        routines.
C
C-&
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
      IF ( FINDEX .GT. 0 ) THEN
         UNIT = FTLUN(FINDEX)
 
      ELSE
         CALL CHKIN  ( 'DAFHLU' )
         CALL SETMSG ( 'There is no file open with handle = #' )
         CALL ERRINT ( '#', HANDLE )
         CALL SIGERR ( 'SPICE(DAFNOSUCHHANDLE)' )
         CALL CHKOUT ( 'DAFHLU' )
      END IF
 
      RETURN
 
 
 
 
 
C$Procedure DAFLUH ( DAF, logical unit to handle )
 
      ENTRY DAFLUH ( UNIT, HANDLE )
 
C$ Abstract
C
C     Return the handle associated with a logical unit.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               UNIT
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     UNIT       I   Logical unit connected to a DAF.
C     HANDLE     O   Corresponding handle.
C
C$ Detailed_Input
C
C     UNIT        is the logical unit to which a DAF has been
C                 connected by a previous call to DAFOPR, DAFOPW, or
C                 DAFOPN.
C
C$ Detailed_Output
C
C     HANDLE      is the handle associated with the file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified unit is not connected to any file that is
C        currently known to be open, the error SPICE(DAFNOSUCHUNIT)
C        is signalled.
C
C$ Particulars
C
C     It is unlikely, but possible, that a calling program would know
C     the logical unit to which a file is connected without knowing the
C     handle associated with the file. DAFLUH is provided mostly for
C     completeness.
C
C$ Examples
C
C     In the following code fragment, the handle associated with
C     a DAF is retrieved using the logical unit to which the
C     file is connected. The handle is then used to determine the
C     name of the file.
C
C        CALL DAFLUH ( UNIT,   HANDLE )
C        CALL DAFHFN ( HANDLE, FNAME )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     logical unit to daf handle
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFLUH' )
      END IF
 
      FINDEX = ISRCHI ( UNIT, NFT, FTLUN )
 
      IF ( FINDEX .GT. 0 ) THEN
         HANDLE = FTHAN(FINDEX)
 
      ELSE
         CALL SETMSG ( 'There is no file open with unit = #' )
         CALL ERRINT ( '#', UNIT )
         CALL SIGERR ( 'SPICE(DAFNOSUCHUNIT)' )
      END IF
 
      CALL CHKOUT ( 'DAFLUH' )
      RETURN
 
 
 
C$Procedure DAFHFN ( DAF, handle to file name )
 
      ENTRY DAFHFN ( HANDLE, FNAME )
 
C$ Abstract
C
C     Return the name of the file associated with a handle.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C     CHARACTER*(*)         FNAME
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of a DAF.
C     FNAME      O   Corresponding file name.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF previously
C                 opened by a call to DAFOPR, DAFOPW, or DAFOPN.
C
C$ Detailed_Output
C
C     UNIT        is the name of the file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified handle does not belong to any file that is
C        currently known to be open, the error SPICE(DAFNOSUCHHANDLE)
C        is signalled.
C
C     2) If the INQUIRE fails, the error SPICE(INQUIREFAILED)
C        is signalled.
C
C$ Particulars
C
C     It may be desirable to recover the names of one or more DAF
C     files in a different part of the program from the one in which
C     they were opened. Note that the names returned by DAFHFN may
C     not be identical to the names used to open the files. Under
C     most operating systems, a particular file can be accessed using
C     many different names. DAFHFN returns one of them.
C
C$ Examples
C
C     In the following code fragment, the name of a DAF is
C     recovered using the handle associated with the file.
C
C        CALL DAFOPR ( 'sample.DAF', HANDLE )
C         .
C         .
C
C        CALL DAFHFN ( HANDLE, FNAME )
C
C     Depending on the circumstances (operating system, compiler,
C     default directory) the value of FNAME might resemble any of
C     the following:
C
C        'USER$DISK:[WYATT.IMAGES]SAMPLE.DAF;4'
C
C        '/wyatt/images/sample.DAF'
C
C        'A:\IMAGES\SAMPLE.DAF'
C
C     On the other hand, it might not.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     J.M. Lynch      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 25-FEB-1993 (JML)
C
C        IOSTAT is checked after the INQUIRE statement.
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
 
C$ Index_Entries
C
C     daf handle to file name
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 25-FEB-1993 (JML)
C
C        IOSTAT is checked after the INQUIRE statement.
C
C-&
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFHFN' )
      END IF
 
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
      IF ( FINDEX .LE. 0 ) THEN
 
         CALL SETMSG ( 'There is no file open with handle = #' )
         CALL ERRINT ( '#', HANDLE              )
         CALL SIGERR ( 'SPICE(DAFNOSUCHHANDLE)' )
         CALL CHKOUT ( 'DAFHFN'                 )
         RETURN
 
      END IF
 
      INQUIRE ( UNIT   = FTLUN(FINDEX),
     .          NAME   = FNAME,
     .          IOSTAT = IOSTAT          )
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'INQUIRE failed. Value of IOSTAT was #.' )
         CALL ERRINT ( '#', IOSTAT                              )
         CALL SIGERR ( 'SPICE(INQUIREFAILED)'                   )
         CALL CHKOUT ( 'DAFHFN'                                 )
         RETURN
 
      END IF
 
 
      CALL CHKOUT ( 'DAFHFN' )
      RETURN
 
 
 
C$Procedure DAFFNH ( DAF, file name to handle )
 
      ENTRY DAFFNH ( FNAME, HANDLE )
 
C$ Abstract
C
C     Return handle associated with a file name.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     DAF
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of a DAF.
C     HANDLE     O   Corresponding handle.
C
C$ Detailed_Input
C
C     FNAME       is the name of a DAF previously opened by a
C                 call to DAFOPR, DAFOPW, or DAFOPN.
C
C$ Detailed_Output
C
C     HANDLE      is the handle associated with the file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified name does not specify any file currently known
C        to be open, the error SPICE(DAFNOSUCHFILE) is signalled.
C
C     2) If the file does not exist, the error SPICE(FILEDOESNOTEXIST)
C        is signalled.
C
C     3) If the INQUIRE fails, the error SPICE(INQUIREFAILED)
C        is signalled.
C
C     4) If the file name is blank, the error SPICE(BLANKFILENAME)
C        is signalled.
 
C$ Particulars
C
C     It is sometimes easier to work with file names (which are
C     meaningful, and often predictable) than with file handles
C     (which are neither), especially in interactive situations.
C     However, nearly every DAF routines requires that you use file
C     handles to refer to files. DAFFNH is provided to bridge the gap
C     between the two representations.
C
C$ Examples
C
C     In the following code fragment, the handle associated with a
C     DAF is recovered using the name of the file.
C
C        CALL DAFOPR ( 'sample.DAF', HANDLE )
C         .
C         .
C
C        CALL DAFFNH ( 'sample.DAF', HANDLE )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     J.M. Lynch      (JPL)
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is open now also
C        checks that the file exists. Two new exceptions were added as
C        a result of this change.
C
C        A RETURN statement was added after the error signalled when
C        the file is not open.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN INQUIRE
C        statement has been chopped at the last non-blank character.
C
C-    SPICELIB Version 1.1.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.1,  18-SEP-1991 (HAN)
C
C        The Revisions section was incorrectly named Version. This has
C        been fixed.
C
C-    SPICELIB Version 1.1.0,  5-NOV-1990 (HAN)
C
C        Call to CHKIN was corrected. The module was checking in
C        as 'DAFFHN'.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     file name to daf handle
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 25-FEB-1993 (JML)
C
C        The INQUIRE statement that checks if the file is open now also
C        checks that the file exists. Two new exceptions were added as
C        a result of this change.
C
C        A RETURN statement was added after the error signalled when
C        the file is not open.
C
C        The file name is checked to see if it is blank.
C
C        The file name string that is passed to the FORTRAN INQUIRE
C        statement has been chopped at the last non-blank character.
C
C-    SPICELIB Version 1.1.0,  5-NOV-1990 (HAN)
C
C        Call to CHKIN was corrected. The module was checking in
C        as 'DAFFHN' instead of 'DAFFNH'.
C
C-&
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFFNH' )
      END IF
 
C
C     Check if the file name is blank. If it isn't, then find the
C     index in the input string of the first and last characters
C     of the name.
C
      IF ( FNAME .EQ. ' ' ) THEN
 
         CALL SETMSG ( 'The file name is blank. ' )
         CALL SIGERR ( 'SPICE(BLANKFILENAME)'     )
         CALL CHKOUT ( 'DAFFNH'                   )
         RETURN
 
      END IF
 
      FCHAR = LTRIM ( FNAME )
      LCHAR = RTRIM ( FNAME )
 
C
C     INQUIRE as to whether the file exists, if it is open, and what
C     logical unit number it is attached to.
C
      INQUIRE ( FILE   = FNAME(FCHAR:LCHAR),
     .          EXIST  = EXIST,
     .          OPENED = OPENED,
     .          NUMBER = NUMBER,
     .          IOSTAT = IOSTAT  )
 
C
C     First make sure that the file exists and that the INQUIRE worked.
C
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'INQUIRE failed. Value of IOSTAT was #.' )
         CALL ERRINT ( '#', IOSTAT                              )
         CALL SIGERR ( 'SPICE(INQUIREFAILED)'                   )
         CALL CHKOUT ( 'DAFFNH'                                 )
         RETURN
 
      END IF
 
      IF ( .NOT. EXIST ) THEN
 
         CALL SETMSG ( 'The file # does not exist.'             )
         CALL ERRCH  ( '#', FNAME                               )
         CALL SIGERR ( 'SPICE(FILEDOESNOTEXIST)'                )
         CALL CHKOUT ( 'DAFFNH'                                 )
         RETURN
 
      END IF
 
C
C     If the file was opened then determine the index of the
C     logical unit number in the file tables.
C
      IF ( .NOT. OPENED ) THEN
 
         CALL SETMSG ( 'There is no file in the table with file '  //
     .                 'name = #'             )
         CALL ERRCH  ( '#', FNAME             )
         CALL SIGERR ( 'SPICE(DAFNOSUCHFILE)' )
         CALL CHKOUT ( 'DAFFNH'               )
         RETURN
 
      END IF
 
      FINDEX = ISRCHI ( NUMBER, NFT, FTLUN )
 
      IF ( FINDEX .GT. 0 ) THEN
 
         HANDLE = FTHAN(FINDEX)
 
      ELSE
 
         CALL SETMSG ( 'There is no file in the table with file '  //
     .                 'name = #'             )
         CALL ERRCH  ( '#', FNAME             )
         CALL SIGERR ( 'SPICE(DAFNOSUCHFILE)' )
 
      END IF
 
      CALL CHKOUT ( 'DAFFNH' )
      RETURN
 
 
 
 
 
 
 
C$Procedure      DAFHOF ( DAF, handles of open files )
 
      ENTRY DAFHOF ( FHSET )
 
C$ Abstract
C
C     Return a SPICELIB set containing the handles of all currently
C     open DAFS.
C
C$ Required_Reading
C
C     DAF
C     SETS
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               LBCELL
C     PARAMETER           ( LBCELL = -5 )
C
C     INTEGER               FHSET ( LBCELL : * )
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FHSET      O   A set containing handles of currently open DAFS.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     FHSET          is a SPICELIB set containing the file handles of
C                    all currently open DAFs.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the set FHSET is not initialized, the error will be
C         diagnosed by routines called by this routine.
C
C     2)  If the set FHSET is too small to accommodate the set of
C         handles to be returned, the error will be diagnosed by
C         routines called by this routine.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine allows subroutines to test file handles for
C     validity before performing operations on them, such as
C     finding the name of the file designated by a handle.  Many
C     DAF operations on handles cause errors to be signalled if
C     the handles are invalid.
C
C$ Examples
C
C     1)  Find out how may DAFs are open for writing.
C
C            C
C            C    Find out which DAFs are open.
C            C
C                 CALL DAFHOF  ( FHSET )
C
C            C
C            C    Count the ones open for writing.  These have
C            C    negative file handles.
C            C
C                 COUNT = 0
C
C                 DO I = 1, CARDC(FHSET)
C
C                    IF ( FHSET(I) .LT. 0 ) THEN
C                       COUNT = COUNT + 1
C                    END IF
C
C                 END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 03-SEP-1991 (NJB) (WLT)
C
C-&
 
C$ Index_Entries
C
C     return the set of handles for open daf files
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFHOF' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Just stuff our local list into the set.
C
      CALL COPYI ( FHLIST, FHSET )
 
      CALL CHKOUT ( 'DAFHOF' )
      RETURN
 
 
 
 
 
C$Procedure      DAFSIH ( DAF, signal invalid handles )
 
      ENTRY DAFSIH ( HANDLE, ACCESS )
 
C$ Abstract
C
C     Signal an error if a DAF file handle does not designate a DAF
C     that is open for a specified type of access.
C
C$ Required_Reading
C
C     DAF
C     ERROR
C     SETS
C
C$ Keywords
C
C     DAF
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C     CHARACTER*(*)         ACCESS
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   HANDLE to be validated.
C     ACCESS     I   String indicating access type.
C
C$ Detailed_Input
C
C     HANDLE         is a DAF handle to validate.  For HANDLE to be
C                    considered valid, it must specify a DAF that is
C                    open for the type of access specified by the input
C                    argument ACCESS.
C
C
C     ACCESS         is a string indicating the type of access that
C                    the DAF specified by the input argument HANDLE
C                    must be open for.  The values of ACCESS may be
C
C
C                       'READ'      File must be open for read access
C                                   by DAF routines.  All open DAFs
C                                   may be read.  The DAF may have
C                                   been opened by DAFOPR, DAFOPW, or
C                                   DAFOPN.
C
C
C                       'WRITE'     File must be open for write access
C                                   by DAF routines.  The file must have
C                                   been opened by DAFOPN or DAFOPW.
C
C                                   Note that files open for write
C                                   access may be read as well as
C                                   written.
C
C
C                    Leading and trailing blanks in ACCESS are ignored,
C                    and case is not significant.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the input argument ACCESS has an unrecognized value,
C         the error SPICE(INVALIDOPTION) is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine signals the error SPICE(DAFINVALIDACCESS) if the
C     DAF designated by the the input argument HANDLE is not open
C     for the specified type of access.  If HANDLE does not designate
C     an open DAF, the error SPICE(DAFNOSUCHHANDLE) is signalled.
C
C     This routine allows subroutines to test file handles for
C     validity before performing operations on them, such as
C     finding the name of the file designated by a handle.  Many
C     DAF operations on handles may cause unpredictable program
C     behavior if the handles are invalid.  This routine should
C     be used in situations where the appropriate action to take upon
C     determining that a handle is invalid is to signal an error.
C     DAFSIH centralizes the error response for this type of error in a
C     single routine.
C
C     In cases where it is necessary to determine the validity of a
C     file handle, but it is not an error for the handle to refer
C     to a closed file, the entry point DAFHOF should be used instead
C     of DAFSIH.
C
C$ Examples
C
C     1)  Add data to a DAF specified by a file handle.  Signal an
C         error if the file is not open for writing.  Check the
C         SPICELIB error status function FAILED after calling
C         DAFSIH, so that the the routine will return if DAFSIH
C         signalled an error (we're presuming that this code
C         fragment would be used in a subroutine).
C
C            C
C            C     Check that HANDLE is valid, then add data to the
C            C     file specified by HANDLE.
C            C
C                  CALL DAFSIH  (  HANDLE, 'WRITE' )
C
C                  IF ( FAILED() ) THEN
C                     RETURN
C                  END IF
C
C                  CALL DAFBNA (  HANDLE,  SUM,    NAME )
C                  CALL DAFADA (  DATA,    N            )
C                  CALL DAFENA
C
C
C     2)  Find the size of an array in a DAF specified by a file
C         handle.  Signal an error if the file is not open for reading.
C
C            C
C            C     Check that HANDLE is valid, then obtain the
C            C     current array summary and compute the size of
C            C     the current array.
C            C
C                  CALL DAFSIH  ( HANDLE, 'READ' )
C
C                  IF ( FAILED() ) THEN
C                     RETURN
C                  END IF
C
C            C
C            C     Obtain the summary format, then the integer and d.p.
C            C     components of the summary.  Finally, compute the
C            C     array length.
C            C
C                  CALL DAFHSF (  HANDLE, ND, NI          )
C                  CALL DAFGS  (  SUMMRY                  )
C                  CALL DAFUS  (  SUMMRY, ND, NI, DC, IC  )
C
C                  IA      =  IC( NI - 1 )
C                  FA      =  IC( NI     )
C                  LENGTH  =  FA  -  IA  +  1
C
C
C
C
C     3)  Make sure that a file handle designates an open DAF.  Signal
C         an error if it does not.
C
C         Note that if a DAF is open at all, read access is allowed.
C
C
C                  CALL DAFSIH ( HANDLE, 'READ' )
C
C                  IF ( FAILED() ) THEN
C                     RETURN
C                  END IF
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     J.M. Lynch     (JPL)
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.2.0, 25-FEB-1993 (JML)
C
C        IOSTAT is now checked after the INQUIRE statement.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 03-SEP-1991 (NJB) (WLT)
C
C-&
 
C$ Index_Entries
C
C     signal an error for invalid daf handles
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.2.0, 25-FEB-1993 (JML)
C
C        IOSTAT is now checked after the INQUIRE statement.
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFSIH' )
      END IF
 
C
C     Initialize the handle list, if necessary.
C
      IF ( FIRST ) THEN
 
         CALL SSIZEI ( FTSIZE, FHLIST )
         FIRST = .FALSE.
 
      END IF
 
C
C     Get an upper case, left-justified copy of ACCESS.
C
      CALL LJUST ( ACCESS, ACC )
      CALL NUCASE ( ACC,    ACC )
 
C
C     Make sure we recognize the access type specified by the caller.
C
      IF (  ( ACC .NE. 'READ' ) .AND. ( ACC .NE. 'WRITE' )  ) THEN
 
         CALL SETMSG ( 'Unrecognized access type.  Type was #. ' )
         CALL ERRCH  ( '#', ACCESS                               )
         CALL SIGERR ( 'SPICE(INVALIDOPTION)'                    )
         CALL CHKOUT ( 'DAFSIH'                                  )
         RETURN
 
      END IF
 
C
C     See whether the input handle is in our list at all.  It's
C     unlawful for the handle to be absent.  All open DAFs are
C     readable, so in the case that ACC is 'READ', we're done if
C     the DAF is open.
C
      IF (  .NOT.  ELEMI ( HANDLE, FHLIST )  )  THEN
 
         CALL SETMSG ( 'Handle # is not attached to an open DAF.' )
         CALL ERRINT ( '#', HANDLE                                )
         CALL SIGERR ( 'SPICE(DAFNOSUCHHANDLE)'                   )
         CALL CHKOUT ( 'DAFSIH'                                   )
         RETURN
 
C
C     If the access type is 'WRITE', the DAF must be open for writing.
C
      ELSE IF (  ( ACC .EQ. 'WRITE' ) .AND. ( HANDLE .GE. 0 )  ) THEN
 
         FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
         INQUIRE ( UNIT   = FTLUN(FINDEX),
     .             NAME   = DAF,
     .             IOSTAT = IOSTAT           )
 
         IF ( IOSTAT .EQ. 0 ) THEN
 
            CALL SETMSG ( 'DAF not open for write.  Handle = #, ' //
     .                    'file = #'                                )
            CALL ERRINT ( '#', HANDLE                               )
            CALL ERRCH  ( '#', DAF                                  )
            CALL SIGERR ( 'SPICE(DAFINVALIDACCESS)'                 )
            CALL CHKOUT ( 'DAFSIH'                                  )
            RETURN
 
         ELSE
 
            CALL SETMSG ( 'DAF not open for write.  Handle = #. ' //
     .                    'Also, INQUIRE failed with IOSTAT = #.'   )
            CALL ERRINT ( '#', HANDLE                               )
            CALL ERRINT ( '#', IOSTAT                               )
            CALL SIGERR ( 'SPICE(DAFINVALIDACCESS)'                 )
            CALL CHKOUT ( 'DAFSIH'                                  )
            RETURN
 
         END IF
 
      END IF
 
C
C     The DAF's handle is o.k.
C
      CALL CHKOUT ( 'DAFSIH' )
      RETURN
      END
