 
C$Procedure DAFRCR ( DAF, read character record )
 
      SUBROUTINE DAFRCR ( HANDLE, RECNO, CREC )
 
C$ Abstract
C
C     Read the contents of a character record from a DAF.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               RECNO
      CHARACTER*(*)         CREC
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of DAF.
C     RECNO      I   Record number of character record.
C     CREC       O   Character record.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF.
C
C     RECNO       is the record number of a character record within
C                 the file.
C
C$ Detailed_Output
C
C     CREC        contains the first 1000 characters of the specified
C                 record from the specified file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the declared length of CREC is not 1000 characters,
C        the error SPICE(DAFBADRECLEN) is signalled.
C
C     2) If the specified record cannot (for some reason) be read,
C        the error SPICE(DAFCRNOTFOUND) is signalled.
C
C$ Particulars
C
C     Unlike double precision records, character records are
C     not buffered. Also, while failing to find a specific double
C     precision record is indicated through the calling sequence,
C     failing to find a character record results in an error.
C
C$ Examples
C
C     In the following example, matching summary and name records are
C     read from a DAF:
C
C        CALL DAFRDR ( HANDLE, NEXT,   DREC, FOUND )
C        CALL DAFRCR ( HANDLE, NEXT+1, CREC        )
C
C     Note that a character record always immediately follows a summary
C     record.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     read daf character record
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               UNIT
      INTEGER               IOSTAT
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFRCR' )
      END IF
 
      CALL DAFHLU ( HANDLE, UNIT )
 
      IF ( LEN ( CREC ) .NE. 1000 ) THEN
         CALL SETMSG ( 'Expected length of character record is 1000. '//
     .                 'Passed string has length #' )
         CALL ERRINT ( '#', LEN( CREC ) )
         CALL SIGERR ( 'SPICE(DAFBADCRECLEN)' )
 
      ELSE
         READ (UNIT,
     .         REC=RECNO,
     .         IOSTAT=IOSTAT) CREC
 
         IF ( IOSTAT .NE. 0 ) THEN
            CALL SETMSG ( 'Could not read record #. IOSTAT was #.' )
            CALL ERRINT ( '#', RECNO                               )
            CALL ERRINT ( '#', IOSTAT                              )
            CALL SIGERR ( 'SPICE(DAFCRNOTFOUND)'                   )
         END IF
      END IF
 
      CALL CHKOUT ( 'DAFRCR' )
      RETURN
      END
