C$Procedure                     DPMAX ( Largest DP number )
 
      DOUBLE PRECISION FUNCTION DPMAX ( )
 
C$ Abstract
C
C     Return the value of the largest (positive) number representable
C     in a double precision variable.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONSTANTS
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     The function returns the value of the largest (positive) number
C     that can be represented in a double precision variable.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     The function returns the value of the largest (positive) number
C     that can be represented in a double precision variable.
C
C     This varies from machine to machine. Values for several popular
C     machines are shown below.
C
C     Environment: VAX/VMS, VAX FORTRAN
C     Value:       1.70141183460469229D+38
C
C     Environment: Sun, Sun FORTRAN
C     Value:       DFLMAX ()
C
C     Environment: PC, MS (Microsoft) FORTRAN
C     Value:       1.79769313486231D+308
C
C     Environment: Macintosh, Language Systems FORTRAN
C     Value:       1.7D+308
C
C     Environment: PC, Lahey F77 EM/32 Version 4.0
C     Value:       1.79769313486231D+308
C
C     Environment: HP-UX 9000/750, FORTRAN/9000 Series 700 computers
C     Value:       1.79769313486231D+308
C
C
C     References for the values above are listed in the
C     Literature_References section of the header.
C
C     Sun Note :
C
C      At the present time the function value for DFLMAX and
C      D_MAX_NORMAL are equivalent. Due to an error in the Sun
C      documentation DFLMAX was not described within the
C      RANGE Sun FORTRAN Library Routines documentation. If
C      D_MAX_NORMAL is a replacement for the function DFLMAX,
C      NAIF will update this module.
C
C     IBM PC Note:
C
C      Although the Microsoft FORTRAN documentation lists
C      1.797693134862316D+308 as the maximum double precision number,
C      it produced a compilation error. As a result, we changed the
C      number to 1.79769313486231D+308, which did not produce a
C      compilation error.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The function always returns a constant value, set by the user
C     prior to compilation.
C
C$ Examples
C
C     The following code fragments illustrate the use of DPMAX.
C     Note in the second example that the smallest negative number
C     is not necessarily the negative of the largest positive number.
C
C        C
C        C     Compute the distance to each object. For stars, use
C        C     a "very large" distance.
C        C
C              DO I = 1, N
C                 IF ( TYPE(I) .NE. 'STAR' ) THEN
C                    RANGE(I) = VNORM ( STATE(1,I) )
C                 ELSE
C                    RANGE(I) = SQRT ( DPMAX() ) / 2.D0
C                 END IF
C              END DO
C
C        C
C        C     The window originally has one interval, from "minus
C        C     infinity" to "plus infinity".
C        C
C              WINDOW(1) =  DPMIN()
C              WINDOW(2) =  DPMAX()
C
C              CALL SCARDD ( 2, WINDOW )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     1. "Programming in VAX FORTRAN", Digital Equipment Corporation,
C        September 1984, Appendix C, FORTRAN Data Representation,
C        page C-3.
C
C     2. "Microsoft FORTRAN Reference", Microsoft Corporation
C        1989, Section 1.3.1, page 13.
C
C     3. "Sun FORTRAN Programmer's Guide", Sun Microsystems,
C        Revision A of 6 May 1988, Appendix F, Manual Pages for FORTRAN,
C        page 306 (LIBM_DOUBLE). This routine includes the function
C        D_MAX_NORMAL.
C
C     4. "FORTRAN Programmer's Guide for the Sun Workstation",
C        Sun Microsystems, Revision E of 17 February 1986,
C        Appendix E, Manual Pages for FORTRAN, page 184 (RANGE).
C        This routine includes the function DFLMAX.
C
C     5. "Language Systems FORTRAN Reference Manual", Language Systems
C        Corporation, version 1.2.1, page 3-12.
C
C     6. "Lahey F77L EM/32 Programmers Reference Manual", version 4.0,
C        page 95.
C
C     7. "FORTRAN/9000 Reference HP 9000 Series 700 Computers",
C        First Edition, June 1991, Hewlett Packard Company, page 4-5.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     M.J. Spencer    (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.4.0, 6-OCT-1992 (HAN)
C
C         Module was updated to include the value of DPMAX for the
C         Hewlett Packard UX 9000/750 environment.
C
C-     SPICELIB Version 1.3.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the value of DPMAX
C         for the Lahey F77L EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the value of DPMAX
C         for the Macintosh.
C
C-     SPICELIB Version 1.1.0, 9-MAR-1990 (HAN)
C
C         Module was changed to include the function value for
C         the Sun, IBM PC value was changed, and references
C         were added to specify the sources of the function
C         values on different machines.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     largest d.p. number
C
C-&
 
C$ Revisions
C
C-     SPICELIB Version 1.4.0, 6-OCT-1992 (HAN)
C
C         Module was updated to include the value of DPMAX for the
C         Hewlett Packard UX 9000/750 environment.
C
C         The code was also reformatted so that a utility program can
C         create the source file for a specific environment given a
C         master source file.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the value of DPMAX
C         for the Lahey F77L EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the value of DPMAX
C         for the Macintosh.
C
C-     SPICELIB Version 1.1.0, 9-MAR-1990 (HAN)
C
C         Code was changed to include the function value
C         for the Sun. Documentation in the Particulars
C         section was changed to include the value, the
C         example was updated and corrected.
C
C         Code was changed to update the function value for
C         the IBM PC. The previous value did not compile
C         under the Microsoft FORTRAN Version 5.0. The last
C         digit was deleted from the value, and the module then
C         compiled.
C
C         All sources for the values contained in this module
C         are now specified in the Literature_References section.
C
C-&
 
 
 
 
C
C     Numbers are provided in a variety of formats: decimal, hex,
C     and binary. These last two formats are not portable; but then,
C     neither are the values.
C
 
C
C     Sun, Sun FORTRAN
C
C     Declare the Sun FORTRAN function DFLMAX.
C
      DOUBLE PRECISION      DFLMAX
 
C
C     The maximum double precision number in this module is assigned
C     the value of the Sun FORTRAN function DFLMAX.
C
      DPMAX = DFLMAX()
 
 
 
 
      RETURN
      END
 
