C$Procedure             EXISTS ( Does the file exist? )
 
      LOGICAL FUNCTION  EXISTS ( FILE )
 
C$ Abstract
C
C      Determine whether a file exists.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      FILES
C
C$ Declarations
 
      CHARACTER*(*)    FILE
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      FILE       I   Name of the file in question.
C
C      The function returns the value TRUE if the file exists, FALSE
C      otherwise.
C
C$ Detailed_Input
C
C      FILE        is the name of the file in question. This may be
C                  a system name, e.g.
C
C                        'DISK:[USER.SUB1.SUB2]DATA.DAT'
C                        '\usr\dir1\dir2\data.dat'
C
C                  or a logical name, e.g.
C
C                        'EPHEMERIS'
C                        'DATA$DIR:SAMPLE.DAT'
C
C$ Detailed_Output
C
C      The function returns the value TRUE if the file exists, FALSE
C      otherwise.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      Use the Fortran INQUIRE statement to determine the existence
C      of FILE.
C
C$ Examples
C
C      The following code fragment illustrates the use of EXISTS.
C
C            IF ( EXISTS ( FILE ) ) THEN
C               CALL UPDATE ( FILE )
C            ELSE
C               ERROR = 'Input file does not exist.'
C               RETURN
C            END IF
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      If an error occurs during the execution of the Fortran INQUIRE
C      statement, the error SPICE(INQUIREFAILED) is signalled.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      H.A. Neilan     (JPL)
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     does the file exist
C
C-&
 
 
 
C$ Revisions
C
C-     Beta Version 2.0.0, 29-DEC-1988 (HAN)
C
C        The IOSTAT specifier was added to the INQUIRE statement.
C        If the value of IOSTAT is not equal to zero, an error
C        occurred during the execution of the INQUIRE statement.
C        In this case, a SPICELIB error is signalled and the routine
C        checks out.
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               IOSTAT
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'EXISTS' )
      END IF
 
C
C     So simple, it defies explanation.
C
      INQUIRE ( FILE = FILE, EXIST = EXISTS, IOSTAT = IOSTAT )
 
      IF ( IOSTAT .NE. 0 ) THEN
         CALL SETMSG ( 'Value of IOSTAT was *.' )
         CALL ERRINT ( '*', IOSTAT              )
         CALL SIGERR ( 'SPICE(INQUIREFAILED)'   )
      END IF
 
      CALL CHKOUT ( 'EXISTS' )
      RETURN
      END
