C$Procedure      FNDNWD ( Find the next word after an index )
 
      SUBROUTINE FNDNWD ( STRING, START, I, J )
 
C$ Abstract
C
C     Find the beginning and end of the first word starting at
C     or after a specified character.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      PARSING,  SEARCH,  WORD
C
C$ Declarations
 
      CHARACTER*(*)    STRING
      INTEGER          START
      INTEGER          I
      INTEGER          J
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      STRING     I   A string to examine for words.
C      START      I   Position in the string to start looking for words.
C      I          O   String position of first character of the word.
C      J          O   String position of last character of the word.
C
C$ Detailed_Input
C
C      STRING     Is a character string that potentially consists of
C                 words of text.
C
C      START      Is the index of a letter within the string from which
C                 to start looking for the next word.
C
C$ Detailed_Output
C
C      I          Is the index of the first letter of the word substring
C                 of STRING that begins at or after position START.  If
C                 there are no such substrings I is returned as 0.
C
C      J          Is the index of the last letter of the word substring
C                 of STRING that begins at or after position START.  If
C                 there are no such substrings J is returned as 0.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      Given a character string and location of a character within that
C      string, this routine finds the first full word of the string
C      that starts on or after the specified location.
C
C$ Examples
C                        1         2         3         4         5
C               12345678901234567890123456789012345678901234567890
C      STRING: 'Now is the time for all good men to go home to bed'
C
C      START    I      J
C      -----   ---    ---
C      1        1      3
C      2        5      6
C      3        5      6
C      4        5      6
C      5        5      6
C      6        8      10
C      7        8      10
C      8        8      10
C      9        12     15
C
C      48       48     50
C      49       0      0
C      111      0      0
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     find the next word after an index
C
C-&
 
 
 
C
C     SPICELIB functions
C
      INTEGER          FRSTNB
      INTEGER          LASTNB
 
C
C     Local Variables
C
      INTEGER          QUIT
      INTEGER          BEGIN
 
C
C
C     Find the first non-blank character following START - 1
C
      QUIT = LASTNB(STRING)
 
      IF ( START .GT. QUIT ) THEN
 
C
C        We're attempting to look for a word beyond the last non-blank
C        character of the string.  We won't find any.
C
         I = 0
         J = 0
         RETURN
 
      END IF
 
C
C     Make sure the user hasn't supplied us with a dumb value for START
C
      BEGIN = MAX (START,1)
 
C
C     Find the first non-blank character of the stuff beginning with
C     START.
C
      I    = FRSTNB ( STRING(BEGIN:) )
 
 
C
C     See if we are in the middle of a word already.
C
      IF (       ( BEGIN                   .GT. 1   )
     .     .AND. ( I                       .EQ. 1   )
     .     .AND. ( STRING(BEGIN-1:BEGIN-1) .NE. ' ' ) ) THEN
 
C
C        we've started in the middle of a word, we need to find
C        the beginning of the next one.  Look for a blank character.
C
         I = BEGIN
 
50001    IF       (     ( I           .LT. QUIT )
     .             .AND.( STRING(I:I) .NE. ' '  ) )
     .   THEN
            I = I + 1
            GO TO 50001
         END IF
 
C
C        If we got to the end of the string without finding a blank
C        we aren't going to have any words following the BEGIN character
C
         IF ( I .EQ. QUIT ) THEN
            I = 0
            J = 0
            RETURN
         END IF
 
C
C        Still here?  Then we haven't reached the last non-blank of
C        the string and we're sitting at a blank.  Look for the next
C        non-blank.  That will be the start of the word we're looking
C        for.
C
50002    IF       ( STRING(I:I) .EQ. ' ' )
     .   THEN
            I = I + 1
            GO TO 50002
         END IF
 
      ELSE
 
C
C        We were at the legitimate beginning of a word of the substring.
C        set I to point at the full string position of this word.
C
         I = BEGIN + I - 1
 
      END IF
 
C
C     The last letter of the word is at or after I but not after the
C     last non-blank character.
C
      J    = I
      QUIT = LASTNB(STRING)
 
C
C     Look until we find a blank or run out of stuff to look at.
C
50003 IF       ( J .LE. QUIT )
     .THEN
         IF ( STRING(J:J) .EQ. ' ' ) THEN
            J = J - 1
            RETURN
         END IF
 
         J = J + 1
         GO TO 50003
      END IF
 
C
C     If we reached this point, the end of the word must be the
C     last non-blank character of STRING.
C
      J    = QUIT
 
      RETURN
      END
