C$Procedure            INTMAX ( Largest integer number )
 
      INTEGER FUNCTION INTMAX ()
 
C$ Abstract
C
C     Return the value of the largest (positive) number representable
C     in an integer variable.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONSTANTS
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     The function returns the value of the largest (positive) number
C     that can be represented in an integer variable.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     The function returns the value of the largest (positive) number
C     that can be represented in an integer variable.
C
C     This varies from machine to machine. Values for several popular
C     machines are shown below.
C
C     Environment: VAX/VMS, VAX FORTRAN
C     Value:       2147483647
C
C     Environment: Sun, Sun FORTRAN
C     Value:       INMAX()
C
C     Environment: PC, MS (Microsoft) FORTRAN
C     Value:       2147483647
C
C     Environment: Macintosh, Language Systems FORTRAN
C     Value:       2147483647
C
C     Environment: PC, Lahey F77 EM/32 Version 4.0
C     Value:       2147483647
C
C     Environment: HP-UX 9000/750, FORTRAN/9000 Series 700 computers
C     Value:       2147483647
C
C
C     References for the values above are listed in the
C     Literature_References section of the header.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The function always returns a constant value, set by the user
C     prior to compilation.
C
C$ Examples
C
C     The following code fragment illustrates the use of INTMAX.
C
C        C
C        C     If the integer component is out of range, avoid overflow
C        C     by making it as large as possible.
C        C
C              IF ( DVALUE .GT. DBLE ( INTMAX() ) ) THEN
C                 IVALUE = INTMAX()
C
C              ELSE IF ( DVALUE .LT. DBLE ( INTMIN() ) THEN
C                 IVALUE = INTMIN()
C
C              ELSE
C                 IVALUE = INT ( DVALUE )
C              END IF
C
C              FRACT = MOD ( DVALUE, 1.D0 )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     1. "Programming in VAX FORTRAN", Digital Equipment Corporation,
C        September 1984, Appendix C, FORTRAN Data Representation,
C        page C-2.
C
C     2. "Microsoft FORTRAN Reference", Microsoft Corporation
C        1989, Section 1.3.1, page 10.
C
C     3. "Sun FORTRAN Programmer's Guide, Sun Microsystems,
C        Revision A of 6 May 1988, Appendix F, Manual Pages for FORTRAN,
C        page 306 (RANGE).
C
C     5. "Language Systems FORTRAN Reference Manual", Language Systems
C        Corporation, version 1.2.1, page 3-2.
C
C     6. "Lahey F77L EM/32 Programmers Reference Manual", version 4.0,
C        page 95.
C
C     7. "FORTRAN/9000 Reference HP 9000 Series 700 Computers",
C        First Edition, June 1991, Hewlett Packard Company, page 4-4.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C     M.J. Spencer    (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.4.0, 9-OCT-1992 (HAN)
C
C         Module was updated to include the value of INTMAX for the
C         Hewlett Packard UX 9000/750 environment.
C
C-     SPICELIB Version 1.3.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the value of INTMAX
C         for the Lahey F77L EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the value of INTMAX for
C         the Macintosh.
C
C-     SPICELIB Version 1.1.0, 12-MAR-1990 (HAN)
C
C         Module was changed to include the function value for
C         the Sun. References were added to specify the sources
C         of the function values on different machines.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     largest integer number
C
C-&
 
 
C$ Revisions
C
C-     SPICELIB Version 1.4.0, 9-OCT-1992 (HAN)
C
C         Module was updated to include the value of INTMAX for the
C         Hewlett Packard UX 9000/750 environment.
C
C         The code was also reformatted so that a utility program can
C         create the source file for a specific environment given a
C         master source file.
C
C-     SPICELIB Version 1.3.0, 13-NOV-1991 (MJS)
C
C         Module was updated to include the value of INTMAX
C         for the Lahey F77L EM/32 environment (PC).
C
C-     SPICELIB Version 1.2.0, 7-DEC-1990 (MJS)
C
C         Module was updated to include the value of INTMAX for
C         the Macintosh. Literature_References section was
C         updated.
C
C-     SPICELIB Version 1.1.0, 12-MAR-1990 (HAN)
C
C         Code was changed to include the function value
C         for the Sun. Documentation in the Particulars
C         section was changed to include the value, and the
C         example was updated and corrected.
C
C         All sources for the values contained in this module
C         are now specified in the Literature_References section.
C
C-     Beta Version 1.1.0, 16-FEB-1989 (HAN) (NJB)
C
C         Contents of the Exceptions section was changed
C         to "error free" to reflect the decision that the
C         module will never participate in error handling.
C
C         Parentheses added to INTMAX declaration.
C-&
 
 
 
C
C     Numbers are provided in a variety of formats: decimal, hex,
C     and binary. These last two formats are not portable; but then,
C     neither are the values.
C
 
C
C     Sun, Sun FORTRAN
C
C     The maximum integer value is assigned the value of the
C     Sun FORTRAN function INMAX.
C
C
C     Declare the Sun FORTRAN function INMAX and assign INTMAX
C     its value.
C
      INTEGER               INMAX
 
      INTMAX = INMAX()
 
      RETURN
      END
