C$Procedure      ORDERC ( Order of a character array )
 
      SUBROUTINE ORDERC ( ARRAY, NDIM, IORDER )
 
C$ Abstract
C
C      Determine the order of elements in an array of character strings.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      ARRAY,  SORT
C
C$ Declarations
 
      CHARACTER*(*)    ARRAY  ( * )
      INTEGER          NDIM
      INTEGER          IORDER ( * )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      ARRAY      I    Input array.
C      NDIM       I    Dimension of ARRAY.
C      IORDER     O    Order vector for ARRAY.
C
C$ Detailed_Input
C
C      ARRAY       is the input array.
C
C      NDIM        is the number of elements in the input array.
C
C$ Detailed_Output
C
C      IORDER      is the order vector for the input array.
C                  IORDER(1) is the index of the smallest element
C                  of ARRAY; IORDER(2) is the index of the next
C                  smallest; and so on. Strings are ordered according
C                  to the ASCII collating sequence.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      ORDERC finds the index of the smallest element of the input
C      array. This becomes the first element of the order vector.
C      The process is repeated for the rest of the elements.
C
C      The order vector returned by ORDERC may be used by any of
C      the REORD routines to sort sets of related arrays, as shown
C      in the example below.
C
C$ Examples
C
C      In the following example, the ORDER and REORD routines are
C      used to sort four related arrays (containing the names,
C      masses, integer ID codes, and visual magnitudes for a group
C      of satellites). This is representative of the typical use of
C      these routines.
C
C            C
C            C     Sort the object arrays by name.
C            C
C                  CALL ORDERC ( NAMES, N, IORDER )
C
C                  CALL REORDC ( IORDER, N, NAMES )
C                  CALL REORDD ( IORDER, N, MASSES )
C                  CALL REORDI ( IORDER, N, CODES )
C                  CALL REORDR ( IORDER, N, VMAGS )
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     order of a character array
C
C-&
 
 
 
 
C
C     Local variables
C
      INTEGER          GAP
      INTEGER          I
      INTEGER          J
      INTEGER          JG
 
 
C
C     Begin with the initial ordering.
C
      DO 50001
     .   I = 1, NDIM
         IORDER(I) = I
50001 CONTINUE
 
C
C     Find the smallest element, then the next smallest, and so on.
C     This uses the Shell Sort algorithm, but swaps the elements of
C     the order vector instead of the array itself.
C
      GAP = NDIM / 2
 
50002 IF       ( GAP .GT. 0 )
     .THEN
 
         DO 50003
     .      I = GAP+1, NDIM
 
            J = I - GAP
50004       IF       ( J .GT. 0 )
     .      THEN
               JG = J + GAP
 
               IF ( LLE ( ARRAY(IORDER(J)), ARRAY(IORDER(JG)) ) ) THEN
                  J = 0
               ELSE
                  CALL SWAPI ( IORDER(J), IORDER(JG) )
               END IF
 
               J = J - GAP
               GO TO 50004
            END IF
 
50003    CONTINUE
 
         GAP = GAP / 2
 
         GO TO 50002
      END IF
 
      RETURN
      END
 
