C$Procedure      OUTMSG ( Output Error Messages )
 
      SUBROUTINE OUTMSG ( LIST )
 
C$ Abstract
C
C     Output error messages.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
 
      CHARACTER*(*)           LIST
 
      INTEGER                FILEN
      PARAMETER            ( FILEN  = 256 )
 
      INTEGER                NAMLEN
      PARAMETER            ( NAMLEN = 256 )
 
      INTEGER                LL
      PARAMETER            ( LL     =  80 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     LIST       I   A list of error message types.
C     FILEN      P   Maximum length of file name.
C     NAMLEN     P   Maximum length of module name.
C     LL         P   Output line length.
C
C$ Detailed_Input
C
C     LIST           is a list of error message types.  A list is a
C                    character string containing one or more words
C                    from the following list, separated by commas.
C
C                       SHORT
C                       EXPLAIN
C                       LONG
C                       TRACEBACK
C                       DEFAULT
C
C                    Each type of error message specified in LIST will
C                    be output when an error is detected, if it is
C                    enabled for output.  Note that DEFAULT does
C                    NOT refer to the "default message selection,"
C                    but rather to a special message that is output
C                    when the error action is 'DEFAULT'.  This message
C                    is a statement referring the user to the error
C                    handling documentation.
C
C                    Messages are never duplicated in the output; for
C                    instance, supplying a value of LIST such as
C
C                       'SHORT, SHORT'
C
C                    does NOT result in the output of two short
C                    messages.
C
C                    The words in LIST may appear in mixed case;
C                    for example, the call
C
C                       CALL OUTMSG ( 'ShOrT' )
C
C                    will work.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     FILEN          is the maximum device name length that can be
C                    accommodated by this routine.
C
C     NAMELN         is the maximum module name length that can be
C                    accommodated in the traceback message.
C
C     LL             is the maximum line length for the output message.
C                    If the output message string is very long, it is
C                    displayed over several lines, each of which has a
C                    maximum length of LL characters.
C
C$ Exceptions
C
C     1)  This routine detects invalid message types in the argument,
C         LIST.   The short error message in this case is
C         'SPICE(INVALIDLISTITEM)'
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      This routine is part of the SPICELIB error handling
C      mechanism.
C
C      This routine outputs the error messages specified in LIST that
C      have been enabled for output (use the SPICELIB routine ERRPRT
C      to enable or disable output of specified types of error
C      messages).  A border is written out preceding and following the
C      messages.  Output is directed to the current error output device.
C
C$ Examples
C
C      1)  Output the short and long error messages:
C
C         C
C         C     Output short and long messages:
C         C
C               CALL OUTMSG ( 'SHORT, LONG' )
C
C$ Restrictions
C
C      1)  This routine is intended for use by the SPICELIB error
C          handling mechanism.  SPICELIB users are not expected to
C          need to call this routine.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C      H.A. Neilan     (JPL)
C      M.J. Spencer    (JPL)
C
C$ Version
C
C-     SPICELIB Version 2.2.0, 12-OCT-1992 (HAN)
C
C        Updated module for multiple environments. Moved the parameter
C        LL to the Declarations section of the header since it's
C        environment dependent.
C
C        The code was also reformatted so that a utility program can
C        create the source file for a specific environment given a
C        master source file.
C
C-     SPICELIB Version 2.1.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 2.1.0, 15-MAY-1991 (MJS)
C
C         Module was updated to include the value of LL for the
C         Macintosh.
C
C-     SPICELIB Version 2.0.0, 28-MAR-1991 (NJB)
C
C         Work-around for MS Fortran compiler error under DOS 3.10
C         was made.  Some substring bounds were simplified using RTRIM.
C         Updates were made to the header to clarify the text and
C         improve the header's appearance.  The default error message
C         was slightly de-uglified.
C
C         The IBM PC version of this routine now uses an output line
C         length of 78 characters rather than 80.  This prevents
C         wrapping of the message borders and default error message.
C
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     None.
C
C-&
 
 
 
 
C$ Revisions
C
C-     SPICELIB Version 2.2.0, 12-OCT-1992 (HAN)
C
C        Updated module for multiple environments. Moved the
C        parameter LL to the Declarations section of the header since
C        it's environment dependent.
C
C        The code was also reformatted so that a utility program can
C        create the source file for a specific environment given a
C        master source file.
C
C-     SPICELIB Version 2.1.0, 15-MAY-1991 (MJS)
C
C         Module was updated to include the value of LL for the
C         Macintosh.
C
C-     SPICELIB Version 2.0.0, 28-MAR-1991 (NJB)
C
C         1)  Work-around for MS Fortran compiler error under DOS 3.10
C             was made.  The compiler did not correctly handle code that
C             concatenated strings whose bounds involved the intrinsic
C             MAX function.
C
C         2)  Some substring bounds were simplified using RTRIM.
C
C         3)  Updates were made to the header to clarify the text and
C             improve the header's appearance.
C
C         4)  Declarations were re-organized.
C
C         5)  The default error message was slightly de-uglified.
C
C         6)  The IBM PC version of this routine now uses an output line
C             length of 78 characters rather than 80.  This prevents
C             wrapping of the message borders and default error message.
C
C
C-     Beta Version 1.3.0, 19-JUL-1989 (NJB)
C
C         Calls to REMSUB removed; blanking and left-justifying used
C         instead.  This was done because REMSUB handles substring
C         bounds differently than in previous versions, and no longer
C         handles all possible inputs as required by this routine.
C         LJUST, which is used now, is error free.
C
C         Also, an instance of .LT. was changed to .LE.   The old code
C         caused a line break one character too soon.  A minor bug, but
C         a bug nonetheless.
C
C         Also, two substring bounds were changed to ensure that they
C         remain greater than zero.
C
C
C-     Beta Version 1.2.0, 16-FEB-1989 (NJB)
C
C         Warnings added to discourage use of this routine in
C         non-error-handling code.  Parameters section updated to
C         describe FILEN and NAMLEN.
C
C         Declaration of unused function FAILED removed.
C
C
C-     Beta Version 1.1.0, 06-OCT-1988 (NJB)
C
C         Test added to ensure substring upper bound is greater than 0.
C         REMAIN must be greater than 0 when used as the upper bound
C         for a substring of NAME.  Also, substring upper bound in
C         WRLINE call is now forced to be greater than 0.
C
C-&
 
 
C
C     SPICELIB functions
C
      INTEGER               LASTNB
      INTEGER               RTRIM
      LOGICAL               MSGSEL
 
 
C
C     Local parameters
C
 
C
C     These parameters are system-independent.
C
      CHARACTER*(*)         BORCHR
      PARAMETER           ( BORCHR = '=' )
 
      INTEGER               DEFLEN
      PARAMETER           ( DEFLEN =   4 )
 
      INTEGER               ENDLEN
      PARAMETER           ( ENDLEN =   1 )
 
      INTEGER               LLEN
      PARAMETER           ( LLEN   = 320 )
 
      INTEGER               LSTLEN
      PARAMETER           ( LSTLEN =  80 )
 
      INTEGER               MAXWRD
      PARAMETER           ( MAXWRD =   5 )
 
      INTEGER               WORDLN
      PARAMETER           ( WORDLN =   9 )
 
      INTEGER               SLEN
      PARAMETER           ( SLEN   =  25 )
 
      INTEGER               STRLEN
      PARAMETER           ( STRLEN =   3 )
 
      INTEGER               XLEN
      PARAMETER           ( XLEN   =  80 )
 
C
C     Local variables
C
      CHARACTER*(LL)        DEFMSG ( DEFLEN )
      CHARACTER*(FILEN)     DEVICE
      CHARACTER*(LL)        ENDBOR ( ENDLEN )
      CHARACTER*(LL)        LINE
      CHARACTER*(LLEN)      LMSG
      CHARACTER*(NAMLEN+4)  NAME
      CHARACTER*(SLEN)      SMSG
      CHARACTER*(LL)        STRBOR ( STRLEN )
      CHARACTER*(SLEN+XLEN) TMPMSG
      CHARACTER*(WORDLN)    UPWORD
      CHARACTER*(WORDLN)    WORDS  ( MAXWRD )
      CHARACTER*(XLEN)      XMSG
 
      INTEGER               DEPTH
      INTEGER               I
      INTEGER               INDEX
      INTEGER               NUMWRD
      INTEGER               REMAIN
 
      LOGICAL               DFAULT
      LOGICAL               EXPL
      LOGICAL               FIRST
      LOGICAL               LONG
      LOGICAL               OUTPUT
      LOGICAL               SHORT
      LOGICAL               TRACE
 
 
C
C     Saved variables
C
      SAVE                  DEFMSG
      SAVE                  FIRST
      SAVE                  STRBOR
      SAVE                  ENDBOR
 
 
 
C
C     Initial Values:
C
      DATA DEFMSG(1)( 1:40) /'Oh, by the way:  The SPICELIB error hand'/
      DATA DEFMSG(1)(41:LL) /'ling actions are USER-TAILORABLE.  You'  /
      DATA DEFMSG(2)( 1:40) /'can choose whether the Toolkit aborts or'/
      DATA DEFMSG(2)(41:LL) /' continues when errors occur, which   '  /
      DATA DEFMSG(3)( 1:40) /'error messages to output, and where to s'/
      DATA DEFMSG(3)(41:LL) /'end the output.  Please read the ERROR'  /
      DATA DEFMSG(4)( 1:40) /'"Required Reading" file, or see the rout'/
      DATA DEFMSG(4)(41:LL) /'ines ERRACT, ERRDEV, and ERRPRT.      '  /
 
      DATA FIRST / .TRUE. /
 
 
C
C     Executable Code:
C
 
C
C     The first time through, set up the output borders.
C
      IF ( FIRST ) THEN
 
         STRBOR(1) = ' '
         STRBOR(3) = ' '
 
         DO 50001
     .      I = 1, LL
            STRBOR(2) (I:I) = BORCHR
50001    CONTINUE
 
         ENDBOR(1)  =  STRBOR(2)
         FIRST      = .FALSE.
 
      END IF
 
C
C     No messages are to be output which are not specified
C     in LIST:
C
      SHORT  = .FALSE.
      EXPL   = .FALSE.
      LONG   = .FALSE.
      TRACE  = .FALSE.
      DFAULT = .FALSE.
 
 
C     We parse the list of message types, and set local flags
C     indicating which ones are to be output.  If we find
C     a word we don't recognize in the list, we signal an error
C     and continue parsing the list.
C
      CALL LPARSE ( LIST,  ',' ,  MAXWRD,  NUMWRD, WORDS )
 
      DO 50002
     .   I = 1, NUMWRD
 
         CALL NUCASE ( WORDS(I), UPWORD )
 
         IF ( UPWORD  .EQ. 'SHORT' ) THEN
 
            SHORT  =  .TRUE.
 
         ELSE IF ( UPWORD  .EQ. 'EXPLAIN'   ) THEN
 
            EXPL   =  .TRUE.
 
         ELSE IF ( UPWORD  .EQ. 'LONG'      ) THEN
 
            LONG   =  .TRUE.
 
         ELSE IF ( UPWORD  .EQ. 'TRACEBACK' ) THEN
 
            TRACE  =  .TRUE.
 
         ELSE IF ( UPWORD  .EQ. 'DEFAULT'   ) THEN
 
            DFAULT =  .TRUE.
 
         ELSE
C
C           Unrecognized word!  This is an error...
C
C           We have a special case on our hands; this routine
C           is itself called by SIGERR, so a recursion error will
C           result if this routine calls SIGERR.  So we output
C           the error message directly:
C
            CALL GETDEV ( DEVICE )
 
            CALL WRLINE ( DEVICE,  'SPICE(INVALIDLISTITEM)'            )
            CALL WRLINE ( DEVICE,  ' '                                 )
            CALL WRLINE ( DEVICE,  'OUTMSG:  An invalid message type '//
     .                             'was specified in the type list. ' //
     .                             'The type was  ' // WORDS(I)        )
         END IF
 
50002 CONTINUE
 
C
C     LIST has been parsed.
C
C     Now, we output those error messages that were specified by LIST
C     and which belong to the set of messages selected for output.
C
 
C
C     We get the default error output device:
C
      CALL GETDEV ( DEVICE )
 
      OUTPUT =        (  SHORT   .AND.  MSGSEL ( 'SHORT'     )  )
     .          .OR.  (  EXPL    .AND.  MSGSEL ( 'EXPLAIN'   )  )
     .          .OR.  (  LONG    .AND.  MSGSEL ( 'LONG'      )  )
     .          .OR.  (  TRACE   .AND.  MSGSEL ( 'TRACEBACK' )  )
     .          .OR.  (  DFAULT  .AND.  MSGSEL ( 'DEFAULT'   )  )
     .          .AND. (  DEVICE  .NE.   'NULL'                  )
C
C     We go ahead and output those messages that have been specified
C     in the list and also are enabled for output.   The order of
C     the cases below IS significant; the order in which the
C     messages appear in the output depends on it.
C
 
C
C     If there's nothing to output, we can leave now.
C
      IF ( .NOT. OUTPUT ) THEN
         RETURN
      END IF
 
C
C     Write the starting border out:
C
      DO 50003
     .   I = 1, STRLEN
         CALL WRLINE ( DEVICE, STRBOR(I) )
50003 CONTINUE
 
C
C     Next, we output the messages specified in the list
C     that have been enabled.
C
C     We start with the short message and its accompanying
C     explanation.  If both are to be output, they are
C     concatenated into a single message.
C
      IF  (        (  SHORT .AND. MSGSEL ( 'SHORT'   )  )
     .      .AND.  (  EXPL  .AND. MSGSEL ( 'EXPLAIN' )  )   ) THEN
 
C
C        Extract the short message from global storage; then get
C        the corresponding explanation.
C
         CALL GETSMS ( SMSG       )
         CALL EXPLN  ( SMSG, XMSG )
 
         TMPMSG  =  SMSG( :RTRIM(SMSG) )  // ' -- '// XMSG
 
         CALL WRLINE ( DEVICE, TMPMSG )
         CALL WRLINE ( DEVICE, ' '    )
 
      ELSE IF  (  SHORT .AND. MSGSEL ( 'SHORT'   )  )  THEN
C
C        Output the short error message without the explanation.
C
         CALL GETSMS ( SMSG )
 
         CALL WRLINE ( DEVICE, SMSG  )
         CALL WRLINE ( DEVICE, ' '   )
 
      ELSE IF (  EXPL .AND. MSGSEL ( 'EXPLAIN' )  )  THEN
C
C        Obtain the explanatory text for the short error
C        message and output it:
C
         CALL GETSMS ( SMSG       )
         CALL EXPLN  ( SMSG, XMSG )
 
         CALL WRLINE ( DEVICE, XMSG )
         CALL WRLINE ( DEVICE, ' '  )
 
      END IF
 
 
      IF (  LONG .AND. MSGSEL ( 'LONG' )  ) THEN
C
C        Extract the long message from global storage and
C        output it:
C
         CALL GETLMS  (  LMSG  )
 
         CALL WRLINE  (  DEVICE, LMSG ( :RTRIM(LMSG) ) )
         CALL WRLINE  (  DEVICE, ' '            )
 
      END IF
 
 
      IF (  TRACE .AND. MSGSEL ( 'TRACEBACK' )  ) THEN
C
C        Extract the traceback from global storage and
C        output it:
C
         CALL TRCDEP ( DEPTH )
 
         IF ( DEPTH .GT. 0 ) THEN
C
C           We know we'll be outputting some trace information.
C           So, write a line telling the reader what's coming.
C
            CALL WRLINE ( DEVICE, 'A traceback follows.  The '  //
     .                            'name of the highest level '  //
     .                            'module is first.'          )
C
C           While there are more names in the traceback
C           representation, we stuff them into output lines and
C           write the lines out when they are full.
C
            INDEX   =   1
            LINE    =   ' '
            REMAIN  =   LL
 
50004       IF       ( INDEX .LE. DEPTH )
     .      THEN
C
C              While there are more module names in the traceback
C              representation, retrieve module names and
C              stuff them into lines of characters.
C
C              Get a name and add the call order sign.  We
C              indicate calling order by a ' --> ' delimiter; e.g.
C              "A calls B" is indicated by 'A --> B'.
C
               CALL TRCNAM ( INDEX, NAME )
 
C
C              If it's not the last name, add the call order sign:
C
               IF ( INDEX .LT. DEPTH ) THEN
                       CALL SUFFIX ( ' -->',   0,  NAME )
               END IF
 
               INDEX = INDEX + 1
 
C
C              We have to stuff the name into one or more lines of
C              text.  Our approach is as follows:
C
C                 1)  If the name fits in the current line,
C                     ram it in there.  (Urmmph!)
C                     If it's the first name on the line, we
C                     don't want a preceding blank; otherwise
C                     we do.
C
C                 2)  If the name doesn't fit, and the length
C                     of the name is <= the line length, write
C                     the current line, and stick the name in
C                     a new line.
C
C                 3)  If the name is longer than a line, we have
C                     no choice but to break it.  We stuff as
C                     much of it as will fit into the current
C                     line, and keep filling up new lines with
C                     chunks of the name, until there's none
C                     left.  As each line is filled, we write
C                     it out.
C
               IF (   LASTNB( NAME ) .LE. REMAIN   ) THEN
C
C                 This is case (1).
C
                  IF ( REMAIN .EQ. LL ) THEN
                     LINE = NAME
                  ELSE
                     CALL SUFFIX ( NAME, 1, LINE )
                  END IF
C
C                 Leave a space between the arrow and the next name.
C
                  REMAIN = REMAIN - LASTNB( NAME ) - 1
 
               ELSE IF ( LASTNB( NAME ) .LT. LL ) THEN
C
C                 Case (2).
C
                  CALL WRLINE ( DEVICE, LINE )
 
                  LINE   =  NAME
C
C                 Leave a space between the arrow and the next name.
C
                  REMAIN =  LL - LASTNB( NAME ) - 1
 
               ELSE
C
C                 Case (3).
C
                  IF ( REMAIN .EQ. LL ) THEN
 
                     LINE = NAME
 
                  ELSE IF ( REMAIN .GT. 0 ) THEN
 
                     CALL SUFFIX (  NAME(:REMAIN), 1, LINE  )
 
                  END IF
 
                  CALL WRLINE ( DEVICE, LINE )
 
                  IF ( REMAIN .GT. 0 ) THEN
 
                     NAME ( :REMAIN )  = ' '
 
                     CALL LJUST ( NAME, NAME )
 
                  END IF
 
 
50005             IF       (  LASTNB( NAME ) .GT. LL  )
     .            THEN
 
                     CALL WRLINE (  DEVICE, NAME(:LL)  )
 
                     NAME ( :LL ) = ' '
 
                     CALL LJUST ( NAME, NAME )
 
                     GO TO 50005
                  END IF
 
                  IF ( NAME .NE. ' ' ) THEN
                     LINE   = NAME
                     REMAIN = LL  -  LASTNB( NAME )  -  1
                  ELSE
                     REMAIN = LL
                  END IF
C
C                 At this point, a name has been inserted into
C                 one or more lines.  If multiple lines have
C                 been used, all lines but the last one have
C                 been written out.
C
               END IF
 
               GO TO 50004
            END IF
 
C
C           At this point, no more names are left in the
C           trace representation.  LINE may still contain
C           names, or part of a long name.  If it does,
C           we now write it out.
C
            IF ( LINE .NE. ' ' ) THEN
               CALL WRLINE ( DEVICE, LINE )
            END IF
 
            CALL WRLINE  ( DEVICE, ' '  )
 
         END IF
C
C        At this point, either we have output the trace
C        representation, or the trace representation was
C        empty.
C
      END IF
 
      IF (  DFAULT  .AND. MSGSEL ( 'DEFAULT' )  ) THEN
C
C        Output the default message:
C
         DO 50006
     .      I = 1, DEFLEN
            CALL WRLINE  ( DEVICE, DEFMSG (I) )
50006    CONTINUE
 
         CALL WRLINE  ( DEVICE, ' '  )
 
      END IF
 
C
C     At this point, we've output all of the enabled messages
C     that were specified in LIST.  At least one message that
C     was specified was enabled.
C
C     Write the ending border out:
C
      DO 50007
     .   I = 1, ENDLEN
         CALL WRLINE ( DEVICE, ENDBOR(I) )
50007 CONTINUE
 
      RETURN
      END
