C$Procedure      PROMPT ( Prompt a user for a string )
 
      SUBROUTINE PROMPT ( PRMPT, STRING )
 
C$ Abstract
C
C     This routine prompts the user for keyboard input.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     UTILITY
C
C$ Declarations
 
      CHARACTER*(*)         PRMPT
      CHARACTER*(*)         STRING
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     PRMPT      I   The prompt to use when asking for input.
C     STRING     O   The response typed by a user.
C
C$ Detailed_Input
C
C     PRMPT      is a character string that will be displayed from the
C                current cursor position and describes the input that
C                the user is expected to enter.  The string PRMPT should
C                be relatively short (.i.e 50 or fewer characters) so
C                the user may type a response on the same line as the
C                line where the prompt is displayed.
C
C                All characters (including trailing blanks) in PRMPT
C                are considered significant and will be displayed.
C
C$ Detailed_Output
C
C     STRING     is a character string that contains the string
C                entered by the user.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This is a utility that allows you to "easily" request information
C     from a program user.  At a high level, it frees you from the
C     peculiarities of a particular implementation of FORTRAN cursor
C     control.
C
C$ Examples
C
C     Suppose you wanted to ask a user to input an answer to
C     a question such as "Do you want to try again? (Y/N) "
C     and leave the cursor at the end of the question as shown here:
C
C        Do you want to try again? (Y/N) _
C
C     (The underscore indicates the cursor position).
C
C     The following line of code will do what you want.
C
C        CALL PROMPT ( 'Do you want to try again? (Y/N) ', ANSWER )
C
C$ Restrictions
C
C     This routine is environment specific.  Standard FORTRAN does not
C     provide for user control of cursor position after write
C     statements.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 15-OCT-1992 (WLT)
C
C-&
 
 
 
C$ Index_Entries
C
C     Prompt for keyboard input
C     Prompt for input with a user supplied message
C
C-&
 
C
C
C     The code below should be used in the following environments
C     SUN/FORTRAN, NeXT/Absoft_FORTRAN, and HP.
C
      WRITE (*,FMT='(A,$)' ) PRMPT
      READ  (*,FMT='(A)'   ) STRING
 
 
C
C     If none of the official files contain a prompt that works on
C     an unsupported platform, read on...
C     Although, this isn't really what you want, if you need to port
C     this quickly to an environment that does not support the format
C     statement in any of the cases above, you can comment out the
C     two lines of code above and un-comment the two lines below.  In
C     this way you can get a program working quickly in the new
C     environment while you figure out how to control cursor
C     positioning.
C
C     WRITE (*,*) PRMPT
C     READ  (*,FMT='(A)' ) STRING
C
      RETURN
      END
