C$Procedure      PUTLMS ( Store Long Error Message )
 
      SUBROUTINE PUTLMS ( MSG )
 
C$ Abstract
C
C     PUTLMS is a low-level data structure access routine which stores
C     the long error message.  DO NOT CALL THIS ROUTINE.  USE SETMSG,
C     NOT PUTLMS, TO SET THE CURRENT LONG ERROR MESSAGE.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
 
      CHARACTER*(*)          MSG
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      MSG        I   A long error message.
C
C$ Detailed_Input
C
C      MSG   The current long error message.  This value will
C            be saved.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      None.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      DO NOT CALL THIS ROUTINE.
C
C      This is a data structure access routine for the
C      toolkit long error message.  This routine should be
C      used for no other purpose; in particular, it should
C      not be used by routines that detect errors
C      to set the long error message.  Use SETMSG for that.
C
C$ Examples
C
C      None.
C
C$ Restrictions
C
C      DO NOT CALL THIS ROUTINE.
C
C      Calls to this routine by routines other than the
C      SPICELIB error handling routines may interfere
C      with error processing.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     None.
C
C-&
 
 
C$ Revisions
C
C-     Beta Version 1.0.1, 08-FEB-1989 (NJB)
C
C         Warnings added to discourage use of this routine in
C         non-error-handling code.
C
C-&
 
 
 
 
 
C
C     Local Variables:
C
 
C
C     The current long error message:
C
 
      INTEGER               LONGLN
      PARAMETER           ( LONGLN = 320 )
 
      CHARACTER*(LONGLN)    SAVMSG
 
      SAVE                  SAVMSG
 
C
C     Initial values:
C
 
      DATA    SAVMSG      / ' ' /
 
 
 
C
C     Executable Code:
C
 
 
      SAVMSG = MSG
 
      RETURN
 
 
 
 
C$Procedure      GETLMS ( Get Long Error Message )
 
       ENTRY GETLMS ( MSG )
 
C$ Abstract
C
C      Return the value of the current long error message.
C
C$ Required_Reading
C
C      ERROR
C
C$ Keywords
C
C      ERROR
C
C$ Declarations
C
C      CHARACTER*(*)          MSG
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C
C      MSG        O   The current long error message.
C
C$ Detailed_Input
C
C      None.
C
C$ Detailed_Output
C
C      MSG    is the current long error message.  See the
C             "required reading" file for a detailed discussion
C             of error messages.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      None.
C
C      However, this routine is part of the SPICELIB error
C      handling mechanism.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      See the required reading file for details of error
C      processing.
C
C$ Examples
C
C      None.
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     None.
C
C-&
 
 
 
 
 
C
C     Executable Code:
C
 
C
C     Grab saved long message:
C
 
      MSG = SAVMSG
 
      END
 
