C$Procedure      TRCPKG ( Trace package )
 
      SUBROUTINE TRCPKG ( DEPTH, INDEX, MODULE, TRACE, NAME )
 
C$ Abstract
C
C     Maintain a trace of subroutine calls for error messages.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
 
      INTEGER               DEPTH
      INTEGER               INDEX
      CHARACTER*(*)         MODULE
      CHARACTER*(*)         TRACE
      CHARACTER*(*)         NAME
 
 
 
      INTEGER               FILEN
      PARAMETER           ( FILEN  = 256 )
 
      INTEGER               NAMLEN
      PARAMETER           ( NAMLEN = 256 )
 
      INTEGER               TRCLEN
      PARAMETER           ( TRCLEN = 512 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  ENTRY
C     --------  ---  --------------------------------------------------
C
C     DEPTH      O   TRCDEP
C     INDEX      I   TRCNAM
C     NAME       O   TRCNAM
C     MODULE     I   CHKIN, CHKOUT
C     TRACE      O   QCKTRC
C
C     FILEN      P
C     NAMLEN     P
C     TRCLEN     P
C
C$ Detailed_Input
C
C     See the ENTRY points for discussions of their arguments.
C
C$ Detailed_Output
C
C     See the ENTRY points for discussions of their arguments.
C
C$ Parameters
C
C     FILEN          is the maximum length of a file name.
C
C     NAMLEN         is the maximum length of a module name.
C
C     TRCLEN         is the maximum length of the traceback string.
C
C$ Exceptions
C
C     1)  If TRCPKG is called directly, the error SPICE(BOGUSENTRY) is
C         signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The entry points declared in this routine are:
C
C     CHKIN
C     CHKOUT
C     TRCDEP
C     TRCNAM
C     QCKTRC
C     FREEZE
C     TRCOFF
C
C     This routine serves as an umbrella that allows the entry
C     points to share data.  TRCPKG should never be called directly.
C
C$ Examples
C
C     See the entry points CHKIN, CHKOUT, TRCDEP, TRCNAM, QCKTRC,
C     FREEZE, and TRCOFF for examples.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C
C$ Version
C
C
C-    SPICELIB Version 1.3.0, 23-OCT-1992 (NJB)
C
C        Bug fix made to routine QCKTRC:  a section of code which
C        itself is exercised only if a bug is present inserted the
C        wrong variable into an error message.
C
C-     SPICELIB Version 1.2.0, 12-OCT-1992 (HAN)
C
C        Module was updated to include the values of the parameters
C        for the Hewlett Packard UX 9000/750 environment.
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 18-JUN-1990 (NJB)
C
C        Added declarations for trace disabling. Re-organized
C        declarations.  Updated comments.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     None.
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 1.3.0, 23-OCT-1992 (NJB)
C
C        Bug fix made to routine QCKTRC:  a section of code which
C        itself is exercised only if a bug is present inserted the
C        wrong variable into an error message.
C
C-     SPICELIB Version 1.2.0, 12-OCT-1992 (HAN)
C
C        Module was updated to include the values of the parameters
C        for the Hewlett Packard UX 9000/750 environment.
C
C        The code was also reformatted so that a utility program can
C        create the source file for a specific environment given a
C        master source file.
C
C
C-    SPICELIB Version 1.1.0, 18-JUN-1990 (NJB)
C
C        Added declarations for trace disabling. Re-organized
C        declarations.  Updated comments to reflect inclusion
C        of the new entry point TRCOFF.  Also updated the header
C        to make the style more parallel to other SPICELIB
C        umbrella routines.  Updated the description line and
C        abstract, in particular.
C
C-    Beta Version 1.0.1, 08-FEB-1989 (NJB)
C
C        Warnings added to discourage use of this routine.
C
C-&
 
C
C     SPICELIB functions:
C
      INTEGER               LASTNB
      INTEGER               FRSTNB
      INTEGER               RTRIM
 
      LOGICAL               FAILED
 
C
C     Local Variables:
C
      INTEGER               BEG
      INTEGER               COUNT
      INTEGER               END
      INTEGER               FIRST
      INTEGER               LAST
      INTEGER               LOC
      INTEGER               MODLEN
      INTEGER               POS
 
      CHARACTER*(10)        ACTION
      CHARACTER*(NAMLEN)    CALLER
      CHARACTER*(FILEN)     DEVICE
      CHARACTER*(NAMLEN)    MODNAM
      CHARACTER*(11)        STRING
 
C
C     Saved variables:
C
      LOGICAL               DOTRC
 
      INTEGER               FRZCNT
      INTEGER               MODCNT
      INTEGER               PSHCNT
      INTEGER               STKPTR
 
      CHARACTER*(TRCLEN)    TRCSTR
      CHARACTER*(TRCLEN)    FRZSTR
 
      SAVE                  DOTRC
      SAVE                  FRZCNT
      SAVE                  FRZSTR
      SAVE                  MODCNT
      SAVE                  PSHCNT
      SAVE                  STKPTR
      SAVE                  TRCSTR
 
C
C     Initial values:
C
      DATA    DOTRC    / .TRUE. /
      DATA    FRZCNT   /  0     /
      DATA    FRZSTR   / ' '    /
      DATA    MODCNT   /  0     /
      DATA    PSHCNT   /  0     /
      DATA    STKPTR   /  1     /
      DATA    TRCSTR   / ' '    /
 
 
C
C     Executable Code:
C
      CALL WRLINE ( 'SCREEN', 'SPICE(BOGUSENTRY)'                  )
      CALL WRLINE ( 'SCREEN', 'TRCPKG: You have called an entry '     //
     .              'that performs no run-time function. '         )
 
      RETURN
 
 
 
 
C$Procedure      CHKIN ( Module Check In )
 
      ENTRY CHKIN ( MODULE )
 
C$ Abstract
C
C     Inform the SPICELIB error handling mechanism of entry into a
C     routine.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     CHARACTER*(*)          MODULE
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  ---------------------------------------------------
C     MODULE     I   The name of the calling routine.
C     FILEN      P   Maximum length of file name.
C
C$ Detailed_Input
C
C     MODULE         is the name of the routine calling CHKIN.  The
C                    named routine is supposed to be `checking in'
C                    when it calls CHKIN; that is, the call should be
C                    the first executable statement following the
C                    reference to the function RETURN (which should be
C                    the first executable statement).
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     FILEN          is the maximum file name length that can be
C                    accommodated by this routine.
C
C$ Exceptions
C
C     CHKIN does not signal errors; rather it writes error messages,
C     so as to avoid recursion.
C
C
C     1)  If the traceback storage area overflows, the short error
C         message SPICE(TRACEBACKOVERFLOW) is written to the error
C         output device.
C
C     2)  If the input argument MODULE is blank, the short error message
C         SPICE(BLANKMODULENAME) is written to the error output device.
C
C     3)  If the input argument MODULE contains embedded blanks, the
C         short error message SPICE(EMBEDDEDBLANK) is written to the
C         error output device.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is part of the SPICELIB error handling mechanism.
C
C     Conceptually, the effect of this routine is to `push' the
C     supplied module name onto a stack.  The routine CHKOUT performs
C     the inverse, or `pop', operation.
C
C     Every routine that participates in the traceback scheme should
C     have a call to CHKIN as the second executable statement.  The
C     first executable statements should be:
C
C        IF ( RETURN() ) THEN
C           RETURN
C        ELSE
C           CALL CHKIN ( module )
C        END IF
C
C     Here module is the name of the routine in which this code appears.
C
C     The line of code preceding the END or any RETURN statement should
C     be
C
C         CALL CHKOUT ( module )
C
C
C     All SPICELIB routines should call CHKIN and CHKOUT, unless they
C     are classified as `error free'.  Programs linked with SPICELIB
C     may also use CHKIN and CHKOUT.
C
C     Routines that don't call CHKIN and CHKOUT won't appear in the
C     traceback.
C
C     All routines that call CHKIN must also call CHKOUT, or else the
C     trace mechanism will become very confused.
C
C     It is possible to disable check-ins (and check-outs) by calling
C     the entry point TRCOFF.  CHKIN and CHKOUT will return immediately
C     upon entry after TRCOFF has been called.  It is not possible to
C     re-enable check-ins and check-outs after calling TRCOFF. Routines
C     that don't call CHKIN and CHKOUT won't appear in the traceback.
C
C$ Examples
C
C     See `Particulars' for an example of how to call this routine.
C
C$ Restrictions
C
C     Routines that call this routine must call CHKOUT immediately
C     prior to any RETURN or END statement.
C
C     Module names are assumed to have no embedded blanks.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 15-JUN-1990  (NJB)
C
C        Disabling of check-ins implemented.  Many parts of the
C        header have be re-written.  Weird spacing ameliorated.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&
 
C$ Index_Entries
C
C     module check in
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 15-JUN-1990  (NJB)
C
C        Disabling of check-ins implemented.  Many parts of the
C        header have be re-written.  Weird spacing ameliorated.
C
C-    Beta Version 1.1.1, 10-FEB-1988  (NJB)
C
C        Parameter declarations documented.  Parameters section added,
C        and parameter declarations listed in `Brief I/O'.
C
C-    Beta Version 1.1.0, 27-OCT-1988  (NJB)
C
C        Cosmetic improvement to code.  Condensed a continued
C        statement into one line.
C
C-&
 
 
C
C     Get out immediately if tracing is disabled.
C
      IF ( .NOT. DOTRC ) RETURN
 
C
C     Although conceptually the trace representation may be thought of
C     as a stack, we implement it as a single character string, to
C     which we append, or from which we remove, module names. The
C     contents of the string are module names, separated by single
C     blanks.  The last module name in the string is the name of the
C     LAST module which `checked in'.  This name can be thought of as
C     being at the `top of the stack'.
C
C     In this implementation, a `push' operation consists of suffixing
C     the current string with a new module name.  A `pop' operation
C     consists of blanking out the last module name in the string, so
C     that the second-to-last name in the string becomes the last name.
C
C     Note:  a count of module names in the stack is maintained so that
C     the traceback representation will not be corrupted in the event
C     of overflow.  The idea is that the number of pushes may exceed
C     the number of module names saved in the traceback string, but
C     `pops' only remove module names from the representation when the
C     pop is `undoing' a push that actually succeeded in placing a new
C     name in the string.
C
C     With this procedure, in the event of overflow, the modules in the
C     traceback string will still correctly represent the top portion
C     of the call chain.
C
 
C
C     Perform a `push', if there's more room in the string. This means
C     suffixing the string on the right with the new name, updating the
C     stack pointer and updating the module count.
C
 
 
      FIRST  =  FRSTNB ( MODULE )
      LAST   =  LASTNB ( MODULE )
 
      MODLEN =  LAST - FIRST  +  1
 
 
      IF ( LAST .EQ. 0  ) THEN
 
         CALL GETDEV (  DEVICE  )
 
         CALL WRLINE (  DEVICE,  'SPICE(BLANKMODULENAME)' )
         CALL WRLINE (  DEVICE,  'CHKIN:  An attempt to check in was' //
     .                 ' made without supplying a module name.'       )
 
         RETURN
 
      ELSE
C
C        Check for embedded blanks:
C
         DO 50001
     .      POS = FIRST, LAST
 
            IF  ( MODULE(POS:POS) .EQ. ' ' ) THEN
 
               CALLER = MODULE
 
               CALL GETDEV (  DEVICE  )
 
               CALL WRLINE (  DEVICE,   'SPICE(EMBEDDEDBLANK)' )
               CALL WRLINE (  DEVICE,
     .                       'CHKIN:  An attempt to check in was '    //
     .                       'made with a module name containing '    //
     .                       'at least  one embedded blank.  The '    //
     .                       'name was:  ' // CALLER(FIRST:LAST) )
 
               RETURN
 
            END IF
 
50001    CONTINUE
 
      END IF
 
 
 
      IF (  MODLEN  .LE.  TRCLEN - STKPTR + 1  )   THEN
 
         MODCNT  =  MODCNT + 1
         PSHCNT  =  PSHCNT + 1
 
         TRCSTR ( STKPTR : STKPTR + MODLEN - 1 )  =  MODULE ( FIRST : )
 
         STKPTR = STKPTR + MODLEN + 1
 
      ELSE
 
         CALL GETDEV ( DEVICE )
 
         CALL WRLINE ( DEVICE,  'SPICE(TRACEBACKOVERFLOW)' )
         CALL WRLINE ( DEVICE,  'CHKIN:  The trace storage is '       //
     .                 'completely full.  No further module names '   //
     .                 'can be added.' )
 
C
C        Even though we can't store another name, we bump the
C        push counter so that checking out won't corrupt the
C        existing traceback information.
C
         PSHCNT =  PSHCNT + 1
 
      END IF
 
      RETURN
 
 
 
 
C$Procedure      CHKOUT ( Module Check Out )
 
      ENTRY CHKOUT ( MODULE )
 
C$ Abstract
C
C     Inform the SPICELIB error handling mechanism of exit from a
C     routine.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     CHARACTER*(*)        MODULE
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     MODULE     I   The name of the calling routine.
C     NAMLEN     P   Maximum module name length.
C     FILEN      P   Maximum file name length.
C
C$ Detailed_Input
C
C     MODULE         is the name of the routine calling CHKOUT.  The
C                    named routine is supposed to be `checking out'
C                    when it calls CHKOUT; that is, the call should be
C                    the last executable statement preceding any exit
C                    from the routine.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     FILEN          is the maximum file name length that can be
C                    accommodated by this routine.
C
C     NAMLEN         is the maximum module name length that can be
C                    accommodated by this routine.
C
C$ Exceptions
C
C     CHKOUT does not signal errors; rather it writes error messages,
C     so as to avoid recursion.
C
C     1)  If the input module name MODULE does not match the name popped
C         from the trace stack, the short error message
C         SPICE(NAMESDONOTMATCH) is written to the error output device.
C
C     2)  If the trace stack is empty, the short error message
C         SPICE(TRACESTACKEMPTY) is written to the error output device.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is part of the SPICELIB error handling mechanism.
C
C     Conceptually, the effect of this routine is to `pop' a module
C     name from a stack.  The routine CHKIN performs the inverse, or
C     `push' operation.
C
C     Every routine that participates in the traceback scheme should
C     have a call to CHKIN as the second executable statement.
C     The first executable statements should be:
C
C        IF ( RETURN() ) THEN
C           RETURN
C        ELSE
C           CALL CHKIN ( module )
C        END IF
C
C     Here module is the name of the routine in which this code appears.
C
C     The line of code preceding the END or any RETURN statement
C     should be
C
C        CALL CHKOUT  ( module )
C
C     All SPICELIB routines should call CHKIN and CHKOUT, unless they
C     are classified as `error free'.  Programs linked with SPICELIB
C     may also use CHKIN and CHKOUT.
C
C     Routines that don't call CHKIN and CHKOUT won't appear in the
C     traceback.
C
C     All routines that call CHKIN must also call CHKOUT, or else the
C     trace mechanism will become very confused.
C
C     It is possible to disable check-ins (and check-outs) by calling
C     the entry point TRCOFF.  CHKIN and CHKOUT will return immediately
C     upon entry after TRCOFF has been called.  It is not possible to
C     re-enable check-ins and check-outs after calling TRCOFF. Routines
C     that don't call CHKIN and CHKOUT won't appear in the traceback.
C
C$ Examples
C
C     1)  Call CHKOUT before a RETURN statement:
C
C            IF ( FAILED() ) THEN
C               CALL CHKOUT ( module )
C               RETURN
C            END IF
C
C
C     2)  Call CHKOUT before an END statement:
C
C            CALL CHKOUT ( module )
C            END
C
C
C     3)  Only ONE call to CHKOUT is needed here:
C
C            CALL CHKOUT ( module )
C            RETURN
C            END
C
C$ Restrictions
C
C     Routines that call this routine must call CHKIN as the second
C     executable statement. (The first is a call to RETURN).
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 15-JUN-1990 (NJB)
C
C        Disabling of check-ins implemented.  Many parts of the
C        header have be re-written.  Weird spacing ameliorated.
C        Removed a bug check.  Short error messages made more
C        specific.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     module check out
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 15-JUN-1990 (NJB)
C
C        Disabling of check-ins implemented.  Many parts of the
C        header have be re-written.  Weird spacing ameliorated.
C        Removed a bug check.  Short error messages changed from
C        SPICE(INVALIDCHECKOUT) to SPICE(NAMESDONOTMATCH) and
C        SPICE(TRACESTACKEMPTY).
C
C-    Beta Version 1.1.1, 10-FEB-1988 (NJB)
C
C        Parameter declarations documented.  Parameters section added,
C        and parameter declarations listed in `Brief I/O'.
C
C-    Beta Version 1.1.0, 27-OCT-1988 (NJB)
C
C        Cosmetic improvement to code.  Removed a blank line
C        separating the first line of a statement from the next
C        continuation line, and condensed and re-organized
C        the statement.  Note:  the precompiler failed to properly
C        convert the orginal statement into standard Fortran.
C
C-&
 
 
C
C     Get out immediately if tracing is disabled.
C
      IF ( .NOT. DOTRC ) RETURN
 
C
C     See the comments at the beginning of the code of CHKIN for a
C     discussion of the implementation ideas used in these routines.
C
      IF ( PSHCNT .GT. 0 ) THEN
 
         IF ( PSHCNT .EQ. MODCNT ) THEN
C
C           We're not in an overflow case, so let's remove the module
C           name at the `top of the stack' and adjust our counters.
C
C           We need to find the beginning of the module name at the top
C           of the stack.  If the module is the only one in the stack,
C           it begins at position 1.  Otherwise, the module name is
C           preceded by a blank.
C
            IF ( MODCNT .LE. 1 ) THEN
 
               BEG = 1
 
            ELSE
C
C              Search for beginning of name...
C
               BEG = STKPTR - 2
 
50002          IF       (       (  TRCSTR ( BEG : BEG ) .NE. ' '  )
     .                    .AND. (  BEG                  .GT.   1  )   )
     .         THEN
 
                  BEG = BEG - 1
 
                  GO TO 50002
               END IF
 
               BEG = BEG + 1
 
            END IF
C
C           At this point, BEG has been set.  END is easy to get;
C           it's just 2 spaces before the stack pointer.
C
            END = STKPTR - 2
 
            PSHCNT               =   PSHCNT - 1
            MODCNT               =   MODCNT - 1
            STKPTR               =   BEG
            MODNAM               =   TRCSTR ( BEG : END )
            TRCSTR ( BEG : END ) =   ' '
 
            IF ( MODNAM .NE. MODULE ) THEN
 
               CALLER = MODULE
 
               CALL GETDEV ( DEVICE )
 
               CALL WRLINE ( DEVICE, 'SPICE(NAMESDONOTMATCH)'    )
               CALL WRLINE ( DEVICE, 'CHKOUT:  Caller is '            //
     .                                CALLER( :RTRIM(CALLER) )        //
     .                               '; popped name is '              //
     .                                MODNAM(:RTRIM(MODNAM))//'.' )
 
            END IF
 
         ELSE
C
C           Overflow case:  just decrement the push count:
C
            PSHCNT  =  PSHCNT - 1
C
C           We don't get any more space, since we didn't remove
C           any names from TRCSTR.
C
         END IF
 
      ELSE
 
         CALL GETDEV ( DEVICE )
 
         CALL WRLINE ( DEVICE, 'SPICE(TRACESTACKEMPTY)'                )
         CALL WRLINE ( DEVICE, 'CHKOUT: An attempt to check out was ' //
     .                         'made when no modules were checked in.' )
 
      END IF
 
      RETURN
 
 
 
 
C$Procedure      TRCDEP ( Traceback depth )
 
      ENTRY TRCDEP ( DEPTH )
 
C$ Abstract
C
C     Return the number of modules in the traceback representation.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     INTEGER               DEPTH
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  ---------------------------------------------------
C
C     DEPTH      O   The number of modules in the traceback.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     DEPTH          Indicates the number of module names in the
C                    traceback representation.
C
C                    The module names represent modules in a call chain,
C                    with the first name being the top-level module,
C                    and the name with index DEPTH being the lowest
C                    level module.
C
C                    The meaning of the traceback depends on the state
C                    of the error handling mechanism.  There are two
C                    cases:
C
C                       1.  In 'RETURN' mode, when an error is
C                           signalled, the traceback at that point is
C                           saved.  TRCDEP, TRCNAM, and QCKTRC will
C                           return values pertaining to the saved
C                           traceback.
C
C                       2.  In all other modes, the traceback represents
C                           the CURRENT call chain.  TRCDEP, TRCNAM,
C                           and QCKTRC will return values pertaining to
C                           the current trace representation.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is part of the SPICELIB error handling mechanism.
C
C$ Examples
C
C     1)  You can use this routine, together with TRCNAM, to create a
C         traceback report. We might wish to create such a report when
C         we have detected an error condition (see FAILED).
C
C         In this example, we assume that the error has already been
C         detected, and that we wish to create a traceback report.  We
C         assume the existence of two user-supplied routines:
C
C            USER_TRACE_FORMAT   --   creates a traceback report in the
C                                     format preferred by the user
C
C            USER_TRACE_INIT     --   indicates that a traceback report
C                                     is to be created; it also
C                                     indicates how many module names
C                                     will be in the report
C
C            C
C            C     Get the trace depth, and retrieve that number of
C            C     module names from the traceback representation.
C            C     Call USER_TRACE_INIT to indicate that a traceback
C            C     report is to be created containing `DEPTH'
C            C     number of module names. Input each of these names,
C            C     as they are retrieved, to USER_TRACE_FORMAT.
C            C
C
C                 CALL TRCDEP           ( DEPTH )
C
C                 CALL USER_TRACE_INIT  ( DEPTH )
C
C
C                 DO INDEX = 1, DEPTH
C
C                    CALL TRCNAM     ( INDEX, MODULE )
C
C                    CALL USER_TRACE_FORMAT ( MODULE )
C
C                 END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 15-JUN-1990  (NJB)
C
C        Some comments updated.  Some cosmetic changes too.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&
 
C$ Index_Entries
C
C     traceback depth
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.0.1, 15-JUN-1990  (NJB)
C
C        Some comments updated.  Some cosmetic changes too.
C-&
 
 
 
C
C     Find the error handling mode.
C
      CALL GETACT ( ACTION )
 
C
C     If we're in 'RETURN' mode, and an error has occurred, we want to
C     use the frozen version of the traceback.  Otherwise, we want to
C     get the module name from the current traceback.
C
      IF  (   ( ACTION .EQ. 'RETURN' )  .AND.   FAILED()   )   THEN
 
         DEPTH = FRZCNT
 
      ELSE
 
         DEPTH = MODCNT
 
      END IF
 
      RETURN
 
 
 
 
C$Procedure      TRCNAM ( Get Module Name from Traceback )
 
      ENTRY TRCNAM ( INDEX, NAME )
 
C$ Abstract
C
C     Return the name of the module having the specified position in
C     the trace representation.  The first module to check in is at
C     position 1.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     INTEGER               INDEX
C     CHARACTER*(*)         NAME
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C
C     INDEX      I   The position of the requested module name.
C     NAME       O   The name in the #INDEX position in the traceback.
C     FILEN      P   Maximum file name length.
C
C$ Detailed_Input
C
C     INDEX          is the position in the traceback of the requested
C                    module name.  The first module to check in is in
C                    the first position; the last to check in the
C                    position indicated by the argument, DEPTH,
C                    returned by TRCDEP.  Note that the first module to
C                    check in is at the top of the traced call chain.
C
C$ Detailed_Output
C
C     NAME           is the name of the module in the position within
C                    the traceback indicated by INDEX.
C
C                    The meaning of the traceback depends on the state
C                    of the error handling mechanism.  There are two
C                    cases:
C
C                       1.  In 'RETURN' mode, when an error is
C                           signalled, the traceback at that point is
C                           saved.  TRCDEP, TRCNAM, and QCKTRC will
C                           return values pertaining to the saved
C                           traceback.
C
C                       2.  In all other modes, the traceback represents
C                           the CURRENT call chain.  TRCDEP, TRCNAM,
C                           and QCKTRC will return values pertaining to
C                           the current trace representation.
C
C$ Parameters
C
C      FILEN   is the maximum file name length that can be
C              accommodated by this routine.
C
C$ Exceptions
C
C     Because this routine is below SIGERR in the calling hierarchy,
C     this routine can not call SIGERR in the event of an error.
C     Therefore, this routine outputs error messages, rather than
C     signalling errors.
C
C     1)  This routine detects the condition of INDEX being out of
C         range.  The short error message set in that case is
C        'SPICE(INVALIDINDEX)'.
C
C     2)  If INDEX is in range, but no module name is found at the
C         indicated location in the trace representation, the error
C         message 'SPICE(INVALIDINDEX)' is set.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is part of the SPICELIB error handling mechanism.
C
C$ Examples
C
C     1)  You can use this routine, together with TRCNAM, to create a
C         traceback report. We might wish to create such a report when
C         we have detected an error condition (see FAILED).
C
C         In this example, we assume that the error has already been
C         detected, and that we wish to create a traceback report.  We
C         assume the existence of two user-supplied routines:
C
C            USER_TRACE_FORMAT   --   creates a traceback report in the
C                                     format preferred by the user
C
C            USER_TRACE_INIT     --   indicates that a traceback report
C                                     is to be created; it also
C                                     indicates how many module names
C                                     will be in the report
C
C            C
C            C     Get the trace depth, and retrieve that number of
C            C     module names from the traceback representation.
C            C     Call USER_TRACE_INIT to indicate that a traceback
C            C     report is to be created containing `DEPTH'
C            C     number of module names. Input each of these names,
C            C     as they are retrieved, to USER_TRACE_FORMAT.
C            C
C
C                 CALL TRCDEP           ( DEPTH )
C
C                 CALL USER_TRACE_INIT  ( DEPTH )
C
C
C                 DO INDEX = 1, DEPTH
C
C                    CALL TRCNAM     ( INDEX, MODULE )
C
C                    CALL USER_TRACE_FORMAT ( MODULE )
C
C                 END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 15-JUN-1990  (NJB)
C
C        Error messages streamlined. Some comments updated.
C        Some cosmetic changes too.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&
 
C$ Index_Entries
C
C     get module name from traceback
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 15-JUN-1990  (NJB)
C
C        Error messages streamlined. Some comments updated.
C        Some cosmetic changes too.
C
C-    Beta Version 1.1.1, 10-FEB-1988  (NJB)
C
C        Parameter declarations documented.  Parameters section added,
C        and parameter declarations listed in `Brief I/O'.
C
C-    Beta Version 1.1.0, 27-OCT-1988  (NJB)
C
C        Added test for failure to remove name from trace
C        representation.  If LOC equals 0 on return from
C        NTHWD, the error SPICE(INVALIDINDEX) is reported.
C        SIGERR is not called; that would be overly recursive.
C
C        Cosmetic changes to header and code were made.  Indentation
C        of some header items was changed, and some blank lines
C        were removed from the code.
C-&
 
 
 
C
C     Get the error handling mode.
C
      CALL GETACT ( ACTION )
 
C
C     If we're in 'RETURN' mode, and an error has occurred, we want to
C     use the frozen version of the traceback.  Otherwise, we want to
C     get the module name from the current traceback.
C
      IF  (   ( ACTION .EQ. 'RETURN' )  .AND.   FAILED()   )   THEN
 
         IF (   ( INDEX .GE. 1 )  .AND.  ( INDEX .LE. FRZCNT )   )  THEN
C
C           We've got a valid index..
C
            CALL NTHWD ( FRZSTR,  INDEX,  NAME,  LOC )
 
         ELSE
C
C           Invalid index...we output the error messages directly
C           in this case:
C
            CALL  GETDEV ( DEVICE )
 
            CALL  WRLINE ( DEVICE,  'SPICE(INVALIDINDEX)' )
 
            CALL  INTSTR ( INDEX, STRING )
 
            CALL  WRLINE ( DEVICE, 'TRCNAM: An invalid index was '    //
     .                     'input.  The value was: '                  //
     .                     STRING( :RTRIM(STRING) )//'.'  )
            RETURN
 
         END IF
 
      ELSE
C
C        Otherwise, use current traceback:
C
         IF (   ( INDEX .GE. 1 )  .AND.  ( INDEX .LE. MODCNT )   )  THEN
C
C           We've got a valid index..
C
            CALL NTHWD ( TRCSTR,  INDEX,  NAME,  LOC )
 
         ELSE
C
C           Invalid index...we output the error messages directly
C           in this case:
C
            CALL  GETDEV ( DEVICE )
 
            CALL  WRLINE ( DEVICE,  'SPICE(INVALIDINDEX)' )
 
            CALL  INTSTR ( INDEX, STRING )
 
            CALL  WRLINE ( DEVICE, 'TRCNAM: An invalid index was '    //
     .                     'input.  The value was: '                  //
     .                     STRING( :RTRIM(STRING) )//'.'  )
 
         END IF
 
      END IF
 
      IF ( LOC .EQ. 0 ) THEN
 
         CALL GETDEV ( DEVICE )
 
         CALL WRLINE ( DEVICE, ' ' )
 
         CALL WRLINE ( DEVICE,  'SPICE(INVALIDINDEX)' )
 
         CALL INTSTR ( INDEX, STRING )
 
         CALL WRLINE ( DEVICE, 'TRCNAM:  No module was found in the ' //
     .                         'traceback string in the position '    //
     .                         'specified by the following index: '   //
     .                         STRING                                  )
 
      END IF
 
      RETURN
 
 
 
C$Procedure      QCKTRC ( Get Quick Traceback )
 
      ENTRY QCKTRC ( TRACE )
 
C$ Abstract
C
C     Return a string containing a traceback.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     CHARACTER*(*)         TRACE
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C
C     TRACE      O   A traceback report.
C     NAMLEN     P   Maximum module name length.
C     FILEN      P   Maximum file name length.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     TRACE          is a list of module names, delimited by the
C                    string, ' -->'.  An example would be
C
C                       'SPUD -->SPAM -->FOOBAR'.
C
C                    In general, the meaning of the trace is as
C                    follows:
C
C                    The first name in the list is the name of the first
C                    module to check in (that hasn't yet checked out).
C                    The last name is the name of the module at the end
C                    of the call chain; this is the last module that
C                    checked in.
C
C                    The meaning of the traceback depends on the state
C                    of the error handling mechanism.  There are two
C                    cases:
C
C                       1.  In 'RETURN' mode, when an error is
C                           signalled, the traceback at that point is
C                           saved.  TRCDEP, TRCNAM, and QCKTRC will
C                           return values pertaining to the saved
C                           traceback.
C
C                       2.  In all other modes, the traceback represents
C                           the CURRENT call chain.  TRCDEP, TRCNAM,
C                           and QCKTRC will return values pertaining to
C                           the current trace representation.
C
C                    Any module names exceeding NAMLEN characters in
C                    length are truncated on the right.
C
C$ Parameters
C
C     FILEN          is the maximum file name length that can be
C                    accommodated by this routine.
C
C     NAMLEN         is the maximum module name length that can be
C                    accommodated by this routine.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is part of the SPICELIB error handling mechanism.
C
C$ Examples
C
C     1)  Here's an example of how to use this routine:
C
C           C
C           C     We call RDTEXT and test for an error condition.
C           C     If an error occurred, we get the traceback and
C           C     long error message and output them using the
C           C     user-defined routine, USER_ERROR.
C           C
C
C                 CALL RDTEXT ( FILE, LINE, EOF )
C
C                 IF ( FAILED() ) THEN
C
C                    CALL QCKTRC     ( TRACE )
C                    CALL USER_ERROR ( TRACE )
C
C                    CALL GETMSG     ( 'LONG', MSG )
C                    CALL USER_ERROR (         MSG )
C
C                 END IF
C
C$ Restrictions
C
C     It is assumed no module names exceed NAMLEN characters in length.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.2.0, 23-OCT-1992 (NJB)
C
C        Bug fix made to routine QCKTRC:  a section of code which
C        itself is exercised only if a bug is present inserted the
C        wrong variable into an error message.
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 15-JUN-1990  (NJB)
C
C        Error messages streamlined. Some comments updated.
C        Some cosmetic changes too.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&
 
C$ Index_Entries
C
C     get quick traceback
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.2.0, 23-OCT-1992 (NJB)
C
C        Bug fix made to routine QCKTRC:  a section of code which
C        itself is exercised only if a bug is present inserted the
C        wrong variable into an error message.  The variable in
C        question was the input argument INDEX; the correct variable
C        to insert in the message is the local variable POS.
C
C-    SPICELIB Version 1.1.0, 15-JUN-1990  (NJB)
C
C        Error messages streamlined. Some comments updated.
C        Some cosmetic changes too. Use of SUFFIX made more
C        rational.
C
C-    Beta Version 1.1.1, 10-FEB-1988  (NJB)
C
C        Parameter declarations documented.  Parameters section added,
C        and parameter declarations listed in `Brief I/O'.
C
C-    Beta Version 1.1.0, 06-OCT-1988  (NJB)
C
C        Added test for failure to remove name from trace
C        representation.  If LOC equals 0 on return from
C        NTHWD, the error SPICE(INVALIDINDEX) is reported.
C        SIGERR is not called; that would be overly recursive.
C
C        Also, some cosmetic changes to code were made.  Some
C        unnecessary continuation lines were removed.
C-&
 
 
C
C     Get the error handling mode.
C
      CALL GETACT ( ACTION )
 
C
C     If we're in 'RETURN' mode, and an error has occurred, we want to
C     use the frozen version of the traceback.  Otherwise, we want to
C     use the current traceback.
C
      IF  (  ( ACTION .EQ. 'RETURN' )  .AND.  FAILED()  ) THEN
 
         COUNT = FRZCNT
 
      ELSE
 
         COUNT = MODCNT
 
      END IF
 
C
C     The names are delimited by ' --> '; e.g.,
C
C     `SPUD --> SPAM --> FOOBAR`.
C
      TRACE = ' '
 
      DO 50003
     .   POS = 1, COUNT
 
         IF  (  ( ACTION .EQ. 'RETURN' )  .AND.  FAILED()  ) THEN
 
            CALL NTHWD  (  FRZSTR, POS, MODNAM, LOC  )
 
         ELSE
 
            CALL NTHWD  (  TRCSTR, POS, MODNAM, LOC  )
 
         END IF
 
         IF ( LOC .EQ. 0 ) THEN
 
            CALL GETDEV ( DEVICE )
 
            CALL WRLINE ( DEVICE,  'SPICE(INVALIDINDEX)' )
 
            CALL INTSTR ( POS, STRING )
 
            CALL WRLINE ( DEVICE, 'QCKTRC:  No module was found in '  //
     .                            'the traceback string in the '      //
     .                            'position numbered '//STRING  )
 
            RETURN
 
         END IF
 
 
         IF ( COUNT .GT. 1 ) THEN
 
            IF ( POS .EQ. 1 ) THEN
 
               TRACE = MODNAM
               CALL SUFFIX ( '-->',  1, TRACE )
 
            ELSEIF ( ( POS .GT. 1 )  .AND. ( POS .LT. COUNT )  )  THEN
 
               CALL SUFFIX ( MODNAM, 1, TRACE )
               CALL SUFFIX ( '-->',  1, TRACE )
 
            ELSE
 
               CALL SUFFIX (   MODNAM, 1, TRACE )
 
            END IF
 
         ELSE IF ( COUNT  .EQ. 1 ) THEN
 
            TRACE = MODNAM
 
         ELSE
 
            TRACE = ' '
 
         END IF
 
50003 CONTINUE
 
      RETURN
 
 
 
C$Procedure   FREEZE   ( Get frozen copy of traceback )
 
      ENTRY FREEZE
 
C$ Abstract
C
C     Make a copy of the current traceback.  This copy is frozen, i.e.
C     unchanged, until the next call to FREEZE. DO NOT CALL THIS
C     ROUTINE.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C
C     None.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     DO NOT CALL THIS ROUTINE.
C
C     When the error response action is 'RETURN', and an error is
C     signalled, a copy of the traceback is saved for later retrieval
C     by the application program.  This is called the `frozen' version
C     of the traceback.  FREEZE is used to create this frozen version.
C
C     This routine is called by the SPICELIB routines SIGERR and RESET.
C
C$ Examples
C
C     1)
C         C
C         C     Create a frozen traceback:
C         C
C               CALL FREEZE
C
C$ Restrictions
C
C     For SPICELIB error handling only.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 15-JUN-1990  (NJB)
C
C       Some comments changed. Cosmetic changes too.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&
 
C$ Index_Entries
C
C     None.
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.0.1, 15-JUN-1990  (NJB)
C
C       Some comments changed. Cosmetic changes too.
C
C-    Beta Version 1.0.1, 08-FEB-1989 (NJB)
C
C        Warnings added to discourage use of this routine in
C        non-error-handling code.
C
C-&
 
 
C
C     Create a frozen version of the traceback:
C
      FRZCNT = MODCNT
      FRZSTR = TRCSTR
 
      RETURN
 
 
 
 
C$Procedure  TRCOFF  ( Turn tracing off )
 
      ENTRY TRCOFF
 
C$ Abstract
C
C     Disable tracing.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C
C     None.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This routine disables tracing.  Checking in or out does not modify
C     the current traceback any further after TRCOFF is called. The
C     routines TRCNAM, TRCDEP, and QCKTRC will return information
C     based on the traceback at the point where TRCOFF is called.
C
C     Once tracing has been disabled, it cannot be re-enabled.
C
C     Additionally, TRCOFF blanks out the existing trace, since the
C     trace will usually be invalid at the time an error is signalled.
C     The frozen copy of the trace, if there is one, is not modified.
C
C$ Examples
C
C     1)    C
C           C     Program initialization:
C           C
C                      .
C                      .
C                      .
C           C
C           C     We disable tracing to enhance speed:
C           C
C                 CALL TRCOFF
C           C
C           C     More initialization code:
C           C
C                      .
C                      .
C                      .
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 11-JUL-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     turn tracing off
C
C-&
 
 
C
C     Indicate that tracing is disabled:
C
      DOTRC = .FALSE.
 
C
C     Blank out the trace.  The stack depth becomes 0 (it will be
C     referenced if TRCDEP is called). The push count is set to 0 for
C     consistency; it will not be referenced again after this code is
C     executed.
C
      TRCSTR = ' '
 
      MODCNT = 0
      PSHCNT = 0
 
      RETURN
      END
