C$Procedure      TXTOPR ( Text file, open for read )
 
      SUBROUTINE TXTOPR ( FNAME, UNIT )
 
C$ Abstract
C
C     Open a text file for read access.
C
C$ Required_Reading
C
C     TEXT
C
C$ Keywords
C
C     FILES
C     TEXT
C
C$ Declarations
 
      CHARACTER*(*)         FNAME
      INTEGER               UNIT
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of file.
C     UNIT       O   Logical unit.
C
C$ Detailed_Input
C
C     FNAME          is the name of the text file to be opened.
C
C$ Detailed_Output
C
C     UNIT           is the logical unit connected to the opened file.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If FNAME is a blank string, the error SPICE(BLANKFILENAME) is
C        signalled.
C
C     2) If the file cannot be opened, the error SPICE(FILEOPENFAILED)
C        is signalled.
C
C$ Files
C
C     See FNAME and UNIT above.
C
C$ Particulars
C
C     In SPICELIB, a text file is formatted and sequential and may
C     contain only printable ASCII characters and blanks (ASCII 32-127).
C     When printing a text file, records are single spaced; the first
C     character will not be interpreted as a carriage control character.
C
C     TXTOPR opens an existing text file for read access and makes use
C     of the SPICELIB mechanism for coordinating use of logical units.
C
C     System Dependencies
C     ===================
C
C     The open statement will include the following keyword = value
C     pairs:
C
C            UNIT   =  UNIT
C            FILE   =  FNAME
C            FORM   = 'FORMATTED'
C            ACCESS = 'SEQUENTIAL'
C            STATUS = 'OLD'
C            IOSTAT =  IOSTAT
C
C     In addition, the statement will include
C
C            CARRIAGECONTROL = 'LIST'
C            READONLY
C
C     for the Vax and Macintosh, or
C
C            MODE            = 'READ'
C
C     for the IBM pc.
C
C$ Examples
C
C     The following example reads the first line from an input file,
C     'INPUT.TXT', and writes it to an output file, 'OUTPUT.TXT'.
C
C        CALL TXTOPR ( 'INPUT.TXT',  IN  )
C        CALL TXTOPN ( 'OUTPUT.TXT', OUT )
C
C        READ  ( IN,  FMT='(A)' ) LINE
C        WRITE ( OUT, FMT='(A)' ) LINE
C
C        CLOSE ( IN  )
C        CLOSE ( OUT )
C
C$ Restrictions
C
C     The file, FNAME, must exist prior to calling TXTOPR.
C
C$ Literature_References
C
C     1. "Lahey F77L EM/32 FORTRAN Language Reference Manual", page
C        145.
C
C$ Author_and_Institution
C
C     J.E. McLean    (JPL)
C     H.A. Neilan    (JPL)
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.2.0, 12-OCT-1992 (HAN)
C
C        The code was also reformatted so that a utility program can
C        create the source file for a specific environment given a
C        master source file.
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 13-NOV-1991 (MJS)
C
C        Module updated to allow portability to the Lahey F77L EM/32
C        FORTRAN V 4.0 environment.
C
C-    SPICELIB Version 1.0.0, 05-APR-1991 (JEM)
C
C-&
 
C$ Index_Entries
C
C     text file open for read
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               IOSTAT
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'TXTOPR' )
      END IF
 
      IF ( FNAME .EQ. ' ' ) THEN
         CALL SETMSG ( 'A blank string is unacceptable as a file name' )
         CALL SIGERR ( 'SPICE(BLANKFILENAME)'                          )
         CALL CHKOUT ( 'TXTOPR'                                        )
         RETURN
      END IF
 
      CALL GETLUN ( UNIT )
 
      OPEN ( UNIT            =  UNIT,
     .       FILE            =  FNAME,
     .       FORM            = 'FORMATTED',
     .       ACCESS          = 'SEQUENTIAL',
     .       STATUS          = 'OLD',
     .       IOSTAT          =  IOSTAT      )
 
      IF ( IOSTAT .NE. 0 ) THEN
         CALL SETMSG ( 'Could not open file #. IOSTAT was #. ' )
         CALL ERRCH  ( '#', FNAME                              )
         CALL ERRINT ( '#', IOSTAT                             )
         CALL SIGERR ( 'SPICE(FILEOPENFAILED)'                 )
         CALL CHKOUT ( 'TXTOPR'                                )
         RETURN
      END IF
 
      CALL CHKOUT ( 'TXTOPR' )
      RETURN
      END
