#! /bin/csh -f
#
# There are some problems when using this as executable...so use an
# alias:
#
#  alias checker 'set checker_args=(\!*);source ~/.checkrc'
#
# Don't modify this file without written consent of Peter Teuben
# Since this file is used for both loading and unloading the
# checker (see the alias for 'exit') be very careful when you
# add features to the checker...
#
# History:
#       xx-mar-93   first off-line version      Peter Teuben
#       10-jun-93   working on-line version     PJT
#	14-jun-93   testing to work from executable instead of alias  PJT
#       18-jun-93   alias 'exit' to remove CHECKER (somewhate) ..     PJT
#       22-jun-93   testing off-line version                          pjt
#       29-jun-93   added arguments to the alias                      pjt
#        2-jul-93   all exit's become 'goto exit', add -x, -q         pjt
#       13-jul-93   exit with cleanup uption                          pjt
#	28-jul-93   modify DATA to /tmp after HATRC load ...	      pjt
#                   hatcat alias to toggle HATCAT ...
#       16-sep-93   deal with new HATDATA variable at HatCreek        pjt
#                   although the HATCHK in hatproject.csh should cure
#                   all problems, and resetting DATA/HATDATA should
#                   not be needed anymore....
#	21-oct      new location at bima
#	 5-nov-93   HATDATA now /tmp/checker$$ with 'mkdir -p'		pjt
#        2-feb-94   added RESET_VAR to allow project to reset variables pjt
#       18-feb-94   added doc about usage and point to manual           pjt
#	10-jun-94   rely on $NEMO, if present 				pjt
#	14-sep-94   added UCAT (if not set yet) as $NEMO/local/cats     pjt
#                   
#
#* checker - load environment to check observing scripts
#& pjt
#: User Utility
#+
#  CHECKER is an alias which modifies your current environment such 
#  that all observing commands become available in emulated mode. This
#  allows you to run observing shell scripts and check them for
#  correctness, on-line (at Hat Creek) as well as off-line. Much more
#  detailed information can be found in the ``Checker'' chapter in
#  the Hat Creek Users Guide.
#
#  Be sure to (re)set your LST clock with the LSTSYMBOL command,
#  using keyword LST=, for example
#	
#	lstsymbol lst=1430    (hhmm notation)
#       lstsymbol lst=14.5    (decimal hours notation)
#
#  are both equivalent. All observing command, except LSTSYMBOL of
#  course, have exactly the same keywords and meaning in CHECKER mode
#  as regular observing mode.
#
#  In CHECKER mode the 'exit' command has been redefined, and will
#  remove the CHECKER environment form your shell.
#
#  Note: CHECKER should not be run on the machine from which real 
#  observing programs are launched (currenty bima.berkeley.edu). For
#  that reason it has been disabled on that machine.
#
#  Note: Because it is an alias, CHECKER is not available from within 
#  the miriad shell, you have to be in the login shell.
#
#  Example of usage (February 1994 onwards):
#       % lstsymbol lst=730
#       % project name=s555a105.test start=730 stop=1600 >& checker.log
#       % summary checker.log
#
#  The following (optional) command line arguments are understood:
#
#@ -h, help
#  inline help, also displayed when a command line argument is not
#  understood.
#
#@ -c, nocommon
#  Does not create the $HATCOM common block
#
#@ -b, bima
#  Attempt to get the common block from bima (the mother machine) 
#  via a remote copy. You need special permission to do this. Alternatively,
#  simply ftp the file bima.berkeley.edu:/tmp/hatcommon to your local
#  version of $HATCOM
#
#@ -p, nopath
#  Does not modify your command search path
#
#@ -m, nomotd
#  Does not show the message of the day  
#
#@ -q, query
#  Query current status of the checker
#
#@ -x, exit
#  Unload (exit) the checker environment (same as 'exit' alias)
#
#@ -r, cleanup
#  Also cleanup your current version of common block, LST clock etc.
#  temporary files. This option only work in exit mode of course.
#--

# set defaults
set add_path=1
set make_common=1
set show_motd=1
set get_remote=0
set do_cleanup=0

# process alias arguments if present, exit out in case of errors.
if ($?checker_args == 1) then
  set tmp_args=($checker_args)
  unset checker_args
  foreach arg ($tmp_args)
    switch ($arg)
    case -h*:
    case help*:
      goto usage
    case -c*:
    case nocommon*:
      set make_common=0
      breaksw
    case -b*:
    case bima*:
      set get_remote=1
      breaksw
    case -p*:
    case -nopath*:
      set add_path=0
      breaksw
    case -m*:
    case nomotd*:
      set show_motd=0
      breaksw
    case -q*:
    case query*:
      goto query
    case -x*:
    case exit*:
      set hat_exit=1
      breaksw
    case -r*:
    case clean*:
      set do_cleanup=1
      breaksw
    default:
      echo "Argument '$arg' not understood, try -h for help"
      goto exit
    endsw
  end
endif

if ($?hat_exit == 1) then
  echo "Unloading BIMA checker..."
  # reset HATCOM
  if ($do_cleanup) then
    echo "Cleanup temp files"
    rm -f $HATCOM*
  endif
  # restore this, but really shouldn't matter
  setenv HATCOM /tmp/hatcommon
  # unset this, since it's the checker trigger
  unsetenv HATCHK
  # restore old PATH by removing NEMOBIN
  set new_path=()
  foreach p ($path)
    if ($p != $NEMOBIN) set new_path = ($new_path $p)
  end
  set path=($new_path)
  rehash
  # get rid of the temporary alias and shell variables
  unalias exit
  #unalias mkdir
  #unalias chdir
  unset hat_exit
  unset new_path
  # try and restore prompt (may not work on all 'cd' alias implementations)
  cd $cwd
  echo " [You need one more 'exit' to exit from this login shell]"
  # and return 
  goto exit
endif

echo "Loading checker..."

# make sure it cannot run on bima - by faking HOST we have a backdoor (:-)
if ($?HOST == 1) then
  set site=$HOST
else
  set site=`hostname`
endif

if ("$site" == bima) then
  echo You cannot run the checker on bima, please use bima2 or bima3
  goto exit
else if (-d /extra/prog) then
  set site=bima
else if (-d /lma && $?NEMO == 0 ) then
  set site=umd
else 
  set site=other
endif

if ( "$site" == umd ) then
  setenv NEMO /lma/nemo
  setenv CHECKER $NEMO/usr/bima/checker
  setenv HAT  $CHECKER/prog
  setenv DATA $CHECKER/data
  source $HAT/HATRC
  echo "UMD setup:  HAT=$HAT"
  echo "         : DATA=$DATA"
  echo "         : NEMO=$NEMO"
  # override to make 'mkdir' in scripts work
  setenv DATA /tmp/checker$$
  setenv HATDATA /tmp/checker$$
  mkdir -p $HATDATA
  echo "Overriding HATDATA=DATA=$DATA for off-line checkers"
endif

if ( "$site" == bima ) then
  # special case: no need to source our HATRC, rely on theirs...
  setenv NEMO ~data/users/teuben/nemo
  setenv CHECKER $NEMO/usr/bima/checker
  # override to make 'mkdir' in scripts work
  setenv DATA /tmp/checker$$
  setenv HATDATA /tmp/checker$$
  mkdir -p $HATDATA
  echo "Overriding HATDATA=DATA=$DATA for off-line checkers"
endif

if ( "$site" == "other") then
  # Assume NEMO has been set
  if ($?NEMO == 0) then
     echo The environment variable NEMO must be set
     goto exit
  endif
  setenv CHECKER $NEMO/usr/bima/checker
  setenv HAT  $CHECKER/prog
  setenv DATA $CHECKER/data
  source $HAT/HATRC
  echo "GENERIC setup:  HAT=$HAT"
  echo "             : DATA=$DATA"
  echo "             : NEMO=$NEMO"
  # override to make 'mkdir' in scripts work
  setenv DATA /tmp/checker$$
  setenv HATDATA /tmp/checker$$
  mkdir -p $HATDATA
  echo "Overriding HATDATA=DATA=$DATA for off-line checkers"

endif

# make sure a random HAT variable has been set
if ($?HATCAT == 0) then
  echo HATCAT not present
  goto exit
endif


if ($cwd == "/tmp") then
   echo Cannot run checker from /tmp.
   goto exit
endif

# set new location of common block
   setenv HATCOM $cwd/hatcommon

# use NEMO as base
   source $NEMO/NEMORC
   setenv CHECKER $NEMO/usr/bima/checker
   # alias some common things...... don't work inside scripts in '-f' startup
   alias exit 'set hat_exit=1;checker'
   #alias mkdir 'echo CHECKER: mkdir=\!*'
   #alias chdir 'echo CHECKER: chdir=\!*'
   setenv HATCAT_OLD .
   alias hatcat 'set tmp=$HATCAT_OLD;setenv HATCAT_OLD $HATCAT;setenv HATCAT $tmp;echo HATCAT now toggled to $HATCAT'
   setenv RESET_VAR 1
   if ($?UCAT == 0) setenv UCAT $NEMO/local/cats

# prepend to searchpath, if it wasn't done yet
   if ($add_path) then
     if ($path[1] == ".") then
       if ($path[2] != $NEMOBIN) then
         set path=(. $NEMOBIN $path[2-])
       else
         echo checker BIN was already loaded
       endif
     else
       if ($path[1] != $NEMOBIN) then
         set path=($NEMOBIN $path)
       else
         echo checker BIN was already loaded
       endif
     endif
     rehash
   endif

#  Create a common block, if none present!!!
   if ($make_common) then
     if (-e $HATCOM) then
       echo Using your old copy of $HATCOM
       echo "Delete this file or start form scratch with 'createcom'"
     else
       if ( "$site" == "bima" || "$get_remote" ) then
	 createcom >& /dev/null
         echo "Attempting to get common from data@bima.berkeley.edu"
         rcp data@bima.berkeley.edu:/tmp/hatcommon $HATCOM
       else
	 createcom
       endif
       if ($status == 0) then
         echo Copied the current common block from bima to:
         echo " $HATCOM"
         echo "Use 'alter' to modify and 'value' to view"
       else
         echo "Error copying common block from bima (bima:/tmp/hatcommon)"
	 echo "Attempting to create a blank common:"
	 createcom
       endif
     endif
     if (-e $HATCOM) then
        chmod 755 $HATCOM* >& /dev/null
     else
	echo "### Fatal error, you must run 'createcom', you have no common"
        echo "### There was a problem copying/creating $HATCOM"
        echo "### Proceed at your own risk"
     endif
   endif

#  show local time to the user

   if (-e $HATCOM.lst) then
      echo The current LST is `lstsymbol`
   else
      echo "### Warning: No current LST available yet (use 'lstsymbol lst=hhmm')"
   endif

#  change the prompt until the next 'cd' command, assuming cd does that job
   set prompt="CHECKER:$cwd:t>"

#  Set HATCHK to be root PID for ID purposes
#  is e.g. used to see if motd has to be displayed, but can also
#  be used in observe scripts to distinguish online and offline situations

   if ($?HATCHK == 0) then
      setenv HATCHK $$
      if (-e $NEMO/etc/motd && $show_motd) cat $NEMO/etc/motd
   endif

#  here's the trick, we switch context to a new shell
#  I don't feel comfortable with this anymore, also creates a bootstrap
#  problem when installing the checker...(pjt)

#   echo "Entering checker... exit when done"
#
#   if ($?SHELL) then
#      exec $SHELL -f
#   else
#      exec csh -f
#   endif
#
  goto exit

# Usage and exit
usage:
  echo "The CHECKER command modifies your environment to emulate"
  echo "BIMA observing programs."
  echo ""
  echo "Optional arguments:"
  echo "  -h, help                   this help"
  echo "  -c, nocommon               do not create common"
  echo "  -b, bima                   attempt to get remote common from bima"
  echo "  -p, nopath                 do not modify command search path"
  echo "  -m, nomotd                 do not display message of the day"
  echo "  -q, query                  query status of checker"
  echo "  -x, exit                   exit the checker environment"
  echo "  -r, cleanup                cleanup temp files at exit"
  echo ""
  echo "Checker commands that deviate from Observing commands:"
  echo "  lstsymbol [lst=hhmm] [inc=hhmm] [format=%g]         reset your LST"
  echo ""
  goto exit


# Query and exit
query:
  echo "Query: Not implemented yet"
  goto exit


# overal exit point (don't actually 'exit' here, since this is an alias)
# this is like a return statement
exit:
  
