#! /bin/csh -f
#
#    Observing Script Example 2 - july 1993
#    Single Source MINT plus multiple Calibrators plus Passband observeration
#    Based on mintobs.csh, but largely rewritten.
#
#    ===> Please, if you modify this template, put your own name / 
#    ===> comments here, and remove these lines.
#

echo "**********************************************************"
echo "       example2.csh  (28jul93 version  -pjt) "
echo "**********************************************************"

# set default commandline keywords for which defaults are accepted        <=== 1
 set tune=f
# unset required variables (done merely for clarity here)
 unset ants
 unset planet
 unset stop

# test for the correct (minimum) number of arguments (# unset's above)    <=== 2
 if ($#argv < 3) goto usage
 if ($1 == help) goto usage

# parse commandline arguments (assumed to be of key=val type)	          <=== 3
 foreach a ($*)
  set check=`echo $a | awk -F= '{print NF}'`
  if ( "$check" > 1 ) then
    set $a
  else
    echo "### Warning: skipping argument $a; not constructed as key=val"
  endif
 end

# setup start-stop logic					          <=== 4
 set start=`lstsymbol`
 set logic='&&'
 if ($start < $stop) set logic='||'
 alias if_timeup 'set lst=`lstsymbol`;if($lst>$stop $logic $lst<$start) \!*'

# define useful things: shell variables and aliases		          <=== 5
 set date=`hatdate`

#				set up source & calibrators
set source=cygnus
set cals=(2007+404  2023+318)
set integ=50
set nchan=0
set grid=cygnus
set scale=1.0
set time3=-1,-.52
alias doint "int ants=$ants itime=$integ nchan=$nchan"
alias domint "mint ants=$ants itime=$integ nchan=$nchan grid=$grid scale=$scale times=$time3"

#
#   Observing can commense ###################################################
# 

#				set up directory

 mkdir -p $DATA/mosaic
 cd $DATA/mosaic

#		    set up frequency and tune and correlator as needed

 if ( "$tune" == "t") then
   setfreq freq=90.400 iffreq=+150.0
   tuner ants=$ants check=y
 endif
 setcorr cormode=4 corbw=100,100 corf=175,375,575,775 ants=$ants


 set loop=`checksource source=$planet elevlim=12 setme=visible`
 if ( "$loop" == "y" ) then
     doint source=$planet vis="$planet.$date" times=-1,-0.5
     echo "Finished Observing $planet"
 endif
#
# Do first calibration observation
#
 foreach cal ($cals)
   set loop=`checksource source=$cal elevlim=12 setme=visible`
   if ( "$loop" == "y") break
 end
 if ( "$loop" == "y") then
   doint source=$cal   vis="$cal.$date"    times=-1,-.15
 endif
#
# Begin source/cal cycle
#
 while (1)
    if_timeup break

    set loop=`checksource source=$source elevlim=12 setme=visible`
    if ( "$loop" != "y") break
    domint source=$source vis=mint_"$source.$date" 

    foreach cal ($cals)
      set loop=`checksource source=$cal elevlim=12 setme=visible`
      if ( "$loop" == "y") break
    end
    if ( "$loop" == "y") then
      doint source=$cal   vis="$cal.$date"    times=-1,-.12
    endif
 end

#			finished observations

 echo "Finished Observing $source"
 exit 0

# Usage message

usage:
 echo "Usage: $0 ants=13456 planet=mars stop=1230 [tune=$tune]"
 echo ""
 echo " Required keywords: ants=, planet, stop="
 echo " Optional keywords: tune=$tune"
 echo ""
 echo "Their order on the commandline is irrelevant"
 exit 0

