#! /bin/csh -f
#
#    Observing Script Serpens SVS-20 - july 1993	          
#
#    ===> Joe McMullin, UMD
#    ===> correlator set to observe CS + others
#  To adapt this script:
#  1. change the number or default of the optional keywords 
#  2. change the unset's for required keywords, and value of minarg
#  3. define useful things (aliases, working directory, etc.)


# set default commandline keywords for which defaults are accepted     <=== EDIT
# As of 31jul93 Rx's 6 and 7 are out of commission. jhowe
 set ants=1345
 set integ=30
# unset required variables (done merely for clarity here) and set minarg <= EDIT
# we prefer minarg to be 0 though, i.e. no 'unset' variables.
 unset stop
 set minarg=1 

# test for the correct (minimum) number of arguments (# unset's above) <=== FIX
 if ($#argv < $minarg) goto usage
 if ($1 == help) goto usage

# parse commandline arguments (assumed to be of key=val type)          <=== FIX
 foreach a ($*)
  set check=`echo $a | awk -F= '{print NF}'`
  if ( "$check" > 1 ) then
    set $a
  else
    echo "### Warning: skipping argument $a; not constructed as key=val"
  endif
 end

# setup start-stop logic                                               <=== FIX
 set start=`lstsymbol`
 set logic='&&'
 if ($start < $stop) set logic='||'
 alias if_timesup 'set lst=`lstsymbol`;if($lst>$stop $logic $lst<$start) \!*'

# define useful things: shell variables and aliases                    <=== EDIT
 set date=`hatdate`
 alias scan "int ants=$ants itime=$integ nchan=2"
 alias isup "checksource elevlim=20 setme=visible"
 set pbdone=no
# your source and calibrators (in lists in descreasing order of importance)
 set source=svs20
 set sel_cal=(1733-130 2148+069)
 set sel_pb=(mars 3c273 3c454.3 3c84)

# Running out of diskspace on /home/bima2, so send the data to the /extra disk. jhowe 31jun93
# set work=$DATA/joe
 set work=/extra/data/joe

#
# Observing Can Commence =======================================================
#

# some questionable, but needed startups
 mkdir -p $work; cd $work
 unlock

# Set up the frequency, correlator, and tune

 setfreq freq=97.981 iffreq=135.5 source=$source catalog=jpm.cat obsline=cs
 setcorr cormode=3 corbw=25,25 corf=148,417,679,748 ants=$ants
 tuner ants=$ants check=n

# Try and observe the passband first (another try occurs after src/cal loop)

 foreach pb ($sel_pb)
    set isup=`isup source=$pb`
    if ( "$isup" == "y" ) then
      scan source=$pb vis=$pb.$date times=-1,-0.4 
      echo "Finished observing $pb"
      set pbdone=yes
      break
    endif
 end

# Begin source/cal cycle; quit when either has set or stoptime has reached

 while (1)
    # stop-time must not be reached
    if_timesup break

    # some calibrator must be up
    foreach cal ($sel_cal)
       set isup=`isup source=$cal`
       if ( "$isup" == "y" ) break
    end
    if ( "$isup" != "y" ) break

    # source must be up
    set isup=`isup source=$source catalog=jpm.cat`
    if ( "$isup" != "y" ) break

    # now integrate for a bit
    scan source=$cal    vis=$cal.$date     times=-1,-0.1
    scan source=$source vis=$source.$date  times=-1,-0.4 catalog=jpm.cat
 end

# Final calibration observation (let's hope some $cal is still up...)

 foreach cal ($sel_cal)
    set isup=`isup source=$cal`
    if ( "$isup" == "y") then
       scan source=$cal   vis="$cal.$date"     times=-1,-0.1
       break
    endif
 end

# If passband had not been up earlier, try again, followed by final check

 if ( "$pbdone" == "no") then
    foreach pb ($sel_pb)
       set isup=`isup source=$pb`
       if ( "$isup" == "y" ) then
         scan source=$pb vis="$pb.$date" times=-1,-0.4
         echo "Finished observing $pb"
         set pbdone=yes
       endif
    end
 endif
 if ( "$pbdone" == "no") then
    echo "### Warning: never did the passband on passband $pb"
 endif

 echo "Finished observing $source"
 exit 0

#
# End of observing ============================================================
#

# 
# Usage message, with a safety exit before usage 
# Note the usage of the default shell variables for optional keywords
#
  exit 0
usage:                                                                 <=== EDIT
  echo "Usage: $0 [ants=$ants]  [integ=$integ] stop=hhmm"
  echo "Example single source + multiple calibrator + passband observing"  
  echo "	[P_i_b] { (C_i,S) ... } C_i [P_i_a]"
  exit 0


