#!/bin/csh -f
#
#  This script is used to reset the variables of a named project.
#  If any setups are explicitly entered, then only these are removed.
#  If no setups are entered, then the entire variable list is removed.
#-----------------------------------------------------------------------
#
#  History:
#    21dec93  jm Original script.
#    08feb94  jm Added optional date argument and modified project name
#                construction to account for 14 character limit.
#
#-----------------------------------------------------------------------
#  Unset required and optional arguments.
#
set project = "project"
unset date
unset name
unset setup
#
#  Make sure the archive directory is specified.
#
if ($?HATDATA == 0) then
  setenv HATDATA $DATA/archive/newdata
endif
if (! -d "$HATDATA") then
  echo "### WARNING: Data directory [$HATDATA] does not exist\!"
  exit 1
endif
#
#  Get the command line arguments.
#
while ($#argv)
  if (`echo "$argv[1]" | awk -F= '{print NF}'` > 1) then
    set $argv[1]
  endif
  shift
end
#
#  Check that the required inputs have been given.
#
if ($?name == 0) then
  echo "### No project name provided."
  echo "### Usage: $0 name=projectName [setup=A,B] [date=ddmmmyy]"
  exit 1
endif
#
if ($?date == 0) then
  set date=`date +%d%h%y`
endif
set thisdate=`echo $date | tr '[A-Z]' '[a-z]'`
#
set thisname = $name:t
if ("${thisname:e}" == "csh") then
  set thisname = $thisname:r
endif
set thisname = `echo $thisname | awk '{print substr($1,1,14)}'`
set thisproject = $HATDATA/$thisdate/$thisname
#
if (! -d $thisproject) exit 0
if (! -d $thisproject/$project) exit 0
if (! -e $thisproject/$project/variables) exit 0
cd /$thisproject/$project
#
#  Set up a flag such that aborting this script will clean up properly.
#
onintr cleanup
#
if ($?setup == 0) then
  rm -fr variables
else
  foreach j ( `echo $setup | sed 's/,/ /g'` )
    grep -iv ${j}/ variables > /tmp/grep.$$
    mv -f /tmp/grep.$$ variables
  end
endif
#
cleanup:
rm -fr /tmp/grep.$$
#
exit 0
