/*
 * STUBS to close off major portions of hatlib.a which are not
 *	 needed for the checker stubs
 *
 *	 Only intended to get checksource+naif (as one binary) to link
 *
 *	march 1993: Created                         Peter Teuben
 *      6-apr-93    properly defined mbusopen_      PJT
 *	6-jun-93    made mystrncpy a static         PJT
 *     29-oct-93    make writeerror check for 'OK'  pjt
 *     16-feb-94    ephem_ -> ephemer_              pjt
 *
 *  Disclaimer: SUN OS Fortran 1.3/4 + F2C interface + SUN OS 4.x
 */

#include <stdio.h>
#include <time.h>
#include <sys/time.h>


#define TWO_PI 6.28318530717958647693
double get_lst();				/* note: this from NEMO !! */


void write_error_(prog, error, plen, elen)
char *prog, *error;
int plen, elen;
{
    char p[256], e[256];
    int i;

    if (strncmp(error,"OK",2)==0) return;
    mystrncpy(p,prog,plen);
    mystrncpy(e,error,elen);
    fprintf(stderr,"WRITE_ERROR: (%s:%s)\n",p,e);
}

static mystrncpy(e,error,elen)
char *e, *error;
int elen;
{
    int i;
    strncpy(e,error,elen);
    for (i=elen; i>0;) {
        e[i--] = 0;
        if (e[i] != ' ') break;
    }
}

void logit_(text, tlen)
char *text;
int tlen;
{
    fprintf(stderr,"LOGIT: ignoring for checker\n");
}
 
void mbusopen_(error, elen)
char *error;
int elen;
{
    int i;

    strcpy(error,"OK");
    for (i=strlen(error); i<elen; i++)  
        error[i] = ' ';                 /* fortran fill */
}

#if 0
int comgetd_(name,dval,nval,error, nlen, elen)
char *name;
double *dval;
int *nval;
char *error;
int nlen;
int elen;
{
    char varname[32];
    int i;

    strncpy(varname,name,nlen);
    varname[nlen] = 0;
    i=nlen-1;
    while (i>0 && varname[i]==' ') varname[i] = 0;
    fprintf(stderr,"COMGETD: %s(%d)\n",varname,*nval);

}
#endif

int peek_(loc)
int *loc;
{
    fprintf(stderr,"PEEK at %d\n",*loc);
    return 0;
}
 

void poke_(loc,contents)
int *loc;
int *contents;
{
    fprintf(stderr,"POKE %d at %d\n",*contents,*loc);
}


static int my_ticks = 0;

void get_times_(ticks,control,month,day,year,ut,lst,error)
int *ticks, *control, *month, *day, *year;
double *ut, *lst;
char *error;
{
    struct timeval tp;
    struct timezone tzp;
    struct tm *t;
    char *ctime();
    long lt, time();
	double get_lst();

    my_ticks++;         /* dunno, just count the number of calls I guess */
    *ticks = my_ticks;

    lt = time(0);
    t = gmtime(&lt);

    /* fprintf(stderr,"GET_TIMES: GMtime=%s\n", asctime(t)); */
    
    switch (*control) {
    case 0:
        return;
    case 1:
    case 2:
    case 3:
        *ut = t->tm_hour + (t->tm_min + (t->tm_sec/60.0))/60.0;
	*ut *= TWO_PI/24.0;
	*lst = get_lst();
	*lst *= TWO_PI/24.0;
        *month = t->tm_mon+1;
        *day = t->tm_mday;
        *year = 1900 + t->tm_year;
        return;
    default:
        printf("### Error: Bad GET_TIMES_ stubbed call\n");
    }
    return;
}


/* This replaces emphemeris.c in $HAT/subs/ephem: */
 
 
void ephemeris_(source,catalog,tjd,freq,s,error, slen, clen, elen)
 char *source;
 char *catalog;
 double *tjd;
 double *freq;
 char *s;               /* opaque poiunter to 'struct asource' */
 char *error;
 int slen, clen, elen;  /* f2c requirements ; by value */
{
        /* call the NAIF routine */
        ephemer_(source,catalog,tjd,freq,s,error,slen,clen,elen);
}

