/* 
 * LST_IO	get and set the time - emulator specific
 *		can also sleep by getting and setting time
 *
 *     10-jun-94    added usleep() for SOLARIS      pjt
 *
 *  lst's are double precision and measured in decimal hours
 */

#include <stdio.h>
#include <math.h>

static FILE *fp = NULL;
static char *file_name = NULL;
static double my_lst;
static void init_lst(), init_name();

double get_lst()
{
    init_lst(0.0);		/* always init to 0 - also sets fp */

    fp = fopen(file_name,"r");
    fscanf(fp,"%lf",&my_lst);
    fclose(fp);

    return my_lst;
}

set_lst(new_lst)
double new_lst;
{
    while (new_lst > 24)
        new_lst -= 24;
    init_name();
    fp = fopen(file_name,"w");
    if (fp==NULL) {
       printf("### Warning: set_lst: Cannot set time in %s\n",file_name);
       return;
    }
    fprintf(fp,"%lf",new_lst);
    fclose(fp);
}

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

static void init_lst(value)
double value;
{
    init_name();
    fp = fopen(file_name,"r");      /* try and open */
    if (fp==NULL) {
        fp  = fopen(file_name,"w");
        if (fp==NULL) {
            fprintf(stderr,"Cannot set time in %s\n",file_name);
            return;
        }
        fprintf(stderr,"Initialize LST to %g\n",value);
        fclose(fp);
        set_lst(value);
    } else
        fclose(fp);
}

static void init_name()
{
    char *cp, *getenv(), *malloc(), name[MAXPATHLEN];

    if (file_name==NULL) {
        cp = getenv("HATCOM");
        if (cp==NULL) {
            fprintf(stderr,"No env.var. $HATCOM\n");
            exit(1);
        }
        sprintf(name,"%s.lst",cp);
        file_name = malloc(strlen(name)+1);
        strcpy(file_name,name);
    }
}


/* 
 * sleep:
 *  !!! this overrides the system library sleep(3V) !!!
 */

int sleep(seconds)
unsigned seconds;
{
    double lst;

    if (seconds <= 0) return 0;
    fprintf(stderr,"### Warning: sleeping %d seconds",seconds);
    lst = get_lst() + (double)seconds/3600.0;
    set_lst(lst);
    return 0;
}


/* warning: returns space to static string, don't reuse */

char *show_lst(lst)
double lst;
{
    static char msg[10];
    int hh = (int)floor(lst);
    int mm = (int) (0.5 + 60.0*(lst-hh));

    sprintf(msg,"%-02d:%-02d", hh, mm);
    return msg;
}



#ifdef SOLARIS

int usleep(unsigned useconds)
{
	/* does nothing */
}
#endif

