/* 
 *  mkants:  transform a human readable antennae configuration
 *		into a bx,by,bz machine readable file....
 *
 *	20-mar-90  proper bx,by,bz transformation  - Peter T. and Joe M.
 *	 2-may-91  WSRT stuff
 */

#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>

#define MAXANTS  100


float lat = 39.0;		/* default latitude */

main (ac,av)
int ac;
char *av[];
{
    FILE *fp;
    char *text, *cp, **words, *malloc(), *strtok();
    int nwords, size;
    struct stat fpbuf;

    if (ac<=1) {
        usage(av[0]);
        exit(0);
    }

    if (ac==2) {
        fp = fopen(av[1],"r");
        if (fp==NULL) {
            fprintf(stderr,"File %s could not be opened\n",av[1]);
            usage(av[0]);
            exit(1);
        }
        if (fstat(fileno(fp),&fpbuf) < 0) {
            fprintf(stderr,"Error accessing file %s\n",av[1]);
            usage(av[0]);
            exit(1);
        }
        size = fpbuf.st_size;           /* file size in bytes */
        text = malloc(size+1);
        if (text==NULL) {
            fprintf(stderr,"Out of memory\n");
            exit(1);
        }
        if (fread(text,1,size,fp) != size) {
            fprintf(stderr,"Warning: error reading into buffer\n");
        }
        words = (char **) malloc( MAXANTS * (sizeof (char **)) );
        nwords = 0;
        cp = strtok(text," ,\n\t");
        while (cp != NULL) {
            words[nwords++] = cp;
            cp = strtok(NULL," ,\n\t");
        }
        process (nwords, words);
    } else
        process(ac-1,&av[1]);       /* take from command line args */
}

#define LAT 39.0
#define DPR 3.141592/180.0

process(nw,w)
int nw;
char *w[];
{
    char *cp;
    int i,l;
    float x,y,z, bx,by,bz, ewmult, nsmult;
    double sin(), cos();
    float coslat,sinlat; 


    coslat = cos(LAT*DPR);
    sinlat = sin(LAT*DPR);
    for (i=0; i<nw; i++) {
        cp = w[i];                  /* point to station ID */
	nsmult = 1.0;
        ewmult = 1.0;                       /* defaults */
        x = y = z = 0.0;                    /* defaults */
        switch (cp[strlen(cp)-1]) {
#if 0
          case 'S':			/* cut that S arm out */
          case 's':
              nsmult = -1.0;
#endif
          case 'N':
          case 'n':
              cp[strlen(cp)-1] = NULL;
              y = nsmult * (float) atoi(cp);
              break;
          case 'W':
          case 'w':
              ewmult = -1.0;
          case 'E':
          case 'e':
              cp[strlen(cp)-1] = NULL;
              x = ewmult * (float) atoi(cp);
              break;
          case '0':
              if (atoi(cp) != 0) {
                fprintf(stderr,"Error: station ID (%s) should be 0\n",cp);
		fprintf(stderr,"       or of the form (%s[NSEW])\n",cp);
                exit(1);
              }
              break;
           default:
              fprintf(stderr,"Error: illegal station ID (%s)\n",cp);
              exit(1);
        }
	/* Now we have the old x,y,z */
	/* transform to bx,by,bz */
    bx = -sinlat*y;
    by = x;
    bz = coslat*y;
    printf("%12.4f%12.4f%12.4f\n",bx,by,bz);
    }
}

    


usage(name)
char *name;
{
    fprintf(stderr,"Usage: %s station-ID-file\n",name);
    fprintf(stderr,"   or: %s station-ID-list\n",name);
    fprintf(stderr,"\n");
    fprintf(stderr,"Station ID's are e.g. 0 10N 40E 80W 100N\n");
}
